import { initAccessibilityCssHelper } from '../accessibility/accessibility-css-helper';
import { initAccessibilityFontScale } from '../accessibility/font-scale';
import { CoreTypes } from '../core-types';
import { CSSUtils } from '../css/system-classes';
import { Device, Screen } from '../platform';
import { profile } from '../profiling';
import { Trace } from '../trace';
import { Builder } from '../ui/builder';
import * as bindableResources from '../ui/core/bindable/bindable-resources';
// prettier-ignore
var ORIENTATION_CSS_CLASSES = [
    "".concat(CSSUtils.CLASS_PREFIX).concat(CoreTypes.DeviceOrientation.portrait),
    "".concat(CSSUtils.CLASS_PREFIX).concat(CoreTypes.DeviceOrientation.landscape),
    "".concat(CSSUtils.CLASS_PREFIX).concat(CoreTypes.DeviceOrientation.unknown),
];
// prettier-ignore
var SYSTEM_APPEARANCE_CSS_CLASSES = [
    "".concat(CSSUtils.CLASS_PREFIX).concat(CoreTypes.SystemAppearance.light),
    "".concat(CSSUtils.CLASS_PREFIX).concat(CoreTypes.SystemAppearance.dark),
];
var globalEvents = global.NativeScriptGlobals.events;
var ApplicationCommon = /** @class */ (function () {
    /**
     * @internal - should not be constructed by the user.
     */
    function ApplicationCommon() {
        var _this = this;
        this.launchEvent = 'launch';
        this.suspendEvent = 'suspend';
        this.displayedEvent = 'displayed';
        this.backgroundEvent = 'background';
        this.foregroundEvent = 'foreground';
        this.resumeEvent = 'resume';
        this.exitEvent = 'exit';
        this.lowMemoryEvent = 'lowMemory';
        this.uncaughtErrorEvent = 'uncaughtError';
        this.discardedErrorEvent = 'discardedError';
        this.orientationChangedEvent = 'orientationChanged';
        this.systemAppearanceChangedEvent = 'systemAppearanceChanged';
        this.fontScaleChangedEvent = 'fontScaleChanged';
        this.livesyncEvent = 'livesync';
        this.loadAppCssEvent = 'loadAppCss';
        this.cssChangedEvent = 'cssChanged';
        this.initRootViewEvent = 'initRootView';
        // Application events go through the global events.
        this.on = globalEvents.on.bind(globalEvents);
        this.once = globalEvents.once.bind(globalEvents);
        this.off = globalEvents.off.bind(globalEvents);
        this.notify = globalEvents.notify.bind(globalEvents);
        this.hasListeners = globalEvents.hasListeners.bind(globalEvents);
        this.cssFile = './app.css';
        /**
         * Boolean to enable/disable systemAppearanceChanged
         */
        this.autoSystemAppearanceChanged = true;
        this._inBackground = false;
        this._suspended = false;
        this.started = false;
        global.NativeScriptGlobals.appInstanceReady = true;
        global.__onUncaughtError = function (error) {
            _this.notify({
                eventName: _this.uncaughtErrorEvent,
                object: _this,
                android: error,
                ios: error,
                error: error,
            });
        };
        global.__onDiscardedError = function (error) {
            _this.notify({
                eventName: _this.discardedErrorEvent,
                object: _this,
                error: error,
            });
        };
        global.__onLiveSync = function (context) {
            if (_this.suspended) {
                return;
            }
            var rootView = _this.getRootView();
            _this.livesync(rootView, context);
        };
    }
    /**
     * @internal
     */
    ApplicationCommon.prototype.livesync = function (rootView, context) {
        this.notify({ eventName: this.livesyncEvent, object: this });
        var liveSyncCore = global.__onLiveSyncCore;
        var reapplyAppStyles = false;
        // ModuleContext is available only for Hot Module Replacement
        if (context && context.path) {
            var styleExtensions = ['css', 'scss'];
            var appStylesFullFileName = this.getCssFileName();
            var appStylesFileName_1 = appStylesFullFileName.substring(0, appStylesFullFileName.lastIndexOf('.') + 1);
            reapplyAppStyles = styleExtensions.some(function (ext) { return context.path === appStylesFileName_1.concat(ext); });
        }
        // Handle application styles
        if (rootView && reapplyAppStyles) {
            rootView._onCssStateChange();
        }
        else if (liveSyncCore) {
            liveSyncCore(context);
        }
    };
    /**
     * Applies the the `newCssClass` to the `rootView` and removes all other css classes from `cssClasses`
     * previously applied to the `rootView`.
     * @param rootView
     * @param cssClasses
     * @param newCssClass
     * @param skipCssUpdate
     */
    ApplicationCommon.prototype.applyCssClass = function (rootView, cssClasses, newCssClass, skipCssUpdate) {
        var _this = this;
        if (skipCssUpdate === void 0) { skipCssUpdate = false; }
        if (!rootView.cssClasses.has(newCssClass)) {
            cssClasses.forEach(function (cssClass) { return _this.removeCssClass(rootView, cssClass); });
            this.addCssClass(rootView, newCssClass);
            this.increaseStyleScopeApplicationCssSelectorVersion(rootView);
            if (!skipCssUpdate) {
                rootView._onCssStateChange();
            }
            if (Trace.isEnabled()) {
                var rootCssClasses = Array.from(rootView.cssClasses);
                Trace.write("Applying root css class: ".concat(newCssClass, ". rootView css classes: ").concat(rootCssClasses.join(' ')), Trace.categories.Style);
            }
        }
    };
    ApplicationCommon.prototype.addCssClass = function (rootView, cssClass) {
        CSSUtils.pushToSystemCssClasses(cssClass);
        rootView.cssClasses.add(cssClass);
    };
    ApplicationCommon.prototype.removeCssClass = function (rootView, cssClass) {
        CSSUtils.removeSystemCssClass(cssClass);
        rootView.cssClasses.delete(cssClass);
    };
    ApplicationCommon.prototype.increaseStyleScopeApplicationCssSelectorVersion = function (rootView) {
        var _a, _b;
        var styleScope = (_a = rootView._styleScope) !== null && _a !== void 0 ? _a : (_b = rootView === null || rootView === void 0 ? void 0 : rootView.currentPage) === null || _b === void 0 ? void 0 : _b._styleScope;
        if (styleScope) {
            styleScope._increaseApplicationCssSelectorVersion();
        }
    };
    ApplicationCommon.prototype.setRootViewCSSClasses = function (rootView) {
        var platform = Device.os.toLowerCase();
        var deviceType = Device.deviceType.toLowerCase();
        var orientation = this.orientation();
        var systemAppearance = this.systemAppearance();
        if (platform) {
            CSSUtils.pushToSystemCssClasses("".concat(CSSUtils.CLASS_PREFIX).concat(platform));
        }
        if (deviceType) {
            CSSUtils.pushToSystemCssClasses("".concat(CSSUtils.CLASS_PREFIX).concat(deviceType));
        }
        if (orientation) {
            CSSUtils.pushToSystemCssClasses("".concat(CSSUtils.CLASS_PREFIX).concat(orientation));
        }
        if (systemAppearance) {
            CSSUtils.pushToSystemCssClasses("".concat(CSSUtils.CLASS_PREFIX).concat(systemAppearance));
        }
        rootView.cssClasses.add(CSSUtils.ROOT_VIEW_CSS_CLASS);
        var rootViewCssClasses = CSSUtils.getSystemCssClasses();
        rootViewCssClasses.forEach(function (c) { return rootView.cssClasses.add(c); });
        this.increaseStyleScopeApplicationCssSelectorVersion(rootView);
        rootView._onCssStateChange();
        if (Trace.isEnabled()) {
            var rootCssClasses = Array.from(rootView.cssClasses);
            Trace.write("Setting root css classes: ".concat(rootCssClasses.join(' ')), Trace.categories.Style);
        }
    };
    /**
     * iOS Only
     * Dynamically change the preferred frame rate
     * For devices (iOS 15+) which support min/max/preferred frame rate you can specify ranges
     * For devices (iOS < 15), you can specify the max frame rate
     * see: https://developer.apple.com/documentation/quartzcore/optimizing_promotion_refresh_rates_for_iphone_13_pro_and_ipad_pro
     * To use, ensure your Info.plist has:
     * ```xml
     *   <key>CADisableMinimumFrameDurationOnPhone</key>
     *   <true/>
     * ```
     * @param options { min?: number; max?: number; preferred?: number }
     */
    ApplicationCommon.prototype.setMaxRefreshRate = function (options) {
        // implement in platform specific files (iOS only for now)
    };
    /**
     * @returns The main entry of the application
     */
    ApplicationCommon.prototype.getMainEntry = function () {
        return this.mainEntry;
    };
    ApplicationCommon.prototype.notifyLaunch = function (additionalLanchEventData) {
        var launchArgs = __assign({ eventName: this.launchEvent, object: this, ios: this.ios, android: this.android }, additionalLanchEventData);
        this.notify(launchArgs);
        this.loadAppCss();
        return launchArgs.root;
    };
    ApplicationCommon.prototype.createRootView = function (view, fireLaunchEvent, additionalLanchEventData) {
        if (fireLaunchEvent === void 0) { fireLaunchEvent = false; }
        var rootView = view;
        if (!rootView) {
            if (fireLaunchEvent) {
                rootView = this.notifyLaunch(additionalLanchEventData);
                // useful for integrations that would like to set rootView asynchronously after app launch
                if (rootView === null) {
                    return null;
                }
            }
            if (!rootView) {
                // try to navigate to the mainEntry (if specified)
                if (!this.mainEntry) {
                    throw new Error('Main entry is missing. App cannot be started. Verify app bootstrap.');
                }
                rootView = Builder.createViewFromEntry(this.mainEntry);
            }
        }
        return rootView;
    };
    ApplicationCommon.prototype.getRootView = function () {
        throw new Error('getRootView() Not implemented.');
    };
    ApplicationCommon.prototype.resetRootView = function (entry) {
        this.mainEntry = typeof entry === 'string' ? { moduleName: entry } : entry;
        // rest of implementation is platform specific
    };
    ApplicationCommon.prototype.initRootView = function (rootView) {
        this.setRootViewCSSClasses(rootView);
        initAccessibilityCssHelper();
        initAccessibilityFontScale();
        this.notify({ eventName: this.initRootViewEvent, rootView: rootView });
    };
    /**
     * Get application level static resources.
     */
    ApplicationCommon.prototype.getResources = function () {
        return bindableResources.get();
    };
    /**
     * Set application level static resources.
     */
    ApplicationCommon.prototype.setResources = function (res) {
        bindableResources.set(res);
    };
    /**
     * Sets css file name for the application.
     */
    ApplicationCommon.prototype.setCssFileName = function (cssFileName) {
        this.cssFile = cssFileName;
        this.notify({
            eventName: this.cssChangedEvent,
            object: this,
            cssFile: cssFileName,
        });
    };
    /**
     * Gets css file name for the application.
     */
    ApplicationCommon.prototype.getCssFileName = function () {
        return this.cssFile;
    };
    /**
     * Loads immediately the app.css.
     * By default the app.css file is loaded shortly after "loaded".
     * For the Android snapshot the CSS can be parsed during the snapshot generation,
     * as the CSS does not depend on runtime APIs, and loadAppCss will be called explicitly.
     */
    ApplicationCommon.prototype.loadAppCss = function () {
        try {
            this.notify({
                eventName: this.loadAppCssEvent,
                object: this,
                ios: this.ios,
                android: this.android,
                cssFile: this.getCssFileName(),
            });
        }
        catch (e) {
            if (Trace.isEnabled()) {
                Trace.write("The app CSS file ".concat(this.getCssFileName(), " couldn't be loaded!"), Trace.categories.Style, Trace.messageType.warn);
            }
        }
    };
    ApplicationCommon.prototype.addCss = function (cssText, attributeScoped) {
        this.notify({
            eventName: this.cssChangedEvent,
            object: this,
            cssText: cssText,
        });
        if (!attributeScoped) {
            var rootView = this.getRootView();
            if (rootView) {
                rootView._onCssStateChange();
            }
        }
    };
    ApplicationCommon.prototype.run = function (entry) {
        throw new Error('run() Not implemented.');
    };
    ApplicationCommon.prototype.getOrientation = function () {
        // override in platform specific Application class
        throw new Error('getOrientation() not implemented');
    };
    ApplicationCommon.prototype.setOrientation = function (value) {
        if (this._orientation === value) {
            return;
        }
        this._orientation = value;
        // Update metrics early enough regardless of the existence of root view
        // Also, CSS will use the correct size values during update trigger
        Screen.mainScreen._updateMetrics();
        this.orientationChanged(this.getRootView(), value);
        this.notify({
            eventName: this.orientationChangedEvent,
            android: this.android,
            ios: this.ios,
            newValue: value,
            object: this,
        });
    };
    ApplicationCommon.prototype.orientation = function () {
        var _a;
        return ((_a = this._orientation) !== null && _a !== void 0 ? _a : (this._orientation = this.getOrientation()));
    };
    ApplicationCommon.prototype.orientationChanged = function (rootView, newOrientation) {
        var _this = this;
        if (!rootView) {
            return;
        }
        var newOrientationCssClass = "".concat(CSSUtils.CLASS_PREFIX).concat(newOrientation);
        this.applyCssClass(rootView, ORIENTATION_CSS_CLASSES, newOrientationCssClass, true);
        var rootModalViews = rootView._getRootModalViews();
        rootModalViews.forEach(function (rootModalView) {
            _this.applyCssClass(rootModalView, ORIENTATION_CSS_CLASSES, newOrientationCssClass, true);
            // Trigger state change for root modal view classes and media queries
            rootModalView._onCssStateChange();
        });
        // Trigger state change for root view classes and media queries
        rootView._onCssStateChange();
    };
    ApplicationCommon.prototype.getNativeApplication = function () {
        // override in platform specific Application class
        throw new Error('getNativeApplication() not implemented');
    };
    ApplicationCommon.prototype.hasLaunched = function () {
        return global.NativeScriptGlobals && global.NativeScriptGlobals.launched;
    };
    ApplicationCommon.prototype.getSystemAppearance = function () {
        // override in platform specific Application class
        throw new Error('getSystemAppearance() not implemented');
    };
    ApplicationCommon.prototype.setSystemAppearance = function (value) {
        if (this._systemAppearance === value) {
            return;
        }
        this._systemAppearance = value;
        this.systemAppearanceChanged(this.getRootView(), value);
        this.notify({
            eventName: this.systemAppearanceChangedEvent,
            android: this.android,
            ios: this.ios,
            newValue: value,
            object: this,
        });
    };
    ApplicationCommon.prototype.systemAppearance = function () {
        var _a;
        // return cached value, or get it from the platform specific override
        return ((_a = this._systemAppearance) !== null && _a !== void 0 ? _a : (this._systemAppearance = this.getSystemAppearance()));
    };
    /**
     * enable/disable systemAppearanceChanged
     */
    ApplicationCommon.prototype.setAutoSystemAppearanceChanged = function (value) {
        this.autoSystemAppearanceChanged = value;
    };
    /**
     * Updates root view classes including those of modals
     * @param rootView the root view
     * @param newSystemAppearance the new appearance change
     */
    ApplicationCommon.prototype.systemAppearanceChanged = function (rootView, newSystemAppearance) {
        var _this = this;
        if (!rootView || !this.autoSystemAppearanceChanged) {
            return;
        }
        var newSystemAppearanceCssClass = "".concat(CSSUtils.CLASS_PREFIX).concat(newSystemAppearance);
        this.applyCssClass(rootView, SYSTEM_APPEARANCE_CSS_CLASSES, newSystemAppearanceCssClass, true);
        var rootModalViews = rootView._getRootModalViews();
        rootModalViews.forEach(function (rootModalView) {
            _this.applyCssClass(rootModalView, SYSTEM_APPEARANCE_CSS_CLASSES, newSystemAppearanceCssClass, true);
            // Trigger state change for root modal view classes and media queries
            rootModalView._onCssStateChange();
        });
        // Trigger state change for root view classes and media queries
        rootView._onCssStateChange();
    };
    Object.defineProperty(ApplicationCommon.prototype, "inBackground", {
        get: function () {
            return this._inBackground;
        },
        enumerable: false,
        configurable: true
    });
    ApplicationCommon.prototype.setInBackground = function (value, additonalData) {
        this._inBackground = value;
        this.notify(__assign({ eventName: value ? this.backgroundEvent : this.foregroundEvent, object: this, ios: this.ios }, additonalData));
    };
    Object.defineProperty(ApplicationCommon.prototype, "suspended", {
        get: function () {
            return this._suspended;
        },
        enumerable: false,
        configurable: true
    });
    ApplicationCommon.prototype.setSuspended = function (value, additonalData) {
        this._suspended = value;
        this.notify(__assign({ eventName: value ? this.suspendEvent : this.resumeEvent, object: this, ios: this.ios, android: this.android }, additonalData));
    };
    Object.defineProperty(ApplicationCommon.prototype, "android", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ApplicationCommon.prototype, "ios", {
        get: function () {
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ApplicationCommon.prototype, "AndroidApplication", {
        get: function () {
            return this.android;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ApplicationCommon.prototype, "iOSApplication", {
        get: function () {
            return this.ios;
        },
        enumerable: false,
        configurable: true
    });
    // Expose statically for backwards compat on AndroidApplication.on etc.
    /**
     * @deprecated Use `Application.android.on()` instead.
     */
    ApplicationCommon.on = globalEvents.on.bind(globalEvents);
    /**
     * @deprecated Use `Application.android.once()` instead.
     */
    ApplicationCommon.once = globalEvents.once.bind(globalEvents);
    /**
     * @deprecated Use `Application.android.off()` instead.
     */
    ApplicationCommon.off = globalEvents.off.bind(globalEvents);
    /**
     * @deprecated Use `Application.android.notify()` instead.
     */
    ApplicationCommon.notify = globalEvents.notify.bind(globalEvents);
    /**
     * @deprecated Use `Application.android.hasListeners()` instead.
     */
    ApplicationCommon.hasListeners = globalEvents.hasListeners.bind(globalEvents);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", Function)
    ], ApplicationCommon.prototype, "notifyLaunch", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Function, Object, Object]),
        __metadata("design:returntype", void 0)
    ], ApplicationCommon.prototype, "createRootView", null);
    return ApplicationCommon;
}());
export { ApplicationCommon };
//# sourceMappingURL=application-common.js.map