var _wrappedIndex = 0;
/**
 * Helper class that is used to fire property change even when real object is the same.
 * By default property change will not be fired for a same object.
 * By wrapping object into a WrappedValue instance `same object restriction` will be passed.
 */
var WrappedValue = /** @class */ (function () {
    /**
     * Creates an instance of WrappedValue object.
     * @param wrapped - the real value which should be wrapped.
     */
    function WrappedValue(
    /**
     * Property which holds the real value.
     */
    wrapped) {
        this.wrapped = wrapped;
    }
    /**
     * Gets the real value of previously wrappedValue.
     * @param value - Value that should be unwraped. If there is no wrappedValue property of the value object then value will be returned.
     */
    WrappedValue.unwrap = function (value) {
        return value instanceof WrappedValue ? value.wrapped : value;
    };
    /**
     * Returns an instance of WrappedValue. The actual instance is get from a WrappedValues pool.
     * @param value - Value that should be wrapped.
     */
    WrappedValue.wrap = function (value) {
        var w = _wrappedValues[_wrappedIndex++ % 5];
        w.wrapped = value;
        return w;
    };
    return WrappedValue;
}());
export { WrappedValue };
var _wrappedValues = [new WrappedValue(null), new WrappedValue(null), new WrappedValue(null), new WrappedValue(null), new WrappedValue(null)];
var _globalEventHandlers = {};
/**
 * Observable is used when you want to be notified when a change occurs. Use on/off methods to add/remove listener.
 * Please note that should you be using the `new Observable({})` constructor, it is **obsolete** since v3.0,
 * and you have to migrate to the "data/observable" `fromObject({})` or the `fromObjectRecursive({})` functions.
 */
var Observable = /** @class */ (function () {
    function Observable() {
        this._observers = {};
    }
    /**
     * Gets the value of the specified property.
     */
    Observable.prototype.get = function (name) {
        return this[name];
    };
    /**
     * Updates the specified property with the provided value.
     */
    Observable.prototype.set = function (name, value) {
        // TODO: Parameter validation
        var oldValue = this[name];
        if (this[name] === value) {
            return;
        }
        var newValue = WrappedValue.unwrap(value);
        this[name] = newValue;
        this.notifyPropertyChange(name, newValue, oldValue);
    };
    /**
     * Updates the specified property with the provided value and raises a property change event and a specific change event based on the property name.
     */
    Observable.prototype.setProperty = function (name, value) {
        var oldValue = this[name];
        if (this[name] === value) {
            return;
        }
        this[name] = value;
        this.notifyPropertyChange(name, value, oldValue);
        var specificPropertyChangeEventName = name + 'Change';
        if (this.hasListeners(specificPropertyChangeEventName)) {
            var eventData = this._createPropertyChangeData(name, value, oldValue);
            eventData.eventName = specificPropertyChangeEventName;
            this.notify(eventData);
        }
    };
    /**
     * Adds a listener for the specified event name.
     *
     * @param eventName The name of the event.
     * @param callback The event listener to add. Will be called when an event of
     * the given name is raised.
     * @param thisArg An optional parameter which, when set, will be bound as the
     * `this` context when the callback is called. Falsy values will be not be
     * bound.
     */
    Observable.prototype.on = function (eventName, callback, thisArg) {
        this.addEventListener(eventName, callback, thisArg);
    };
    /**
     * Adds a listener for the specified event name, which, once fired, will
     * remove itself.
     *
     * @param eventName The name of the event.
     * @param callback The event listener to add. Will be called when an event of
     * the given name is raised.
     * @param thisArg An optional parameter which, when set, will be bound as the
     * `this` context when the callback is called. Falsy values will be not be
     * bound.
     */
    Observable.prototype.once = function (eventName, callback, thisArg) {
        this.addEventListener(eventName, callback, thisArg, true);
    };
    /**
     * Removes the listener(s) for the specified event name.
     *
     * @param eventName The name of the event.
     * @param callback An optional specific event listener to remove (if omitted,
     * all event listeners by this name will be removed).
     * @param thisArg An optional parameter which, when set, will be used to
     * refine search of the correct event listener to be removed.
     */
    Observable.prototype.off = function (eventName, callback, thisArg) {
        this.removeEventListener(eventName, callback, thisArg);
    };
    /**
     * Adds a listener for the specified event name.
     * @param eventName Name of the event to attach to.
     * @param callback A function to be called when some of the specified event(s) is raised.
     * @param thisArg An optional parameter which when set will be used as "this" in callback method call.
     */
    Observable.prototype.addEventListener = function (eventName, callback, thisArg, once) {
        once = once || undefined;
        thisArg = thisArg || undefined;
        if (typeof eventName !== 'string') {
            throw new TypeError('Event name must be a string.');
        }
        if (typeof callback !== 'function') {
            throw new TypeError('Callback, if provided, must be a function.');
        }
        var list = this._getEventList(eventName, true);
        if (Observable._indexOfListener(list, callback, thisArg) !== -1) {
            // Already added.
            return;
        }
        list.push({
            callback: callback,
            thisArg: thisArg,
            once: once,
        });
    };
    /**
     * Removes listener(s) for the specified event name.
     * @param eventName Name of the event to attach to.
     * @param callback An optional parameter pointing to a specific listener. If not defined, all listeners for the event names will be removed.
     * @param thisArg An optional parameter which when set will be used to refine search of the correct callback which will be removed as event listener.
     */
    Observable.prototype.removeEventListener = function (eventName, callback, thisArg) {
        thisArg = thisArg || undefined;
        if (typeof eventName !== 'string') {
            throw new TypeError('Events name(s) must be string.');
        }
        if (callback && typeof callback !== 'function') {
            throw new TypeError('callback must be function.');
        }
        var entries = this._observers[eventName];
        if (!entries) {
            return;
        }
        Observable.innerRemoveEventListener(entries, callback, thisArg);
        if (!entries.length) {
            // Clear all entries of this type
            delete this._observers[eventName];
        }
    };
    /**
     * Please avoid using the static event-handling APIs as they will be removed
     * in future.
     * @deprecated
     */
    Observable.on = function (eventName, callback, thisArg, once) {
        this.addEventListener(eventName, callback, thisArg, once);
    };
    /**
     * Please avoid using the static event-handling APIs as they will be removed
     * in future.
     * @deprecated
     */
    Observable.once = function (eventName, callback, thisArg) {
        this.addEventListener(eventName, callback, thisArg, true);
    };
    /**
     * Please avoid using the static event-handling APIs as they will be removed
     * in future.
     * @deprecated
     */
    Observable.off = function (eventName, callback, thisArg) {
        this.removeEventListener(eventName, callback, thisArg);
    };
    Observable.innerRemoveEventListener = function (entries, callback, thisArg) {
        for (var i = 0; i < entries.length; i++) {
            var entry = entries[i];
            // If we have a `thisArg`, refine on both `callback` and `thisArg`.
            if (thisArg && (entry.callback !== callback || entry.thisArg !== thisArg)) {
                continue;
            }
            // If we don't have a `thisArg`, refine only on `callback`.
            if (callback && entry.callback !== callback) {
                continue;
            }
            // If we have neither `thisArg` nor `callback`, just remove all events
            // of this type regardless.
            entries.splice(i, 1);
            i--;
        }
    };
    /**
     * Please avoid using the static event-handling APIs as they will be removed
     * in future.
     * @deprecated
     */
    Observable.removeEventListener = function (eventName, callback, thisArg) {
        var _a;
        thisArg = thisArg || undefined;
        if (typeof eventName !== 'string') {
            throw new TypeError('Event name must be a string.');
        }
        if (callback && typeof callback !== 'function') {
            throw new TypeError('Callback, if provided, must be function.');
        }
        var eventClass = this.name === 'Observable' ? '*' : this.name;
        var entries = (_a = _globalEventHandlers === null || _globalEventHandlers === void 0 ? void 0 : _globalEventHandlers[eventClass]) === null || _a === void 0 ? void 0 : _a[eventName];
        if (!entries) {
            return;
        }
        Observable.innerRemoveEventListener(entries, callback, thisArg);
        if (!entries.length) {
            // Clear all entries of this type
            delete _globalEventHandlers[eventClass][eventName];
        }
        // Clear the primary class grouping if no list are left
        var keys = Object.keys(_globalEventHandlers[eventClass]);
        if (keys.length === 0) {
            delete _globalEventHandlers[eventClass];
        }
    };
    /**
     * Please avoid using the static event-handling APIs as they will be removed
     * in future.
     * @deprecated
     */
    Observable.addEventListener = function (eventName, callback, thisArg, once) {
        once = once || undefined;
        thisArg = thisArg || undefined;
        if (typeof eventName !== 'string') {
            throw new TypeError('Event name must be a string.');
        }
        if (typeof callback !== 'function') {
            throw new TypeError('Callback must be a function.');
        }
        var eventClass = this.name === 'Observable' ? '*' : this.name;
        if (!_globalEventHandlers[eventClass]) {
            _globalEventHandlers[eventClass] = {};
        }
        if (!_globalEventHandlers[eventClass][eventName]) {
            _globalEventHandlers[eventClass][eventName] = [];
        }
        if (Observable._indexOfListener(_globalEventHandlers[eventClass][eventName], callback, thisArg) !== -1) {
            // Already added.
            return;
        }
        _globalEventHandlers[eventClass][eventName].push({ callback: callback, thisArg: thisArg, once: once });
    };
    Observable.prototype._globalNotify = function (eventClass, eventType, data) {
        // Check for the Global handlers for JUST this class
        if (_globalEventHandlers[eventClass]) {
            var eventName = data.eventName + eventType;
            var entries = _globalEventHandlers[eventClass][eventName];
            if (entries) {
                Observable._handleEvent(entries, data);
            }
        }
        // Check for the Global handlers for ALL classes
        if (_globalEventHandlers['*']) {
            var eventName = data.eventName + eventType;
            var entries = _globalEventHandlers['*'][eventName];
            if (entries) {
                Observable._handleEvent(entries, data);
            }
        }
    };
    /**
     * Notify this Observable instance with some data. This causes all event
     * handlers on the Observable instance to be called, as well as any 'global'
     * event handlers set on the instance's class.
     *
     * @param data an object that satisfies the EventData interface, though with
     * an optional 'object' property. If left undefined, the 'object' property
     * will implicitly be set as this Observable instance.
     */
    Observable.prototype.notify = function (data) {
        data.object = data.object || this;
        var dataWithObject = data;
        var eventClass = this.constructor.name;
        this._globalNotify(eventClass, 'First', dataWithObject);
        var observers = this._observers[data.eventName];
        if (observers) {
            Observable._handleEvent(observers, dataWithObject);
        }
        this._globalNotify(eventClass, '', dataWithObject);
    };
    Observable._handleEvent = function (observers, data) {
        if (!observers.length) {
            return;
        }
        for (var i = observers.length - 1; i >= 0; i--) {
            var entry = observers[i];
            if (!entry) {
                continue;
            }
            if (entry.once) {
                observers.splice(i, 1);
            }
            var returnValue = entry.thisArg ? entry.callback.apply(entry.thisArg, [data]) : entry.callback(data);
            // This ensures errors thrown inside asynchronous functions do not get swallowed
            if (returnValue instanceof Promise) {
                returnValue.catch(function (err) {
                    console.error(err);
                });
            }
        }
    };
    /**
     * Notifies all the registered listeners for the property change event.
     */
    Observable.prototype.notifyPropertyChange = function (name, value, oldValue) {
        this.notify(this._createPropertyChangeData(name, value, oldValue));
    };
    /**
     * Checks whether a listener is registered for the specified event name.
     * @param eventName The name of the event to check for.
     */
    Observable.prototype.hasListeners = function (eventName) {
        return eventName in this._observers;
    };
    /**
     * This method is intended to be overriden by inheritors to provide additional implementation.
     */
    Observable.prototype._createPropertyChangeData = function (propertyName, value, oldValue) {
        return {
            eventName: Observable.propertyChangeEvent,
            object: this,
            propertyName: propertyName,
            value: value,
            oldValue: oldValue,
        };
    };
    Observable.prototype._emit = function (eventName) {
        this.notify({ eventName: eventName, object: this });
    };
    Observable.prototype._getEventList = function (eventName, createIfNeeded) {
        if (!eventName) {
            throw new TypeError('eventName must be a valid string.');
        }
        var list = this._observers[eventName];
        if (!list && createIfNeeded) {
            list = [];
            this._observers[eventName] = list;
        }
        return list;
    };
    Observable._indexOfListener = function (list, callback, thisArg) {
        thisArg = thisArg || undefined;
        return list.findIndex(function (entry) { return entry.callback === callback && entry.thisArg === thisArg; });
    };
    /**
     * String value used when hooking to propertyChange event.
     * @nsEvent  {PropertyChangeData} propertyChange
     */
    Observable.propertyChangeEvent = 'propertyChange';
    return Observable;
}());
export { Observable };
var ObservableFromObject = /** @class */ (function (_super) {
    __extends(ObservableFromObject, _super);
    function ObservableFromObject() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._map = {};
        return _this;
    }
    ObservableFromObject.prototype.get = function (name) {
        return this._map[name];
    };
    /**
     * Updates the specified property with the provided value.
     */
    ObservableFromObject.prototype.set = function (name, value) {
        var currentValue = this._map[name];
        if (currentValue === value) {
            return;
        }
        var newValue = WrappedValue.unwrap(value);
        this._map[name] = newValue;
        this.notifyPropertyChange(name, newValue, currentValue);
    };
    return ObservableFromObject;
}(Observable));
function defineNewProperty(target, propertyName) {
    Object.defineProperty(target, propertyName, {
        get: function () {
            return target._map[propertyName];
        },
        set: function (value) {
            target.set(propertyName, value);
        },
        enumerable: true,
        configurable: true,
    });
}
function addPropertiesFromObject(observable, source, recursive) {
    if (recursive === void 0) { recursive = false; }
    Object.keys(source).forEach(function (prop) {
        var value = source[prop];
        if (recursive && !Array.isArray(value) && value && typeof value === 'object' && !(value instanceof Observable)) {
            value = fromObjectRecursive(value);
        }
        defineNewProperty(observable, prop);
        observable.set(prop, value);
    });
}
/**
 * Creates an Observable instance and sets its properties according to the supplied JavaScript object.
 * param obj - A JavaScript object used to initialize nativescript Observable instance.
 */
export function fromObject(source) {
    var observable = new ObservableFromObject();
    addPropertiesFromObject(observable, source, false);
    return observable;
}
/**
 * Creates an Observable instance and sets its properties according to the supplied JavaScript object.
 * This function will create new Observable for each nested object (expect arrays and functions) from supplied JavaScript object.
 * param obj - A JavaScript object used to initialize nativescript Observable instance.
 */
export function fromObjectRecursive(source) {
    var observable = new ObservableFromObject();
    addPropertiesFromObject(observable, source, true);
    return observable;
}
//# sourceMappingURL=index.js.map