import { Observable } from '../observable';
import { ChangeType } from '../observable-array';
/**
 * Advanced array like class that helps loading items on demand.
 */
var VirtualArray = /** @class */ (function (_super) {
    __extends(VirtualArray, _super);
    function VirtualArray(length) {
        if (length === void 0) { length = 0; }
        var _this = _super.call(this) || this;
        _this._length = length;
        _this._cache = {};
        _this._requestedIndexes = [];
        _this._loadedIndexes = [];
        return _this;
    }
    Object.defineProperty(VirtualArray.prototype, "length", {
        /**
         * Gets or sets length for the virtual array.
         */
        get: function () {
            return this._length;
        },
        set: function (value) {
            if (this._length !== value) {
                var index = this._length;
                var count = value - this._length;
                this._length = value;
                this.notify({
                    eventName: ChangeType.Change,
                    object: this,
                    action: count > 0 ? ChangeType.Add : ChangeType.Delete,
                    index: index,
                    removed: new Array(count < 0 ? Math.abs(count) : 0),
                    addedCount: count > 0 ? count : 0,
                });
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(VirtualArray.prototype, "loadSize", {
        get: function () {
            return this._loadSize;
        },
        set: function (value) {
            this._loadSize = value;
        },
        enumerable: false,
        configurable: true
    });
    VirtualArray.prototype.getItem = function (index) {
        var item = this._cache[index];
        if (item === undefined) {
            if (index >= 0 && index < this.length && this._requestedIndexes.indexOf(index) < 0 && this._loadedIndexes.indexOf(index) < 0) {
                this.requestItems(index);
            }
        }
        return item;
    };
    VirtualArray.prototype.setItem = function (index, value) {
        if (this._cache[index] !== value) {
            this.load(index, [value]);
        }
    };
    /**
     * Loads items from an array starting at index.
     */
    VirtualArray.prototype.load = function (index, items) {
        for (var i = 0; i < items.length; i++) {
            var itemIndex = index + i;
            this._cache[itemIndex] = items[i];
            this._requestedIndexes.splice(this._requestedIndexes.indexOf(itemIndex), 1);
            if (this._loadedIndexes.indexOf(itemIndex) < 0) {
                this._loadedIndexes.push(itemIndex);
            }
        }
        // Remove requested but never loaded indexes.
        if (this._requestedIndexes.length > 0) {
            for (var i = 0; i < this.loadSize - items.length; i++) {
                this._requestedIndexes.splice(this._requestedIndexes.indexOf(index + i), 1);
            }
        }
        this.notify({
            eventName: ChangeType.Change,
            object: this,
            action: ChangeType.Update,
            index: index,
            removed: new Array(items.length),
            addedCount: items.length,
        });
    };
    VirtualArray.prototype.requestItems = function (index) {
        var indexesToLoad = [];
        var pageIndex = this._loadSize > 0 ? this._loadSize * Math.floor(index / this._loadSize) : index;
        var count = 0;
        var start = -1;
        for (var i = 0; i < this.loadSize; i++) {
            var itemIndex = pageIndex + i;
            if (itemIndex >= this._length) {
                break;
            }
            if (this._loadedIndexes.indexOf(itemIndex) < 0) {
                if (start < 0) {
                    start = itemIndex;
                }
                indexesToLoad.push(itemIndex);
                if (this._requestedIndexes.indexOf(itemIndex) < 0) {
                    this._requestedIndexes.push(itemIndex);
                }
                count++;
            }
            else {
                if (count > 0) {
                    this.notify({
                        eventName: VirtualArray.itemsLoadingEvent,
                        object: this,
                        index: start,
                        count: count,
                    });
                }
                start = -1;
                count = 0;
            }
        }
        if (start >= 0 && count > 0) {
            this.notify({
                eventName: VirtualArray.itemsLoadingEvent,
                object: this,
                index: start,
                count: count,
            });
        }
    };
    /**
     * String value used when hooking to change event.
     */
    VirtualArray.changeEvent = ChangeType.Change;
    /**
     * String value used when hooking to itemsLoading event.
     */
    VirtualArray.itemsLoadingEvent = 'itemsLoading';
    return VirtualArray;
}(Observable));
export { VirtualArray };
//# sourceMappingURL=index.js.map