import * as textModule from '../text';
import { Application } from '../application';
import { getFileExtension } from '../utils';
import { SDK_VERSION } from '../utils/constants';
var applicationContext;
function getApplicationContext() {
    if (!applicationContext) {
        applicationContext = Application.android.getNativeApplication().getApplicationContext();
    }
    return applicationContext;
}
function getOrSetHelper(path) {
    return org.nativescript.widgets.FileHelper.fromString(getApplicationContext(), path);
}
function isContentUri(path) {
    if (typeof path === 'string' && path.startsWith('content:')) {
        return true;
    }
    return false;
}
var FileSystemAccess = /** @class */ (function () {
    function FileSystemAccess() {
        this._pathSeparator = '/';
        this.copy = this.copySync.bind(this);
        this.readBuffer = this.readBufferSync.bind(this);
        this.read = this.readSync.bind(this);
        this.appendBuffer = this.appendBufferSync.bind(this);
        this.append = this.appendSync.bind(this);
        this.writeBuffer = this.writeBufferSync.bind(this);
        this.write = this.writeSync.bind(this);
        this.readText = this.readTextSync.bind(this);
        this.appendText = this.appendTextSync.bind(this);
        this.writeText = this.writeTextSync.bind(this);
    }
    FileSystemAccess.prototype.getLastModified = function (path) {
        var javaFile = new java.io.File(path);
        return new Date(javaFile.lastModified());
    };
    FileSystemAccess.prototype.getFileSize = function (path) {
        var javaFile = new java.io.File(path);
        return javaFile.length();
    };
    FileSystemAccess.prototype.getParent = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            var parent_1 = javaFile.getParentFile();
            return { path: parent_1.getAbsolutePath(), name: parent_1.getName() };
        }
        catch (exception) {
            // TODO: unified approach for error messages
            if (onError) {
                onError(exception);
            }
            return undefined;
        }
    };
    FileSystemAccess.prototype.getFile = function (path, onError) {
        return this.ensureFile(new java.io.File(path), false, onError);
    };
    FileSystemAccess.prototype.getFolder = function (path, onError) {
        var javaFile = new java.io.File(path);
        var dirInfo = this.ensureFile(javaFile, true, onError);
        if (!dirInfo) {
            return undefined;
        }
        return { path: dirInfo.path, name: dirInfo.name };
    };
    FileSystemAccess.prototype.eachEntity = function (path, onEntity, onError) {
        if (!onEntity) {
            return;
        }
        this.enumEntities(path, onEntity, onError);
    };
    FileSystemAccess.prototype.getEntities = function (path, onError) {
        var fileInfos = new Array();
        var onEntity = function (entity) {
            fileInfos.push(entity);
            return true;
        };
        var errorOccurred;
        var localError = function (error) {
            if (onError) {
                onError(error);
            }
            errorOccurred = true;
        };
        this.enumEntities(path, onEntity, localError);
        if (!errorOccurred) {
            return fileInfos;
        }
        return null;
    };
    FileSystemAccess.prototype.fileExists = function (path) {
        var file = new java.io.File(path);
        return file.exists();
    };
    FileSystemAccess.prototype.folderExists = function (path) {
        var file = new java.io.File(path);
        return file.exists() && file.isDirectory();
    };
    FileSystemAccess.prototype.deleteFile = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            if (!javaFile.isFile()) {
                if (onError) {
                    onError({
                        message: 'The specified parameter is not a File entity.',
                    });
                }
                return;
            }
            if (!javaFile.delete()) {
                if (onError) {
                    onError({ message: 'File deletion failed' });
                }
            }
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.deleteFolder = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            if (!javaFile.getCanonicalFile().isDirectory()) {
                if (onError) {
                    onError({
                        message: 'The specified parameter is not a Folder entity.',
                    });
                }
                return;
            }
            // TODO: Asynchronous
            this.deleteFolderContent(javaFile);
            if (!javaFile.delete()) {
                if (onError) {
                    onError({ message: 'Folder deletion failed.' });
                }
            }
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.emptyFolder = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            if (!javaFile.getCanonicalFile().isDirectory()) {
                if (onError) {
                    onError({
                        message: 'The specified parameter is not a Folder entity.',
                    });
                }
                return;
            }
            // TODO: Asynchronous
            this.deleteFolderContent(javaFile);
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.rename = function (path, newPath, onError) {
        var javaFile = new java.io.File(path);
        if (!javaFile.exists()) {
            if (onError) {
                onError(new Error('The file to rename does not exist'));
            }
            return;
        }
        var newFile = new java.io.File(newPath);
        if (newFile.exists()) {
            if (onError) {
                onError(new Error('A file with the same name already exists.'));
            }
            return;
        }
        if (!javaFile.renameTo(newFile)) {
            if (onError) {
                onError(new Error("Failed to rename file '" + path + "' to '" + newPath + "'"));
            }
        }
    };
    FileSystemAccess.prototype.getDocumentsFolderPath = function () {
        var dir = getApplicationContext().getFilesDir();
        return dir.getAbsolutePath();
    };
    FileSystemAccess.prototype.getExternalDocumentsFolderPath = function () {
        var dirs = getApplicationContext().getExternalFilesDirs(null);
        var dir;
        if (dirs && dirs.length > 1) {
            dir = dirs[dirs.length - 1];
        }
        if (!dir) {
            dir = getApplicationContext().getExternalFilesDir(null);
        }
        return dir.getAbsolutePath();
    };
    FileSystemAccess.prototype.getLogicalRootPath = function () {
        var dir = getApplicationContext().getFilesDir();
        return dir.getCanonicalPath();
    };
    FileSystemAccess.prototype.getTempFolderPath = function () {
        var dir = getApplicationContext().getCacheDir();
        return dir.getAbsolutePath();
    };
    FileSystemAccess.prototype.getCurrentAppPath = function () {
        return this.getLogicalRootPath() + '/app';
    };
    FileSystemAccess.prototype.copySync = function (src, dest, onError) {
        try {
            return org.nativescript.widgets.Async.File.copySync(src, dest, getApplicationContext());
        }
        catch (error) {
            if (onError) {
                onError(error);
            }
        }
        return false;
    };
    FileSystemAccess.prototype.copyAsync = function (src, dest) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.copy(src, dest, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function (result) {
                        resolve(result);
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), getApplicationContext());
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.readBufferAsync = function (path) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.readBuffer(path, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function (result) {
                        resolve(ArrayBuffer.from(result));
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.readBufferSync = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileInputStream(javaFile);
            var channel = stream.getChannel();
            var buffer = new ArrayBuffer(javaFile.length());
            channel.read(buffer);
            return buffer;
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.readAsync = function (path) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.read(path, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function (result) {
                        resolve(result);
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.readSync = function (path, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileInputStream(javaFile);
            var bytes = Array.create('byte', javaFile.length());
            var dataInputStream = new java.io.DataInputStream(stream);
            dataInputStream.readFully(bytes);
            return bytes;
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.getBuffer = function (buffer) {
        var _a;
        if (buffer instanceof ArrayBuffer) {
            return buffer.nativeObject || buffer;
        }
        else {
            return ((_a = buffer === null || buffer === void 0 ? void 0 : buffer.buffer) === null || _a === void 0 ? void 0 : _a.nativeObject) || buffer;
        }
    };
    FileSystemAccess.prototype.appendBufferAsync = function (path, buffer) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.appendBuffer(path, FileSystemAccess.getBuffer(buffer), new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.appendBufferSync = function (path, buffer, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile);
            var channel = stream.getChannel();
            channel.write(FileSystemAccess.getBuffer(buffer));
            stream.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.appendAsync = function (path, bytes) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.append(path, bytes, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.appendSync = function (path, bytes, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile, true);
            stream.write(bytes, 0, bytes.length);
            stream.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.writeBufferAsync = function (path, buffer) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.writeBuffer(path, FileSystemAccess.getBuffer(buffer), new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.writeBufferSync = function (path, buffer, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile);
            var channel = stream.getChannel();
            channel.write(FileSystemAccess.getBuffer(buffer));
            stream.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.writeAsync = function (path, bytes) {
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.write(path, bytes, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.writeSync = function (path, bytes, onError) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile);
            stream.write(bytes, 0, bytes.length);
            stream.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.readTextAsync = function (path, encoding) {
        var actualEncoding = encoding;
        if (!actualEncoding) {
            actualEncoding = textModule.encoding.UTF_8;
        }
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.readText(path, actualEncoding, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function (result) {
                        if (actualEncoding === textModule.encoding.UTF_8) {
                            // Remove UTF8 BOM if present. http://www.rgagnon.com/javadetails/java-handle-utf8-file-with-bom.html
                            result = FileSystemAccess._removeUtf8Bom(result);
                        }
                        resolve(result);
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.readTextSync = function (path, onError, encoding) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileInputStream(javaFile);
            var actualEncoding = encoding;
            if (!actualEncoding) {
                actualEncoding = textModule.encoding.UTF_8;
            }
            var reader = new java.io.InputStreamReader(stream, actualEncoding);
            var bufferedReader = new java.io.BufferedReader(reader);
            // TODO: We will need to read the entire file to a CharBuffer instead of reading it line by line
            // TODO: bufferedReader.read(CharBuffer) does not currently work
            var line = undefined;
            var result = '';
            while (line !== null) {
                line = bufferedReader.readLine();
                if (line === null) {
                    break;
                }
                if (result.length > 0) {
                    // add the new line manually to the result
                    // TODO: Try with CharBuffer at a later stage, when the Bridge allows it
                    result += '\n';
                }
                result += line;
            }
            if (actualEncoding === textModule.encoding.UTF_8) {
                // Remove UTF8 BOM if present. http://www.rgagnon.com/javadetails/java-handle-utf8-file-with-bom.html
                result = FileSystemAccess._removeUtf8Bom(result);
            }
            bufferedReader.close();
            return result;
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess._removeUtf8Bom = function (s) {
        if (s.charCodeAt(0) === 0xfeff) {
            s = s.slice(1);
            //console.log("Removed UTF8 BOM.");
        }
        return s;
    };
    FileSystemAccess.prototype.appendTextAsync = function (path, content, encoding) {
        var actualEncoding = encoding;
        if (!actualEncoding) {
            actualEncoding = textModule.encoding.UTF_8;
        }
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.appendText(path, content, actualEncoding, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.appendTextSync = function (path, content, onError, encoding) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile, true);
            var actualEncoding = encoding;
            if (!actualEncoding) {
                actualEncoding = textModule.encoding.UTF_8;
            }
            var writer = new java.io.OutputStreamWriter(stream, actualEncoding);
            writer.write(content);
            writer.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.writeTextAsync = function (path, content, encoding) {
        var actualEncoding = encoding;
        if (!actualEncoding) {
            actualEncoding = textModule.encoding.UTF_8;
        }
        return new Promise(function (resolve, reject) {
            try {
                org.nativescript.widgets.Async.File.writeText(path, content, actualEncoding, new org.nativescript.widgets.Async.CompleteCallback({
                    onComplete: function () {
                        resolve();
                    },
                    onError: function (err) {
                        reject(err);
                    },
                }), null);
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.writeTextSync = function (path, content, onError, encoding) {
        try {
            var javaFile = new java.io.File(path);
            var stream = new java.io.FileOutputStream(javaFile);
            var actualEncoding = encoding;
            if (!actualEncoding) {
                actualEncoding = textModule.encoding.UTF_8;
            }
            var writer = new java.io.OutputStreamWriter(stream, actualEncoding);
            writer.write(content);
            writer.close();
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.deleteFolderContent = function (file) {
        var filesList = file.listFiles();
        if (filesList.length === 0) {
            return true; // Nothing to delete, so success!
        }
        var childFile;
        var success = false;
        for (var i = 0; i < filesList.length; i++) {
            childFile = filesList[i];
            if (childFile.getCanonicalFile().isDirectory()) {
                success = this.deleteFolderContent(childFile);
                if (!success) {
                    break;
                }
            }
            success = childFile.delete();
        }
        return success;
    };
    FileSystemAccess.prototype.ensureFile = function (javaFile, isFolder, onError) {
        try {
            if (!javaFile.exists()) {
                var created = void 0;
                if (isFolder) {
                    created = javaFile.mkdirs();
                }
                else {
                    javaFile.getParentFile().mkdirs();
                    created = javaFile.createNewFile();
                }
                if (!created) {
                    // TODO: unified approach for error messages
                    if (onError) {
                        onError('Failed to create new java File for path ' + javaFile.getAbsolutePath());
                    }
                    return undefined;
                }
                else {
                    javaFile.setReadable(true);
                    javaFile.setWritable(true);
                }
            }
            var path = javaFile.getAbsolutePath();
            return {
                path: path,
                name: javaFile.getName(),
                extension: this.getFileExtension(path),
            };
        }
        catch (exception) {
            // TODO: unified approach for error messages
            if (onError) {
                onError(exception);
            }
            return undefined;
        }
    };
    // TODO: This method is the same as in the iOS implementation.
    // Make it in a separate file / module so it can be reused from both implementations.
    FileSystemAccess.prototype.getFileExtension = function (path) {
        return getFileExtension(path);
    };
    FileSystemAccess.prototype.enumEntities = function (path, callback, onError) {
        try {
            var javaFile = new java.io.File(path);
            if (!javaFile.getCanonicalFile().isDirectory()) {
                if (onError) {
                    onError('There is no folder existing at path ' + path);
                }
                return;
            }
            var filesList = javaFile.listFiles();
            var length_1 = filesList.length;
            var info = void 0;
            var retVal = void 0;
            for (var i = 0; i < length_1; i++) {
                javaFile = filesList[i];
                info = {
                    path: javaFile.getAbsolutePath(),
                    name: javaFile.getName(),
                };
                if (javaFile.isFile()) {
                    info.extension = this.getFileExtension(info.path);
                }
                retVal = callback(info);
                if (retVal === false) {
                    break;
                }
            }
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
        }
    };
    FileSystemAccess.prototype.getPathSeparator = function () {
        return this._pathSeparator;
    };
    FileSystemAccess.prototype.normalizePath = function (path) {
        if (SDK_VERSION >= 26) {
            // the [''] is a trick to not have to create a android.net.URI
            // and use the method with string signature
            return java.nio.file.Paths.get(path, ['']).normalize().toString();
        }
        else {
            // for now it wont work on pre 26 as File does not normalize
            var file = new java.io.File(path);
            return file.getAbsolutePath();
        }
    };
    FileSystemAccess.prototype.joinPath = function (left, right) {
        var file1 = new java.io.File(left);
        var file2 = new java.io.File(file1, right);
        return file2.getPath();
    };
    FileSystemAccess.prototype.joinPaths = function (paths) {
        if (!paths || paths.length === 0) {
            return '';
        }
        if (paths.length === 1) {
            return paths[0];
        }
        var result = paths[0];
        for (var i = 1; i < paths.length; i++) {
            result = this.joinPath(result, paths[i]);
        }
        return result;
    };
    return FileSystemAccess;
}());
export { FileSystemAccess };
var FileSystemAccess29 = /** @class */ (function (_super) {
    __extends(FileSystemAccess29, _super);
    function FileSystemAccess29() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.__skip = true;
        _this.appendBuffer = _this.appendBufferSync.bind(_this);
        _this.append = _this.appendSync.bind(_this);
        _this.appendText = _this.appendTextSync.bind(_this);
        _this.readText = _this.readTextSync.bind(_this);
        _this.readBuffer = _this.readBufferSync.bind(_this);
        _this.read = _this.readSync.bind(_this);
        _this.writeText = _this.writeTextSync.bind(_this);
        _this.writeBuffer = _this.writeBufferSync.bind(_this);
        _this.write = _this.writeSync.bind(_this);
        return _this;
    }
    FileSystemAccess29.prototype.getLastModified = function (path) {
        if (isContentUri(path)) {
            return new Date(getOrSetHelper(path).getLastModified() * 1000);
        }
        return _super.prototype.getLastModified.call(this, path);
    };
    FileSystemAccess29.prototype.getFileSize = function (path) {
        if (isContentUri(path)) {
            return getOrSetHelper(path).getSize();
        }
        return _super.prototype.getFileSize.call(this, path);
    };
    FileSystemAccess29.prototype.getParent = function (path, onError) {
        if (isContentUri(path)) {
            return null;
        }
        return _super.prototype.getParent.call(this, path, onError);
    };
    FileSystemAccess29.prototype.getFile = function (path, onError) {
        if (isContentUri(path)) {
            try {
                var file = getOrSetHelper(path);
                return {
                    path: path,
                    name: file.getName(),
                    extension: file.getExtension(),
                };
            }
            catch (e) {
                if (typeof onError === 'function') {
                    onError(e);
                }
                return;
            }
        }
        return _super.prototype.getFile.call(this, path, onError);
    };
    FileSystemAccess29.prototype.getFolder = function (path, onError) {
        if (isContentUri(path)) {
            return null;
        }
        return _super.prototype.getFolder.call(this, path, onError);
    };
    FileSystemAccess29.prototype.getEntities = function (path, onError) {
        if (isContentUri(path)) {
            return null;
        }
        return _super.prototype.getEntities.call(this, path, onError);
    };
    FileSystemAccess29.prototype.eachEntity = function (path, onEntity, onError) {
        if (isContentUri(path)) {
            return null;
        }
        _super.prototype.eachEntity.call(this, path, onEntity);
    };
    FileSystemAccess29.prototype.fileExists = function (path) {
        if (isContentUri(path)) {
            return org.nativescript.widgets.FileHelper.exists(applicationContext, path);
        }
        return _super.prototype.fileExists.call(this, path);
    };
    FileSystemAccess29.prototype.folderExists = function (path) {
        if (isContentUri(path)) {
            return null;
        }
        return _super.prototype.folderExists.call(this, path);
    };
    FileSystemAccess29.prototype.deleteFile = function (path, onError) {
        if (isContentUri(path)) {
            try {
                getOrSetHelper(path).delete(applicationContext);
            }
            catch (e) {
                onError === null || onError === void 0 ? void 0 : onError(e);
            }
        }
        else {
            _super.prototype.deleteFile.call(this, path, onError);
        }
    };
    FileSystemAccess29.prototype.deleteFolder = function (path, onError) {
        if (!isContentUri(path)) {
            _super.prototype.deleteFolder.call(this, path, onError);
        }
    };
    FileSystemAccess29.prototype.emptyFolder = function (path, onError) {
        if (!isContentUri(path)) {
            _super.prototype.emptyFolder.call(this, path, onError);
        }
    };
    FileSystemAccess29.prototype.rename = function (path, newPath, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).renameSync(applicationContext, newPath, callback);
        }
        else {
            _super.prototype.rename.call(this, path, newPath, onError);
        }
    };
    FileSystemAccess29.prototype.renameAsync = function (path, newPath) {
        return new Promise(function (resolve, reject) {
            getOrSetHelper(path).renameSync(applicationContext, newPath, new org.nativescript.widgets.FileHelper.Callback({
                onSuccess: function (result) {
                    resolve();
                },
                onError: function (error) {
                    reject(error);
                },
            }));
        });
    };
    FileSystemAccess29.prototype.getDocumentsFolderPath = function () {
        return _super.prototype.getDocumentsFolderPath.call(this);
    };
    FileSystemAccess29.prototype.getTempFolderPath = function () {
        return _super.prototype.getTempFolderPath.call(this);
    };
    FileSystemAccess29.prototype.getLogicalRootPath = function () {
        return _super.prototype.getLogicalRootPath.call(this);
    };
    FileSystemAccess29.prototype.getCurrentAppPath = function () {
        return _super.prototype.getCurrentAppPath.call(this);
    };
    FileSystemAccess29.prototype.appendBufferAsync = function (path, content) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).appendBuffer(applicationContext, FileSystemAccess.getBuffer(content), new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.appendAsync.call(this, path, content);
    };
    FileSystemAccess29.prototype.appendBufferSync = function (path, content, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).appendSync(applicationContext, FileSystemAccess.getBuffer(content), callback);
        }
        else {
            _super.prototype.appendSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.appendAsync = function (path, content) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).append(applicationContext, content, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.appendAsync.call(this, path, content);
    };
    FileSystemAccess29.prototype.appendSync = function (path, content, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).appendSync(applicationContext, content, callback);
        }
        else {
            _super.prototype.appendSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.appendTextAsync = function (path, content, encoding) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).appendText(applicationContext, content, encoding !== null && encoding !== void 0 ? encoding : null, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.appendTextAsync.call(this, path, content, encoding);
    };
    FileSystemAccess29.prototype.appendTextSync = function (path, content, onError, encoding) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).appendTextSync(applicationContext, content, encoding !== null && encoding !== void 0 ? encoding : null, callback);
        }
        else {
            _super.prototype.appendTextSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.readTextAsync = function (path, encoding) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).readText(applicationContext, encoding !== null && encoding !== void 0 ? encoding : null, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve(result);
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.readTextAsync.call(this, path, encoding);
    };
    FileSystemAccess29.prototype.readTextSync = function (path, onError, encoding) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            return getOrSetHelper(path).readTextSync(applicationContext, encoding !== null && encoding !== void 0 ? encoding : null, callback);
        }
        else {
            return _super.prototype.readTextSync.call(this, path, onError, encoding);
        }
    };
    FileSystemAccess29.prototype.readBufferAsync = function (path) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).readBuffer(applicationContext, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve(result);
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.readBufferAsync.call(this, path);
    };
    FileSystemAccess29.prototype.readBufferSync = function (path, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            var ret = getOrSetHelper(path).readBufferSync(applicationContext, callback);
            if (ret) {
                return null;
            }
            return ArrayBuffer.from(ret);
        }
        return _super.prototype.readBufferSync.call(this, path, onError);
    };
    FileSystemAccess29.prototype.readAsync = function (path) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).read(applicationContext, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve(result);
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.readAsync.call(this, path);
    };
    FileSystemAccess29.prototype.readSync = function (path, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            return getOrSetHelper(path).readSync(applicationContext, callback);
        }
        return _super.prototype.readSync.call(this, path, onError);
    };
    FileSystemAccess29.prototype.writeTextAsync = function (path, content, encoding) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).writeText(applicationContext, content, encoding !== null && encoding !== void 0 ? encoding : null, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.writeTextAsync.call(this, path, content, encoding);
    };
    FileSystemAccess29.prototype.writeTextSync = function (path, content, onError, encoding) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).writeTextSync(applicationContext, content, encoding !== null && encoding !== void 0 ? encoding : null, callback);
        }
        else {
            _super.prototype.writeTextSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.writeBufferAsync = function (path, content) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).writeBuffer(applicationContext, FileSystemAccess.getBuffer(content), new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.writeAsync.call(this, path, content);
    };
    FileSystemAccess29.prototype.writeBufferSync = function (path, content, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).writeSync(applicationContext, FileSystemAccess.getBuffer(content), callback);
        }
        else {
            _super.prototype.writeSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.writeAsync = function (path, content) {
        if (isContentUri(path)) {
            return new Promise(function (resolve, reject) {
                getOrSetHelper(path).write(applicationContext, content, new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) {
                        resolve();
                    },
                    onError: function (error) {
                        reject(error);
                    },
                }));
            });
        }
        return _super.prototype.writeAsync.call(this, path, content);
    };
    FileSystemAccess29.prototype.writeSync = function (path, content, onError) {
        if (isContentUri(path)) {
            var callback = null;
            if (typeof onError === 'function') {
                callback = new org.nativescript.widgets.FileHelper.Callback({
                    onSuccess: function (result) { },
                    onError: function (error) {
                        onError(error);
                    },
                });
            }
            getOrSetHelper(path).writeSync(applicationContext, content, callback);
        }
        else {
            _super.prototype.writeSync.call(this, path, content, onError);
        }
    };
    FileSystemAccess29.prototype.getFileExtension = function (path) {
        if (isContentUri(path)) {
            return getOrSetHelper(path).getExtension();
        }
        return _super.prototype.getFileExtension.call(this, path);
    };
    FileSystemAccess29.prototype.getPathSeparator = function () {
        return _super.prototype.getPathSeparator.call(this);
    };
    FileSystemAccess29.prototype.normalizePath = function (path) {
        return _super.prototype.normalizePath.call(this, path);
    };
    FileSystemAccess29.prototype.joinPath = function (left, right) {
        return _super.prototype.joinPath.call(this, left, right);
    };
    FileSystemAccess29.prototype.joinPaths = function (paths) {
        return _super.prototype.joinPaths.call(this, paths);
    };
    return FileSystemAccess29;
}(FileSystemAccess));
export { FileSystemAccess29 };
//# sourceMappingURL=file-system-access.android.js.map