import { encoding as textEncoding } from '../text';
import { iOSNativeHelper } from '../utils';
// TODO: Implement all the APIs receiving callback using async blocks
// TODO: Check whether we need try/catch blocks for the iOS implementation
var FileSystemAccess = /** @class */ (function () {
    function FileSystemAccess() {
        this.copy = this.copySync.bind(this);
        this.readText = this.readTextSync.bind(this);
        this.readBuffer = this.readBufferSync.bind(this);
        this.read = this.readSync.bind(this);
        this.writeText = this.writeTextSync.bind(this);
        this.appendBuffer = this.appendBufferSync.bind(this);
        this.append = this.appendSync.bind(this);
        this.appendText = this.appendTextSync.bind(this);
        this.writeBuffer = this.writeBufferSync.bind(this);
        this.write = this.writeSync.bind(this);
    }
    FileSystemAccess.prototype.getLastModified = function (path) {
        var fileManager = NSFileManager.defaultManager;
        var attributes = fileManager.attributesOfItemAtPathError(path);
        if (attributes) {
            return attributes.objectForKey('NSFileModificationDate');
        }
        else {
            return new Date();
        }
    };
    FileSystemAccess.prototype.getFileSize = function (path) {
        var fileManager = NSFileManager.defaultManager;
        var attributes = fileManager.attributesOfItemAtPathError(path);
        if (attributes) {
            return attributes.objectForKey('NSFileSize');
        }
        else {
            return 0;
        }
    };
    FileSystemAccess.prototype.getParent = function (path, onError) {
        try {
            var fileManager = NSFileManager.defaultManager;
            var nsString = NSString.stringWithString(path);
            var parentPath = nsString.stringByDeletingLastPathComponent;
            var name_1 = fileManager.displayNameAtPath(parentPath);
            return {
                path: parentPath.toString(),
                name: name_1,
            };
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
            return undefined;
        }
    };
    FileSystemAccess.prototype.getFile = function (path, onError) {
        try {
            var fileManager = NSFileManager.defaultManager;
            var exists = fileManager.fileExistsAtPath(path);
            if (!exists) {
                var parentPath = this.getParent(path, onError).path;
                if (!fileManager.createDirectoryAtPathWithIntermediateDirectoriesAttributesError(parentPath, true, null) || !fileManager.createFileAtPathContentsAttributes(path, null, null)) {
                    if (onError) {
                        onError(new Error("Failed to create file at path '" + path + "'"));
                    }
                    return undefined;
                }
            }
            var fileName = fileManager.displayNameAtPath(path);
            return {
                path: path,
                name: fileName,
                extension: this.getFileExtension(path),
            };
        }
        catch (exception) {
            if (onError) {
                onError(exception);
            }
            return undefined;
        }
    };
    FileSystemAccess.prototype.getFolder = function (path, onError) {
        try {
            var fileManager = NSFileManager.defaultManager;
            var exists = this.folderExists(path);
            if (!exists) {
                try {
                    fileManager.createDirectoryAtPathWithIntermediateDirectoriesAttributesError(path, true, null);
                }
                catch (ex) {
                    if (onError) {
                        onError(new Error("Failed to create folder at path '" + path + "': " + ex));
                    }
                    return undefined;
                }
            }
            var dirName = fileManager.displayNameAtPath(path);
            return {
                path: path,
                name: dirName,
            };
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to create folder at path '" + path + "'"));
            }
            return undefined;
        }
    };
    FileSystemAccess.prototype.getExistingFolder = function (path, onError) {
        try {
            var fileManager = NSFileManager.defaultManager;
            var exists = this.folderExists(path);
            if (exists) {
                var dirName = fileManager.displayNameAtPath(path);
                return {
                    path: path,
                    name: dirName,
                };
            }
            return undefined;
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to get folder at path '" + path + "'"));
            }
            return undefined;
        }
    };
    FileSystemAccess.prototype.eachEntity = function (path, onEntity, onError) {
        if (!onEntity) {
            return;
        }
        this.enumEntities(path, onEntity, onError);
    };
    FileSystemAccess.prototype.getEntities = function (path, onError) {
        var fileInfos = new Array();
        var onEntity = function (entity) {
            fileInfos.push(entity);
            return true;
        };
        var errorOccurred;
        var localError = function (error) {
            if (onError) {
                onError(error);
            }
            errorOccurred = true;
        };
        this.enumEntities(path, onEntity, localError);
        if (!errorOccurred) {
            return fileInfos;
        }
        return null;
    };
    FileSystemAccess.prototype.fileExists = function (path) {
        var result = this.exists(path);
        return result.exists;
    };
    FileSystemAccess.prototype.folderExists = function (path) {
        var result = this.exists(path);
        return result.exists && result.isDirectory;
    };
    FileSystemAccess.prototype.exists = function (path) {
        var fileManager = NSFileManager.defaultManager;
        var isDirectory = new interop.Reference(interop.types.bool, false);
        var exists = fileManager.fileExistsAtPathIsDirectory(path, isDirectory);
        return { exists: exists, isDirectory: isDirectory.value };
    };
    FileSystemAccess.prototype.concatPath = function (left, right) {
        return NSString.pathWithComponents([left, right]).toString();
    };
    FileSystemAccess.prototype.deleteFile = function (path, onError) {
        this.deleteEntity(path, onError);
    };
    FileSystemAccess.prototype.deleteFolder = function (path, onError) {
        this.deleteEntity(path, onError);
    };
    FileSystemAccess.prototype.emptyFolder = function (path, onError) {
        var fileManager = NSFileManager.defaultManager;
        var entities = this.getEntities(path, onError);
        if (!entities) {
            return;
        }
        for (var i = 0; i < entities.length; i++) {
            try {
                fileManager.removeItemAtPathError(entities[i].path);
            }
            catch (ex) {
                if (onError) {
                    onError(new Error("Failed to empty folder '" + path + "': " + ex));
                }
                return;
            }
        }
    };
    FileSystemAccess.prototype.rename = function (path, newPath, onError) {
        var fileManager = NSFileManager.defaultManager;
        try {
            fileManager.moveItemAtPathToPathError(path, newPath);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to rename '" + path + "' to '" + newPath + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.getLogicalRootPath = function () {
        var mainBundlePath = NSBundle.mainBundle.bundlePath;
        var resolvedPath = NSString.stringWithString(mainBundlePath).stringByResolvingSymlinksInPath;
        return resolvedPath;
    };
    FileSystemAccess.prototype.getDocumentsFolderPath = function () {
        return this.getKnownPath(9 /* NSSearchPathDirectory.DocumentDirectory */);
    };
    FileSystemAccess.prototype.getExternalDocumentsFolderPath = function () {
        return this.getDocumentsFolderPath();
    };
    FileSystemAccess.prototype.getTempFolderPath = function () {
        return this.getKnownPath(13 /* NSSearchPathDirectory.CachesDirectory */);
    };
    FileSystemAccess.prototype.getCurrentAppPath = function () {
        return iOSNativeHelper.getCurrentAppPath();
    };
    FileSystemAccess.prototype.copySync = function (src, dest, onError) {
        var fileManager = NSFileManager.defaultManager;
        try {
            return fileManager.copyItemAtPathToPathError(src, dest);
        }
        catch (error) {
            if (error.message.indexOf('exists') > -1) {
                // check the size of file if empty remove then try copying again
                // this could be zero due to using File.fromPath passing in a new file
                var didRemove = false;
                try {
                    didRemove = fileManager.removeItemAtPathError(dest);
                    return fileManager.copyItemAtPathToPathError(src, dest);
                }
                catch (exception) {
                    if (onError) {
                        if (didRemove) {
                            onError(error);
                        }
                        else {
                            onError(exception);
                        }
                    }
                }
            }
            if (onError) {
                onError(error);
            }
        }
        return false;
    };
    FileSystemAccess.prototype.copyAsync = function (src, dest) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                NSData.dataWithContentsOfFileCompletion(src, function (data) {
                    if (!data) {
                        reject(new Error("Failed to read file at path '" + src));
                    }
                    else {
                        data.writeToFileAtomicallyCompletion(dest, true, function () {
                            if (_this.fileExists(dest)) {
                                var size = _this.getFileSize(dest);
                                if (size === data.length) {
                                    resolve(true);
                                }
                                else {
                                    reject(new Error("Failed to write file at path '" + dest));
                                }
                            }
                            else {
                                reject(new Error("Failed to write file at path '" + dest));
                            }
                        });
                    }
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    FileSystemAccess.prototype.readTextAsync = function (path, encoding) {
        var actualEncoding = encoding || textEncoding.UTF_8;
        return new Promise(function (resolve, reject) {
            try {
                NSString.stringWithContentsOfFileEncodingCompletion(path, actualEncoding, function (result, error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve(result.toString());
                    }
                });
            }
            catch (ex) {
                reject(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.readTextSync = function (path, onError, encoding) {
        var actualEncoding = encoding || textEncoding.UTF_8;
        try {
            var nsString = NSString.stringWithContentsOfFileEncodingError(path, actualEncoding);
            return nsString.toString();
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.readBufferAsync = function (path) {
        return new Promise(function (resolve, reject) {
            try {
                NSData.dataWithContentsOfFileCompletion(path, function (data) {
                    resolve(interop.bufferFromData(data));
                });
            }
            catch (ex) {
                reject(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.readBufferSync = function (path, onError) {
        try {
            return interop.bufferFromData(NSData.dataWithContentsOfFile(path));
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.readAsync = function (path) {
        return new Promise(function (resolve, reject) {
            try {
                NSData.dataWithContentsOfFileCompletion(path, resolve);
            }
            catch (ex) {
                reject(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.readSync = function (path, onError) {
        try {
            return NSData.dataWithContentsOfFile(path);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to read file at path '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.writeTextAsync = function (path, content, encoding) {
        var nsString = NSString.stringWithString(content);
        var actualEncoding = encoding || textEncoding.UTF_8;
        return new Promise(function (resolve, reject) {
            try {
                nsString.writeToFileAtomicallyEncodingCompletion(path, true, actualEncoding, function (error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                });
            }
            catch (ex) {
                reject(new Error("Failed to write file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.writeTextSync = function (path, content, onError, encoding) {
        var nsString = NSString.stringWithString(content);
        var actualEncoding = encoding || textEncoding.UTF_8;
        // TODO: verify the useAuxiliaryFile parameter should be false
        try {
            nsString.writeToFileAtomicallyEncodingError(path, false, actualEncoding);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to write to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.getBuffer = function (buffer) {
        var _a;
        if (buffer instanceof ArrayBuffer) {
            return NSData.dataWithData(buffer);
        }
        else {
            var buf = NSData.dataWithData(buffer === null || buffer === void 0 ? void 0 : buffer.buffer);
            var len = buffer.byteLength;
            return NSData.dataWithBytesNoCopyLength(buf.bytes.add((_a = buffer === null || buffer === void 0 ? void 0 : buffer.byteOffset) !== null && _a !== void 0 ? _a : 0), len);
        }
    };
    FileSystemAccess.prototype.appendBufferAsync = function (path, content) {
        return new Promise(function (resolve, reject) {
            try {
                var handle_1 = NSFileHandle.fileHandleForWritingAtPath(path);
                handle_1.appendDataCompletion(FileSystemAccess.getBuffer(content), function (error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                    handle_1.closeFile();
                });
            }
            catch (ex) {
                reject(new Error("Failed to write file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.appendBufferSync = function (path, content, onError) {
        try {
            var handle = NSFileHandle.fileHandleForWritingAtPath(path);
            handle.seekToEndOfFile();
            handle.writeData(FileSystemAccess.getBuffer(content));
            handle.closeFile();
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to write to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.appendAsync = function (path, content) {
        return new Promise(function (resolve, reject) {
            try {
                var handle_2 = NSFileHandle.fileHandleForWritingAtPath(path);
                handle_2.appendDataCompletion(content, function (error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                    handle_2.closeFile();
                });
            }
            catch (ex) {
                reject(new Error("Failed to write file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.appendSync = function (path, content, onError) {
        try {
            var handle = NSFileHandle.fileHandleForWritingAtPath(path);
            handle.seekToEndOfFile();
            handle.writeData(content);
            handle.closeFile();
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to write to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.appendTextAsync = function (path, content, encoding) {
        var nsString = NSString.stringWithString(content);
        var actualEncoding = encoding || textEncoding.UTF_8;
        return new Promise(function (resolve, reject) {
            try {
                var data = nsString.dataUsingEncoding(actualEncoding);
                var handle_3 = NSFileHandle.fileHandleForWritingAtPath(path);
                handle_3.appendDataCompletion(data, function (error) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve();
                    }
                    handle_3.closeFile();
                });
            }
            catch (ex) {
                reject(new Error("Failed to append file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.appendTextSync = function (path, content, onError, encoding) {
        var nsString = NSString.stringWithString(content);
        var actualEncoding = encoding || textEncoding.UTF_8;
        // TODO: verify the useAuxiliaryFile parameter should be false
        try {
            var data = nsString.dataUsingEncoding(actualEncoding);
            var handle = NSFileHandle.fileHandleForWritingAtPath(path);
            handle.seekToEndOfFile();
            handle.writeData(data);
            handle.closeFile();
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to append to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.writeBufferAsync = function (path, content) {
        return new Promise(function (resolve, reject) {
            try {
                FileSystemAccess.getBuffer(content).writeToFileAtomicallyCompletion(path, true, function () {
                    resolve();
                });
            }
            catch (ex) {
                reject(new Error("Failed to write file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.writeBufferSync = function (path, content, onError) {
        try {
            FileSystemAccess.getBuffer(content).writeToFileAtomically(path, true);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to write to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.writeAsync = function (path, content) {
        return new Promise(function (resolve, reject) {
            try {
                content.writeToFileAtomicallyCompletion(path, true, function () {
                    resolve();
                });
            }
            catch (ex) {
                reject(new Error("Failed to write file at path '" + path + "': " + ex));
            }
        });
    };
    FileSystemAccess.prototype.writeSync = function (path, content, onError) {
        try {
            content.writeToFileAtomically(path, true);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to write to file '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.getKnownPath = function (folderType) {
        var fileManager = NSFileManager.defaultManager;
        var paths = fileManager.URLsForDirectoryInDomains(folderType, 1 /* NSSearchPathDomainMask.UserDomainMask */);
        var url = paths.objectAtIndex(0);
        return url.path;
    };
    // TODO: This method is the same as in the iOS implementation.
    // Make it in a separate file / module so it can be reused from both implementations.
    FileSystemAccess.prototype.getFileExtension = function (path) {
        // TODO [For Panata]: The definitions currently specify "any" as a return value of this method
        //const nsString = Foundation.NSString.stringWithString(path);
        //const extension = nsString.pathExtension();
        //if (extension && extension.length > 0) {
        //    extension = extension.concat(".", extension);
        //}
        //return extension;
        var dotIndex = path.lastIndexOf('.');
        if (dotIndex && dotIndex >= 0 && dotIndex < path.length) {
            return path.substring(dotIndex);
        }
        return '';
    };
    FileSystemAccess.prototype.deleteEntity = function (path, onError) {
        var fileManager = NSFileManager.defaultManager;
        try {
            fileManager.removeItemAtPathError(path);
        }
        catch (ex) {
            if (onError) {
                onError(new Error("Failed to delete file at path '" + path + "': " + ex));
            }
        }
    };
    FileSystemAccess.prototype.enumEntities = function (path, callback, onError) {
        try {
            var fileManager = NSFileManager.defaultManager;
            var files = void 0;
            try {
                files = fileManager.contentsOfDirectoryAtPathError(path);
            }
            catch (ex) {
                if (onError) {
                    onError(new Error("Failed to enum files for folder '" + path + "': " + ex));
                }
                return;
            }
            for (var i = 0; i < files.count; i++) {
                var file = files.objectAtIndex(i);
                var info = {
                    path: this.concatPath(path, file),
                    name: file,
                    extension: '',
                };
                if (!this.folderExists(this.joinPath(path, file))) {
                    info.extension = this.getFileExtension(info.path);
                }
                var retVal = callback(info);
                if (retVal === false) {
                    // the callback returned false meaning we should stop the iteration
                    break;
                }
            }
        }
        catch (ex) {
            if (onError) {
                onError(ex);
            }
        }
    };
    FileSystemAccess.prototype.getPathSeparator = function () {
        return '/';
    };
    FileSystemAccess.prototype.normalizePath = function (path) {
        var nsString = NSString.stringWithString(path);
        var normalized = nsString.stringByStandardizingPath;
        return normalized;
    };
    FileSystemAccess.prototype.joinPath = function (left, right) {
        var nsString = NSString.stringWithString(left);
        return nsString.stringByAppendingPathComponent(right);
    };
    FileSystemAccess.prototype.joinPaths = function (paths) {
        return iOSNativeHelper.joinPaths.apply(iOSNativeHelper, paths);
    };
    return FileSystemAccess;
}());
export { FileSystemAccess };
// stub to avoid cross platform warning
var FileSystemAccess29 = /** @class */ (function (_super) {
    __extends(FileSystemAccess29, _super);
    function FileSystemAccess29() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return FileSystemAccess29;
}(FileSystemAccess));
export { FileSystemAccess29 };
//# sourceMappingURL=file-system-access.ios.js.map