import { FileSystemAccess, FileSystemAccess29 } from './file-system-access';
import { SDK_VERSION } from '../utils';
import { Application } from '../application';
// The FileSystemAccess implementation, used through all the APIs.
var fileAccess;
/**
 * Returns FileSystemAccess, a shared singleton utility class to provide methods to access and work with the file system. This is used under the hood of all the file system apis in @nativescript/core and provided as a lower level convenience if needed.
 * @returns FileSystemAccess
 */
export function getFileAccess() {
    if (!fileAccess) {
        if (__ANDROID__ && SDK_VERSION >= 29) {
            fileAccess = new FileSystemAccess29();
        }
        else {
            fileAccess = new FileSystemAccess();
        }
    }
    return fileAccess;
}
function createFile(info) {
    var file = new File();
    file._path = info.path;
    file._name = info.name;
    file._extension = info.extension;
    return file;
}
function createFolder(info) {
    var documents = knownFolders.documents();
    if (info.path === documents.path) {
        return documents;
    }
    var temp = knownFolders.temp();
    if (info.path === temp.path) {
        return temp;
    }
    var folder = new Folder();
    folder._path = info.path;
    folder._name = info.name;
    return folder;
}
var FileSystemEntity = /** @class */ (function () {
    function FileSystemEntity() {
    }
    Object.defineProperty(FileSystemEntity.prototype, "parent", {
        get: function () {
            var onError = function (error) {
                throw error;
            };
            var folderInfo = getFileAccess().getParent(this.path, onError);
            if (!folderInfo) {
                return undefined;
            }
            return createFolder(folderInfo);
        },
        enumerable: false,
        configurable: true
    });
    FileSystemEntity.prototype.remove = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var hasError = false;
            var localError = function (error) {
                hasError = true;
                reject(error);
            };
            _this.removeSync(localError);
            if (!hasError) {
                resolve(true);
            }
        });
    };
    FileSystemEntity.prototype.removeSync = function (onError) {
        if (this._isKnown) {
            if (onError) {
                onError({ message: 'Cannot delete known folder.' });
            }
            return;
        }
        var fileAccess = getFileAccess();
        if (this instanceof File) {
            fileAccess.deleteFile(this.path, onError);
        }
        else if (this instanceof Folder) {
            fileAccess.deleteFolder(this.path, onError);
        }
    };
    FileSystemEntity.prototype.rename = function (newName) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var hasError = false;
            var localError = function (error) {
                hasError = true;
                reject(error);
            };
            _this.renameSync(newName, localError);
            if (!hasError) {
                resolve(true);
            }
        });
    };
    FileSystemEntity.prototype.renameSync = function (newName, onError) {
        if (this._isKnown) {
            if (onError) {
                onError(new Error('Cannot rename known folder.'));
            }
            return;
        }
        var localError = function (error) {
            if (onError) {
                onError(error);
            }
            return null;
        };
        var fileAccess = getFileAccess();
        // call rename for FileSystemAccess29
        if (fileAccess.__skip) {
            fileAccess.rename(this.path, newName, localError);
            var fileInfo = getFileAccess().getFile(this.path, null);
            if (fileInfo) {
                this._name = fileInfo.name;
                this._extension = fileInfo.extension;
            }
            return;
        }
        var parentFolder = this.parent;
        if (!parentFolder) {
            if (onError) {
                onError(new Error('No parent folder.'));
            }
            return;
        }
        var path = parentFolder.path;
        var newPath = fileAccess.joinPath(path, newName);
        fileAccess.rename(this.path, newPath, localError);
        this._path = newPath;
        this._name = newName;
        if (this instanceof File) {
            this._extension = fileAccess.getFileExtension(newPath);
        }
    };
    Object.defineProperty(FileSystemEntity.prototype, "name", {
        get: function () {
            return this._name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FileSystemEntity.prototype, "path", {
        get: function () {
            return this._path;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FileSystemEntity.prototype, "lastModified", {
        get: function () {
            return getFileAccess().getLastModified(this.path);
        },
        enumerable: false,
        configurable: true
    });
    return FileSystemEntity;
}());
export { FileSystemEntity };
var applicationContext;
function getApplicationContext() {
    if (!applicationContext) {
        applicationContext = Application.android.getNativeApplication().getApplicationContext();
    }
    return applicationContext;
}
export var AndroidDirectory;
(function (AndroidDirectory) {
    AndroidDirectory["ALARMS"] = "alarms";
    AndroidDirectory["AUDIOBOOKS"] = "audiobooks";
    AndroidDirectory["DCIM"] = "dcim";
    AndroidDirectory["DOCUMENTS"] = "documents";
    AndroidDirectory["DOWNLOADS"] = "downloads";
    AndroidDirectory["MOVIES"] = "movies";
    AndroidDirectory["MUSIC"] = "music";
    AndroidDirectory["NOTIFICATIONS"] = "notifications";
    AndroidDirectory["PICTURES"] = "pictures";
    AndroidDirectory["PODCASTS"] = "podcasts";
    AndroidDirectory["RINGTONES"] = "ringtones";
    AndroidDirectory["SCREENSHOTS"] = "screenshots";
})(AndroidDirectory || (AndroidDirectory = {}));
function getAndroidDirectory(value) {
    switch (value) {
        case AndroidDirectory.ALARMS:
            return {
                path: android.os.Environment.DIRECTORY_ALARMS,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.AUDIOBOOKS:
            return {
                path: android.os.Environment.DIRECTORY_AUDIOBOOKS,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.DCIM:
            return {
                path: android.os.Environment.DIRECTORY_DCIM,
                column: android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.DOCUMENTS:
            return {
                path: android.os.Environment.DIRECTORY_DOCUMENTS,
                column: android.provider.MediaStore.Files.getContentUri('external'),
            };
        case AndroidDirectory.DOWNLOADS:
            return {
                path: android.os.Environment.DIRECTORY_DOWNLOADS,
                column: android.provider.MediaStore.Downloads.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.MOVIES:
            return {
                path: android.os.Environment.DIRECTORY_MOVIES,
                column: android.provider.MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.MUSIC:
            return {
                path: android.os.Environment.DIRECTORY_MUSIC,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.NOTIFICATIONS:
            return {
                path: android.os.Environment.DIRECTORY_NOTIFICATIONS,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.PICTURES:
            return {
                path: android.os.Environment.DIRECTORY_PICTURES,
                column: android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.PODCASTS:
            return {
                path: android.os.Environment.DIRECTORY_PODCASTS,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.RINGTONES:
            return {
                path: android.os.Environment.DIRECTORY_RINGTONES,
                column: android.provider.MediaStore.Audio.Media.EXTERNAL_CONTENT_URI,
            };
        case AndroidDirectory.SCREENSHOTS:
            return {
                path: android.os.Environment.DIRECTORY_SCREENSHOTS,
                column: android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
            };
        default:
            return null;
    }
}
var Android = /** @class */ (function () {
    function Android() {
    }
    Android.prototype.createFile = function (options) {
        if (!__ANDROID__) {
            throw new Error("createFile is available on Android only!");
        }
        var context = getApplicationContext();
        var meta = new android.content.ContentValues();
        meta.put(android.provider.MediaStore.MediaColumns.DISPLAY_NAME, options.name);
        meta.put(android.provider.MediaStore.MediaColumns.MIME_TYPE, options.mime);
        //meta.put(android.provider.MediaStore.MediaColumns.DATE_ADDED, java.lang.System.currentTimeMillis() as any);
        var externalDirectory = getAndroidDirectory(options.directory);
        if (SDK_VERSION >= 29) {
            var relativePath = (options === null || options === void 0 ? void 0 : options.relativePath) ? "/".concat(options.relativePath) : '';
            meta.put(android.provider.MediaStore.MediaColumns.RELATIVE_PATH, "".concat(externalDirectory.path).concat(relativePath));
            // todo
            //	meta.put(android.provider.MediaStore.MediaColumns.IS_PENDING, java.lang.Integer.valueOf(1));
        }
        else {
            var relativePath = (options === null || options === void 0 ? void 0 : options.relativePath) ? "".concat(options.relativePath, "/") : '';
            var directory = android.os.Environment.getExternalStoragePublicDirectory(externalDirectory.path);
            var file = new java.io.File(directory, "".concat(relativePath).concat(options.name));
            meta.put(android.provider.MediaStore.MediaColumns.DATA, file.getAbsolutePath());
        }
        var uri = context.getContentResolver().insert(externalDirectory.column, meta);
        return File.fromPath(uri.toString());
    };
    return Android;
}());
var ad = new Android();
var iOS = /** @class */ (function () {
    function iOS() {
    }
    return iOS;
}());
var ios = new iOS();
var File = /** @class */ (function (_super) {
    __extends(File, _super);
    function File() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(File, "ios", {
        get: function () {
            return ios;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(File, "android", {
        get: function () {
            return ad;
        },
        enumerable: false,
        configurable: true
    });
    File.fromPath = function (path, copy) {
        if (copy === void 0) { copy = false; }
        var onError = function (error) {
            throw error;
        };
        if (__ANDROID__ && copy) {
            if (path.startsWith('content:')) {
                var fileInfo_1 = getFileAccess().getFile(path, onError);
                // falls back to creating a temp file without a known extension.
                if (!fileInfo_1) {
                    var tempFile = "".concat(knownFolders.temp().path, "/").concat(java.util.UUID.randomUUID().toString());
                    org.nativescript.widgets.Async.File.copySync(path, tempFile, getApplicationContext());
                    path = tempFile;
                }
                else {
                    var ext = fileInfo_1.extension;
                    var name_1 = "".concat(fileInfo_1.name.replace(".".concat(ext), ''), ".").concat(ext);
                    var tempFile = "".concat(knownFolders.temp().path, "/").concat(name_1);
                    getFileAccess().copySync(path, tempFile);
                    path = tempFile;
                }
            }
        }
        var fileInfo = getFileAccess().getFile(path, onError);
        if (!fileInfo) {
            return undefined;
        }
        return createFile(fileInfo);
    };
    File.exists = function (path) {
        return getFileAccess().fileExists(path);
    };
    Object.defineProperty(File.prototype, "extension", {
        get: function () {
            return this._extension;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(File.prototype, "isLocked", {
        get: function () {
            // !! is a boolean conversion/cast, handling undefined as well
            return !!this._locked;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(File.prototype, "size", {
        get: function () {
            return getFileAccess().getFileSize(this.path);
        },
        enumerable: false,
        configurable: true
    });
    File.prototype.append = function (content) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .appendAsync(_this.path, content)
                .then(function () {
                resolve();
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.appendSync = function (content, onError) {
        this._checkAccess();
        try {
            this._locked = true;
            var that_1 = this;
            var localError = function (error) {
                that_1._locked = false;
                if (onError) {
                    onError(error);
                }
            };
            getFileAccess().appendSync(this.path, content, localError);
        }
        finally {
            this._locked = false;
        }
    };
    File.prototype.appendText = function (content, encoding) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .appendTextAsync(_this.path, content, encoding)
                .then(function () {
                resolve(true);
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.appendTextSync = function (content, onError, encoding) {
        this._checkAccess();
        try {
            this._locked = true;
            var that_2 = this;
            var localError = function (error) {
                that_2._locked = false;
                if (onError) {
                    onError(error);
                }
            };
            getFileAccess().appendTextSync(this.path, content, localError, encoding);
        }
        finally {
            this._locked = false;
        }
    };
    File.prototype.copy = function (dest) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .copyAsync(_this.path, dest)
                .then(function (result) {
                resolve(result);
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.copySync = function (dest, onError) {
        this._checkAccess();
        this._locked = true;
        var that = this;
        var localError = function (error) {
            that._locked = false;
            if (onError) {
                onError(error);
            }
        };
        var content = getFileAccess().copySync(this.path, dest, localError);
        this._locked = false;
        return content;
    };
    File.prototype.read = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .readAsync(_this.path)
                .then(function (result) {
                resolve(result);
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.readSync = function (onError) {
        this._checkAccess();
        this._locked = true;
        var that = this;
        var localError = function (error) {
            that._locked = false;
            if (onError) {
                onError(error);
            }
        };
        var content = getFileAccess().readSync(this.path, localError);
        this._locked = false;
        return content;
    };
    File.prototype.write = function (content) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .writeAsync(_this.path, content)
                .then(function () {
                resolve();
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.writeSync = function (content, onError) {
        this._checkAccess();
        try {
            this._locked = true;
            var that_3 = this;
            var localError = function (error) {
                that_3._locked = false;
                if (onError) {
                    onError(error);
                }
            };
            getFileAccess().writeSync(this.path, content, localError);
        }
        finally {
            this._locked = false;
        }
    };
    File.prototype.readText = function (encoding) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .readTextAsync(_this.path, encoding)
                .then(function (result) {
                resolve(result);
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.readTextSync = function (onError, encoding) {
        this._checkAccess();
        this._locked = true;
        var that = this;
        var localError = function (error) {
            that._locked = false;
            if (onError) {
                onError(error);
            }
        };
        var content = getFileAccess().readTextSync(this.path, localError, encoding);
        this._locked = false;
        return content;
    };
    File.prototype.writeText = function (content, encoding) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this._checkAccess();
            }
            catch (ex) {
                reject(ex);
                return;
            }
            _this._locked = true;
            getFileAccess()
                .writeTextAsync(_this.path, content, encoding)
                .then(function () {
                resolve(true);
                _this._locked = false;
            }, function (error) {
                reject(error);
                _this._locked = false;
            });
        });
    };
    File.prototype.writeTextSync = function (content, onError, encoding) {
        this._checkAccess();
        try {
            this._locked = true;
            var that_4 = this;
            var localError = function (error) {
                that_4._locked = false;
                if (onError) {
                    onError(error);
                }
            };
            getFileAccess().writeTextSync(this.path, content, localError, encoding);
        }
        finally {
            this._locked = false;
        }
    };
    File.prototype._checkAccess = function () {
        if (this.isLocked) {
            throw new Error('Cannot access a locked file.');
        }
    };
    return File;
}(FileSystemEntity));
export { File };
var Folder = /** @class */ (function (_super) {
    __extends(Folder, _super);
    function Folder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Folder.fromPath = function (path) {
        var onError = function (error) {
            throw error;
        };
        var folderInfo = getFileAccess().getFolder(path, onError);
        if (!folderInfo) {
            return undefined;
        }
        return createFolder(folderInfo);
    };
    Folder.exists = function (path) {
        return getFileAccess().folderExists(path);
    };
    Folder.prototype.contains = function (name) {
        var fileAccess = getFileAccess();
        var path = fileAccess.joinPath(this.path, name);
        if (fileAccess.fileExists(path)) {
            return true;
        }
        return fileAccess.folderExists(path);
    };
    Folder.prototype.clear = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var hasError = false;
            var onError = function (error) {
                hasError = true;
                reject(error);
            };
            _this.clearSync(onError);
            if (!hasError) {
                resolve(true);
            }
        });
    };
    Folder.prototype.clearSync = function (onError) {
        getFileAccess().emptyFolder(this.path, onError);
    };
    Object.defineProperty(Folder.prototype, "isKnown", {
        get: function () {
            return this._isKnown;
        },
        enumerable: false,
        configurable: true
    });
    Folder.prototype.getFile = function (name) {
        var fileAccess = getFileAccess();
        var path = fileAccess.joinPath(this.path, name);
        var onError = function (error) {
            throw error;
        };
        var fileInfo = fileAccess.getFile(path, onError);
        if (!fileInfo) {
            return undefined;
        }
        return createFile(fileInfo);
    };
    Folder.prototype.getFolder = function (name) {
        var fileAccess = getFileAccess();
        var path = fileAccess.joinPath(this.path, name);
        var onError = function (error) {
            throw error;
        };
        var folderInfo = fileAccess.getFolder(path, onError);
        if (!folderInfo) {
            return undefined;
        }
        return createFolder(folderInfo);
    };
    Folder.prototype.getEntities = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var hasError = false;
            var localError = function (error) {
                hasError = true;
                reject(error);
            };
            var entities = _this.getEntitiesSync(localError);
            if (!hasError) {
                resolve(entities);
            }
        });
    };
    Folder.prototype.getEntitiesSync = function (onError) {
        var fileInfos = getFileAccess().getEntities(this.path, onError);
        if (!fileInfos) {
            return null;
        }
        var entities = new Array();
        for (var i = 0; i < fileInfos.length; i++) {
            if (fileInfos[i].extension) {
                entities.push(createFile(fileInfos[i]));
            }
            else {
                entities.push(createFolder(fileInfos[i]));
            }
        }
        return entities;
    };
    Folder.prototype.eachEntity = function (onEntity) {
        if (!onEntity) {
            return;
        }
        var onSuccess = function (fileInfo) {
            var entity;
            if (fileInfo.extension) {
                entity = createFile(fileInfo);
            }
            else {
                entity = createFolder(fileInfo);
            }
            return onEntity(entity);
        };
        var onError = function (error) {
            throw error;
        };
        getFileAccess().eachEntity(this.path, onSuccess, onError);
    };
    return Folder;
}(FileSystemEntity));
export { Folder };
export var knownFolders;
(function (knownFolders) {
    var _documents;
    var _externalDocuments;
    var _temp;
    var _app;
    function documents() {
        if (!_documents) {
            var path_1 = getFileAccess().getDocumentsFolderPath();
            _documents = new Folder();
            _documents._path = path_1;
            _documents._isKnown = true;
        }
        return _documents;
    }
    knownFolders.documents = documents;
    function externalDocuments() {
        if (!_externalDocuments) {
            var path_2 = getFileAccess().getExternalDocumentsFolderPath();
            _externalDocuments = new Folder();
            _externalDocuments._path = path_2;
            _externalDocuments._isKnown = true;
        }
        return _externalDocuments;
    }
    knownFolders.externalDocuments = externalDocuments;
    function temp() {
        if (!_temp) {
            var path_3 = getFileAccess().getTempFolderPath();
            _temp = new Folder();
            _temp._path = path_3;
            _temp._isKnown = true;
        }
        return _temp;
    }
    knownFolders.temp = temp;
    function currentApp() {
        if (!_app) {
            var path_4 = getFileAccess().getCurrentAppPath();
            _app = new Folder();
            _app._path = path_4;
            _app._isKnown = true;
        }
        return _app;
    }
    knownFolders.currentApp = currentApp;
    var ios;
    (function (ios) {
        function _checkPlatform(knownFolderName) {
            if (!__APPLE__) {
                throw new Error("The \"".concat(knownFolderName, "\" known folder is available on iOS only!"));
            }
        }
        var _library;
        function library() {
            _checkPlatform('library');
            if (!_library) {
                var existingFolderInfo = getExistingFolderInfo(5 /* NSSearchPathDirectory.LibraryDirectory */);
                if (existingFolderInfo) {
                    _library = existingFolderInfo.folder;
                    _library._path = existingFolderInfo.path;
                    _library._isKnown = true;
                }
            }
            return _library;
        }
        ios.library = library;
        var _developer;
        function developer() {
            _checkPlatform('developer');
            if (!_developer) {
                var existingFolderInfo = getExistingFolderInfo(6 /* NSSearchPathDirectory.DeveloperDirectory */);
                if (existingFolderInfo) {
                    _developer = existingFolderInfo.folder;
                    _developer._path = existingFolderInfo.path;
                    _developer._isKnown = true;
                }
            }
            return _developer;
        }
        ios.developer = developer;
        var _desktop;
        function desktop() {
            _checkPlatform('desktop');
            if (!_desktop) {
                var existingFolderInfo = getExistingFolderInfo(12 /* NSSearchPathDirectory.DesktopDirectory */);
                if (existingFolderInfo) {
                    _desktop = existingFolderInfo.folder;
                    _desktop._path = existingFolderInfo.path;
                    _desktop._isKnown = true;
                }
            }
            return _desktop;
        }
        ios.desktop = desktop;
        var _downloads;
        function downloads() {
            _checkPlatform('downloads');
            if (!_downloads) {
                var existingFolderInfo = getExistingFolderInfo(15 /* NSSearchPathDirectory.DownloadsDirectory */);
                if (existingFolderInfo) {
                    _downloads = existingFolderInfo.folder;
                    _downloads._path = existingFolderInfo.path;
                    _downloads._isKnown = true;
                }
            }
            return _downloads;
        }
        ios.downloads = downloads;
        var _movies;
        function movies() {
            _checkPlatform('movies');
            if (!_movies) {
                var existingFolderInfo = getExistingFolderInfo(17 /* NSSearchPathDirectory.MoviesDirectory */);
                if (existingFolderInfo) {
                    _movies = existingFolderInfo.folder;
                    _movies._path = existingFolderInfo.path;
                    _movies._isKnown = true;
                }
            }
            return _movies;
        }
        ios.movies = movies;
        var _music;
        function music() {
            _checkPlatform('music');
            if (!_music) {
                var existingFolderInfo = getExistingFolderInfo(18 /* NSSearchPathDirectory.MusicDirectory */);
                if (existingFolderInfo) {
                    _music = existingFolderInfo.folder;
                    _music._path = existingFolderInfo.path;
                    _music._isKnown = true;
                }
            }
            return _music;
        }
        ios.music = music;
        var _pictures;
        function pictures() {
            _checkPlatform('pictures');
            if (!_pictures) {
                var existingFolderInfo = getExistingFolderInfo(19 /* NSSearchPathDirectory.PicturesDirectory */);
                if (existingFolderInfo) {
                    _pictures = existingFolderInfo.folder;
                    _pictures._path = existingFolderInfo.path;
                    _pictures._isKnown = true;
                }
            }
            return _pictures;
        }
        ios.pictures = pictures;
        var _sharedPublic;
        function sharedPublic() {
            _checkPlatform('sharedPublic');
            if (!_sharedPublic) {
                var existingFolderInfo = getExistingFolderInfo(21 /* NSSearchPathDirectory.SharedPublicDirectory */);
                if (existingFolderInfo) {
                    _sharedPublic = existingFolderInfo.folder;
                    _sharedPublic._path = existingFolderInfo.path;
                    _sharedPublic._isKnown = true;
                }
            }
            return _sharedPublic;
        }
        ios.sharedPublic = sharedPublic;
        function getExistingFolderInfo(pathDirectory /* NSSearchPathDirectory */) {
            var fileAccess = getFileAccess();
            var folderPath = fileAccess.getKnownPath(pathDirectory);
            var folderInfo = fileAccess.getExistingFolder(folderPath);
            if (folderInfo) {
                return {
                    folder: createFolder(folderInfo),
                    path: folderPath,
                };
            }
            return undefined;
        }
    })(ios = knownFolders.ios || (knownFolders.ios = {}));
})(knownFolders || (knownFolders = {}));
export var path;
(function (path_5) {
    function normalize(path) {
        return getFileAccess().normalizePath(path);
    }
    path_5.normalize = normalize;
    function join() {
        var paths = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            paths[_i] = arguments[_i];
        }
        var fileAccess = getFileAccess();
        return fileAccess.joinPaths(paths);
    }
    path_5.join = join;
    path_5.separator = getFileAccess().getPathSeparator();
})(path || (path = {}));
//# sourceMappingURL=index.js.map