var tslib = require('tslib');
import { Observable } from '../data/observable';
import { trace as profilingTrace, time, uptime, level as profilingLevel } from '../profiling';
function registerOnGlobalContext(moduleName, exportName) {
    Object.defineProperty(global, exportName, {
        get: function () {
            // We do not need to cache require() call since it is already cached in the runtime.
            var m = global.loadModule(moduleName);
            // Redefine the property to make sure the above code is executed only once.
            var resolvedValue = m[exportName];
            Object.defineProperty(global, exportName, {
                value: resolvedValue,
                configurable: true,
                writable: true,
            });
            return resolvedValue;
        },
        configurable: true,
    });
}
/**
 * Manages internal framework global state
 */
var NativeScriptGlobalState = /** @class */ (function () {
    function NativeScriptGlobalState() {
        this.launched = false;
        this._appInstanceReady = false;
        // console.log('creating NativeScriptGlobals...')
        this.events = new Observable();
        this._setLaunched = this._setLaunchedFn.bind(this);
        this.events.on('launch', this._setLaunched);
        if (profilingLevel() > 0) {
            this.events.on('displayed', function () {
                var duration = uptime();
                var end = time();
                var start = end - duration;
                profilingTrace("Displayed in ".concat(duration.toFixed(2), "ms"), start, end);
            });
        }
    }
    Object.defineProperty(NativeScriptGlobalState.prototype, "appInstanceReady", {
        get: function () {
            return this._appInstanceReady;
        },
        set: function (value) {
            this._appInstanceReady = value;
            // app instance ready, wire up any app events waiting in startup queue
            if (this.appEventWiring && this.appEventWiring.length) {
                for (var _i = 0, _a = this.appEventWiring; _i < _a.length; _i++) {
                    var callback = _a[_i];
                    callback();
                }
                // cleanup
                this.appEventWiring = null;
            }
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Ability for classes to initialize app event handling early even before the app instance is ready during boot cycle avoiding boot race conditions
     * @param callback wire up any global event handling inside the callback
     */
    NativeScriptGlobalState.prototype.addEventWiring = function (callback) {
        if (this._appInstanceReady) {
            callback();
        }
        else {
            if (!this.appEventWiring) {
                this.appEventWiring = [];
            }
            this.appEventWiring.push(callback);
        }
    };
    NativeScriptGlobalState.prototype._setLaunchedFn = function () {
        // console.log('NativeScriptGlobals launch fired!');
        this.launched = true;
        this.events.off('launch', this._setLaunched);
        this._setLaunched = null;
    };
    return NativeScriptGlobalState;
}());
export { NativeScriptGlobalState };
export function installPolyfills(moduleName, exportNames) {
    if (global.__snapshot) {
        var loadedModule_1 = global.loadModule(moduleName);
        exportNames.forEach(function (exportName) { return (global[exportName] = loadedModule_1[exportName]); });
    }
    else {
        exportNames.forEach(function (exportName) { return registerOnGlobalContext(moduleName, exportName); });
    }
}
export function initGlobal() {
    if (!global.NativeScriptHasInitGlobal) {
        global.NativeScriptHasInitGlobal = true;
        // init global state handler
        global.NativeScriptGlobals = new NativeScriptGlobalState();
        // ts-helpers
        // Required by V8 snapshot generator
        if (!global.__extends) {
            global.__extends = function (d, b) {
                for (var p in b) {
                    if (b.hasOwnProperty(p)) {
                        d[p] = b[p];
                    }
                }
                function __() {
                    this.constructor = d;
                }
                d.prototype = b === null ? Object.create(b) : ((__.prototype = b.prototype), new __());
            };
        }
        // Bind the tslib helpers to global scope.
        // This is needed when we don't use importHelpers, which
        // breaks extending native-classes
        for (var _i = 0, _a = Object.getOwnPropertyNames(tslib); _i < _a.length; _i++) {
            var fnName = _a[_i];
            if (typeof tslib[fnName] !== 'function') {
                continue;
            }
            if (fnName in global) {
                // Don't override globals that are already defined (ex. __extends)
                continue;
            }
            global[fnName] = tslib[fnName];
        }
        // module helpers
        var modules_1 = new Map();
        var modulesLoadedForUI_1 = new Set();
        var defaultExtensionMap_1 = {
            '.js': '.js',
            '.ts': '.js',
            '.kt': '.js',
            '.css': '.css',
            '.scss': '.css',
            '.less': '.css',
            '.sass': '.css',
            '.xml': '.xml',
        };
        // Cast to <any> because moduleResolvers is read-only in definitions
        global.moduleResolvers = [global.require];
        global.registerModule = function (name, loader) {
            modules_1.set(name, { loader: loader, moduleId: name });
        };
        global._unregisterModule = function _unregisterModule(name) {
            modules_1.delete(name);
        };
        global._isModuleLoadedForUI = function _isModuleLoadedForUI(moduleName) {
            return modulesLoadedForUI_1.has(moduleName);
        };
        global.registerWebpackModules = function registerWebpackModules(context, extensionMap) {
            if (extensionMap === void 0) { extensionMap = {}; }
            context.keys().forEach(function (moduleId) {
                var extDotIndex = moduleId.lastIndexOf('.');
                var base = moduleId.substr(0, extDotIndex);
                var originalExt = moduleId.substr(extDotIndex);
                var registerExt = extensionMap[originalExt] || defaultExtensionMap_1[originalExt] || originalExt;
                // We prefer source files for webpack scenarios before compilation leftovers,
                // e. g. if we get a .js and .ts for the same module, the .js is probably the compiled version of the .ts file,
                // so we register the .ts with higher priority, similar is the case with us preferring the .scss to .css
                var isSourceFile = originalExt !== registerExt;
                var registerName = base + registerExt;
                var registerWithName = function (nickName) {
                    modules_1.set(nickName, {
                        moduleId: moduleId,
                        loader: function () {
                            return context(moduleId);
                        },
                    });
                };
                if (registerName.startsWith('./') && registerName.endsWith('.js')) {
                    var jsNickNames = [
                        // This is extremely short version like "main-page" that was promoted to be used with global.registerModule("module-name", loaderFunc);
                        registerName.substr(2, registerName.length - 5),
                        // This is for supporting module names like "./main/main-page"
                        registerName.substr(0, registerName.length - 3),
                        // This is for supporting module names like "main/main-page.js"
                        registerName.substr(2),
                    ];
                    jsNickNames.forEach(function (jsNickName) {
                        if (isSourceFile || !global.moduleExists(jsNickName)) {
                            registerWithName(jsNickName);
                        }
                    });
                }
                else if (registerName.startsWith('./')) {
                    var moduleNickNames = [
                        // This is for supporting module names like "main/main-page.xml"
                        registerName.substr(2),
                    ];
                    moduleNickNames.forEach(function (moduleNickName) {
                        if (!global.moduleExists(moduleNickName)) {
                            registerWithName(moduleNickName);
                        }
                    });
                }
                if (isSourceFile || !global.moduleExists(registerName)) {
                    registerWithName(registerName);
                }
            });
        };
        global.moduleExists = function moduleExists(name) {
            return modules_1.has(name);
        };
        global.loadModule = function loadModule(name, isUIModule) {
            if (isUIModule === void 0) { isUIModule = false; }
            var moduleInfo = modules_1.get(name);
            if (moduleInfo) {
                if (isUIModule) {
                    modulesLoadedForUI_1.add(moduleInfo.moduleId);
                }
                var result = moduleInfo.loader(name);
                if (result.enableAutoAccept) {
                    result.enableAutoAccept();
                }
                return result;
            }
            var _loop_1 = function (resolver) {
                var result = resolver(name);
                if (result) {
                    modules_1.set(name, { moduleId: name, loader: function () { return result; } });
                    return { value: result };
                }
            };
            for (var _i = 0, _a = global.moduleResolvers; _i < _a.length; _i++) {
                var resolver = _a[_i];
                var state_1 = _loop_1(resolver);
                if (typeof state_1 === "object")
                    return state_1.value;
            }
        };
        global.getRegisteredModules = function getRegisteredModules() {
            return Array.from(modules_1.keys());
        };
        /**
         * Polyfills
         */
        // This method iterates all the keys in the source exports object and copies them to the destination exports one.
        // Note: the method will not check for naming collisions and will override any already existing entries in the destination exports.
        global.moduleMerge = function (sourceExports, destExports) {
            for (var key in sourceExports) {
                destExports[key] = sourceExports[key];
            }
        };
        global.zonedCallback = function (callback) {
            if (global.zone) {
                // Zone v0.5.* style callback wrapping
                return global.zone.bind(callback);
            }
            if (global.Zone) {
                // Zone v0.6.* style callback wrapping
                return global.Zone.current.wrap(callback);
            }
            else {
                return callback;
            }
        };
        global.System = {
            import: function (path) {
                return new Promise(function (resolve, reject) {
                    try {
                        resolve(global.require(path));
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
        };
        // DOM api polyfills
        global.registerModule('timer', function () { return require('../timer'); });
        installPolyfills('timer', ['setTimeout', 'clearTimeout', 'setInterval', 'clearInterval']);
        global.registerModule('animation', function () { return require('../animation-frame'); });
        installPolyfills('animation', ['requestAnimationFrame', 'cancelAnimationFrame']);
        global.registerModule('media-query-list', function () { return require('../media-query-list'); });
        installPolyfills('media-query-list', ['matchMedia', 'MediaQueryList']);
        global.registerModule('ui-dialogs', function () { return require('../ui/dialogs'); });
        installPolyfills('ui-dialogs', ['alert', 'confirm', 'prompt', 'login', 'action']);
        global.registerModule('text', function () { return require('../text'); });
        installPolyfills('text', ['TextDecoder', 'TextEncoder']);
        global.registerModule('xhr', function () { return require('../xhr'); });
        installPolyfills('xhr', ['XMLHttpRequest', 'FormData', 'Blob', 'File', 'FileReader']);
        global.registerModule('fetch', function () { return require('../fetch'); });
        installPolyfills('fetch', ['fetch', 'Headers', 'Request', 'Response']);
        global.registerModule('wgc', function () { return require('../wgc'); });
        installPolyfills('wgc', ['atob', 'btoa']);
        global.registerModule('crypto', function () { return require('../wgc/crypto'); });
        installPolyfills('crypto', ['Crypto']);
        global.registerModule('subtle', function () { return require('../wgc/crypto/SubtleCrypto'); });
        installPolyfills('subtle-crypto', ['Subtle']);
        global.crypto = new global.Crypto();
        // global.registerModule('abortcontroller', () => require('../abortcontroller'));
        // installPolyfills('abortcontroller', ['AbortController', 'AbortSignal']);
        // Custom decorators
        global.Deprecated = function (target, key, descriptor) {
            if (descriptor) {
                var originalMethod_1 = descriptor.value;
                descriptor.value = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    console.log("".concat(key.toString(), " is deprecated"));
                    return originalMethod_1.apply(this, args);
                };
                return descriptor;
            }
            else {
                console.log("".concat((target && target.name) || target, " is deprecated"));
                return target;
            }
        };
        global.Experimental = function (target, key, descriptor) {
            if (descriptor) {
                var originalMethod_2 = descriptor.value;
                descriptor.value = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    console.log("".concat(key.toString(), " is experimental"));
                    return originalMethod_2.apply(this, args);
                };
                return descriptor;
            }
            else {
                console.log("".concat((target && target.name) || target, " is experimental"));
                return target;
            }
        };
    }
}
function isTestingEnv() {
    return typeof jest !== 'undefined' || global.__UNIT_TEST__;
}
if (!global.NativeScriptHasInitGlobal && !isTestingEnv()) {
    initGlobal();
}
if (!isTestingEnv()) {
    // ensure the Application instance is initialized before any other module imports it.
    require('@nativescript/core/application');
}
//# sourceMappingURL=index.js.map