import { Font } from '../ui/styling/font';
import { Trace } from '../trace';
// Types.
import { path as fsPath, knownFolders } from '../file-system';
import { isFileOrResourcePath, RESOURCE_PREFIX, SYSTEM_PREFIX } from '../utils';
import { getScaledDimensions } from './image-source-common';
export { isFileOrResourcePath };
var http;
function ensureHttp() {
    if (!http) {
        http = require('../http');
    }
}
var ImageSource = /** @class */ (function () {
    function ImageSource(nativeSource) {
        if (nativeSource) {
            this.setNativeSource(nativeSource);
        }
    }
    Object.defineProperty(ImageSource.prototype, "height", {
        get: function () {
            if (this.ios) {
                return this.ios.size.height;
            }
            return NaN;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ImageSource.prototype, "width", {
        get: function () {
            if (this.ios) {
                return this.ios.size.width;
            }
            return NaN;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ImageSource.prototype, "rotationAngle", {
        get: function () {
            return NaN;
        },
        set: function (_value) {
            // compatibility with Android
        },
        enumerable: false,
        configurable: true
    });
    ImageSource.fromAsset = function (asset) {
        return new Promise(function (resolve, reject) {
            asset.getImageAsync(function (image, err) {
                if (image) {
                    resolve(new ImageSource(image));
                }
                else {
                    reject(err);
                }
            });
        });
    };
    ImageSource.fromUrl = function (url) {
        ensureHttp();
        return http.getImage(url);
    };
    ImageSource.iosSystemScaleFor = function (scale) {
        switch (scale) {
            case 'small':
                return 1 /* UIImageSymbolScale.Small */;
            case 'medium':
                return 2 /* UIImageSymbolScale.Medium */;
            case 'large':
                return 3 /* UIImageSymbolScale.Large */;
            default:
                return -1 /* UIImageSymbolScale.Default */;
        }
    };
    ImageSource.fromSystemImageSync = function (name, scale) {
        if (scale) {
            var image = UIImage.systemImageNamedWithConfiguration(name, UIImageSymbolConfiguration.configurationWithScale(ImageSource.iosSystemScaleFor(scale)));
            return image ? new ImageSource(image) : null;
        }
        else {
            var image = UIImage.systemImageNamed(name);
            return image ? new ImageSource(image) : null;
        }
    };
    ImageSource.fromSystemImage = function (name, scale) {
        return new Promise(function (resolve, reject) {
            try {
                var image = void 0;
                if (scale) {
                    image = UIImage.systemImageNamedWithConfiguration(name, UIImageSymbolConfiguration.configurationWithScale(ImageSource.iosSystemScaleFor(scale)));
                }
                else {
                    image = UIImage.systemImageNamed(name);
                }
                if (image) {
                    resolve(new ImageSource(image));
                }
                else {
                    reject(new Error("Failed to load system icon with name: ".concat(name)));
                }
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.fromResourceSync = function (name) {
        var nativeSource = UIImage.tns_safeImageNamed(name) || UIImage.tns_safeImageNamed("".concat(name, ".jpg"));
        return nativeSource ? new ImageSource(nativeSource) : null;
    };
    ImageSource.fromResource = function (name) {
        return new Promise(function (resolve, reject) {
            try {
                UIImage.tns_safeDecodeImageNamedCompletion(name, function (image) {
                    if (image) {
                        resolve(new ImageSource(image));
                    }
                    else {
                        UIImage.tns_safeDecodeImageNamedCompletion("".concat(name, ".jpg"), function (img) {
                            if (img) {
                                resolve(new ImageSource(img));
                            }
                        });
                    }
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.fromFileSync = function (path) {
        var uiImage = UIImage.imageWithContentsOfFile(getFileName(path));
        return uiImage ? new ImageSource(uiImage) : null;
    };
    ImageSource.fromFile = function (path) {
        return new Promise(function (resolve, reject) {
            try {
                UIImage.tns_decodeImageWidthContentsOfFileCompletion(getFileName(path), function (uiImage) {
                    if (uiImage) {
                        resolve(new ImageSource(uiImage));
                    }
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.fromFileOrResourceSync = function (path) {
        if (!isFileOrResourcePath(path)) {
            if (Trace.isEnabled()) {
                Trace.write('Path "' + path + '" is not a valid file or resource.', Trace.categories.Binding, Trace.messageType.error);
            }
            return null;
        }
        if (path.indexOf(RESOURCE_PREFIX) === 0) {
            return ImageSource.fromResourceSync(path.slice(RESOURCE_PREFIX.length));
        }
        if (path.indexOf(SYSTEM_PREFIX) === 0) {
            return ImageSource.fromSystemImageSync(path.slice(SYSTEM_PREFIX.length));
        }
        return ImageSource.fromFileSync(path);
    };
    ImageSource.fromDataSync = function (data) {
        var uiImage = UIImage.imageWithData(data);
        return uiImage ? new ImageSource(uiImage) : null;
    };
    ImageSource.fromData = function (data) {
        return new Promise(function (resolve, reject) {
            try {
                UIImage.tns_decodeImageWithDataCompletion(data, function (uiImage) {
                    if (uiImage) {
                        resolve(new ImageSource(uiImage));
                    }
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.fromBase64Sync = function (source) {
        var uiImage;
        if (typeof source === 'string') {
            var data = NSData.alloc().initWithBase64EncodedStringOptions(source, 1 /* NSDataBase64DecodingOptions.IgnoreUnknownCharacters */);
            uiImage = UIImage.imageWithData(data);
        }
        return uiImage ? new ImageSource(uiImage) : null;
    };
    ImageSource.fromBase64 = function (source) {
        return new Promise(function (resolve, reject) {
            try {
                var data_1 = NSData.alloc().initWithBase64EncodedStringOptions(source, 1 /* NSDataBase64DecodingOptions.IgnoreUnknownCharacters */);
                var main_queue_1 = dispatch_get_current_queue();
                var background_queue = dispatch_get_global_queue(21 /* qos_class_t.QOS_CLASS_DEFAULT */, 0);
                dispatch_async(background_queue, function () {
                    var uiImage = UIImage.imageWithData(data_1);
                    dispatch_async(main_queue_1, function () {
                        resolve(new ImageSource(uiImage));
                    });
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.fromFontIconCodeSync = function (source, font, color) {
        var _a;
        font = font || Font.default;
        // TODO: Consider making 36 font size as default for optimal look on TabView and ActionBar
        var attributes = (_a = {},
            _a[NSFontAttributeName] = font.getUIFont(UIFont.systemFontOfSize(UIFont.labelFontSize)),
            _a);
        if (color) {
            attributes[NSForegroundColorAttributeName] = color.ios;
        }
        var attributedString = NSAttributedString.alloc().initWithStringAttributes(source, attributes);
        UIGraphicsBeginImageContextWithOptions(attributedString.size(), false, 0.0);
        attributedString.drawAtPoint(CGPointMake(0, 0));
        var iconImage = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        return iconImage ? new ImageSource(iconImage) : null;
    };
    ImageSource.prototype.fromAsset = function (asset) {
        var _this = this;
        console.log('fromAsset() is deprecated. Use ImageSource.fromAsset() instead.');
        return ImageSource.fromAsset(asset).then(function (imgSource) {
            _this.setNativeSource(imgSource.ios);
            return _this;
        });
    };
    ImageSource.prototype.loadFromResource = function (name) {
        console.log('loadFromResource() is deprecated. Use ImageSource.fromResourceSync() instead.');
        var imgSource = ImageSource.fromResourceSync(name);
        this.ios = imgSource ? imgSource.ios : null;
        return !!this.ios;
    };
    ImageSource.prototype.fromResource = function (name) {
        var _this = this;
        console.log('fromResource() is deprecated. Use ImageSource.fromResource() instead.');
        return ImageSource.fromResource(name).then(function (imgSource) {
            _this.ios = imgSource.ios;
            return !!_this.ios;
        });
    };
    ImageSource.prototype.loadFromFile = function (path) {
        console.log('loadFromFile() is deprecated. Use ImageSource.fromFileSync() instead.');
        var imgSource = ImageSource.fromFileSync(path);
        this.ios = imgSource ? imgSource.ios : null;
        return !!this.ios;
    };
    ImageSource.prototype.fromFile = function (path) {
        var _this = this;
        console.log('fromFile() is deprecated. Use ImageSource.fromFile() instead.');
        return ImageSource.fromFile(path).then(function (imgSource) {
            _this.ios = imgSource.ios;
            return !!_this.ios;
        });
    };
    ImageSource.prototype.loadFromData = function (data) {
        console.log('loadFromData() is deprecated. Use ImageSource.fromDataSync() instead.');
        var imgSource = ImageSource.fromDataSync(data);
        this.ios = imgSource ? imgSource.ios : null;
        return !!this.ios;
    };
    ImageSource.prototype.fromData = function (data) {
        var _this = this;
        console.log('fromData() is deprecated. Use ImageSource.fromData() instead.');
        return ImageSource.fromData(data).then(function (imgSource) {
            _this.ios = imgSource.ios;
            return !!_this.ios;
        });
    };
    ImageSource.prototype.loadFromBase64 = function (source) {
        console.log('loadFromBase64() is deprecated. Use ImageSource.fromBase64Sync() instead.');
        var imgSource = ImageSource.fromBase64Sync(source);
        this.ios = imgSource ? imgSource.ios : null;
        return !!this.ios;
    };
    ImageSource.prototype.fromBase64 = function (source) {
        var _this = this;
        console.log('fromBase64() is deprecated. Use ImageSource.fromBase64() instead.');
        return ImageSource.fromBase64(source).then(function (imgSource) {
            _this.ios = imgSource.ios;
            return !!_this.ios;
        });
    };
    ImageSource.prototype.loadFromFontIconCode = function (source, font, color) {
        console.log('loadFromFontIconCode() is deprecated. Use ImageSource.fromFontIconCodeSync() instead.');
        var imgSource = ImageSource.fromFontIconCodeSync(source, font, color);
        this.ios = imgSource ? imgSource.ios : null;
        return !!this.ios;
    };
    ImageSource.prototype.getNativeSource = function () {
        return this.ios;
    };
    ImageSource.prototype.setNativeSource = function (source) {
        if (!source) {
            this.ios = null;
        }
        else if (source instanceof UIImage) {
            this.ios = source;
        }
        else {
            if (Trace.isEnabled()) {
                Trace.write('The method setNativeSource() expects UIImage instance.', Trace.categories.Binding, Trace.messageType.error);
            }
        }
    };
    ImageSource.prototype.saveToFile = function (path, format, quality) {
        if (!this.ios) {
            return false;
        }
        if (quality) {
            quality = (quality - 0) / (100 - 0); // Normalize quality on a scale of 0 to 1
        }
        var data = getImageData(this.ios, format, quality);
        if (data) {
            return NSFileManager.defaultManager.createFileAtPathContentsAttributes(path, data, null);
        }
        return false;
    };
    ImageSource.prototype.saveToFileAsync = function (path, format, quality) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.ios) {
                reject(false);
            }
            var isSuccess = false;
            try {
                if (quality) {
                    quality = (quality - 0) / (100 - 0); // Normalize quality on a scale of 0 to 1
                }
                var main_queue_2 = dispatch_get_current_queue();
                var background_queue = dispatch_get_global_queue(21 /* qos_class_t.QOS_CLASS_DEFAULT */, 0);
                dispatch_async(background_queue, function () {
                    var data = getImageData(_this.ios, format, quality);
                    if (data) {
                        isSuccess = NSFileManager.defaultManager.createFileAtPathContentsAttributes(path, data, null);
                    }
                    dispatch_async(main_queue_2, function () {
                        resolve(isSuccess);
                    });
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.prototype.toBase64String = function (format, quality) {
        var res = null;
        if (!this.ios) {
            return res;
        }
        if (quality) {
            quality = (quality - 0) / (100 - 0); // Normalize quality on a scale of 0 to 1
        }
        var data = getImageData(this.ios, format, quality);
        if (data) {
            res = data.base64Encoding();
        }
        return res;
    };
    ImageSource.prototype.toBase64StringAsync = function (format, quality) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.ios) {
                reject(null);
            }
            var result = null;
            try {
                if (quality) {
                    quality = (quality - 0) / (100 - 0); // Normalize quality on a scale of 0 to 1
                }
                var main_queue_3 = dispatch_get_current_queue();
                var background_queue = dispatch_get_global_queue(21 /* qos_class_t.QOS_CLASS_DEFAULT */, 0);
                dispatch_async(background_queue, function () {
                    var data = getImageData(_this.ios, format, quality);
                    if (data) {
                        result = data.base64Encoding();
                    }
                    dispatch_async(main_queue_3, function () {
                        resolve(result);
                    });
                });
            }
            catch (ex) {
                reject(ex);
            }
        });
    };
    ImageSource.prototype.resize = function (maxSize, options) {
        var _a;
        var size = this.ios.size;
        var dim = getScaledDimensions(size.width, size.height, maxSize);
        var newSize = CGSizeMake(dim.width, dim.height);
        UIGraphicsBeginImageContextWithOptions(newSize, (_a = options === null || options === void 0 ? void 0 : options.opaque) !== null && _a !== void 0 ? _a : false, this.ios.scale);
        this.ios.drawInRect(CGRectMake(0, 0, newSize.width, newSize.height));
        var resizedImage = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        return new ImageSource(resizedImage);
    };
    ImageSource.prototype.resizeAsync = function (maxSize, options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.ios) {
                reject(null);
            }
            var main_queue = dispatch_get_current_queue();
            var background_queue = dispatch_get_global_queue(21 /* qos_class_t.QOS_CLASS_DEFAULT */, 0);
            dispatch_async(background_queue, function () {
                var _a;
                var size = _this.ios.size;
                var dim = getScaledDimensions(size.width, size.height, maxSize);
                var newSize = CGSizeMake(dim.width, dim.height);
                UIGraphicsBeginImageContextWithOptions(newSize, (_a = options === null || options === void 0 ? void 0 : options.opaque) !== null && _a !== void 0 ? _a : false, _this.ios.scale);
                _this.ios.drawInRect(CGRectMake(0, 0, newSize.width, newSize.height));
                var resizedImage = UIGraphicsGetImageFromCurrentImageContext();
                UIGraphicsEndImageContext();
                dispatch_async(main_queue, function () {
                    resolve(new ImageSource(resizedImage));
                });
            });
        });
    };
    return ImageSource;
}());
export { ImageSource };
function getFileName(path) {
    var fileName = typeof path === 'string' ? path.trim() : '';
    if (fileName.indexOf('~/') === 0) {
        fileName = fsPath.join(knownFolders.currentApp().path, fileName.replace('~/', ''));
    }
    return fileName;
}
function getImageData(instance, format, quality) {
    if (quality === void 0) { quality = 0.9; }
    return NativeScriptUtils.getImageDataFormatQuality(instance, format, quality);
}
export function fromAsset(asset) {
    console.log('fromAsset() is deprecated. Use ImageSource.fromAsset() instead.');
    return ImageSource.fromAsset(asset);
}
export function fromResource(name) {
    console.log('fromResource() is deprecated. Use ImageSource.fromResourceSync() instead.');
    return ImageSource.fromResourceSync(name);
}
export function fromFile(path) {
    console.log('fromFile() is deprecated. Use ImageSource.fromFileSync() instead.');
    return ImageSource.fromFileSync(path);
}
export function fromData(data) {
    console.log('fromData() is deprecated. Use ImageSource.fromDataSync() instead.');
    return ImageSource.fromDataSync(data);
}
export function fromFontIconCode(source, font, color) {
    console.log('fromFontIconCode() is deprecated. Use ImageSource.fromFontIconCodeSync() instead.');
    return ImageSource.fromFontIconCodeSync(source, font, color);
}
export function fromBase64(source) {
    console.log('fromBase64() is deprecated. Use ImageSource.fromBase64Sync() instead.');
    return ImageSource.fromBase64Sync(source);
}
export function fromNativeSource(nativeSource) {
    console.log('fromNativeSource() is deprecated. Use ImageSource constructor instead.');
    return new ImageSource(nativeSource);
}
export function fromUrl(url) {
    console.log('fromUrl() is deprecated. Use ImageSource.fromUrl() instead.');
    return ImageSource.fromUrl(url);
}
export function fromFileOrResource(path) {
    console.log('fromFileOrResource() is deprecated. Use ImageSource.fromFileOrResourceSync() instead.');
    return ImageSource.fromFileOrResourceSync(path);
}
//# sourceMappingURL=index.ios.js.map