import { Screen, Device } from '../platform';
import { findMatch, stripQualifiers } from './qualifier-matcher';
import { Trace } from '../trace';
import { Application } from '../application';
var ModuleNameResolver = /** @class */ (function () {
    function ModuleNameResolver(context, moduleListProvider) {
        if (moduleListProvider === void 0) { moduleListProvider = global.getRegisteredModules; }
        this.context = context;
        this.moduleListProvider = moduleListProvider;
        this._cache = {};
        Application.on('livesync', function (args) { return clearCache(); });
        Application.on('orientationChanged', function (args) {
            resolverInstance = undefined;
        });
    }
    ModuleNameResolver.prototype.resolveModuleName = function (path, ext) {
        var key = path + ext;
        var result = this._cache[key];
        if (result === undefined) {
            result = this.resolveModuleNameImpl(path, ext);
            this._cache[key] = result;
        }
        if (Trace.isEnabled()) {
            Trace.write("path: '".concat(path, "' with ext: '").concat(ext, "' resolved: '").concat(result, "'"), Trace.categories.ModuleNameResolver);
        }
        return result;
    };
    ModuleNameResolver.prototype.clearCache = function () {
        this._cache = {};
    };
    ModuleNameResolver.prototype.resolveModuleNameImpl = function (path, ext) {
        var result = null;
        ext = ext ? '.' + ext : '';
        // This call will return a clean path without qualifiers
        path = stripQualifiers(path);
        var candidates = this.getCandidates(path, ext);
        result = findMatch(path, ext, candidates, this.context);
        return result;
    };
    ModuleNameResolver.prototype.getCandidates = function (path, ext) {
        var candidates = this.moduleListProvider().filter(function (moduleName) { return moduleName.startsWith(path) && (!ext || moduleName.endsWith(ext)); });
        return candidates;
    };
    return ModuleNameResolver;
}());
export { ModuleNameResolver };
var resolverInstance;
export function resolveModuleName(path, ext) {
    if (global.__snapshot) {
        return resolveModuleSnapshot(path, ext);
    }
    if (!resolverInstance) {
        resolverInstance = new ModuleNameResolver({
            width: Screen.mainScreen.widthDIPs,
            height: Screen.mainScreen.heightDIPs,
            os: Device.os,
            deviceType: Device.deviceType,
        });
    }
    return resolverInstance.resolveModuleName(path, ext);
}
function resolveModuleSnapshot(path, ext) {
    Trace.write("Resolving module in SNAPSHOT context - path: '".concat(path, "' with ext: '").concat(ext, "'"), Trace.categories.ModuleNameResolver);
    // Platform module when in snapshot. So resolve modules with default android phone.
    // NB: The only module name that should ever be resolved while in snapshot is app.css, because it is
    // applied explicitly in the snapshot by [NativeScriptSnapshotPlugin](https://github.com/NativeScript/nativescript-dev-webpack/blob/48b26f412fd70c19dc0b9c7763e08e9505a0ae11/plugins/NativeScriptSnapshotPlugin/index.js#L48-L56)
    return new ModuleNameResolver({
        width: 400,
        height: 800,
        os: 'Android',
        deviceType: 'Phone',
    }).resolveModuleName(path, ext);
}
export function clearCache() {
    if (resolverInstance) {
        resolverInstance.clearCache();
    }
}
export function _setResolver(resolver) {
    resolverInstance = resolver;
}
//# sourceMappingURL=index.js.map