// Types
import { AnimationBase, Properties } from './animation-common';
import { CubicBezierAnimationCurve } from './animation-interfaces';
import { Trace } from '../../trace';
import { opacityProperty, backgroundColorProperty, rotateProperty, rotateXProperty, rotateYProperty, translateXProperty, translateYProperty, scaleXProperty, scaleYProperty, heightProperty, widthProperty, PercentLength } from '../styling/style-properties';
import { ios as iosBackground } from '../styling/background';
import { ios as iosViewUtils } from '../utils';
import { ios as iosHelper } from '../../utils/native-helper';
import { Screen } from '../../platform';
export * from './animation-common';
export { KeyframeAnimation, KeyframeAnimationInfo, KeyframeDeclaration, KeyframeInfo } from './keyframe-animation';
var _transform = '_transform';
var _skip = '_skip';
var FLT_MAX = 340282346638528859811704183484516925440.0;
var AnimationInfo = /** @class */ (function () {
    function AnimationInfo() {
    }
    return AnimationInfo;
}());
var AnimationDelegateImpl = /** @class */ (function (_super) {
    __extends(AnimationDelegateImpl, _super);
    function AnimationDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AnimationDelegateImpl.initWithFinishedCallback = function (finishedCallback, propertyAnimation, valueSource) {
        var delegate = AnimationDelegateImpl.new();
        delegate._finishedCallback = finishedCallback;
        delegate._propertyAnimation = propertyAnimation;
        delegate._valueSource = valueSource;
        return delegate;
    };
    AnimationDelegateImpl.prototype.animationDidStart = function (anim) {
        var value = this._propertyAnimation.value;
        var setLocal = this._valueSource === 'animation';
        var targetStyle = this._propertyAnimation.target.style;
        this._propertyAnimation.target._suspendPresentationLayerUpdates();
        switch (this._propertyAnimation.property) {
            case Properties.backgroundColor:
                targetStyle[setLocal ? backgroundColorProperty.name : backgroundColorProperty.keyframe] = value;
                break;
            case Properties.opacity:
                targetStyle[setLocal ? opacityProperty.name : opacityProperty.keyframe] = value;
                break;
            case Properties.rotate:
                targetStyle[setLocal ? rotateXProperty.name : rotateXProperty.keyframe] = value.x;
                targetStyle[setLocal ? rotateYProperty.name : rotateYProperty.keyframe] = value.y;
                targetStyle[setLocal ? rotateProperty.name : rotateProperty.keyframe] = value.z;
                break;
            case Properties.translate:
                targetStyle[setLocal ? translateXProperty.name : translateXProperty.keyframe] = value.x;
                targetStyle[setLocal ? translateYProperty.name : translateYProperty.keyframe] = value.y;
                break;
            case Properties.height:
                targetStyle[setLocal ? heightProperty.name : heightProperty.keyframe] = value;
                break;
            case Properties.width:
                targetStyle[setLocal ? widthProperty.name : widthProperty.keyframe] = value;
                break;
            case Properties.scale:
                targetStyle[setLocal ? scaleXProperty.name : scaleXProperty.keyframe] = value.x === 0 ? 0.001 : value.x;
                targetStyle[setLocal ? scaleYProperty.name : scaleYProperty.keyframe] = value.y === 0 ? 0.001 : value.y;
                break;
            case _transform:
                if (value[Properties.translate] !== undefined) {
                    targetStyle[setLocal ? translateXProperty.name : translateXProperty.keyframe] = value[Properties.translate].x;
                    targetStyle[setLocal ? translateYProperty.name : translateYProperty.keyframe] = value[Properties.translate].y;
                }
                if (value[Properties.rotate] !== undefined) {
                    targetStyle[setLocal ? rotateXProperty.name : rotateXProperty.keyframe] = value[Properties.rotate].x;
                    targetStyle[setLocal ? rotateYProperty.name : rotateYProperty.keyframe] = value[Properties.rotate].y;
                    targetStyle[setLocal ? rotateProperty.name : rotateProperty.keyframe] = value[Properties.rotate].z;
                }
                if (value[Properties.scale] !== undefined) {
                    var x = value[Properties.scale].x;
                    var y = value[Properties.scale].y;
                    targetStyle[setLocal ? scaleXProperty.name : scaleXProperty.keyframe] = x === 0 ? 0.001 : x;
                    targetStyle[setLocal ? scaleYProperty.name : scaleYProperty.keyframe] = y === 0 ? 0.001 : y;
                }
                break;
        }
        this._propertyAnimation.target._resumePresentationLayerUpdates();
    };
    AnimationDelegateImpl.prototype.animationDidStopFinished = function (anim, finished) {
        if (this._finishedCallback) {
            this._finishedCallback(!finished);
        }
        if (finished && this.nextAnimation) {
            this.nextAnimation();
        }
    };
    // The CAAnimationDelegate protocol has been introduced in the iOS 10 SDK
    AnimationDelegateImpl.ObjCProtocols = global.CAAnimationDelegate ? [global.CAAnimationDelegate] : [];
    return AnimationDelegateImpl;
}(NSObject));
export function _resolveAnimationCurve(curve) {
    switch (curve) {
        case 'easeIn':
            return CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionEaseIn);
        case 'easeOut':
            return CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionEaseOut);
        case 'easeInOut':
            return CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionEaseInEaseOut);
        case 'linear':
            return CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionLinear);
        case 'spring':
            return curve;
        case 'ease':
            return CAMediaTimingFunction.functionWithControlPoints(0.25, 0.1, 0.25, 1.0);
        default:
            if (curve instanceof CAMediaTimingFunction) {
                return curve;
            }
            else if (curve instanceof CubicBezierAnimationCurve) {
                return CAMediaTimingFunction.functionWithControlPoints(curve.x1, curve.y1, curve.x2, curve.y2);
            }
            else {
                console.error("Invalid animation curve: ".concat(curve));
            }
    }
}
var Animation = /** @class */ (function (_super) {
    __extends(Animation, _super);
    function Animation(animationDefinitions, playSequentially) {
        var _this = _super.call(this, animationDefinitions, playSequentially) || this;
        _this._valueSource = 'animation';
        if (animationDefinitions.length > 0 && animationDefinitions[0].valueSource !== undefined) {
            _this._valueSource = animationDefinitions[0].valueSource;
        }
        if (!playSequentially) {
            if (Trace.isEnabled()) {
                Trace.write('Non-merged Property Animations: ' + _this._propertyAnimations.length, Trace.categories.Animation);
            }
            _this._mergedPropertyAnimations = Animation._mergeAffineTransformAnimations(_this._propertyAnimations);
            if (Trace.isEnabled()) {
                Trace.write('Merged Property Animations: ' + _this._mergedPropertyAnimations.length, Trace.categories.Animation);
            }
        }
        else {
            _this._mergedPropertyAnimations = _this._propertyAnimations;
        }
        var animationFinishedCallback = function (cancelled) {
            if (_this._playSequentially) {
                // This function will be called by the last animation when done or by another animation if the user cancels them halfway through.
                if (!cancelled) {
                    _this._resolveAnimationFinishedPromise();
                }
            }
            else {
                // This callback will be called by each INDIVIDUAL animation when it finishes or is cancelled.
                if (cancelled) {
                    _this._cancelledAnimations++;
                }
                else {
                    _this._finishedAnimations++;
                }
                if (_this._cancelledAnimations > 0 && _this._cancelledAnimations + _this._finishedAnimations === _this._mergedPropertyAnimations.length) {
                    if (Trace.isEnabled()) {
                        Trace.write(_this._cancelledAnimations + ' animations cancelled.', Trace.categories.Animation);
                    }
                }
                else if (_this._finishedAnimations === _this._mergedPropertyAnimations.length) {
                    if (Trace.isEnabled()) {
                        Trace.write(_this._finishedAnimations + ' animations finished.', Trace.categories.Animation);
                    }
                    _this._resolveAnimationFinishedPromise();
                }
            }
        };
        _this._iOSAnimationFunction = Animation._createiOSAnimationFunction(_this._mergedPropertyAnimations, 0, _this._playSequentially, _this._valueSource, animationFinishedCallback);
        return _this;
    }
    Animation.prototype.play = function () {
        if (this.isPlaying) {
            return this._rejectAlreadyPlaying();
        }
        var animationFinishedPromise = _super.prototype.play.call(this);
        this._finishedAnimations = 0;
        this._cancelledAnimations = 0;
        this._iOSAnimationFunction();
        return animationFinishedPromise;
    };
    Animation.prototype.cancel = function () {
        var _a;
        if (!this.isPlaying) {
            Trace.write('Animation is not currently playing.', Trace.categories.Animation, Trace.messageType.warn);
            return;
        }
        if (this._mergedPropertyAnimations) {
            for (var i = 0; i < this._mergedPropertyAnimations.length; i++) {
                var propertyAnimation = this._mergedPropertyAnimations[i];
                if (propertyAnimation) {
                    if ((_a = propertyAnimation.target) === null || _a === void 0 ? void 0 : _a.nativeViewProtected) {
                        var nativeView = propertyAnimation.target.nativeViewProtected;
                        if (nativeView.layer.mask) {
                            nativeView.layer.mask.removeAllAnimations();
                        }
                        nativeView.layer.removeAllAnimations();
                        // Gradient background animations
                        if (nativeView.gradientLayer) {
                            nativeView.gradientLayer.removeAllAnimations();
                        }
                        // Border animations
                        if (nativeView.borderLayer) {
                            if (nativeView.borderLayer.mask) {
                                nativeView.borderLayer.mask.removeAllAnimations();
                            }
                            var borderLayers = nativeView.borderLayer.sublayers;
                            if (borderLayers === null || borderLayers === void 0 ? void 0 : borderLayers.count) {
                                for (var i_1 = 0, count = borderLayers.count; i_1 < count; i_1++) {
                                    borderLayers[i_1].removeAllAnimations();
                                }
                            }
                            nativeView.borderLayer.removeAllAnimations();
                        }
                        // Shadow animations
                        if (nativeView.outerShadowContainerLayer) {
                            if (nativeView.outerShadowContainerLayer.mask) {
                                nativeView.outerShadowContainerLayer.mask.removeAllAnimations();
                            }
                            var outerShadowLayers = nativeView.outerShadowContainerLayer.sublayers;
                            if (outerShadowLayers === null || outerShadowLayers === void 0 ? void 0 : outerShadowLayers.count) {
                                for (var i_2 = 0, count = outerShadowLayers.count; i_2 < count; i_2++) {
                                    var shadowLayer = outerShadowLayers[i_2];
                                    if (shadowLayer.mask) {
                                        shadowLayer.mask.removeAllAnimations();
                                    }
                                    shadowLayer.removeAllAnimations();
                                }
                            }
                            nativeView.outerShadowContainerLayer.removeAllAnimations();
                        }
                    }
                    if (propertyAnimation._propertyResetCallback) {
                        propertyAnimation._propertyResetCallback(propertyAnimation._originalValue, this._valueSource);
                    }
                }
            }
        }
    };
    Animation.prototype._resolveAnimationCurve = function (curve) {
        return _resolveAnimationCurve(curve);
    };
    Animation._createiOSAnimationFunction = function (propertyAnimations, index, playSequentially, valueSource, finishedCallback) {
        return function (cancelled) {
            if (cancelled && finishedCallback) {
                if (Trace.isEnabled()) {
                    Trace.write('Animation ' + (index - 1).toString() + ' was cancelled. Will skip the rest of animations and call finishedCallback(true).', Trace.categories.Animation);
                }
                finishedCallback(cancelled);
                return;
            }
            var animation = propertyAnimations[index];
            var args = Animation._getNativeAnimationArguments(animation, valueSource);
            if (animation.curve === 'spring') {
                Animation._createNativeSpringAnimation(propertyAnimations, index, playSequentially, args, animation, valueSource, finishedCallback);
            }
            else {
                Animation._createNativeAnimation(propertyAnimations, index, playSequentially, args, animation, valueSource, finishedCallback);
            }
        };
    };
    Animation._getNativeAnimationArguments = function (animation, valueSource) {
        var _a;
        var view = animation.target;
        var style = view.style;
        var nativeView = view.nativeViewProtected;
        var parent = view.parent;
        var propertyNameToAnimate = animation.property;
        var subPropertyNameToAnimate;
        var toValue = animation.value;
        var fromValue;
        if (nativeView) {
            var setLocal_1 = valueSource === 'animation';
            switch (animation.property) {
                case Properties.backgroundColor:
                    animation._originalValue = view.backgroundColor;
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? backgroundColorProperty.name : backgroundColorProperty.keyframe] = value;
                    };
                    fromValue = nativeView.layer.backgroundColor;
                    if (nativeView instanceof UILabel) {
                        nativeView.setValueForKey(UIColor.clearColor, 'backgroundColor');
                    }
                    toValue = (_a = toValue === null || toValue === void 0 ? void 0 : toValue.ios) === null || _a === void 0 ? void 0 : _a.CGColor;
                    break;
                case Properties.opacity:
                    animation._originalValue = view.opacity;
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? opacityProperty.name : opacityProperty.keyframe] = value;
                    };
                    fromValue = nativeView.layer.opacity;
                    break;
                case Properties.rotate:
                    animation._originalValue = {
                        x: view.rotateX,
                        y: view.rotateY,
                        z: view.rotate,
                    };
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? rotateProperty.name : rotateProperty.keyframe] = value.z;
                        style[setLocal_1 ? rotateXProperty.name : rotateXProperty.keyframe] = value.x;
                        style[setLocal_1 ? rotateYProperty.name : rotateYProperty.keyframe] = value.y;
                    };
                    propertyNameToAnimate = 'transform.rotation';
                    subPropertyNameToAnimate = ['x', 'y', 'z'];
                    fromValue = {
                        x: nativeView.layer.valueForKeyPath('transform.rotation.x'),
                        y: nativeView.layer.valueForKeyPath('transform.rotation.y'),
                        z: nativeView.layer.valueForKeyPath('transform.rotation.z'),
                    };
                    if (animation.target.rotateX !== undefined && animation.target.rotateX !== 0 && Math.floor(toValue / 360) - toValue / 360 === 0) {
                        fromValue.x = (animation.target.rotateX * Math.PI) / 180;
                    }
                    if (animation.target.rotateY !== undefined && animation.target.rotateY !== 0 && Math.floor(toValue / 360) - toValue / 360 === 0) {
                        fromValue.y = (animation.target.rotateY * Math.PI) / 180;
                    }
                    if (animation.target.rotate !== undefined && animation.target.rotate !== 0 && Math.floor(toValue / 360) - toValue / 360 === 0) {
                        fromValue.z = (animation.target.rotate * Math.PI) / 180;
                    }
                    // Respect only value.z for back-compat until 3D rotations are implemented
                    toValue = {
                        x: (toValue.x * Math.PI) / 180,
                        y: (toValue.y * Math.PI) / 180,
                        z: (toValue.z * Math.PI) / 180,
                    };
                    break;
                case Properties.translate:
                    animation._originalValue = {
                        x: view.translateX,
                        y: view.translateY,
                    };
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? translateXProperty.name : translateXProperty.keyframe] = value.x;
                        style[setLocal_1 ? translateYProperty.name : translateYProperty.keyframe] = value.y;
                    };
                    propertyNameToAnimate = 'transform';
                    fromValue = NSValue.valueWithCATransform3D(nativeView.layer.transform);
                    toValue = NSValue.valueWithCATransform3D(CATransform3DTranslate(nativeView.layer.transform, toValue.x, toValue.y, 0));
                    break;
                case Properties.scale:
                    if (toValue.x === 0) {
                        toValue.x = 0.001;
                    }
                    if (toValue.y === 0) {
                        toValue.y = 0.001;
                    }
                    animation._originalValue = { x: view.scaleX, y: view.scaleY };
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? scaleXProperty.name : scaleXProperty.keyframe] = value.x;
                        style[setLocal_1 ? scaleYProperty.name : scaleYProperty.keyframe] = value.y;
                    };
                    propertyNameToAnimate = 'transform';
                    fromValue = NSValue.valueWithCATransform3D(nativeView.layer.transform);
                    toValue = NSValue.valueWithCATransform3D(CATransform3DScale(nativeView.layer.transform, toValue.x, toValue.y, 1));
                    break;
                case _transform:
                    fromValue = NSValue.valueWithCATransform3D(nativeView.layer.transform);
                    animation._originalValue = {
                        xs: view.scaleX,
                        ys: view.scaleY,
                        xt: view.translateX,
                        yt: view.translateY,
                        rx: view.rotateX,
                        ry: view.rotateY,
                        rz: view.rotate,
                    };
                    animation._propertyResetCallback = function (value, valueSource) {
                        style[setLocal_1 ? translateXProperty.name : translateXProperty.keyframe] = value.xt;
                        style[setLocal_1 ? translateYProperty.name : translateYProperty.keyframe] = value.yt;
                        style[setLocal_1 ? scaleXProperty.name : scaleXProperty.keyframe] = value.xs;
                        style[setLocal_1 ? scaleYProperty.name : scaleYProperty.keyframe] = value.ys;
                        style[setLocal_1 ? rotateXProperty.name : rotateXProperty.keyframe] = value.rx;
                        style[setLocal_1 ? rotateYProperty.name : rotateYProperty.keyframe] = value.ry;
                        style[setLocal_1 ? rotateProperty.name : rotateProperty.keyframe] = value.rz;
                    };
                    propertyNameToAnimate = 'transform';
                    toValue = NSValue.valueWithCATransform3D(Animation._createNativeAffineTransform(animation));
                    break;
                case Properties.width:
                case Properties.height: {
                    var direction = animation.property;
                    var isHeight_1 = direction === 'height';
                    propertyNameToAnimate = 'bounds';
                    if (!parent) {
                        console.error("cannot animate ".concat(direction, " on root view"));
                    }
                    var parentExtent = isHeight_1 ? parent.getMeasuredHeight() : parent.getMeasuredWidth();
                    var asNumber = PercentLength.toDevicePixels(PercentLength.parse(toValue), parentExtent, parentExtent) / Screen.mainScreen.scale;
                    var currentBounds = nativeView.layer.bounds;
                    var extentX = isHeight_1 ? currentBounds.size.width : asNumber;
                    var extentY = isHeight_1 ? asNumber : currentBounds.size.height;
                    fromValue = NSValue.valueWithCGRect(currentBounds);
                    toValue = NSValue.valueWithCGRect(CGRectMake(currentBounds.origin.x, currentBounds.origin.y, extentX, extentY));
                    animation._originalValue = view[isHeight_1 ? 'height' : 'width'];
                    animation._propertyResetCallback = function (value, valueSource) {
                        var prop = isHeight_1 ? heightProperty : widthProperty;
                        style[setLocal_1 ? prop.name : prop.keyframe] = value;
                    };
                    break;
                }
                default:
                    console.error("Animating property '".concat(animation.property, "' is unsupported"));
            }
        }
        var duration = 0.3;
        if (animation.duration !== undefined) {
            duration = animation.duration / 1000.0;
        }
        var delay = undefined;
        if (animation.delay) {
            delay = animation.delay / 1000.0;
        }
        var repeatCount = undefined;
        if (animation.iterations !== undefined) {
            if (animation.iterations === Number.POSITIVE_INFINITY) {
                repeatCount = FLT_MAX;
            }
            else {
                repeatCount = animation.iterations;
            }
        }
        return {
            propertyNameToAnimate: propertyNameToAnimate,
            fromValue: fromValue,
            subPropertiesToAnimate: subPropertyNameToAnimate,
            toValue: toValue,
            duration: duration,
            repeatCount: repeatCount,
            delay: delay,
        };
    };
    Animation._createNativeAnimation = function (propertyAnimations, index, playSequentially, args, animation, valueSource, finishedCallback) {
        var nativeView = animation.target.nativeViewProtected;
        var nativeAnimation;
        if (args.subPropertiesToAnimate) {
            nativeAnimation = this._createGroupAnimation(args, animation);
        }
        else {
            nativeAnimation = this._createBasicAnimation(args, animation);
        }
        var animationDelegate = AnimationDelegateImpl.initWithFinishedCallback(finishedCallback, animation, valueSource);
        nativeAnimation.setValueForKey(animationDelegate, 'delegate');
        if (nativeView) {
            nativeView.layer.addAnimationForKey(nativeAnimation, args.propertyNameToAnimate);
            if (args.propertyNameToAnimate === 'bounds') {
                this.animateNestedLayerSizeUsingBasicAnimation(nativeView, args.toValue.CGRectValue, animation, args, nativeAnimation);
            }
            // Shadow layers do not inherit from animating view layer
            if (nativeView.outerShadowContainerLayer) {
                nativeView.outerShadowContainerLayer.addAnimationForKey(nativeAnimation, args.propertyNameToAnimate);
            }
        }
        var callback = undefined;
        if (index + 1 < propertyAnimations.length) {
            callback = Animation._createiOSAnimationFunction(propertyAnimations, index + 1, playSequentially, valueSource, finishedCallback);
            if (!playSequentially) {
                callback();
            }
            else {
                animationDelegate.nextAnimation = callback;
            }
        }
    };
    Animation._createGroupAnimation = function (args, animation) {
        var _this = this;
        var groupAnimation = CAAnimationGroup.new();
        groupAnimation.duration = args.duration;
        if (args.repeatCount !== undefined) {
            groupAnimation.repeatCount = args.repeatCount;
        }
        if (args.delay !== undefined) {
            groupAnimation.beginTime = CACurrentMediaTime() + args.delay;
        }
        if (animation.curve !== undefined) {
            groupAnimation.timingFunction = animation.curve;
        }
        var animations = NSMutableArray.alloc().initWithCapacity(3);
        args.subPropertiesToAnimate.forEach(function (property) {
            var basicAnimationArgs = __assign(__assign({}, args), { duration: undefined, repeatCount: undefined, delay: undefined, curve: undefined });
            basicAnimationArgs.propertyNameToAnimate = "".concat(args.propertyNameToAnimate, ".").concat(property);
            basicAnimationArgs.fromValue = args.fromValue[property];
            basicAnimationArgs.toValue = args.toValue[property];
            var basicAnimation = _this._createBasicAnimation(basicAnimationArgs, animation);
            animations.addObject(basicAnimation);
        });
        groupAnimation.animations = animations;
        return groupAnimation;
    };
    Animation._createBasicAnimation = function (args, animation) {
        var basicAnimation = CABasicAnimation.animationWithKeyPath(args.propertyNameToAnimate);
        basicAnimation.fromValue = args.fromValue;
        basicAnimation.toValue = args.toValue;
        basicAnimation.duration = args.duration;
        if (args.repeatCount !== undefined) {
            basicAnimation.repeatCount = args.repeatCount;
        }
        if (args.delay !== undefined) {
            basicAnimation.beginTime = CACurrentMediaTime() + args.delay;
        }
        if (animation.curve !== undefined) {
            basicAnimation.timingFunction = animation.curve;
        }
        return basicAnimation;
    };
    Animation._createNativeSpringAnimation = function (propertyAnimations, index, playSequentially, args, animation, valueSource, finishedCallback) {
        var nativeView = animation.target.nativeViewProtected;
        var callback = undefined;
        var nextAnimation;
        if (index + 1 < propertyAnimations.length) {
            callback = Animation._createiOSAnimationFunction(propertyAnimations, index + 1, playSequentially, valueSource, finishedCallback);
            if (!playSequentially) {
                callback();
            }
            else {
                nextAnimation = callback;
            }
        }
        var delay = 0;
        if (args.delay) {
            delay = args.delay;
        }
        UIView.animateWithDurationDelayUsingSpringWithDampingInitialSpringVelocityOptionsAnimationsCompletion(args.duration, delay, 0.2, 0, 196608 /* UIViewAnimationOptions.CurveLinear */, function () {
            if (args.repeatCount !== undefined) {
                UIView.setAnimationRepeatCount(args.repeatCount);
            }
            switch (animation.property) {
                case Properties.backgroundColor:
                    animation.target.backgroundColor = args.toValue;
                    break;
                case Properties.opacity:
                    animation.target.opacity = args.toValue;
                    break;
                case Properties.height:
                case Properties.width:
                    animation._originalValue = animation.target[animation.property];
                    nativeView.layer.setValueForKey(args.toValue, args.propertyNameToAnimate);
                    // Resize background during animation
                    iosBackground.drawBackgroundVisualEffects(animation.target);
                    animation._propertyResetCallback = function (value) {
                        animation.target[animation.property] = value;
                    };
                    break;
                case _transform:
                    animation._originalValue = nativeView.layer.transform;
                    nativeView.layer.setValueForKey(args.toValue, args.propertyNameToAnimate);
                    // Shadow layers do not inherit from animating view layer
                    if (nativeView.outerShadowContainerLayer) {
                        nativeView.outerShadowContainerLayer.setValueForKey(args.toValue, args.propertyNameToAnimate);
                    }
                    animation._propertyResetCallback = function (value) {
                        nativeView.layer.transform = value;
                    };
                    break;
            }
        }, function (animationDidFinish) {
            if (animationDidFinish) {
                if (animation.property === _transform) {
                    if (animation.value[Properties.translate] !== undefined) {
                        animation.target.translateX = animation.value[Properties.translate].x;
                        animation.target.translateY = animation.value[Properties.translate].y;
                    }
                    if (animation.value[Properties.rotate] !== undefined) {
                        animation.target.rotateX = animation.value[Properties.rotate].x;
                        animation.target.rotateY = animation.value[Properties.rotate].y;
                        animation.target.rotate = animation.value[Properties.rotate].z;
                    }
                    if (animation.value[Properties.scale] !== undefined) {
                        animation.target.scaleX = animation.value[Properties.scale].x;
                        animation.target.scaleY = animation.value[Properties.scale].y;
                    }
                }
            }
            else {
                if (animation._propertyResetCallback) {
                    animation._propertyResetCallback(animation._originalValue);
                }
            }
            if (finishedCallback) {
                var cancelled = !animationDidFinish;
                finishedCallback(cancelled);
            }
            if (animationDidFinish && nextAnimation) {
                nextAnimation();
            }
        });
    };
    Animation._createNativeAffineTransform = function (animation) {
        var value = animation.value;
        var result = CATransform3DIdentity;
        if (value[Properties.translate] !== undefined) {
            var x = value[Properties.translate].x;
            var y = value[Properties.translate].y;
            result = CATransform3DTranslate(result, x, y, 0);
        }
        if (value[Properties.scale] !== undefined) {
            var x = value[Properties.scale].x;
            var y = value[Properties.scale].y;
            result = CATransform3DScale(result, x === 0 ? 0.001 : x, y === 0 ? 0.001 : y, 1);
        }
        return result;
    };
    Animation._isAffineTransform = function (property) {
        return property === _transform || property === Properties.translate || property === Properties.scale;
    };
    Animation._canBeMerged = function (animation1, animation2) {
        var result = Animation._isAffineTransform(animation1.property) && Animation._isAffineTransform(animation2.property) && animation1.target === animation2.target && animation1.duration === animation2.duration && animation1.delay === animation2.delay && animation1.iterations === animation2.iterations && animation1.curve === animation2.curve;
        return result;
    };
    Animation._mergeAffineTransformAnimations = function (propertyAnimations) {
        var result = new Array();
        var i = 0;
        var j;
        var length = propertyAnimations.length;
        for (; i < length; i++) {
            if (propertyAnimations[i][_skip]) {
                continue;
            }
            if (!Animation._isAffineTransform(propertyAnimations[i].property)) {
                // This is not an affine transform animation, so there is nothing to merge.
                result.push(propertyAnimations[i]);
            }
            else {
                // This animation has not been merged anywhere. Create a new transform animation.
                // The value becomes a JSON object combining all affine transforms together like this:
                // {
                //    translate: {x: 100, y: 100 },
                //    rotate: 90,
                //    scale: {x: 2, y: 2 }
                // }
                var newTransformAnimation = {
                    target: propertyAnimations[i].target,
                    property: _transform,
                    value: {},
                    duration: propertyAnimations[i].duration,
                    delay: propertyAnimations[i].delay,
                    iterations: propertyAnimations[i].iterations,
                    curve: propertyAnimations[i].curve,
                };
                if (Trace.isEnabled()) {
                    Trace.write('Curve: ' + propertyAnimations[i].curve, Trace.categories.Animation);
                }
                newTransformAnimation.value[propertyAnimations[i].property] = propertyAnimations[i].value;
                if (Trace.isEnabled()) {
                    Trace.write('Created new transform animation: ' + Animation._getAnimationInfo(newTransformAnimation), Trace.categories.Animation);
                }
                // Merge all compatible affine transform animations to the right into this new animation.
                j = i + 1;
                if (j < length) {
                    for (; j < length; j++) {
                        if (Animation._canBeMerged(propertyAnimations[i], propertyAnimations[j])) {
                            if (Trace.isEnabled()) {
                                Trace.write('Merging animations: ' + Animation._getAnimationInfo(newTransformAnimation) + ' + ' + Animation._getAnimationInfo(propertyAnimations[j]) + ';', Trace.categories.Animation);
                            }
                            newTransformAnimation.value[propertyAnimations[j].property] = propertyAnimations[j].value;
                            // Mark that it has been merged so we can skip it on our outer loop.
                            propertyAnimations[j][_skip] = true;
                        }
                    }
                }
                result.push(newTransformAnimation);
            }
        }
        return result;
    };
    Animation.animateNestedLayerSizeUsingBasicAnimation = function (nativeView, bounds, animation, args, nativeAnimation) {
        var view = animation.target;
        // Gradient background animation
        if (nativeView.gradientLayer) {
            nativeView.gradientLayer.addAnimationForKey(nativeAnimation, 'bounds');
        }
        var clipPath; // This is also used for animating shadow
        // Clipping mask animation
        if (nativeView.layer.mask instanceof CAShapeLayer) {
            var toValue = void 0;
            if (nativeView.maskType === iosViewUtils.LayerMask.BORDER) {
                toValue = iosBackground.generateNonUniformBorderOuterClipRoundedPath(view, bounds);
            }
            else if (nativeView.maskType === iosViewUtils.LayerMask.CLIP_PATH) {
                clipPath = iosBackground.generateClipPath(view, bounds);
                toValue = clipPath;
            }
            else {
                Trace.write('Unknown mask on animating view: ' + view, Trace.categories.Animation, Trace.messageType.info);
            }
            if (toValue) {
                nativeView.layer.mask.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: nativeView.layer.mask.path, toValue: toValue }), animation), 'path');
            }
        }
        // Border animations (uniform and non-uniform)
        if (nativeView.hasNonUniformBorder) {
            if (nativeView.borderLayer) {
                var innerClipPath = iosBackground.generateNonUniformBorderInnerClipRoundedPath(animation.target, bounds);
                if (nativeView.hasNonUniformBorderColor) {
                    var borderMask = nativeView.borderLayer.mask;
                    if (borderMask instanceof CAShapeLayer) {
                        borderMask.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: borderMask.path, toValue: innerClipPath }), animation), 'path');
                    }
                    var borderLayers = nativeView.borderLayer.sublayers;
                    if (borderLayers === null || borderLayers === void 0 ? void 0 : borderLayers.count) {
                        var paths = iosBackground.generateNonUniformMultiColorBorderRoundedPaths(animation.target, bounds);
                        for (var i = 0, count = borderLayers.count; i < count; i++) {
                            var layer = nativeView.borderLayer.sublayers[i];
                            if (layer instanceof CAShapeLayer) {
                                layer.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: layer.path, toValue: paths[i] }), animation), 'path');
                            }
                        }
                    }
                }
                else {
                    nativeView.borderLayer.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: nativeView.borderLayer.path, toValue: innerClipPath }), animation), 'path');
                }
            }
        }
        else {
            // TODO: Animate border width when borders get support for percentage values
            // Uniform corner radius also relies on view size
            if (nativeView.layer.cornerRadius) {
                nativeView.layer.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'cornerRadius', fromValue: nativeView.layer.cornerRadius, toValue: iosBackground.getUniformBorderRadius(animation.target, bounds) }), animation), 'cornerRadius');
            }
        }
        // Shadow layers do not inherit from animating view layer
        if (nativeView.outerShadowContainerLayer) {
            var shadowClipMask = nativeView.outerShadowContainerLayer.mask;
            // This is for animating view clip path on shadow
            if (clipPath && shadowClipMask instanceof CAShapeLayer) {
                shadowClipMask.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: shadowClipMask.path, toValue: clipPath }), animation), 'path');
            }
            var outerShadowLayers = nativeView.outerShadowContainerLayer.sublayers;
            if (outerShadowLayers === null || outerShadowLayers === void 0 ? void 0 : outerShadowLayers.count) {
                var _a = iosBackground.generateShadowLayerPaths(view, bounds), maskPath = _a.maskPath, shadowPath = _a.shadowPath;
                for (var i = 0, count = outerShadowLayers.count; i < count; i++) {
                    var shadowLayer = outerShadowLayers[i];
                    shadowLayer.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'shadowPath', fromValue: shadowLayer.shadowPath, toValue: shadowPath }), animation), 'shadowPath');
                    if (shadowLayer.mask instanceof CAShapeLayer) {
                        shadowLayer.mask.addAnimationForKey(this._createBasicAnimation(__assign(__assign({}, args), { propertyNameToAnimate: 'path', fromValue: shadowLayer.mask.path, toValue: maskPath }), animation), 'path');
                    }
                }
            }
        }
    };
    return Animation;
}(AnimationBase));
export { Animation };
export function _getTransformMismatchErrorMessage(view) {
    var expectedTransform = calculateTransform(view);
    var expectedTransformString = getCATransform3DString(expectedTransform);
    var actualTransformString = getCATransform3DString(view.nativeViewProtected.layer.transform);
    if (actualTransformString !== expectedTransformString) {
        return 'View and Native transforms do not match.\nActual: ' + actualTransformString + ';\nExpected: ' + expectedTransformString;
    }
    return undefined;
}
function calculateTransform(view) {
    var scaleX = view.scaleX || 1e-6;
    var scaleY = view.scaleY || 1e-6;
    var perspective = view.perspective || 300;
    // Order is important: translate, rotate, scale
    var expectedTransform = new CATransform3D(CATransform3DIdentity);
    // Only set perspective if there is 3D rotation
    if (view.rotateX || view.rotateY) {
        expectedTransform.m34 = -1 / perspective;
    }
    expectedTransform = CATransform3DTranslate(expectedTransform, view.translateX, view.translateY, 0);
    expectedTransform = iosHelper.applyRotateTransform(expectedTransform, view.rotateX, view.rotateY, view.rotate);
    expectedTransform = CATransform3DScale(expectedTransform, scaleX, scaleY, 1);
    return expectedTransform;
}
function getCATransform3DString(t) {
    return "[\n    ".concat(t.m11, ", ").concat(t.m12, ", ").concat(t.m13, ", ").concat(t.m14, ",\n    ").concat(t.m21, ", ").concat(t.m22, ", ").concat(t.m23, ", ").concat(t.m24, ",\n    ").concat(t.m31, ", ").concat(t.m32, ", ").concat(t.m33, ", ").concat(t.m34, ",\n    ").concat(t.m41, ", ").concat(t.m42, ", ").concat(t.m43, ", ").concat(t.m44, "]");
}
//# sourceMappingURL=index.ios.js.map