// Types.
import { debug } from '../../utils/debug';
import { isDefined } from '../../utils/types';
import { sanitizeModuleName } from '../../utils/common';
import { setPropertyValue, getComponentModule } from './component-builder';
import { platformNames } from '../../platform';
import { resolveModuleName } from '../../module-name-resolver';
import { xml2ui } from './xml2ui';
export var ios = platformNames.ios.toLowerCase();
export var android = platformNames.android.toLowerCase();
export var visionos = platformNames.visionos.toLowerCase();
export var defaultNameSpaceMatcher = /tns\.xsd$/i;
var Builder = /** @class */ (function () {
    function Builder() {
    }
    Builder.createViewFromEntry = function (entry) {
        if (entry.create) {
            var view = entry.create();
            if (!view) {
                throw new Error('Failed to create View with entry.create() function.');
            }
            return view;
        }
        else if (entry.moduleName) {
            var moduleName = sanitizeModuleName(entry.moduleName);
            var resolvedCodeModuleName = resolveModuleName(moduleName, ''); //`${moduleName}.xml`;
            var moduleExports = resolvedCodeModuleName ? global.loadModule(resolvedCodeModuleName, true) : null;
            if (moduleExports && moduleExports.createPage) {
                // Exports has a createPage() method
                var view = moduleExports.createPage();
                var resolvedCssModuleName = resolveModuleName(moduleName, 'css'); //entry.moduleName + ".css";
                if (resolvedCssModuleName) {
                    view.addCssFile(resolvedCssModuleName);
                }
                return view;
            }
            else {
                var componentView = void 0;
                if (__UI_USE_XML_PARSER__) {
                    var componentModule = loadInternal(moduleName, moduleExports);
                    componentView = componentModule && componentModule.component;
                }
                else {
                    var resolvedXmlModuleName = resolveModuleName(moduleName, 'xml');
                    var componentModule = resolvedXmlModuleName ? global.loadModule(resolvedXmlModuleName, true) : null;
                    if (componentModule === null || componentModule === void 0 ? void 0 : componentModule.default) {
                        componentView = new componentModule.default();
                    }
                    else {
                        throw new Error('Failed to load component from module: ' + moduleName);
                    }
                }
                return componentView;
            }
        }
        throw new Error('Failed to load page XML file for module: ' + entry.moduleName);
    };
    Builder.parse = function (value, context) {
        if (typeof value === 'function') {
            return value();
        }
        else {
            var exports_1 = context ? getExports(context) : undefined;
            var componentModule = parseInternal(value, exports_1);
            return componentModule && componentModule.component;
        }
    };
    Builder.load = function (pathOrOptions, context) {
        var componentModule;
        if (typeof pathOrOptions === 'string') {
            var moduleName = sanitizeModuleName(pathOrOptions);
            componentModule = loadInternal(moduleName, context);
        }
        else {
            componentModule = loadCustomComponent(pathOrOptions.path, pathOrOptions.name, pathOrOptions.attributes, pathOrOptions.exports, pathOrOptions.page, true);
        }
        return componentModule && componentModule.component;
    };
    Builder.parseMultipleTemplates = function (value, context) {
        var dummyComponent = "<ListView><ListView.itemTemplates>".concat(value, "</ListView.itemTemplates></ListView>");
        return parseInternal(dummyComponent, context).component['itemTemplates'];
    };
    // ui plugin developers can add to these to define their own custom types if needed
    Builder.knownTemplates = new Set(['itemTemplate']);
    Builder.knownMultiTemplates = new Set(['itemTemplates']);
    Builder.knownCollections = new Set(['items', 'spans', 'actionItems']);
    return Builder;
}());
export { Builder };
export function parse(value, context) {
    console.log('parse() is deprecated. Use Builder.parse() instead.');
    return Builder.parse(value, context);
}
export function parseMultipleTemplates(value, context) {
    console.log('parseMultipleTemplates() is deprecated. Use Builder.parseMultipleTemplates() instead.');
    return Builder.parseMultipleTemplates(value, context);
}
export function load(pathOrOptions, context) {
    console.log('load() is deprecated. Use Builder.load() instead.');
    return Builder.load(pathOrOptions, context);
}
export function createViewFromEntry(entry) {
    console.log('createViewFromEntry() is deprecated. Use Builder.createViewFromEntry() instead.');
    return Builder.createViewFromEntry(entry);
}
function loadInternal(moduleName, moduleExports) {
    var componentModule;
    var resolvedXmlModule = resolveModuleName(moduleName, 'xml');
    if (resolvedXmlModule) {
        var text = global.loadModule(resolvedXmlModule, true);
        componentModule = parseInternal(text, moduleExports, resolvedXmlModule, moduleName);
    }
    var componentView = componentModule && componentModule.component;
    if (componentView) {
        // Save exports to root component (will be used for templates).
        componentView.exports = moduleExports;
        // Save _moduleName - used for livesync
        componentView._moduleName = moduleName;
    }
    if (!componentModule) {
        throw new Error('Failed to load component from module: ' + moduleName);
    }
    return componentModule;
}
export function loadCustomComponent(componentNamespace, componentName, attributes, context, parentPage, isRootComponent, moduleNamePath) {
    if (isRootComponent === void 0) { isRootComponent = true; }
    if (!parentPage && context) {
        // Read the parent page that was passed down below
        // https://github.com/NativeScript/NativeScript/issues/1639
        parentPage = context['_parentPage'];
        delete context['_parentPage'];
    }
    var result;
    componentNamespace = sanitizeModuleName(componentNamespace);
    var moduleName = "".concat(componentNamespace, "/").concat(componentName);
    var resolvedCodeModuleName = resolveModuleName(moduleName, '');
    var resolvedXmlModuleName = resolveModuleName(moduleName, 'xml');
    var resolvedCssModuleName = resolveModuleName(moduleName, 'css');
    if (resolvedXmlModuleName) {
        // Custom components with XML
        var subExports = context;
        if (resolvedCodeModuleName) {
            // Component has registered code module.
            subExports = global.loadModule(resolvedCodeModuleName, true);
        }
        // Pass the parent page down the chain in case of custom components nested on many levels. Use the context for piggybacking.
        // https://github.com/NativeScript/NativeScript/issues/1639
        if (!subExports) {
            subExports = {};
        }
        subExports['_parentPage'] = parentPage;
        result = loadInternal(moduleName, subExports);
        // Attributes will be transferred to the custom component
        if (isDefined(result) && isDefined(result.component) && isDefined(attributes)) {
            for (var attr in attributes) {
                setPropertyValue(result.component, subExports, context, attr, attributes[attr]);
            }
        }
    }
    else {
        // Custom components without XML
        result = getComponentModule(componentName, componentNamespace, attributes, context, moduleNamePath, isRootComponent);
        // The namespace is the JS module and the (componentName is the name of the class in the module)
        // So if there is no componentNamespace/componentName.{qualifiers}.css we should also look for
        // componentNamespace.{qualifiers}.css
        if (!resolvedCssModuleName) {
            resolvedCssModuleName = resolveModuleName(componentNamespace, 'css');
        }
    }
    // Add CSS from webpack module if exists.
    if (parentPage && resolvedCssModuleName) {
        parentPage.addCssFile(resolvedCssModuleName);
    }
    return result;
}
export function getExports(instance) {
    var isView = !!instance._domId;
    if (!isView) {
        return instance.exports || instance;
    }
    var exportObject = instance.exports;
    var parent = instance.parent;
    while (exportObject === undefined && parent) {
        exportObject = parent.exports;
        parent = parent.parent;
    }
    return exportObject;
}
function parseInternal(value, context, xmlModule, moduleName) {
    if (__UI_USE_XML_PARSER__) {
        var start = void 0;
        var ui = void 0;
        var errorFormat = debug && xmlModule ? xml2ui.SourceErrorFormat(xmlModule) : xml2ui.PositionErrorFormat;
        var componentSourceTracker = debug && xmlModule
            ? xml2ui.ComponentSourceTracker(xmlModule)
            : function () {
                // no-op
            };
        (start = new xml2ui.XmlStringParser(errorFormat)).pipe(new xml2ui.PlatformFilter()).pipe(new xml2ui.XmlStateParser((ui = new xml2ui.ComponentParser(context, errorFormat, componentSourceTracker, moduleName))));
        start.parse(value);
        return ui.rootComponentModule;
    }
    else {
        return null;
    }
}
//# sourceMappingURL=index.js.map