import { parse } from 'acorn';
import { isFunction, isNullOrUndefined, isObject } from '../../../utils/types';
var expressionsCache = {};
var FORCED_CHAIN_VALUE = Symbol('forcedChain');
// prettier-ignore
var unaryOperators = {
    '+': function (v) { return +v; },
    '-': function (v) { return -v; },
    '!': function (v) { return !v; },
    'void': function (v) { return void v; },
    'typeof': function (v) { return typeof v; }
};
// prettier-ignore
var binaryOperators = {
    '+': function (l, r) { return l + r; },
    '-': function (l, r) { return l - r; },
    '*': function (l, r) { return l * r; },
    '**': function (l, r) { return Math.pow(l, r); },
    '/': function (l, r) { return l / r; },
    '%': function (l, r) { return l % r; },
    '<': function (l, r) { return l < r; },
    '>': function (l, r) { return l > r; },
    '<=': function (l, r) { return l <= r; },
    '>=': function (l, r) { return l >= r; },
    '==': function (l, r) { return l == r; },
    '!=': function (l, r) { return l != r; },
    '===': function (l, r) { return l === r; },
    '!==': function (l, r) { return l !== r; },
    '|': function (l, r) { return l | r; },
    'in': function (l, r) { return l in r; },
    'instanceof': function (l, r) { return l instanceof r; }
};
// prettier-ignore
var logicalOperators = {
    '&&': function (l, r) { return l && r(); },
    '||': function (l, r) { return l || r(); },
    '??': function (l, r) { return l !== null && l !== void 0 ? l : r(); }
};
// prettier-ignore
var expressionParsers = {
    'ArrayExpression': function (expression, model, isBackConvert, changedModel) {
        var parsed = [];
        for (var _i = 0, _a = expression.elements; _i < _a.length; _i++) {
            var element = _a[_i];
            var value = convertExpressionToValue(element, model, isBackConvert, changedModel);
            element.type == 'SpreadElement' ? parsed.push.apply(parsed, value) : parsed.push(value);
        }
        return parsed;
    },
    'BinaryExpression': function (expression, model, isBackConvert, changedModel) {
        if (binaryOperators[expression.operator] == null) {
            throw new Error('Disallowed binary operator: ' + expression.operator);
        }
        var left = convertExpressionToValue(expression.left, model, isBackConvert, changedModel);
        var converterExpression = expression.right;
        if (expression.operator == '|') {
            if (converterExpression.type == 'ChainExpression') {
                converterExpression = converterExpression.expression;
            }
            if (converterExpression.type == 'CallExpression') {
                !converterExpression.arguments.includes(expression.left) && converterExpression.arguments.unshift(expression.left);
                expression.right.nsIsCallable = true;
                converterExpression = converterExpression.callee;
            }
            switch (converterExpression.type) {
                case 'Identifier':
                case 'MemberExpression':
                case 'NewExpression':
                    converterExpression.nsRequiresConverter = true;
                    converterExpression.nsIsPendingCall = true;
                    break;
                default:
                    throw new Error('Invalid converter syntax');
            }
        }
        var right = convertExpressionToValue(expression.right, model, isBackConvert, changedModel);
        if (expression.operator == '|') {
            if (converterExpression.nsRequiresConverter) {
                if (expression.right.nsIsCallable) {
                    return right;
                }
                if (isFunction(right)) {
                    return right(left);
                }
                if (isNullOrUndefined(right)) {
                    throw new Error('Cannot perform a call using a null or undefined property');
                }
            }
            throw new Error('Invalid converter syntax');
        }
        return binaryOperators[expression.operator](left, right);
    },
    'CallExpression': function (expression, model, isBackConvert, changedModel) {
        expression.callee.nsIsPendingCall = true;
        var callback = convertExpressionToValue(expression.callee, model, isBackConvert, changedModel);
        if (!expression.optional && isNullOrUndefined(callback)) {
            throw new Error('Cannot perform a call using a null or undefined property');
        }
        var parsedArgs = [];
        for (var _i = 0, _a = expression.arguments; _i < _a.length; _i++) {
            var argument = _a[_i];
            var value = convertExpressionToValue(argument, model, isBackConvert, changedModel);
            argument.type == 'SpreadElement' ? parsedArgs.push.apply(parsedArgs, value) : parsedArgs.push(value);
        }
        return expression.optional ? callback === null || callback === void 0 ? void 0 : callback.apply(void 0, parsedArgs) : callback.apply(void 0, parsedArgs);
    },
    'ChainExpression': function (expression, model, isBackConvert, changedModel) {
        return convertExpressionToValue(expression.expression, model, isBackConvert, changedModel);
    },
    'ConditionalExpression': function (expression, model, isBackConvert, changedModel) {
        var test = convertExpressionToValue(expression.test, model, isBackConvert, changedModel);
        return convertExpressionToValue(expression[test ? 'consequent' : 'alternate'], model, isBackConvert, changedModel);
    },
    'Identifier': function (expression, model, isBackConvert, changedModel) {
        var context = getContext(expression.name, model, changedModel);
        var value = context[expression.name];
        if (expression.nsRequiresConverter) {
            value = getConverterCallback(value, isBackConvert);
        }
        return expression.nsIsPendingCall && typeof value === 'function' ? value.bind(context) : value;
    },
    'Literal': function (expression, model, isBackConvert, changedModel) {
        return expression.regex != null ? new RegExp(expression.regex.pattern, expression.regex.flags) : expression.value;
    },
    'LogicalExpression': function (expression, model, isBackConvert, changedModel) {
        if (logicalOperators[expression.operator] == null) {
            throw new Error('Disallowed logical operator: ' + expression.operator);
        }
        var left = convertExpressionToValue(expression.left, model, isBackConvert, changedModel);
        return logicalOperators[expression.operator](left, function () { return convertExpressionToValue(expression.right, model, isBackConvert, changedModel); });
    },
    'MemberExpression': function (expression, model, isBackConvert, changedModel) {
        var _a;
        if (expression.object.type == 'MemberExpression') {
            expression.object.nsIsChained = true;
        }
        var object = convertExpressionToValue(expression.object, model, isBackConvert, changedModel);
        var property = expression.computed ? convertExpressionToValue(expression.property, model, isBackConvert, changedModel) : (_a = expression.property) === null || _a === void 0 ? void 0 : _a.name;
        /**
         * If an expression parent property is null or undefined, apply null-safety.
         * This behaviour also helps cope with components whose binding context takes a bit longer to load.
         * Old parser would be null-safe for properties and sub-properties
         * even if expression as a whole consisted of undefined ones.
         * The new parser will keep the same principle only if parent property is null or undefined, resulting in better control over code and errors.
         * It meddles with members specifically, so that it will not affect expression result as a whole.
         * For example, an 'isLoading || isBusy' expression will be validated as 'undefined || undefined'
         * if context is not ready.
         */
        if (object == null && expression.object.type == 'Identifier') {
            return expression.nsIsChained ? FORCED_CHAIN_VALUE : undefined;
        }
        if (object == FORCED_CHAIN_VALUE) {
            return expression.nsIsChained ? object : undefined;
        }
        var value = expression.optional ? object === null || object === void 0 ? void 0 : object[property] : object[property];
        if (expression.nsRequiresConverter) {
            value = getConverterCallback(value, isBackConvert);
        }
        return expression.nsIsPendingCall && typeof value === 'function' ? value.bind(object) : value;
    },
    'NewExpression': function (expression, model, isBackConvert, changedModel) {
        var callback = convertExpressionToValue(expression.callee, model, isBackConvert, changedModel);
        var parsedArgs = [];
        for (var _i = 0, _a = expression.arguments; _i < _a.length; _i++) {
            var argument = _a[_i];
            var value_1 = convertExpressionToValue(argument, model, isBackConvert, changedModel);
            argument.type == 'SpreadElement' ? parsedArgs.push.apply(parsedArgs, value_1) : parsedArgs.push(value_1);
        }
        var value = new (callback.bind.apply(callback, __spreadArray([void 0], parsedArgs, false)))();
        if (expression.nsRequiresConverter) {
            value = getConverterCallback(value, isBackConvert);
        }
        return value;
    },
    'ObjectExpression': function (expression, model, isBackConvert, changedModel) {
        var parsedObject = {};
        for (var _i = 0, _a = expression.properties; _i < _a.length; _i++) {
            var property = _a[_i];
            var value = convertExpressionToValue(property, model, isBackConvert, changedModel);
            Object.assign(parsedObject, value);
        }
        return parsedObject;
    },
    'Property': function (expression, model, isBackConvert, changedModel) {
        var _a;
        var _b;
        var key = expression.computed ? convertExpressionToValue(expression.key, model, isBackConvert, changedModel) : (_b = expression.key) === null || _b === void 0 ? void 0 : _b.name;
        var value = convertExpressionToValue(expression.value, model, isBackConvert, changedModel);
        return _a = {}, _a[key] = value, _a;
    },
    'SpreadElement': function (expression, model, isBackConvert, changedModel) {
        var argument = convertExpressionToValue(expression.argument, model, isBackConvert, changedModel);
        return argument;
    },
    'TemplateElement': function (expression, model, isBackConvert, changedModel) {
        return expression.value.cooked;
    },
    'TemplateLiteral': function (expression, model, isBackConvert, changedModel) {
        var parsedText = '';
        var length = expression.quasis.length;
        for (var i = 0; i < length; i++) {
            var q = expression.quasis[i];
            parsedText += convertExpressionToValue(q, model, isBackConvert, changedModel);
            if (!q.tail) {
                parsedText += convertExpressionToValue(expression.expressions[i], model, isBackConvert, changedModel);
            }
        }
        return parsedText;
    },
    'UnaryExpression': function (expression, model, isBackConvert, changedModel) {
        if (unaryOperators[expression.operator] == null) {
            throw Error('Disallowed unary operator: ' + expression.operator);
        }
        var argument = convertExpressionToValue(expression.argument, model, isBackConvert, changedModel);
        return unaryOperators[expression.operator](argument);
    }
};
function getContext(key, model, changedModel) {
    var context = key in changedModel ? changedModel : model;
    if (!(key in context)) {
        context = global;
    }
    return context;
}
function getConverterCallback(value, isBackConvert) {
    var callback = null;
    if (isNullOrUndefined(value)) {
        callback = value;
    }
    else if (isFunction(value)) {
        callback = isBackConvert ? Function.prototype : value;
    }
    else if (isObject(value) && (isFunction(value.toModel) || isFunction(value.toView))) {
        callback = (isBackConvert ? value.toModel : value.toView) || Function.prototype;
    }
    else {
        callback = value;
    }
    return callback;
}
export function parseExpression(expressionText) {
    var expression = expressionsCache[expressionText];
    if (expression == null) {
        var program = parse(expressionText, { ecmaVersion: 2020 });
        var statements = program.body;
        for (var _i = 0, statements_1 = statements; _i < statements_1.length; _i++) {
            var statement = statements_1[_i];
            if (statement.type == 'ExpressionStatement') {
                expression = statement.expression;
                break;
            }
        }
        expressionsCache[expressionText] = expression;
    }
    return expression;
}
export function convertExpressionToValue(expression, model, isBackConvert, changedModel) {
    if (!(expression.type in expressionParsers)) {
        throw Error('Invalid expression syntax');
    }
    return expressionParsers[expression.type](expression, model, isBackConvert, changedModel);
}
//# sourceMappingURL=bindable-expressions.js.map