// Requires
import { ViewCommon, isEnabledProperty, originXProperty, originYProperty, isUserInteractionEnabledProperty, testIDProperty } from './view-common';
import { hiddenProperty } from '../view-base';
import { Trace } from '../../../trace';
import { layout, ios as iosUtils, SDK_VERSION } from '../../../utils';
import { IOSHelper } from './view-helper';
import { ios as iosBackground } from '../../styling/background';
import { perspectiveProperty, visibilityProperty, opacityProperty, rotateProperty, rotateXProperty, rotateYProperty, scaleXProperty, scaleYProperty, translateXProperty, translateYProperty, zIndexProperty, backgroundInternalProperty } from '../../styling/style-properties';
import { profile } from '../../../profiling';
import { accessibilityEnabledProperty, accessibilityHiddenProperty, accessibilityHintProperty, accessibilityIdentifierProperty, accessibilityLabelProperty, accessibilityLanguageProperty, accessibilityLiveRegionProperty, accessibilityMediaSessionProperty, accessibilityRoleProperty, accessibilityStateProperty, accessibilityValueProperty, accessibilityIgnoresInvertColorsProperty } from '../../../accessibility/accessibility-properties';
import { IOSPostAccessibilityNotificationType, isAccessibilityServiceEnabled, updateAccessibilityProperties } from '../../../accessibility';
import { CoreTypes } from '../../../core-types';
import { SharedTransition } from '../../transition/shared-transition';
export * from './view-common';
// helpers (these are okay re-exported here)
export * from './view-helper';
// This one can eventually be cleaned up but causes issues with a lot of ui-suite plugins in particular if not exported here
export * from '../properties';
var PFLAG_FORCE_LAYOUT = 1;
var PFLAG_MEASURED_DIMENSION_SET = 1 << 1;
var PFLAG_LAYOUT_REQUIRED = 1 << 2;
var View = /** @class */ (function (_super) {
    __extends(View, _super);
    function View() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._isLaidOut = false;
        _this._hasTransform = false;
        _this._hasPendingTransform = false;
        _this._privateFlags = PFLAG_LAYOUT_REQUIRED | PFLAG_FORCE_LAYOUT;
        _this._suspendCATransaction = false;
        return _this;
    }
    Object.defineProperty(View.prototype, "isLayoutRequired", {
        get: function () {
            return (this._privateFlags & PFLAG_LAYOUT_REQUIRED) === PFLAG_LAYOUT_REQUIRED;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(View.prototype, "isLayoutRequested", {
        get: function () {
            return (this._privateFlags & PFLAG_FORCE_LAYOUT) === PFLAG_FORCE_LAYOUT;
        },
        enumerable: false,
        configurable: true
    });
    View.prototype.disposeNativeView = function () {
        _super.prototype.disposeNativeView.call(this);
        this._cachedFrame = null;
        this._isLaidOut = false;
        this._hasTransform = false;
        this._hasPendingTransform = false;
    };
    View.prototype.requestLayout = function () {
        this._privateFlags |= PFLAG_FORCE_LAYOUT;
        _super.prototype.requestLayout.call(this);
        var nativeView = this.nativeViewProtected;
        if (nativeView && nativeView.setNeedsLayout) {
            nativeView.setNeedsLayout();
        }
        if (this.viewController && this.viewController.view !== nativeView) {
            this.viewController.view.setNeedsLayout();
        }
    };
    View.prototype.measure = function (widthMeasureSpec, heightMeasureSpec) {
        var measureSpecsChanged = this._setCurrentMeasureSpecs(widthMeasureSpec, heightMeasureSpec);
        var forceLayout = (this._privateFlags & PFLAG_FORCE_LAYOUT) === PFLAG_FORCE_LAYOUT;
        if (this.nativeViewProtected && (forceLayout || measureSpecsChanged)) {
            // first clears the measured dimension flag
            this._privateFlags &= ~PFLAG_MEASURED_DIMENSION_SET;
            // measure ourselves, this should set the measured dimension flag back
            this.onMeasure(widthMeasureSpec, heightMeasureSpec);
            this._privateFlags |= PFLAG_LAYOUT_REQUIRED;
            // flag not set, setMeasuredDimension() was not invoked, we trace
            // the exception to warn the developer
            if ((this._privateFlags & PFLAG_MEASURED_DIMENSION_SET) !== PFLAG_MEASURED_DIMENSION_SET) {
                if (Trace.isEnabled()) {
                    Trace.write('onMeasure() did not set the measured dimension by calling setMeasuredDimension() ' + this, Trace.categories.Layout, Trace.messageType.error);
                }
            }
        }
    };
    View.prototype.layout = function (left, top, right, bottom, setFrame) {
        if (setFrame === void 0) { setFrame = true; }
        var result = this._setCurrentLayoutBounds(left, top, right, bottom);
        var sizeChanged = result.sizeChanged;
        if (setFrame) {
            this.layoutNativeView(left, top, right, bottom);
        }
        var needsLayout = result.boundsChanged || (this._privateFlags & PFLAG_LAYOUT_REQUIRED) === PFLAG_LAYOUT_REQUIRED;
        if (needsLayout) {
            var position = void 0;
            if (this.nativeViewProtected && SDK_VERSION > 10) {
                // on iOS 11+ it is possible to have a changed layout frame due to safe area insets
                // get the frame and adjust the position, so that onLayout works correctly
                position = IOSHelper.getPositionFromFrame(this.nativeViewProtected.frame);
                if (!sizeChanged) {
                    // If frame has actually changed, there is the need to update view background and border styles as they depend on native view bounds
                    // To trigger the needed visual update, mark size as changed
                    if (position.left !== left || position.top !== top || position.right !== right || position.bottom !== bottom) {
                        sizeChanged = true;
                    }
                }
            }
            else {
                position = { left: left, top: top, right: right, bottom: bottom };
            }
            this.onLayout(position.left, position.top, position.right, position.bottom);
            this._privateFlags &= ~PFLAG_LAYOUT_REQUIRED;
        }
        this.updateBackground(sizeChanged, needsLayout);
        if (this._hasPendingTransform) {
            this.updateNativeTransform();
            this._hasPendingTransform = false;
        }
        this._privateFlags &= ~PFLAG_FORCE_LAYOUT;
    };
    View.prototype.updateBackground = function (sizeChanged, needsLayout) {
        if (sizeChanged) {
            this._onSizeChanged();
        }
        else if (this._nativeBackgroundState === 'invalid') {
            var background = this.style.backgroundInternal;
            this._redrawNativeBackground(background);
        }
        else {
            // Update layers that don't belong to view's layer (e.g. shadow layers)
            if (needsLayout) {
                this.layoutOuterShadows();
            }
        }
    };
    View.prototype.layoutOuterShadows = function () {
        var nativeView = this.nativeViewProtected;
        if (nativeView === null || nativeView === void 0 ? void 0 : nativeView.outerShadowContainerLayer) {
            CATransaction.setDisableActions(true);
            nativeView.outerShadowContainerLayer.bounds = nativeView.bounds;
            nativeView.outerShadowContainerLayer.position = nativeView.center;
            CATransaction.setDisableActions(false);
        }
    };
    View.prototype.setMeasuredDimension = function (measuredWidth, measuredHeight) {
        _super.prototype.setMeasuredDimension.call(this, measuredWidth, measuredHeight);
        this._privateFlags |= PFLAG_MEASURED_DIMENSION_SET;
    };
    View.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var view = this.nativeViewProtected;
        var width = layout.getMeasureSpecSize(widthMeasureSpec);
        var widthMode = layout.getMeasureSpecMode(widthMeasureSpec);
        var height = layout.getMeasureSpecSize(heightMeasureSpec);
        var heightMode = layout.getMeasureSpecMode(heightMeasureSpec);
        var nativeWidth = 0;
        var nativeHeight = 0;
        if (view) {
            var nativeSize = layout.measureNativeView(view, width, widthMode, height, heightMode);
            nativeWidth = nativeSize.width;
            nativeHeight = nativeSize.height;
        }
        var measureWidth = Math.max(nativeWidth, this.effectiveMinWidth);
        var measureHeight = Math.max(nativeHeight, this.effectiveMinHeight);
        var widthAndState = View.resolveSizeAndState(measureWidth, width, widthMode, 0);
        var heightAndState = View.resolveSizeAndState(measureHeight, height, heightMode, 0);
        this.setMeasuredDimension(widthAndState, heightAndState);
    };
    View.prototype.onLayout = function (left, top, right, bottom) {
        //
    };
    View.prototype._setNativeViewFrame = function (nativeView, frame) {
        var oldFrame = this._cachedFrame || nativeView.frame;
        if (!CGRectEqualToRect(oldFrame, frame)) {
            if (Trace.isEnabled()) {
                Trace.write(this + ' :_setNativeViewFrame: ' + JSON.stringify(IOSHelper.getPositionFromFrame(frame)), Trace.categories.Layout);
            }
            this._cachedFrame = frame;
            var adjustedFrame = null;
            var transform = null;
            if (this._hasTransform) {
                // Always set identity transform before setting frame;
                transform = nativeView.layer.transform;
                nativeView.layer.transform = CATransform3DIdentity;
                nativeView.frame = frame;
            }
            else {
                nativeView.frame = frame;
            }
            adjustedFrame = this.applySafeAreaInsets(frame);
            if (adjustedFrame) {
                nativeView.frame = adjustedFrame;
            }
            if (this._hasTransform) {
                // re-apply the transform after the frame is adjusted
                nativeView.layer.transform = transform;
            }
            var boundsOrigin = nativeView.bounds.origin;
            var boundsFrame = adjustedFrame || frame;
            nativeView.bounds = CGRectMake(boundsOrigin.x, boundsOrigin.y, boundsFrame.size.width, boundsFrame.size.height);
            nativeView.layoutIfNeeded();
            this._raiseLayoutChangedEvent();
            this._isLaidOut = true;
        }
        else if (!this._isLaidOut) {
            // Rects could be equal on the first layout and an event should be raised.
            this._raiseLayoutChangedEvent();
            // But make sure event is raised only once if rects are equal on the first layout as
            // this method is called twice with equal rects in landscape mode (vs only once in portrait)
            this._isLaidOut = true;
        }
    };
    Object.defineProperty(View.prototype, "isLayoutValid", {
        get: function () {
            if (this.nativeViewProtected) {
                return this._isLayoutValid;
            }
            return false;
        },
        enumerable: false,
        configurable: true
    });
    View.prototype.layoutNativeView = function (left, top, right, bottom) {
        if (!this.nativeViewProtected) {
            return;
        }
        var nativeView = this.nativeViewProtected;
        var frame = IOSHelper.getFrameFromPosition({
            left: left,
            top: top,
            right: right,
            bottom: bottom,
        });
        this._setNativeViewFrame(nativeView, frame);
    };
    View.prototype._layoutParent = function () {
        if (this.nativeViewProtected) {
            var frame = this.nativeViewProtected.frame;
            var origin_1 = frame.origin;
            var size = frame.size;
            var left = layout.toDevicePixels(origin_1.x);
            var top_1 = layout.toDevicePixels(origin_1.y);
            var width = layout.toDevicePixels(size.width);
            var height = layout.toDevicePixels(size.height);
            this._setLayoutFlags(left, top_1, width + left, height + top_1);
        }
        _super.prototype._layoutParent.call(this);
    };
    View.prototype._setLayoutFlags = function (left, top, right, bottom) {
        var width = right - left;
        var height = bottom - top;
        var widthSpec = layout.makeMeasureSpec(width, layout.EXACTLY);
        var heightSpec = layout.makeMeasureSpec(height, layout.EXACTLY);
        this._setCurrentMeasureSpecs(widthSpec, heightSpec);
        this._privateFlags &= ~PFLAG_FORCE_LAYOUT;
        this.setMeasuredDimension(width, height);
        var _a = this._setCurrentLayoutBounds(left, top, right, bottom), boundsChanged = _a.boundsChanged, sizeChanged = _a.sizeChanged;
        this.updateBackground(sizeChanged, boundsChanged);
        this._privateFlags &= ~PFLAG_LAYOUT_REQUIRED;
    };
    View.prototype.focus = function () {
        if (this.ios) {
            return this.ios.becomeFirstResponder();
        }
        return false;
    };
    View.prototype.applySafeAreaInsets = function (frame) {
        if (!__VISIONOS__ && SDK_VERSION <= 10) {
            return null;
        }
        if (this.iosIgnoreSafeArea) {
            return frame;
        }
        if (!this.iosOverflowSafeArea || !this.iosOverflowSafeAreaEnabled) {
            return IOSHelper.shrinkToSafeArea(this, frame);
        }
        else if (this.nativeViewProtected && this.nativeViewProtected.window) {
            return IOSHelper.expandBeyondSafeArea(this, frame);
        }
        return null;
    };
    View.prototype.getSafeAreaInsets = function () {
        var safeAreaInsets = this.nativeViewProtected && this.nativeViewProtected.safeAreaInsets;
        var insets = { left: 0, top: 0, right: 0, bottom: 0 };
        if (this.iosIgnoreSafeArea) {
            return insets;
        }
        if (safeAreaInsets) {
            insets.left = layout.round(layout.toDevicePixels(safeAreaInsets.left));
            insets.top = layout.round(layout.toDevicePixels(safeAreaInsets.top));
            insets.right = layout.round(layout.toDevicePixels(safeAreaInsets.right));
            insets.bottom = layout.round(layout.toDevicePixels(safeAreaInsets.bottom));
        }
        return insets;
    };
    View.prototype.getLocationInWindow = function () {
        if (!this.nativeViewProtected || !this.nativeViewProtected.window) {
            return undefined;
        }
        var pointInWindow = this.nativeViewProtected.convertPointToView(this.nativeViewProtected.bounds.origin, null);
        return {
            x: pointInWindow.x,
            y: pointInWindow.y,
        };
    };
    View.prototype.getLocationOnScreen = function () {
        if (!this.nativeViewProtected || !this.nativeViewProtected.window) {
            return undefined;
        }
        var pointInWindow = this.nativeViewProtected.convertPointToView(this.nativeViewProtected.bounds.origin, null);
        var pointOnScreen = this.nativeViewProtected.window.convertPointToWindow(pointInWindow, null);
        return {
            x: pointOnScreen.x,
            y: pointOnScreen.y,
        };
    };
    View.prototype.getLocationRelativeTo = function (otherView) {
        if (!this.nativeViewProtected || !this.nativeViewProtected.window || !otherView.nativeViewProtected || !otherView.nativeViewProtected.window || this.nativeViewProtected.window !== otherView.nativeViewProtected.window) {
            return undefined;
        }
        var myPointInWindow = this.nativeViewProtected.convertPointToView(this.nativeViewProtected.bounds.origin, null);
        var otherPointInWindow = otherView.nativeViewProtected.convertPointToView(otherView.nativeViewProtected.bounds.origin, null);
        return {
            x: myPointInWindow.x - otherPointInWindow.x,
            y: myPointInWindow.y - otherPointInWindow.y,
        };
    };
    View.prototype._onSizeChanged = function () {
        var nativeView = this.nativeViewProtected;
        if (!nativeView) {
            return;
        }
        var background = this.style.backgroundInternal;
        var backgroundDependsOnSize = (background.image && background.image !== 'none') || background.clipPath || !background.hasUniformBorder() || background.hasBorderRadius() || background.hasBoxShadow();
        if (this._nativeBackgroundState === 'invalid' || (this._nativeBackgroundState === 'drawn' && backgroundDependsOnSize)) {
            this._redrawNativeBackground(background);
        }
    };
    View.prototype.updateNativeTransform = function () {
        if (!this.isLayoutValid) {
            this._hasPendingTransform = true;
            return;
        }
        var scaleX = this.scaleX || 1e-6;
        var scaleY = this.scaleY || 1e-6;
        var perspective = this.perspective || 300;
        var nativeView = this.nativeViewProtected;
        var transform = new CATransform3D(CATransform3DIdentity);
        // Only set perspective if there is 3D rotation
        if (this.rotateX || this.rotateY) {
            transform.m34 = -1 / perspective;
        }
        transform = CATransform3DTranslate(transform, this.translateX, this.translateY, 0);
        transform = iosUtils.applyRotateTransform(transform, this.rotateX, this.rotateY, this.rotate);
        transform = CATransform3DScale(transform, scaleX, scaleY, 1);
        var needsTransform = !CATransform3DEqualToTransform(this.nativeViewProtected.layer.transform, transform) || (nativeView.outerShadowContainerLayer && !CATransform3DEqualToTransform(nativeView.outerShadowContainerLayer.transform, transform));
        if (needsTransform) {
            var updateSuspended = this._isPresentationLayerUpdateSuspended();
            if (!updateSuspended) {
                CATransaction.begin();
            }
            // Disable CALayer animatable property changes
            CATransaction.setDisableActions(true);
            this.nativeViewProtected.layer.transform = transform;
            if (nativeView.outerShadowContainerLayer) {
                nativeView.outerShadowContainerLayer.transform = transform;
            }
            this._hasTransform = this.nativeViewProtected && !CATransform3DEqualToTransform(this.nativeViewProtected.transform3D, CATransform3DIdentity);
            CATransaction.setDisableActions(false);
            if (!updateSuspended) {
                CATransaction.commit();
            }
        }
    };
    View.prototype.updateOriginPoint = function (originX, originY) {
        var nativeView = this.nativeViewProtected;
        var newPoint = CGPointMake(originX, originY);
        // Disable CALayer animatable property changes
        CATransaction.setDisableActions(true);
        nativeView.layer.anchorPoint = newPoint;
        // Bounds have to be recalculated after anchor point update
        if (this._cachedFrame) {
            var frame = this._cachedFrame;
            this._cachedFrame = null;
            this._setNativeViewFrame(nativeView, frame);
        }
        // Make sure new origin also applies to outer shadow layers
        if (nativeView.outerShadowContainerLayer) {
            // This is the new frame after view origin point update
            var frame = nativeView.frame;
            nativeView.outerShadowContainerLayer.anchorPoint = newPoint;
            nativeView.outerShadowContainerLayer.position = CGPointMake(frame.origin.x + frame.size.width * originX, frame.origin.y + frame.size.height * originY);
        }
        CATransaction.setDisableActions(false);
    };
    // By default we update the view's presentation layer when setting backgroundColor and opacity properties.
    // This is done by calling CATransaction begin and commit methods.
    // This action should be disabled when updating those properties during an animation.
    View.prototype._suspendPresentationLayerUpdates = function () {
        this._suspendCATransaction = true;
    };
    View.prototype._resumePresentationLayerUpdates = function () {
        this._suspendCATransaction = false;
    };
    View.prototype._isPresentationLayerUpdateSuspended = function () {
        return this._suspendCATransaction || this._suspendNativeUpdatesCount > 0;
    };
    View.prototype._showNativeModalView = function (parent, options) {
        var _this = this;
        var _a;
        var parentWithController = IOSHelper.getParentWithViewController(parent);
        if (!parentWithController) {
            Trace.write("Could not find parent with viewController for ".concat(parent, " while showing modal view."), Trace.categories.ViewHierarchy, Trace.messageType.error);
            return;
        }
        var parentController = parentWithController.viewController;
        if (parentController.presentedViewController) {
            Trace.write('Parent is already presenting view controller. Close the current modal page before showing another one!', Trace.categories.ViewHierarchy, Trace.messageType.error);
            return;
        }
        if (!parentController.view || !parentController.view.window) {
            Trace.write('Parent page is not part of the window hierarchy.', Trace.categories.ViewHierarchy, Trace.messageType.error);
            return;
        }
        this._setupAsRootView({});
        _super.prototype._showNativeModalView.call(this, parentWithController, options);
        var controller = this.viewController;
        if (!controller) {
            var nativeView = this.ios || this.nativeViewProtected;
            controller = IOSHelper.UILayoutViewController.initWithOwner(new WeakRef(this));
            if (nativeView instanceof UIView) {
                controller.view.addSubview(nativeView);
            }
            this.viewController = controller;
        }
        if (options.transition) {
            controller.modalPresentationStyle = 4 /* UIModalPresentationStyle.Custom */;
            if (options.transition.instance) {
                this._transitioningDelegate = UIViewControllerTransitioningDelegateImpl.initWithOwner(new WeakRef(options.transition.instance));
                controller.transitioningDelegate = this._transitioningDelegate;
                this.transitionId = options.transition.instance.id;
                var transitionState = SharedTransition.getState(options.transition.instance.id);
                if ((_a = transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactive) === null || _a === void 0 ? void 0 : _a.dismiss) {
                    // interactive transitions via gestures
                    // TODO - these could be typed as: boolean | (view: View) => void
                    // to allow users to define their own custom gesture dismissals
                    options.transition.instance.setupInteractiveGesture(this._closeModalCallback.bind(this), this);
                }
            }
        }
        else if (options.fullscreen) {
            controller.modalPresentationStyle = 0 /* UIModalPresentationStyle.FullScreen */;
        }
        else {
            controller.modalPresentationStyle = 2 /* UIModalPresentationStyle.FormSheet */;
            //check whether both height and width is provided and are positive numbers
            // set it has prefered content size to the controller presenting the dialog
            if (options.ios && options.ios.width > 0 && options.ios.height > 0) {
                controller.preferredContentSize = CGSizeMake(options.ios.width, options.ios.height);
            }
            else {
                //use CSS & attribute width & height if option is not provided
                var handler_1 = function () {
                    if (_this.viewController) {
                        var w = (_this.width || _this.style.width);
                        var h = (_this.height || _this.style.height);
                        //TODO: only numeric value is supported, percentage value is not supported like Android
                        if (w > 0 && h > 0) {
                            _this.viewController.preferredContentSize = CGSizeMake(w, h);
                        }
                    }
                    _this.off(View.loadedEvent, handler_1);
                };
                this.on(View.loadedEvent, handler_1);
            }
        }
        if (options.ios && options.ios.presentationStyle) {
            var presentationStyle = options.ios.presentationStyle;
            controller.modalPresentationStyle = presentationStyle;
            if (presentationStyle === 7 /* UIModalPresentationStyle.Popover */) {
                this._setupPopoverControllerDelegate(controller, parent);
            }
        }
        var cancelable = options.cancelable !== undefined ? !!options.cancelable : true;
        if (SDK_VERSION >= 13) {
            if (cancelable) {
                // Listen for dismiss modal callback.
                this._setupAdaptiveControllerDelegate(controller);
            }
            else {
                // Prevent users from dismissing the modal.
                controller.modalInPresentation = true;
            }
        }
        this.horizontalAlignment = 'stretch';
        this.verticalAlignment = 'stretch';
        this._raiseShowingModallyEvent();
        var animated = options.animated === undefined ? true : !!options.animated;
        if (!this._modalAnimatedOptions) {
            // track the user's animated options to use upon close as well
            this._modalAnimatedOptions = [];
        }
        this._modalAnimatedOptions.push(animated);
        // TODO: a11y
        // controller.accessibilityViewIsModal = true;
        // controller.accessibilityPerformEscape = () => {
        //   console.log('accessibilityPerformEscape!!')
        //   return true;
        // }
        parentController.presentViewControllerAnimatedCompletion(controller, animated, null);
        var transitionCoordinator = parentController.transitionCoordinator;
        if (transitionCoordinator) {
            transitionCoordinator.animateAlongsideTransitionCompletion(null, function () {
                setTimeout(function () {
                    // ensure raised on main queue
                    _this._raiseShownModallyEvent();
                });
            });
        }
        else {
            // Apparently iOS 9+ stops all transitions and animations upon application suspend and transitionCoordinator becomes null here in this case.
            // Since we are not waiting for any transition to complete, i.e. transitionCoordinator is null, we can directly raise our shownModally event.
            // Take a look at https://github.com/NativeScript/NativeScript/issues/2173 for more info and a sample project.
            this._raiseShownModallyEvent();
        }
        controller = null;
    };
    View.prototype._hideNativeModalView = function (parent, whenClosedCallback) {
        var _this = this;
        var _a;
        if (!parent || !parent.viewController) {
            Trace.error('Trying to hide modal view but no parent with viewController specified.');
            return;
        }
        // modal view has already been closed by UI, probably as a popover
        if (!parent.viewController.presentedViewController) {
            whenClosedCallback();
            return;
        }
        var parentController = parent.viewController;
        var animated = true;
        if ((_a = this._modalAnimatedOptions) === null || _a === void 0 ? void 0 : _a.length) {
            animated = this._modalAnimatedOptions.slice(-1)[0];
        }
        parentController.dismissViewControllerAnimatedCompletion(animated, function () {
            var transitionState = SharedTransition.getState(_this.transitionId);
            if (!(transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveCancelled)) {
                _this._transitioningDelegate = null;
                // this.off('pan', this._interactiveDismissGesture);
                if (_this._modalAnimatedOptions) {
                    _this._modalAnimatedOptions.pop();
                }
            }
            whenClosedCallback();
        });
    };
    View.prototype[isEnabledProperty.getDefault] = function () {
        var nativeView = this.nativeViewProtected;
        return nativeView instanceof UIControl ? nativeView.enabled : true;
    };
    View.prototype[isEnabledProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        if (nativeView instanceof UIControl) {
            nativeView.enabled = value;
        }
    };
    View.prototype[originXProperty.getDefault] = function () {
        return this.nativeViewProtected.layer.anchorPoint.x;
    };
    View.prototype[originXProperty.setNative] = function (value) {
        this.updateOriginPoint(value, this.originY);
    };
    View.prototype[originYProperty.getDefault] = function () {
        return this.nativeViewProtected.layer.anchorPoint.y;
    };
    View.prototype[originYProperty.setNative] = function (value) {
        this.updateOriginPoint(this.originX, value);
    };
    View.prototype[testIDProperty.setNative] = function (value) {
        this.setAccessibilityIdentifier(this.nativeViewProtected, value);
    };
    View.prototype.setAccessibilityIdentifier = function (view, value) {
        view.accessibilityIdentifier = value;
        if (this.testID && this.testID !== value)
            this.testID = value;
        if (this.accessibilityIdentifier !== value)
            this.accessibilityIdentifier = value;
    };
    View.prototype[accessibilityEnabledProperty.setNative] = function (value) {
        this.nativeViewProtected.isAccessibilityElement = !!value;
        updateAccessibilityProperties(this);
    };
    View.prototype[accessibilityIdentifierProperty.getDefault] = function () {
        return this.nativeViewProtected.accessibilityIdentifier;
    };
    View.prototype[accessibilityIdentifierProperty.setNative] = function (value) {
        this.setAccessibilityIdentifier(this.nativeViewProtected, value);
    };
    View.prototype[accessibilityRoleProperty.setNative] = function (value) {
        this.accessibilityRole = value;
        updateAccessibilityProperties(this);
    };
    View.prototype[accessibilityValueProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityValue = value;
    };
    View.prototype[accessibilityLabelProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        // not sure if needed for Label:
        // if ((<any>this).nativeTextViewProtected) {
        //   (<any>this).nativeTextViewProtected.accessibilityLabel = value;
        // } else {
        this.nativeViewProtected.accessibilityLabel = value;
        // }
    };
    View.prototype[accessibilityHintProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityHint = value;
    };
    View.prototype[accessibilityIgnoresInvertColorsProperty.setNative] = function (value) {
        this.nativeViewProtected.accessibilityIgnoresInvertColors = !!value;
    };
    View.prototype[accessibilityLanguageProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityLanguage = value;
    };
    View.prototype[accessibilityHiddenProperty.setNative] = function (value) {
        this.nativeViewProtected.accessibilityElementsHidden = !!value;
        updateAccessibilityProperties(this);
    };
    View.prototype[accessibilityLiveRegionProperty.setNative] = function () {
        updateAccessibilityProperties(this);
    };
    View.prototype[accessibilityStateProperty.setNative] = function (value) {
        this.accessibilityState = value;
        updateAccessibilityProperties(this);
    };
    View.prototype[accessibilityMediaSessionProperty.setNative] = function () {
        updateAccessibilityProperties(this);
    };
    View.prototype[isUserInteractionEnabledProperty.getDefault] = function () {
        return this.nativeViewProtected.userInteractionEnabled;
    };
    View.prototype[isUserInteractionEnabledProperty.setNative] = function (value) {
        this.nativeViewProtected.userInteractionEnabled = value;
    };
    View.prototype[hiddenProperty.getDefault] = function () {
        return this.nativeViewProtected.hidden;
    };
    View.prototype[hiddenProperty.setNative] = function (value) {
        this.nativeViewProtected.hidden = value;
    };
    View.prototype[visibilityProperty.getDefault] = function () {
        return this.nativeViewProtected.hidden ? CoreTypes.Visibility.collapse : CoreTypes.Visibility.visible;
    };
    View.prototype[visibilityProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        switch (value) {
            case CoreTypes.Visibility.visible:
                nativeView.hidden = false;
                break;
            case CoreTypes.Visibility.hidden:
            case CoreTypes.Visibility.collapse:
                nativeView.hidden = true;
                break;
            default:
                throw new Error("Invalid visibility value: ".concat(value, ". Valid values are: \"").concat(CoreTypes.Visibility.visible, "\", \"").concat(CoreTypes.Visibility.hidden, "\", \"").concat(CoreTypes.Visibility.collapse, "\"."));
        }
        // Apply visibility value to shadows as well
        if (nativeView.outerShadowContainerLayer) {
            nativeView.outerShadowContainerLayer.hidden = nativeView.hidden;
        }
    };
    View.prototype[opacityProperty.getDefault] = function () {
        return this.nativeViewProtected.alpha;
    };
    View.prototype[opacityProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        var updateSuspended = this._isPresentationLayerUpdateSuspended();
        if (!updateSuspended) {
            CATransaction.begin();
        }
        // Disable CALayer animatable property changes
        CATransaction.setDisableActions(true);
        nativeView.alpha = value;
        // Apply opacity value to shadows as well
        if (nativeView.outerShadowContainerLayer) {
            nativeView.outerShadowContainerLayer.opacity = value;
        }
        CATransaction.setDisableActions(false);
        if (!updateSuspended) {
            CATransaction.commit();
        }
    };
    View.prototype[rotateProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[rotateProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[rotateXProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[rotateXProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[rotateYProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[rotateYProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[perspectiveProperty.getDefault] = function () {
        return 300;
    };
    View.prototype[perspectiveProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[scaleXProperty.getDefault] = function () {
        return 1;
    };
    View.prototype[scaleXProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[scaleYProperty.getDefault] = function () {
        return 1;
    };
    View.prototype[scaleYProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[translateXProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[translateXProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[translateYProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[translateYProperty.setNative] = function (value) {
        this.updateNativeTransform();
    };
    View.prototype[zIndexProperty.getDefault] = function () {
        return 0;
    };
    View.prototype[zIndexProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        nativeView.layer.zPosition = value;
        // Apply z-index to shadows as well
        if (nativeView.outerShadowContainerLayer) {
            nativeView.outerShadowContainerLayer.zPosition = value;
        }
    };
    View.prototype[backgroundInternalProperty.getDefault] = function () {
        return this.nativeViewProtected.backgroundColor;
    };
    View.prototype[backgroundInternalProperty.setNative] = function (value) {
        this._nativeBackgroundState = 'invalid';
        if (this.isLayoutValid) {
            this._redrawNativeBackground(value);
        }
    };
    View.prototype.sendAccessibilityEvent = function (options) {
        if (!isAccessibilityServiceEnabled()) {
            return;
        }
        if (!options.iosNotificationType) {
            return;
        }
        var notification;
        var args = this.nativeViewProtected;
        if (options === null || options === void 0 ? void 0 : options.message) {
            args = options.message;
        }
        switch (options.iosNotificationType) {
            case IOSPostAccessibilityNotificationType.Announcement: {
                notification = UIAccessibilityAnnouncementNotification;
                break;
            }
            case IOSPostAccessibilityNotificationType.Layout: {
                notification = UIAccessibilityLayoutChangedNotification;
                break;
            }
            case IOSPostAccessibilityNotificationType.Screen: {
                notification = UIAccessibilityScreenChangedNotification;
                break;
            }
            default: {
                return;
            }
        }
        UIAccessibilityPostNotification(notification, args !== null && args !== void 0 ? args : null);
    };
    View.prototype.accessibilityAnnouncement = function (msg) {
        if (msg === void 0) { msg = this.accessibilityLabel; }
        this.sendAccessibilityEvent({
            iosNotificationType: IOSPostAccessibilityNotificationType.Announcement,
            message: msg,
        });
    };
    View.prototype.accessibilityScreenChanged = function () {
        this.sendAccessibilityEvent({
            iosNotificationType: IOSPostAccessibilityNotificationType.Screen,
        });
    };
    View.prototype._getCurrentLayoutBounds = function () {
        var nativeView = this.nativeViewProtected;
        if (nativeView && !this.isCollapsed) {
            var frame = nativeView.frame;
            var origin_2 = frame.origin;
            var size = frame.size;
            return {
                left: Math.round(layout.toDevicePixels(origin_2.x)),
                top: Math.round(layout.toDevicePixels(origin_2.y)),
                right: Math.round(layout.toDevicePixels(origin_2.x + size.width)),
                bottom: Math.round(layout.toDevicePixels(origin_2.y + size.height)),
            };
        }
        else {
            return { left: 0, top: 0, right: 0, bottom: 0 };
        }
    };
    View.prototype._redrawNativeBackground = function (value) {
        var updateSuspended = this._isPresentationLayerUpdateSuspended();
        if (!updateSuspended) {
            CATransaction.begin();
        }
        // Disable CALayer animatable property changes
        CATransaction.setDisableActions(true);
        var nativeView = this.nativeViewProtected;
        if (nativeView) {
            if (value instanceof UIColor) {
                nativeView.backgroundColor = value;
            }
            else {
                iosBackground.createBackgroundUIColor(this, function (color) {
                    nativeView.backgroundColor = color;
                });
                this._setNativeClipToBounds();
            }
        }
        CATransaction.setDisableActions(false);
        if (!updateSuspended) {
            CATransaction.commit();
        }
        this._nativeBackgroundState = 'drawn';
    };
    View.prototype._setNativeClipToBounds = function () {
        var view = this.nativeViewProtected;
        if (view) {
            var backgroundInternal = this.style.backgroundInternal;
            view.clipsToBounds = view instanceof UIScrollView || backgroundInternal.hasBorderWidth() || backgroundInternal.hasBorderRadius();
        }
    };
    View.prototype._setupPopoverControllerDelegate = function (controller, parent) {
        var popoverPresentationController = controller.popoverPresentationController;
        this._popoverPresentationDelegate = IOSHelper.UIPopoverPresentationControllerDelegateImp.initWithOwnerAndCallback(new WeakRef(this), this._closeModalCallback);
        popoverPresentationController.delegate = this._popoverPresentationDelegate;
        var view;
        do {
            view = parent.nativeViewProtected;
            parent = parent.parent;
        } while (parent && !view);
        // Note: sourceView and sourceRect are needed to specify the anchor location for the popover.
        // Note: sourceView should be the button triggering the modal. If it the Page the popover might appear "behind" the page content
        popoverPresentationController.sourceView = view;
        popoverPresentationController.sourceRect = CGRectMake(0, 0, view.frame.size.width, view.frame.size.height);
    };
    View.prototype._setupAdaptiveControllerDelegate = function (controller) {
        this._adaptivePresentationDelegate = IOSHelper.UIAdaptivePresentationControllerDelegateImp.initWithOwnerAndCallback(new WeakRef(this), this._closeModalCallback);
        if (controller === null || controller === void 0 ? void 0 : controller.presentationController) {
            controller.presentationController.delegate = this._adaptivePresentationDelegate;
        }
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Number, Number, Number, Number, Object]),
        __metadata("design:returntype", void 0)
    ], View.prototype, "layout", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Number, Number]),
        __metadata("design:returntype", void 0)
    ], View.prototype, "onMeasure", null);
    return View;
}(ViewCommon));
export { View };
View.prototype._nativeBackgroundState = 'unset';
var UIViewControllerTransitioningDelegateImpl = /** @class */ (function (_super) {
    __extends(UIViewControllerTransitioningDelegateImpl, _super);
    function UIViewControllerTransitioningDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UIViewControllerTransitioningDelegateImpl.initWithOwner = function (owner) {
        var delegate = UIViewControllerTransitioningDelegateImpl.new();
        delegate.owner = owner;
        return delegate;
    };
    UIViewControllerTransitioningDelegateImpl.prototype.animationControllerForDismissedController = function (dismissed) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner === null || owner === void 0 ? void 0 : owner.iosDismissedController) {
            return owner.iosDismissedController(dismissed);
        }
        return null;
    };
    UIViewControllerTransitioningDelegateImpl.prototype.animationControllerForPresentedControllerPresentingControllerSourceController = function (presented, presenting, source) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner === null || owner === void 0 ? void 0 : owner.iosPresentedController) {
            return owner.iosPresentedController(presented, presenting, source);
        }
        return null;
    };
    UIViewControllerTransitioningDelegateImpl.prototype.interactionControllerForDismissal = function (animator) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner === null || owner === void 0 ? void 0 : owner.iosInteractionDismiss) {
            var transitionState = SharedTransition.getState(owner.id);
            if (transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveBegan) {
                return owner.iosInteractionDismiss(animator);
            }
        }
        return null;
    };
    UIViewControllerTransitioningDelegateImpl.prototype.interactionControllerForPresentation = function (animator) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner === null || owner === void 0 ? void 0 : owner.iosInteractionPresented) {
            return owner.iosInteractionPresented(animator);
        }
        return null;
    };
    UIViewControllerTransitioningDelegateImpl.ObjCProtocols = [UIViewControllerTransitioningDelegate];
    return UIViewControllerTransitioningDelegateImpl;
}(NSObject));
var ContainerView = /** @class */ (function (_super) {
    __extends(ContainerView, _super);
    function ContainerView() {
        var _this = _super.call(this) || this;
        _this.iosOverflowSafeArea = true;
        return _this;
    }
    return ContainerView;
}(View));
export { ContainerView };
var CustomLayoutView = /** @class */ (function (_super) {
    __extends(CustomLayoutView, _super);
    function CustomLayoutView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CustomLayoutView.prototype.createNativeView = function () {
        var window = iosUtils.getWindow();
        return UIView.alloc().initWithFrame(window ? window.screen.bounds : UIScreen.mainScreen.bounds);
    };
    Object.defineProperty(CustomLayoutView.prototype, "ios", {
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    CustomLayoutView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        // Don't call super because it will set MeasureDimension. This method must be overridden and calculate its measuredDimensions.
    };
    CustomLayoutView.prototype._addViewToNativeVisualTree = function (child, atIndex) {
        _super.prototype._addViewToNativeVisualTree.call(this, child, atIndex);
        var parentNativeView = this.nativeViewProtected;
        var childNativeView = child.nativeViewProtected;
        if (parentNativeView && childNativeView) {
            if (typeof atIndex !== 'number' || atIndex >= parentNativeView.subviews.count) {
                parentNativeView.addSubview(childNativeView);
            }
            else {
                parentNativeView.insertSubviewAtIndex(childNativeView, atIndex);
            }
            // Add outer shadow layer manually as it belongs to parent layer tree (this is needed for reusable views)
            if (childNativeView.outerShadowContainerLayer && !childNativeView.outerShadowContainerLayer.superlayer) {
                parentNativeView.layer.insertSublayerBelow(childNativeView.outerShadowContainerLayer, childNativeView.layer);
            }
            return true;
        }
        return false;
    };
    CustomLayoutView.prototype._removeViewFromNativeVisualTree = function (child) {
        _super.prototype._removeViewFromNativeVisualTree.call(this, child);
        if (child.nativeViewProtected) {
            var nativeView = child.nativeViewProtected;
            // Remove outer shadow layer manually as it belongs to parent layer tree
            if (nativeView.outerShadowContainerLayer) {
                nativeView.outerShadowContainerLayer.removeFromSuperlayer();
            }
            nativeView.removeFromSuperview();
        }
    };
    return CustomLayoutView;
}(ContainerView));
export { CustomLayoutView };
//# sourceMappingURL=index.ios.js.map