import { booleanConverter, ViewBase } from '../view-base';
import { getEventOrGestureName } from '../bindable';
import { layout } from '../../../utils';
import { isObject } from '../../../utils/types';
import { sanitizeModuleName } from '../../../utils/common';
import { Property, InheritedProperty } from '../properties';
import { Trace } from '../../../trace';
import { ViewHelper } from './view-helper';
import { setupAccessibleView } from '../../../accessibility';
import { PercentLength } from '../../styling/style-properties';
import { observe as gestureObserve, GestureTypes, fromString as gestureFromString, toString as gestureToString, TouchManager } from '../../gestures';
import { CSSUtils } from '../../../css/system-classes';
import { Builder } from '../../builder';
import { StyleScope } from '../../styling/style-scope';
import { accessibilityHintProperty, accessibilityIdentifierProperty, accessibilityLabelProperty, accessibilityValueProperty, accessibilityIgnoresInvertColorsProperty } from '../../../accessibility/accessibility-properties';
import { accessibilityBlurEvent, accessibilityFocusChangedEvent, accessibilityFocusEvent, accessibilityPerformEscapeEvent, getCurrentFontScale } from '../../../accessibility';
import { SharedTransition } from '../../transition/shared-transition';
// helpers (these are okay re-exported here)
export * from './view-helper';
var animationModule;
function ensureAnimationModule() {
    if (!animationModule) {
        animationModule = require('../../animation');
    }
}
export function CSSType(type) {
    return function (cls) {
        cls.prototype.cssType = type;
    };
}
export function viewMatchesModuleContext(view, context, types) {
    return context && view._moduleName && context.type && types.some(function (type) { return type === context.type; }) && context.path && context.path.includes(view._moduleName);
}
export function PseudoClassHandler() {
    var pseudoClasses = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        pseudoClasses[_i] = arguments[_i];
    }
    var stateEventNames = pseudoClasses.map(function (s) { return ':' + s; });
    var listeners = Symbol('listeners');
    return function (target, propertyKey, descriptor) {
        function update(change) {
            var prev = this[listeners] || 0;
            var next = prev + change;
            if (prev <= 0 && next > 0) {
                this[propertyKey](true);
            }
            else if (prev > 0 && next <= 0) {
                this[propertyKey](false);
            }
        }
        stateEventNames.forEach(function (s) { return (target[s] = update); });
    };
}
export var _rootModalViews = new Array();
// TODO: remove once we fully switch to the new event system
var warnedEvent = new Set();
var ViewCommon = /** @class */ (function (_super) {
    __extends(ViewCommon, _super);
    function ViewCommon() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._gestureObservers = {};
        return _this;
    }
    Object.defineProperty(ViewCommon.prototype, "css", {
        get: function () {
            var scope = this._styleScope;
            return scope && scope.css;
        },
        set: function (value) {
            this._updateStyleScope(undefined, undefined, value);
        },
        enumerable: false,
        configurable: true
    });
    ViewCommon.prototype.addCss = function (cssString) {
        this._updateStyleScope(undefined, cssString);
    };
    ViewCommon.prototype.addCssFile = function (cssFileName) {
        this._updateStyleScope(cssFileName);
    };
    ViewCommon.prototype.changeCssFile = function (cssFileName) {
        var scope = this._styleScope;
        if (scope && cssFileName) {
            scope.changeCssFile(cssFileName);
            this._onCssStateChange();
        }
    };
    ViewCommon.prototype._updateStyleScope = function (cssFileName, cssString, css) {
        var scope = this._styleScope;
        if (!scope) {
            scope = new StyleScope();
            this.setScopeProperty(scope, cssFileName, cssString, css);
            this._inheritStyleScope(scope);
            this._isStyleScopeHost = true;
        }
        else {
            this.setScopeProperty(scope, cssFileName, cssString, css);
            this._onCssStateChange();
        }
    };
    ViewCommon.prototype.setScopeProperty = function (scope, cssFileName, cssString, css) {
        if (cssFileName !== undefined) {
            scope.addCssFile(cssFileName);
        }
        else if (cssString !== undefined) {
            scope.addCss(cssString);
        }
        else if (css !== undefined) {
            scope.css = css;
        }
    };
    ViewCommon.prototype.onLoaded = function () {
        if (!this.isLoaded) {
            var hasTap = this.hasListeners('tap') || this.hasListeners('tapChange') || !!this.getGestureObservers(GestureTypes.tap);
            var enableTapAnimations = TouchManager.enableGlobalTapAnimations && hasTap;
            if (!this.ignoreTouchAnimation && (this.touchAnimation || enableTapAnimations)) {
                TouchManager.addAnimations(this);
            }
            if (__VISIONOS__) {
                var enableHoverStyle = TouchManager.enableGlobalHoverWhereTap && hasTap;
                if (!this.visionIgnoreHoverStyle && (this.visionHoverStyle || enableHoverStyle)) {
                    TouchManager.addHoverStyle(this);
                }
            }
        }
        _super.prototype.onLoaded.call(this);
        setupAccessibleView(this);
    };
    ViewCommon.prototype._closeAllModalViewsInternal = function () {
        if (_rootModalViews && _rootModalViews.length > 0) {
            _rootModalViews.forEach(function (v) {
                v.closeModal();
            });
            return true;
        }
        return false;
    };
    ViewCommon.prototype._getRootModalViews = function () {
        return _rootModalViews;
    };
    ViewCommon.prototype._onLivesync = function (context) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(this, "._onLivesync(").concat(JSON.stringify(context), ")"), Trace.categories.Livesync);
        }
        var handled = false;
        if (this._closeAllModalViewsInternal()) {
            handled = true;
        }
        if (this._handleLivesync(context)) {
            return true;
        }
        this.eachChildView(function (child) {
            if (child._onLivesync(context)) {
                handled = true;
                return false;
            }
        });
        return handled;
    };
    ViewCommon.prototype._handleLivesync = function (context) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(this, "._handleLivesync(").concat(JSON.stringify(context), ")"), Trace.categories.Livesync);
        }
        // Handle local CSS
        if (viewMatchesModuleContext(this, context, ['style'])) {
            if (Trace.isEnabled()) {
                Trace.write("Change Handled: Changing CSS for ".concat(this), Trace.categories.Livesync);
            }
            // Always load styles with ".css" extension. Even when changes are in ".scss" ot ".less" files
            var cssModuleName = "".concat(sanitizeModuleName(context.path), ".css");
            this.changeCssFile(cssModuleName);
            return true;
        }
        // Handle script/markup changes in custom components by falling back to page refresh
        if (viewMatchesModuleContext(this, context, ['markup', 'script']) && this.page && this.page.frame) {
            if (Trace.isEnabled()) {
                Trace.write("Change Handled: Changing ".concat(context.type, " for ").concat(this, " inside ").concat(this.page), Trace.categories.Livesync);
            }
            return this.page.frame._handleLivesync({
                type: context.type,
                path: this.page._moduleName,
            });
        }
        return false;
    };
    ViewCommon.prototype._setupAsRootView = function (context) {
        _super.prototype._setupAsRootView.call(this, context);
        if (!this._styleScope) {
            this._updateStyleScope();
        }
    };
    ViewCommon.prototype._observe = function (type, callback, thisArg) {
        var _a;
        thisArg = thisArg || undefined;
        if ((_a = this._gestureObservers[type]) === null || _a === void 0 ? void 0 : _a.find(function (observer) { return observer.callback === callback && observer.context === thisArg; })) {
            // Already added.
            return;
        }
        if (!this._gestureObservers[type]) {
            this._gestureObservers[type] = [];
        }
        this._gestureObservers[type].push(gestureObserve(this, type, callback, thisArg));
    };
    ViewCommon.prototype.getGestureObservers = function (type) {
        return this._gestureObservers[type];
    };
    ViewCommon.prototype.addEventListener = function (eventNames, callback, thisArg) {
        thisArg = thisArg || undefined;
        // TODO: Remove this once we fully switch to the new event system
        if (typeof eventNames === 'number') {
            // likely a gesture type from a plugin
            var gestureName = gestureToString(eventNames);
            if (!warnedEvent.has(gestureName)) {
                console.warn("Using a gesture type (".concat(gestureName, ") as an event name is deprecated. Please use the event name instead."));
                warnedEvent.add(gestureName);
            }
            eventNames = gestureName;
        }
        // Normalize "ontap" -> "tap"
        var normalizedName = getEventOrGestureName(eventNames);
        // Coerce "tap" -> GestureTypes.tap
        // Coerce "loaded" -> undefined
        var gestureType = gestureFromString(normalizedName);
        // If it's a gesture (and this Observable declares e.g. `static tapEvent`)
        if (gestureType && !this._isEvent(normalizedName)) {
            this._observe(gestureType, callback, thisArg);
            return;
        }
        _super.prototype.addEventListener.call(this, normalizedName, callback, thisArg);
    };
    ViewCommon.prototype.removeEventListener = function (eventNames, callback, thisArg) {
        thisArg = thisArg || undefined;
        // TODO: Remove this once we fully switch to the new event system
        if (typeof eventNames === 'number') {
            // likely a gesture type from a plugin
            var gestureName = gestureToString(eventNames);
            if (!warnedEvent.has(gestureName)) {
                console.warn("Using a gesture type (".concat(gestureName, ") as an event name is deprecated. Please use the event name instead."));
                warnedEvent.add(gestureName);
            }
            eventNames = gestureName;
        }
        // Normalize "ontap" -> "tap"
        var normalizedName = getEventOrGestureName(eventNames);
        // Coerce "tap" -> GestureTypes.tap
        // Coerce "loaded" -> undefined
        var gestureType = gestureFromString(normalizedName);
        // If it's a gesture (and this Observable declares e.g. `static tapEvent`)
        if (gestureType && !this._isEvent(normalizedName)) {
            this._disconnectGestureObservers(gestureType, callback, thisArg);
            return;
        }
        _super.prototype.removeEventListener.call(this, normalizedName, callback, thisArg);
    };
    ViewCommon.prototype.onBackPressed = function () {
        return false;
    };
    ViewCommon.prototype._getFragmentManager = function () {
        return undefined;
    };
    ViewCommon.prototype.getModalOptions = function (args) {
        if (args.length === 0) {
            throw new Error('showModal without parameters is deprecated. Please call showModal on a view instance instead.');
        }
        else {
            var options = null;
            if (args.length === 2) {
                options = args[1];
            }
            else {
                if (args[0] instanceof ViewCommon) {
                    console.log('showModal(view: ViewBase, context: any, closeCallback: Function, fullscreen?: boolean, animated?: boolean, stretched?: boolean) ' + 'is deprecated. Use showModal(view: ViewBase, modalOptions: ShowModalOptions) instead.');
                }
                else {
                    console.log('showModal(moduleName: string, context: any, closeCallback: Function, fullscreen?: boolean, animated?: boolean, stretched?: boolean) ' + 'is deprecated. Use showModal(moduleName: string, modalOptions: ShowModalOptions) instead.');
                }
                options = {
                    context: args[1],
                    closeCallback: args[2],
                    fullscreen: args[3],
                    animated: args[4],
                    stretched: args[5],
                };
            }
            var firstArgument = args[0];
            var view = firstArgument instanceof ViewCommon ? firstArgument : Builder.createViewFromEntry({
                moduleName: firstArgument,
            });
            return { view: view, options: options };
        }
    };
    ViewCommon.prototype.showModal = function () {
        var _a, _b;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _c = this.getModalOptions(args), view = _c.view, options = _c.options;
        if ((_a = options.transition) === null || _a === void 0 ? void 0 : _a.instance) {
            SharedTransition.updateState((_b = options.transition) === null || _b === void 0 ? void 0 : _b.instance.id, {
                page: this,
                toPage: view,
            });
        }
        view._showNativeModalView(this, options);
        return view;
    };
    ViewCommon.prototype.closeModal = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var closeCallback = this._closeModalCallback;
        if (closeCallback) {
            closeCallback.apply(void 0, args);
        }
        else {
            var parent_1 = this.parent;
            if (parent_1) {
                parent_1.closeModal.apply(parent_1, args);
            }
        }
    };
    Object.defineProperty(ViewCommon.prototype, "modal", {
        get: function () {
            return this._modal;
        },
        enumerable: false,
        configurable: true
    });
    ViewCommon.prototype._showNativeModalView = function (parent, options) {
        var _this = this;
        _rootModalViews.push(this);
        this.cssClasses.add(CSSUtils.MODAL_ROOT_VIEW_CSS_CLASS);
        var modalRootViewCssClasses = CSSUtils.getSystemCssClasses();
        modalRootViewCssClasses.forEach(function (c) { return _this.cssClasses.add(c); });
        parent._modal = this;
        this.style.fontScaleInternal = getCurrentFontScale();
        this._modalParent = parent;
        this._modalContext = options.context;
        this._closeModalCallback = function () {
            var originalArgs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                originalArgs[_i] = arguments[_i];
            }
            var cleanupModalViews = function () {
                var modalIndex = _rootModalViews.indexOf(_this);
                _rootModalViews.splice(modalIndex, 1);
                _this._modalParent = null;
                _this._modalContext = null;
                _this._closeModalCallback = null;
                _this._dialogClosed();
                parent._modal = null;
            };
            var whenClosedCallback = function () {
                var transitionState = SharedTransition.getState(_this.transitionId);
                if (transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveBegan) {
                    SharedTransition.updateState(_this.transitionId, {
                        interactiveBegan: false,
                    });
                    if (!(transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveCancelled)) {
                        cleanupModalViews();
                    }
                }
                if (!(transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveCancelled)) {
                    if (typeof options.closeCallback === 'function') {
                        options.closeCallback.apply(undefined, originalArgs);
                    }
                    _this._tearDownUI(true);
                }
            };
            var transitionState = SharedTransition.getState(_this.transitionId);
            if (!(transitionState === null || transitionState === void 0 ? void 0 : transitionState.interactiveBegan)) {
                cleanupModalViews();
            }
            _this._hideNativeModalView(parent, whenClosedCallback);
        };
    };
    ViewCommon.prototype._hideNativeModalView = function (parent, whenClosedCallback) { };
    ViewCommon.prototype._raiseLayoutChangedEvent = function () {
        var args = {
            eventName: ViewCommon.layoutChangedEvent,
            object: this,
        };
        this.notify(args);
    };
    ViewCommon.prototype._raiseShownModallyEvent = function () {
        var args = {
            eventName: ViewCommon.shownModallyEvent,
            object: this,
            context: this._modalContext,
            closeCallback: this._closeModalCallback,
        };
        this.notify(args);
    };
    ViewCommon.prototype._raiseShowingModallyEvent = function () {
        var args = {
            eventName: ViewCommon.showingModallyEvent,
            object: this,
            context: this._modalContext,
            closeCallback: this._closeModalCallback,
        };
        this.notify(args);
    };
    ViewCommon.prototype._isEvent = function (name) {
        return this.constructor && "".concat(name, "Event") in this.constructor;
    };
    ViewCommon.prototype._disconnectGestureObservers = function (type, callback, thisArg) {
        // Largely mirrors the implementation of Observable.innerRemoveEventListener().
        var observers = this.getGestureObservers(type);
        if (!observers) {
            return;
        }
        for (var i = 0; i < observers.length; i++) {
            var observer = observers[i];
            // If we have a `thisArg`, refine on both `callback` and `thisArg`.
            if (thisArg && (observer.callback !== callback || observer.context !== thisArg)) {
                continue;
            }
            // If we don't have a `thisArg`, refine only on `callback`.
            if (callback && observer.callback !== callback) {
                continue;
            }
            observer.disconnect();
            observers.splice(i, 1);
            i--;
        }
        if (!observers.length) {
            delete this._gestureObservers[type];
        }
    };
    Object.defineProperty(ViewCommon.prototype, "flexFlow", {
        // START Style property shortcuts
        get: function () {
            return this.style.flexFlow;
        },
        set: function (value) {
            this.style.flexFlow = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "flex", {
        get: function () {
            return this.style.flex;
        },
        set: function (value) {
            this.style.flex = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderColor", {
        get: function () {
            return this.style.borderColor;
        },
        set: function (value) {
            this.style.borderColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderTopColor", {
        get: function () {
            return this.style.borderTopColor;
        },
        set: function (value) {
            this.style.borderTopColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderRightColor", {
        get: function () {
            return this.style.borderRightColor;
        },
        set: function (value) {
            this.style.borderRightColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderBottomColor", {
        get: function () {
            return this.style.borderBottomColor;
        },
        set: function (value) {
            this.style.borderBottomColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderLeftColor", {
        get: function () {
            return this.style.borderLeftColor;
        },
        set: function (value) {
            this.style.borderLeftColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderWidth", {
        get: function () {
            return this.style.borderWidth;
        },
        set: function (value) {
            this.style.borderWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderTopWidth", {
        get: function () {
            return this.style.borderTopWidth;
        },
        set: function (value) {
            this.style.borderTopWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderRightWidth", {
        get: function () {
            return this.style.borderRightWidth;
        },
        set: function (value) {
            this.style.borderRightWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderBottomWidth", {
        get: function () {
            return this.style.borderBottomWidth;
        },
        set: function (value) {
            this.style.borderBottomWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderLeftWidth", {
        get: function () {
            return this.style.borderLeftWidth;
        },
        set: function (value) {
            this.style.borderLeftWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderRadius", {
        get: function () {
            return this.style.borderRadius;
        },
        set: function (value) {
            this.style.borderRadius = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderTopLeftRadius", {
        get: function () {
            return this.style.borderTopLeftRadius;
        },
        set: function (value) {
            this.style.borderTopLeftRadius = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderTopRightRadius", {
        get: function () {
            return this.style.borderTopRightRadius;
        },
        set: function (value) {
            this.style.borderTopRightRadius = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderBottomRightRadius", {
        get: function () {
            return this.style.borderBottomRightRadius;
        },
        set: function (value) {
            this.style.borderBottomRightRadius = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "borderBottomLeftRadius", {
        get: function () {
            return this.style.borderBottomLeftRadius;
        },
        set: function (value) {
            this.style.borderBottomLeftRadius = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "color", {
        get: function () {
            return this.style.color;
        },
        set: function (value) {
            this.style.color = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "background", {
        get: function () {
            return this.style.background;
        },
        set: function (value) {
            this.style.background = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "backgroundColor", {
        get: function () {
            return this.style.backgroundColor;
        },
        set: function (value) {
            this.style.backgroundColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "backgroundImage", {
        get: function () {
            return this.style.backgroundImage;
        },
        set: function (value) {
            this.style.backgroundImage = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "backgroundSize", {
        get: function () {
            return this.style.backgroundSize;
        },
        set: function (value) {
            this.style.backgroundSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "backgroundPosition", {
        get: function () {
            return this.style.backgroundPosition;
        },
        set: function (value) {
            this.style.backgroundPosition = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "backgroundRepeat", {
        get: function () {
            return this.style.backgroundRepeat;
        },
        set: function (value) {
            this.style.backgroundRepeat = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "boxShadow", {
        get: function () {
            return this.style.boxShadow;
        },
        set: function (value) {
            this.style.boxShadow = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "minWidth", {
        get: function () {
            return this.style.minWidth;
        },
        set: function (value) {
            this.style.minWidth = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "minHeight", {
        get: function () {
            return this.style.minHeight;
        },
        set: function (value) {
            this.style.minHeight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "width", {
        get: function () {
            return this.style.width;
        },
        set: function (value) {
            this.style.width = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "height", {
        get: function () {
            return this.style.height;
        },
        set: function (value) {
            this.style.height = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "margin", {
        get: function () {
            return this.style.margin;
        },
        set: function (value) {
            this.style.margin = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "marginLeft", {
        get: function () {
            return this.style.marginLeft;
        },
        set: function (value) {
            this.style.marginLeft = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "marginTop", {
        get: function () {
            return this.style.marginTop;
        },
        set: function (value) {
            this.style.marginTop = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "marginRight", {
        get: function () {
            return this.style.marginRight;
        },
        set: function (value) {
            this.style.marginRight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "marginBottom", {
        get: function () {
            return this.style.marginBottom;
        },
        set: function (value) {
            this.style.marginBottom = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "horizontalAlignment", {
        get: function () {
            return this.style.horizontalAlignment;
        },
        set: function (value) {
            this.style.horizontalAlignment = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "verticalAlignment", {
        get: function () {
            return this.style.verticalAlignment;
        },
        set: function (value) {
            this.style.verticalAlignment = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "visibility", {
        get: function () {
            return this.style.visibility;
        },
        set: function (value) {
            this.style.visibility = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "opacity", {
        get: function () {
            return this.style.opacity;
        },
        set: function (value) {
            this.style.opacity = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "rotate", {
        get: function () {
            return this.style.rotate;
        },
        set: function (value) {
            this.style.rotate = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "rotateX", {
        get: function () {
            return this.style.rotateX;
        },
        set: function (value) {
            this.style.rotateX = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "rotateY", {
        get: function () {
            return this.style.rotateY;
        },
        set: function (value) {
            this.style.rotateY = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "perspective", {
        get: function () {
            return this.style.perspective;
        },
        set: function (value) {
            this.style.perspective = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "textTransform", {
        get: function () {
            return this.style.textTransform;
        },
        set: function (value) {
            this.style.textTransform = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "translateX", {
        get: function () {
            return this.style.translateX;
        },
        set: function (value) {
            this.style.translateX = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "translateY", {
        get: function () {
            return this.style.translateY;
        },
        set: function (value) {
            this.style.translateY = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "scaleX", {
        get: function () {
            return this.style.scaleX;
        },
        set: function (value) {
            this.style.scaleX = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "scaleY", {
        get: function () {
            return this.style.scaleY;
        },
        set: function (value) {
            this.style.scaleY = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessible", {
        get: function () {
            return this.style.accessible;
        },
        set: function (value) {
            this.style.accessible = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityHidden", {
        get: function () {
            return this.style.accessibilityHidden;
        },
        set: function (value) {
            this.style.accessibilityHidden = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityRole", {
        get: function () {
            return this.style.accessibilityRole;
        },
        set: function (value) {
            this.style.accessibilityRole = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityState", {
        get: function () {
            return this.style.accessibilityState;
        },
        set: function (value) {
            this.style.accessibilityState = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityLiveRegion", {
        get: function () {
            return this.style.accessibilityLiveRegion;
        },
        set: function (value) {
            this.style.accessibilityLiveRegion = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityLanguage", {
        get: function () {
            return this.style.accessibilityLanguage;
        },
        set: function (value) {
            this.style.accessibilityLanguage = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "accessibilityMediaSession", {
        get: function () {
            return this.style.accessibilityMediaSession;
        },
        set: function (value) {
            this.style.accessibilityMediaSession = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "iosAccessibilityAdjustsFontSize", {
        get: function () {
            return this.style.iosAccessibilityAdjustsFontSize;
        },
        set: function (value) {
            this.style.iosAccessibilityAdjustsFontSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "iosAccessibilityMinFontScale", {
        get: function () {
            return this.style.iosAccessibilityMinFontScale;
        },
        set: function (value) {
            this.style.iosAccessibilityMinFontScale = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "iosAccessibilityMaxFontScale", {
        get: function () {
            return this.style.iosAccessibilityMaxFontScale;
        },
        set: function (value) {
            this.style.iosAccessibilityMaxFontScale = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "automationText", {
        get: function () {
            return this.accessibilityIdentifier;
        },
        set: function (value) {
            this.accessibilityIdentifier = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "androidElevation", {
        get: function () {
            return this.style.androidElevation;
        },
        set: function (value) {
            this.style.androidElevation = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "androidDynamicElevationOffset", {
        get: function () {
            return this.style.androidDynamicElevationOffset;
        },
        set: function (value) {
            this.style.androidDynamicElevationOffset = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "isLayoutValid", {
        get: function () {
            return this._isLayoutValid;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "cssType", {
        get: function () {
            if (!this._cssType) {
                this._cssType = this.typeName.toLowerCase();
            }
            return this._cssType;
        },
        set: function (type) {
            this._cssType = type.toLowerCase();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "isLayoutRequired", {
        get: function () {
            return true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewCommon.prototype, "needsNativeDrawableFill", {
        get: function () {
            return false;
        },
        enumerable: false,
        configurable: true
    });
    ViewCommon.prototype.measure = function (widthMeasureSpec, heightMeasureSpec) {
        this._setCurrentMeasureSpecs(widthMeasureSpec, heightMeasureSpec);
    };
    ViewCommon.prototype.layout = function (left, top, right, bottom) {
        this._setCurrentLayoutBounds(left, top, right, bottom);
    };
    ViewCommon.prototype.getMeasuredWidth = function () {
        return this._measuredWidth & layout.MEASURED_SIZE_MASK || 0;
    };
    ViewCommon.prototype.getMeasuredHeight = function () {
        return this._measuredHeight & layout.MEASURED_SIZE_MASK || 0;
    };
    ViewCommon.prototype.getMeasuredState = function () {
        return (this._measuredWidth & layout.MEASURED_STATE_MASK) | ((this._measuredHeight >> layout.MEASURED_HEIGHT_STATE_SHIFT) & (layout.MEASURED_STATE_MASK >> layout.MEASURED_HEIGHT_STATE_SHIFT));
    };
    ViewCommon.prototype.setMeasuredDimension = function (measuredWidth, measuredHeight) {
        this._measuredWidth = measuredWidth;
        this._measuredHeight = measuredHeight;
        if (Trace.isEnabled()) {
            Trace.write(this + ' :setMeasuredDimension: ' + measuredWidth + ', ' + measuredHeight, Trace.categories.Layout);
        }
    };
    ViewCommon.prototype.requestLayout = function () {
        this._isLayoutValid = false;
        _super.prototype.requestLayout.call(this);
    };
    ViewCommon.resolveSizeAndState = function (size, specSize, specMode, childMeasuredState) {
        return ViewHelper.resolveSizeAndState(size, specSize, specMode, childMeasuredState);
    };
    ViewCommon.combineMeasuredStates = function (curState, newState) {
        return ViewHelper.combineMeasuredStates(curState, newState);
    };
    ViewCommon.layoutChild = function (parent, child, left, top, right, bottom, setFrame) {
        if (setFrame === void 0) { setFrame = true; }
        ViewHelper.layoutChild(parent, child, left, top, right, bottom);
    };
    ViewCommon.measureChild = function (parent, child, widthMeasureSpec, heightMeasureSpec) {
        return ViewHelper.measureChild(parent, child, widthMeasureSpec, heightMeasureSpec);
    };
    ViewCommon.prototype._setCurrentMeasureSpecs = function (widthMeasureSpec, heightMeasureSpec) {
        var changed = this._currentWidthMeasureSpec !== widthMeasureSpec || this._currentHeightMeasureSpec !== heightMeasureSpec;
        this._currentWidthMeasureSpec = widthMeasureSpec;
        this._currentHeightMeasureSpec = heightMeasureSpec;
        return changed;
    };
    ViewCommon.prototype._getCurrentLayoutBounds = function () {
        return { left: 0, top: 0, right: 0, bottom: 0 };
    };
    /**
     * Returns two booleans - the first if "boundsChanged" the second is "sizeChanged".
     */
    ViewCommon.prototype._setCurrentLayoutBounds = function (left, top, right, bottom) {
        this._isLayoutValid = true;
        var boundsChanged = this._oldLeft !== left || this._oldTop !== top || this._oldRight !== right || this._oldBottom !== bottom;
        var sizeChanged = this._oldRight - this._oldLeft !== right - left || this._oldBottom - this._oldTop !== bottom - top;
        this._oldLeft = left;
        this._oldTop = top;
        this._oldRight = right;
        this._oldBottom = bottom;
        return { boundsChanged: boundsChanged, sizeChanged: sizeChanged };
    };
    ViewCommon.prototype.eachChild = function (callback) {
        this.eachChildView(callback);
    };
    ViewCommon.prototype.eachChildView = function (callback) {
        //
    };
    ViewCommon.prototype._getNativeViewsCount = function () {
        return this._isAddedToNativeVisualTree ? 1 : 0;
    };
    ViewCommon.prototype._eachLayoutView = function (callback) {
        return callback(this);
    };
    ViewCommon.prototype.focus = function () {
        return undefined;
    };
    ViewCommon.prototype.getSafeAreaInsets = function () {
        return { left: 0, top: 0, right: 0, bottom: 0 };
    };
    ViewCommon.prototype.getLocationInWindow = function () {
        return undefined;
    };
    ViewCommon.prototype.getLocationOnScreen = function () {
        return undefined;
    };
    ViewCommon.prototype.getLocationRelativeTo = function (otherView) {
        return undefined;
    };
    ViewCommon.prototype.getActualSize = function () {
        var currentBounds = this._getCurrentLayoutBounds();
        if (!currentBounds) {
            return undefined;
        }
        return {
            width: layout.toDeviceIndependentPixels(currentBounds.right - currentBounds.left),
            height: layout.toDeviceIndependentPixels(currentBounds.bottom - currentBounds.top),
        };
    };
    ViewCommon.prototype.animate = function (animation) {
        return this.createAnimation(animation).play();
    };
    ViewCommon.prototype.createAnimation = function (animation) {
        ensureAnimationModule();
        if (!this._localAnimations) {
            this._localAnimations = new Set();
        }
        animation.target = this;
        var anim = new animationModule.Animation([animation]);
        this._localAnimations.add(anim);
        return anim;
    };
    ViewCommon.prototype._removeAnimation = function (animation) {
        var localAnimations = this._localAnimations;
        if (localAnimations && localAnimations.has(animation)) {
            localAnimations.delete(animation);
            if (animation.isPlaying) {
                animation.cancel();
            }
            return true;
        }
        return false;
    };
    ViewCommon.prototype.resetNativeView = function () {
        var _this = this;
        if (this._localAnimations) {
            this._localAnimations.forEach(function (a) { return _this._removeAnimation(a); });
        }
        _super.prototype.resetNativeView.call(this);
    };
    ViewCommon.prototype._setNativeViewFrame = function (nativeView, frame) {
        //
    };
    ViewCommon.prototype._getValue = function () {
        throw new Error('The View._getValue is obsolete. There is a new property system.');
    };
    ViewCommon.prototype._setValue = function () {
        throw new Error('The View._setValue is obsolete. There is a new property system.');
    };
    ViewCommon.prototype._updateEffectiveLayoutValues = function (parentWidthMeasureSize, parentWidthMeasureMode, parentHeightMeasureSize, parentHeightMeasureMode) {
        var style = this.style;
        var availableWidth = parentWidthMeasureMode === layout.UNSPECIFIED ? -1 : parentWidthMeasureSize;
        this.effectiveWidth = PercentLength.toDevicePixels(style.width, -2, availableWidth);
        this.effectiveMarginLeft = PercentLength.toDevicePixels(style.marginLeft, 0, availableWidth);
        this.effectiveMarginRight = PercentLength.toDevicePixels(style.marginRight, 0, availableWidth);
        var availableHeight = parentHeightMeasureMode === layout.UNSPECIFIED ? -1 : parentHeightMeasureSize;
        this.effectiveHeight = PercentLength.toDevicePixels(style.height, -2, availableHeight);
        this.effectiveMarginTop = PercentLength.toDevicePixels(style.marginTop, 0, availableHeight);
        this.effectiveMarginBottom = PercentLength.toDevicePixels(style.marginBottom, 0, availableHeight);
    };
    ViewCommon.prototype._setNativeClipToBounds = function () {
        //
    };
    ViewCommon.prototype._redrawNativeBackground = function (value) {
        //
    };
    ViewCommon.prototype._applyBackground = function (background, isBorderDrawable, onlyColor, backgroundDrawable) {
        //
    };
    ViewCommon.prototype._onAttachedToWindow = function () {
        //
    };
    ViewCommon.prototype._onDetachedFromWindow = function () {
        //
    };
    ViewCommon.prototype._hasAncestorView = function (ancestorView) {
        var matcher = function (view) { return view === ancestorView; };
        for (var parent_2 = this.parent; parent_2 != null; parent_2 = parent_2.parent) {
            if (matcher(parent_2)) {
                return true;
            }
        }
        return false;
    };
    ViewCommon.prototype.sendAccessibilityEvent = function (options) {
        return;
    };
    ViewCommon.prototype.accessibilityAnnouncement = function (msg) {
        return;
    };
    ViewCommon.prototype.accessibilityScreenChanged = function () {
        return;
    };
    ViewCommon.prototype.setAccessibilityIdentifier = function (view, value) {
        return;
    };
    ViewCommon.layoutChangedEvent = 'layoutChanged';
    ViewCommon.shownModallyEvent = 'shownModally';
    ViewCommon.showingModallyEvent = 'showingModally';
    ViewCommon.accessibilityBlurEvent = accessibilityBlurEvent;
    ViewCommon.accessibilityFocusEvent = accessibilityFocusEvent;
    ViewCommon.accessibilityFocusChangedEvent = accessibilityFocusChangedEvent;
    ViewCommon.accessibilityPerformEscapeEvent = accessibilityPerformEscapeEvent;
    return ViewCommon;
}(ViewBase));
export { ViewCommon };
export var originXProperty = new Property({
    name: 'originX',
    defaultValue: 0.5,
    valueConverter: function (v) { return parseFloat(v); },
});
originXProperty.register(ViewCommon);
export var originYProperty = new Property({
    name: 'originY',
    defaultValue: 0.5,
    valueConverter: function (v) { return parseFloat(v); },
});
originYProperty.register(ViewCommon);
export var isEnabledProperty = new Property({
    name: 'isEnabled',
    defaultValue: true,
    valueConverter: booleanConverter,
    valueChanged: function (target, oldValue, newValue) {
        var state = 'disabled';
        if (newValue) {
            target._removeVisualState(state);
        }
        else {
            target._addVisualState(state);
        }
    },
});
isEnabledProperty.register(ViewCommon);
export var isUserInteractionEnabledProperty = new Property({
    name: 'isUserInteractionEnabled',
    defaultValue: true,
    valueConverter: booleanConverter,
});
isUserInteractionEnabledProperty.register(ViewCommon);
export var iosOverflowSafeAreaProperty = new Property({
    name: 'iosOverflowSafeArea',
    defaultValue: false,
    valueConverter: booleanConverter,
});
iosOverflowSafeAreaProperty.register(ViewCommon);
export var iosOverflowSafeAreaEnabledProperty = new InheritedProperty({
    name: 'iosOverflowSafeAreaEnabled',
    defaultValue: true,
    valueConverter: booleanConverter,
});
iosOverflowSafeAreaEnabledProperty.register(ViewCommon);
export var iosIgnoreSafeAreaProperty = new InheritedProperty({
    name: 'iosIgnoreSafeArea',
    defaultValue: false,
    valueConverter: booleanConverter,
});
iosIgnoreSafeAreaProperty.register(ViewCommon);
export var visionHoverStyleProperty = new Property({
    name: 'visionHoverStyle',
    valueChanged: function (view, oldValue, newValue) {
        view.visionHoverStyle = newValue;
    },
});
visionHoverStyleProperty.register(ViewCommon);
var visionIgnoreHoverStyleProperty = new Property({
    name: 'visionIgnoreHoverStyle',
    valueChanged: function (view, oldValue, newValue) {
        view.visionIgnoreHoverStyle = newValue;
    },
    valueConverter: booleanConverter,
});
visionIgnoreHoverStyleProperty.register(ViewCommon);
var touchAnimationProperty = new Property({
    name: 'touchAnimation',
    valueChanged: function (view, oldValue, newValue) {
        view.touchAnimation = newValue;
    },
    valueConverter: function (value) {
        if (isObject(value)) {
            return value;
        }
        else {
            return booleanConverter(value);
        }
    },
});
touchAnimationProperty.register(ViewCommon);
var ignoreTouchAnimationProperty = new Property({
    name: 'ignoreTouchAnimation',
    valueChanged: function (view, oldValue, newValue) {
        view.ignoreTouchAnimation = newValue;
    },
    valueConverter: booleanConverter,
});
ignoreTouchAnimationProperty.register(ViewCommon);
var touchDelayProperty = new Property({
    name: 'touchDelay',
    valueChanged: function (view, oldValue, newValue) {
        view.touchDelay = newValue;
    },
    valueConverter: function (v) { return parseFloat(v); },
});
touchDelayProperty.register(ViewCommon);
export var testIDProperty = new Property({
    name: 'testID',
});
testIDProperty.register(ViewCommon);
accessibilityIdentifierProperty.register(ViewCommon);
accessibilityLabelProperty.register(ViewCommon);
accessibilityValueProperty.register(ViewCommon);
accessibilityHintProperty.register(ViewCommon);
accessibilityIgnoresInvertColorsProperty.register(ViewCommon);
//# sourceMappingURL=view-common.js.map