/**
 * iOS specific dialogs functions implementation.
 */
import { Trace } from '../../trace';
import { getLabelColor, getButtonColors, getTextFieldColor, isDialogOptions, inputType, capitalizationType, DialogStrings, parseLoginOptions } from './dialogs-common';
import { isString, isDefined, isFunction } from '../../utils/types';
import { Application } from '../../application';
export * from './dialogs-common';
function addButtonsToAlertController(alertController, options, callback) {
    if (!options) {
        return;
    }
    if (isString(options.cancelButtonText)) {
        alertController.addAction(UIAlertAction.actionWithTitleStyleHandler(options.cancelButtonText, 0 /* UIAlertActionStyle.Default */, function () {
            raiseCallback(callback, false);
        }));
    }
    if (isString(options.neutralButtonText)) {
        alertController.addAction(UIAlertAction.actionWithTitleStyleHandler(options.neutralButtonText, 0 /* UIAlertActionStyle.Default */, function () {
            raiseCallback(callback, undefined);
        }));
    }
    if (isString(options.okButtonText)) {
        alertController.addAction(UIAlertAction.actionWithTitleStyleHandler(options.okButtonText, 0 /* UIAlertActionStyle.Default */, function () {
            raiseCallback(callback, true);
        }));
    }
}
function raiseCallback(callback, result) {
    if (isFunction(callback)) {
        callback(result);
    }
}
function showUIAlertController(alertController) {
    var _a, _b;
    var viewController = Application.ios.rootController;
    while (viewController && viewController.presentedViewController && !viewController.presentedViewController.beingDismissed) {
        viewController = viewController.presentedViewController;
    }
    if (!viewController) {
        Trace.write("No root controller found to open dialog.", Trace.categories.Error, Trace.messageType.warn);
        return;
    }
    if (alertController.popoverPresentationController) {
        alertController.popoverPresentationController.sourceView = viewController.view;
        alertController.popoverPresentationController.sourceRect = CGRectMake(viewController.view.bounds.size.width / 2.0, viewController.view.bounds.size.height / 2.0, 1.0, 1.0);
        alertController.popoverPresentationController.permittedArrowDirections = 0;
    }
    var color = getButtonColors().color;
    if (color) {
        alertController.view.tintColor = color.ios;
    }
    var lblColor = getLabelColor();
    if (lblColor) {
        if (alertController.title) {
            var title = NSAttributedString.alloc().initWithStringAttributes(alertController.title, (_a = {}, _a[NSForegroundColorAttributeName] = lblColor.ios, _a));
            alertController.setValueForKey(title, 'attributedTitle');
        }
        if (alertController.message) {
            var message = NSAttributedString.alloc().initWithStringAttributes(alertController.message, (_b = {}, _b[NSForegroundColorAttributeName] = lblColor.ios, _b));
            alertController.setValueForKey(message, 'attributedMessage');
        }
    }
    viewController.presentModalViewControllerAnimated(alertController, true);
}
export function alert(arg) {
    return new Promise(function (resolve, reject) {
        try {
            var options = !isDialogOptions(arg)
                ? {
                    title: DialogStrings.ALERT,
                    okButtonText: DialogStrings.OK,
                    message: arg + '',
                }
                : arg;
            var alertController = UIAlertController.alertControllerWithTitleMessagePreferredStyle(options.title, options.message, 1 /* UIAlertControllerStyle.Alert */);
            addButtonsToAlertController(alertController, options, function () {
                resolve();
            });
            showUIAlertController(alertController);
        }
        catch (ex) {
            reject(ex);
        }
    });
}
export function confirm(arg) {
    return new Promise(function (resolve, reject) {
        try {
            var options = !isDialogOptions(arg)
                ? {
                    title: DialogStrings.CONFIRM,
                    okButtonText: DialogStrings.OK,
                    cancelButtonText: DialogStrings.CANCEL,
                    message: arg + '',
                }
                : arg;
            var alertController = UIAlertController.alertControllerWithTitleMessagePreferredStyle(options.title, options.message, 1 /* UIAlertControllerStyle.Alert */);
            addButtonsToAlertController(alertController, options, function (r) {
                resolve(r);
            });
            showUIAlertController(alertController);
        }
        catch (ex) {
            reject(ex);
        }
    });
}
export function prompt() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var options;
    var defaultOptions = {
        title: DialogStrings.PROMPT,
        okButtonText: DialogStrings.OK,
        cancelButtonText: DialogStrings.CANCEL,
        inputType: inputType.text,
    };
    var arg = args[0];
    if (args.length === 1) {
        if (isString(arg)) {
            options = defaultOptions;
            options.message = arg;
        }
        else {
            options = arg;
        }
    }
    else if (args.length === 2) {
        if (isString(arg) && isString(args[1])) {
            options = defaultOptions;
            options.message = arg;
            options.defaultText = args[1];
        }
    }
    return new Promise(function (resolve, reject) {
        try {
            var alertController = UIAlertController.alertControllerWithTitleMessagePreferredStyle(options.title, options.message, 1 /* UIAlertControllerStyle.Alert */);
            alertController.addTextFieldWithConfigurationHandler(function (arg) {
                arg.text = isString(options.defaultText) ? options.defaultText : '';
                arg.secureTextEntry = options && options.inputType === inputType.password;
                if (options && options.inputType === inputType.email) {
                    arg.keyboardType = 7 /* UIKeyboardType.EmailAddress */;
                }
                else if (options && options.inputType === inputType.number) {
                    arg.keyboardType = 4 /* UIKeyboardType.NumberPad */;
                }
                else if (options && options.inputType === inputType.decimal) {
                    arg.keyboardType = 8 /* UIKeyboardType.DecimalPad */;
                }
                else if (options && options.inputType === inputType.phone) {
                    arg.keyboardType = 5 /* UIKeyboardType.PhonePad */;
                }
                var color = getTextFieldColor();
                if (color) {
                    arg.textColor = arg.tintColor = color.ios;
                }
            });
            var textField_1 = alertController.textFields.firstObject;
            if (options) {
                switch (options.capitalizationType) {
                    case capitalizationType.all: {
                        textField_1.autocapitalizationType = 3 /* UITextAutocapitalizationType.AllCharacters */;
                        break;
                    }
                    case capitalizationType.sentences: {
                        textField_1.autocapitalizationType = 2 /* UITextAutocapitalizationType.Sentences */;
                        break;
                    }
                    case capitalizationType.words: {
                        textField_1.autocapitalizationType = 1 /* UITextAutocapitalizationType.Words */;
                        break;
                    }
                    default: {
                        textField_1.autocapitalizationType = 0 /* UITextAutocapitalizationType.None */;
                    }
                }
            }
            addButtonsToAlertController(alertController, options, function (r) {
                resolve({ result: r, text: textField_1.text });
            });
            showUIAlertController(alertController);
        }
        catch (ex) {
            reject(ex);
        }
    });
}
export function login() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var options = parseLoginOptions(args);
    return new Promise(function (resolve, reject) {
        try {
            var alertController = UIAlertController.alertControllerWithTitleMessagePreferredStyle(options.title, options.message, 1 /* UIAlertControllerStyle.Alert */);
            var textFieldColor_1 = getTextFieldColor();
            alertController.addTextFieldWithConfigurationHandler(function (arg) {
                arg.placeholder = 'Login';
                arg.placeholder = options.userNameHint ? options.userNameHint : '';
                arg.text = isString(options.userName) ? options.userName : '';
                if (textFieldColor_1) {
                    arg.textColor = arg.tintColor = textFieldColor_1.ios;
                }
            });
            alertController.addTextFieldWithConfigurationHandler(function (arg) {
                arg.placeholder = 'Password';
                arg.secureTextEntry = true;
                arg.placeholder = options.passwordHint ? options.passwordHint : '';
                arg.text = isString(options.password) ? options.password : '';
                if (textFieldColor_1) {
                    arg.textColor = arg.tintColor = textFieldColor_1.ios;
                }
            });
            var userNameTextField_1 = alertController.textFields.firstObject;
            var passwordTextField_1 = alertController.textFields.lastObject;
            addButtonsToAlertController(alertController, options, function (r) {
                resolve({
                    result: r,
                    userName: userNameTextField_1.text,
                    password: passwordTextField_1.text,
                });
            });
            showUIAlertController(alertController);
        }
        catch (ex) {
            reject(ex);
        }
    });
}
export function action() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var options;
    var defaultOptions = { title: null, cancelButtonText: DialogStrings.CANCEL };
    if (args.length === 1) {
        if (isString(args[0])) {
            options = defaultOptions;
            options.message = args[0];
        }
        else {
            options = args[0];
        }
    }
    else if (args.length === 2) {
        if (isString(args[0]) && isString(args[1])) {
            options = defaultOptions;
            options.message = args[0];
            options.cancelButtonText = args[1];
        }
    }
    else if (args.length === 3) {
        if (isString(args[0]) && isString(args[1]) && isDefined(args[2])) {
            options = defaultOptions;
            options.message = args[0];
            options.cancelButtonText = args[1];
            options.actions = args[2];
        }
    }
    return new Promise(function (resolve, reject) {
        try {
            var i = void 0;
            var action_1;
            var alertController = UIAlertController.alertControllerWithTitleMessagePreferredStyle(options.title, options.message, 0 /* UIAlertControllerStyle.ActionSheet */);
            if (options.actions) {
                for (i = 0; i < options.actions.length; i++) {
                    action_1 = options.actions[i];
                    if (isString(action_1)) {
                        var thisActionIsDestructive = options.destructiveActionsIndexes && options.destructiveActionsIndexes.indexOf(i) !== -1;
                        var dialogType = thisActionIsDestructive ? 2 /* UIAlertActionStyle.Destructive */ : 0 /* UIAlertActionStyle.Default */;
                        alertController.addAction(UIAlertAction.actionWithTitleStyleHandler(action_1, dialogType, function (arg) {
                            resolve(arg.title);
                        }));
                    }
                }
            }
            if (isString(options.cancelButtonText)) {
                alertController.addAction(UIAlertAction.actionWithTitleStyleHandler(options.cancelButtonText, 1 /* UIAlertActionStyle.Cancel */, function (arg) {
                    resolve(arg.title);
                }));
            }
            showUIAlertController(alertController);
        }
        catch (ex) {
            reject(ex);
        }
    });
}
/**
 * Singular rollup for convenience of all dialog methods
 */
export var Dialogs = {
    alert: alert,
    confirm: confirm,
    prompt: prompt,
    login: login,
    action: action,
};
//# sourceMappingURL=index.ios.js.map