import { profile } from '../../../profiling';
import { getFrameByNumberId } from '..';
import { Trace } from '../../../trace';
import { Application } from '../../../application';
import { Color } from '../../../color';
import { _updateTransitions } from '../fragment.transitions';
var FRAMEID = '_frameId';
var CALLBACKS = '_callbacks';
var FragmentCallbacksImplementation = /** @class */ (function () {
    function FragmentCallbacksImplementation() {
        this.backgroundBitmap = null;
    }
    FragmentCallbacksImplementation.prototype.onHiddenChanged = function (fragment, hidden, superFunc) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(fragment, ".onHiddenChanged(").concat(hidden, ")"), Trace.categories.NativeLifecycle);
        }
        superFunc.call(fragment, hidden);
    };
    FragmentCallbacksImplementation.prototype.onCreateAnimator = function (fragment, transit, enter, nextAnim, superFunc) {
        var animator = null;
        var entry = this.entry;
        // Return enterAnimator only when new (no current entry) nested transition.
        if (enter && entry.isNestedDefaultTransition) {
            animator = entry.enterAnimator;
            entry.isNestedDefaultTransition = false;
        }
        return animator || superFunc.call(fragment, transit, enter, nextAnim);
    };
    FragmentCallbacksImplementation.prototype.onCreate = function (fragment, savedInstanceState, superFunc) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(fragment, ".onCreate(").concat(savedInstanceState, ")"), Trace.categories.NativeLifecycle);
        }
        superFunc.call(fragment, savedInstanceState);
        // There is no entry set to the fragment, so this must be destroyed fragment that was recreated by Android.
        // We should find its corresponding page in our backstack and set it manually.
        if (!this.entry) {
            var args = fragment.getArguments();
            var frameId = args.getInt(FRAMEID);
            var frame = getFrameByNumberId(frameId);
            if (!frame) {
                throw new Error("Cannot find Frame for ".concat(fragment));
            }
            findPageForFragment(fragment, frame);
        }
    };
    FragmentCallbacksImplementation.prototype.onCreateView = function (fragment, inflater, container, savedInstanceState, superFunc) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(fragment, ".onCreateView(inflater, container, ").concat(savedInstanceState, ")"), Trace.categories.NativeLifecycle);
        }
        var entry = this.entry;
        if (!entry) {
            Trace.error("".concat(fragment, ".onCreateView: entry is null or undefined"));
            return null;
        }
        var page = entry.resolvedPage;
        if (!page) {
            Trace.error("".concat(fragment, ".onCreateView: entry has no resolvedPage"));
            return null;
        }
        var frame = this.frame;
        if (!frame) {
            Trace.error("".concat(fragment, ".onCreateView: this.frame is null or undefined"));
            return null;
        }
        frame._resolvedPage = page;
        if (page.parent === frame) {
            frame._inheritStyles(page);
            // If we are navigating to a page that was destroyed
            // reinitialize its UI.
            if (!page._context) {
                var context = (container && container.getContext()) || (inflater && inflater.getContext());
                page._setupUI(context);
            }
            if (frame.isLoaded && !page.isLoaded) {
                page.callLoaded();
            }
        }
        else {
            if (!page.parent) {
                if (!frame._styleScope) {
                    // Make sure page will have styleScope even if parents don't.
                    page._updateStyleScope();
                }
                frame._addView(page);
            }
            else {
                throw new Error('Page is already shown on another frame.');
            }
        }
        var savedState = entry.viewSavedState;
        if (savedState) {
            page.nativeViewProtected.restoreHierarchyState(savedState);
            entry.viewSavedState = null;
        }
        // fixes 'java.lang.IllegalStateException: The specified child already has a parent. You must call removeView() on the child's parent first'.
        // on app resume in nested frame scenarios with support library version greater than 26.0.0
        // HACK: this whole code block shouldn't be necessary as the native view is supposedly removed from its parent
        // right after onDestroyView(...) is called but for some reason the fragment view (page) still thinks it has a
        // parent while its supposed parent believes it properly removed its children; in order to "force" the child to
        // lose its parent we temporarily add it to the parent, and then remove it (addViewInLayout doesn't trigger layout pass)
        var nativeView = page.nativeViewProtected;
        if (nativeView != null) {
            var parentView = nativeView.getParent();
            if (parentView instanceof android.view.ViewGroup) {
                if (parentView.getChildCount() === 0) {
                    parentView.addViewInLayout(nativeView, -1, new org.nativescript.widgets.CommonLayoutParams());
                }
                parentView.removeAllViews();
            }
        }
        return page.nativeViewProtected;
    };
    FragmentCallbacksImplementation.prototype.onSaveInstanceState = function (fragment, outState, superFunc) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(fragment, ".onSaveInstanceState(").concat(outState, ")"), Trace.categories.NativeLifecycle);
        }
        superFunc.call(fragment, outState);
    };
    FragmentCallbacksImplementation.prototype.onDestroyView = function (fragment, superFunc) {
        try {
            if (Trace.isEnabled()) {
                Trace.write("".concat(fragment, ".onDestroyView()"), Trace.categories.NativeLifecycle);
            }
            var hasRemovingParent = fragment.getRemovingParentFragment();
            if (hasRemovingParent) {
                var nativeFrameView = this.frame.nativeViewProtected;
                if (nativeFrameView) {
                    var bitmapDrawable = new android.graphics.drawable.BitmapDrawable(Application.android.context.getResources(), this.backgroundBitmap);
                    this.frame._originalBackground = this.frame.backgroundColor || new Color('White');
                    nativeFrameView.setBackgroundDrawable(bitmapDrawable);
                    this.backgroundBitmap = null;
                }
            }
        }
        finally {
            superFunc.call(fragment);
        }
    };
    FragmentCallbacksImplementation.prototype.onDestroy = function (fragment, superFunc) {
        if (Trace.isEnabled()) {
            Trace.write("".concat(fragment, ".onDestroy()"), Trace.categories.NativeLifecycle);
        }
        superFunc.call(fragment);
        var entry = this.entry;
        if (!entry) {
            Trace.error("".concat(fragment, ".onDestroy: entry is null or undefined"));
            return null;
        }
        // [nested frames / fragments] see https://github.com/NativeScript/NativeScript/issues/6629
        // retaining reference to a destroyed fragment here somehow causes a cryptic
        // "IllegalStateException: Failure saving state: active fragment has cleared index: -1"
        // in a specific mixed parent / nested frame navigation scenario
        entry.fragment = null;
        var page = entry.resolvedPage;
        if (!page) {
            // todo: check why this happens when using shared element transition!!!
            // commented out the Trace.error to prevent a crash (the app will still work interestingly)
            console.log("".concat(fragment, ".onDestroy: entry has no resolvedPage"));
            // Trace.error(`${fragment}.onDestroy: entry has no resolvedPage`);
            return null;
        }
    };
    FragmentCallbacksImplementation.prototype.onPause = function (fragment, superFunc) {
        try {
            // Get view as bitmap and set it as background. This is workaround for the disapearing nested fragments.
            // TODO: Consider removing it when update to androidx.fragment:1.2.0
            var hasRemovingParent = fragment.getRemovingParentFragment();
            if (hasRemovingParent) {
                this.backgroundBitmap = this.loadBitmapFromView(this.frame.nativeViewProtected);
            }
        }
        finally {
            superFunc.call(fragment);
        }
    };
    FragmentCallbacksImplementation.prototype.onResume = function (fragment, superFunc) {
        var frame = this.entry.resolvedPage.frame;
        // on some cases during the first navigation on nested frames the animation doesn't trigger
        // we depend on the animation (even None animation) to set the entry as the current entry
        // animation should start between start and resume, so if we have an executing navigation here it probably means the animation was skipped
        // so we manually set the entry
        // also, to be compatible with fragments 1.2.x we need this setTimeout as animations haven't run on onResume yet
        var weakRef = new WeakRef(this);
        setTimeout(function () {
            var owner = weakRef.get();
            if (!owner) {
                return;
            }
            if (frame._executingContext && !owner.entry.isAnimationRunning) {
                frame.setCurrent(owner.entry, frame._executingContext.navigationType);
            }
        }, 0);
        superFunc.call(fragment);
    };
    FragmentCallbacksImplementation.prototype.onStop = function (fragment, superFunc) {
        superFunc.call(fragment);
    };
    FragmentCallbacksImplementation.prototype.toStringOverride = function (fragment, superFunc) {
        var entry = this.entry;
        if (entry) {
            return "".concat(entry.fragmentTag, "<").concat(entry.resolvedPage, ">");
        }
        else {
            return 'NO ENTRY, ' + superFunc.call(fragment);
        }
    };
    FragmentCallbacksImplementation.prototype.loadBitmapFromView = function (view) {
        // Don't try to create bitmaps with no dimensions as this causes a crash
        // This might happen when showing and closing dialogs fast.
        if (!(view && view.getWidth() > 0 && view.getHeight() > 0)) {
            return undefined;
        }
        // Another way to get view bitmap. Test performance vs setDrawingCacheEnabled
        // const width = view.getWidth();
        // const height = view.getHeight();
        // const bitmap = android.graphics.Bitmap.createBitmap(width, height, android.graphics.Bitmap.Config.ARGB_8888);
        // const canvas = new android.graphics.Canvas(bitmap);
        // view.layout(0, 0, width, height);
        // view.draw(canvas);
        // view.setDrawingCacheEnabled(true);
        // const drawCache = view.getDrawingCache();
        // const bitmap = android.graphics.Bitmap.createBitmap(drawCache);
        // view.setDrawingCacheEnabled(false);
        return org.nativescript.widgets.Utils.getBitmapFromView(view);
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, Boolean, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onHiddenChanged", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, Number, Boolean, Number, Function]),
        __metadata("design:returntype", android.animation.Animator)
    ], FragmentCallbacksImplementation.prototype, "onCreateAnimator", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, android.os.Bundle, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onCreate", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle, Function]),
        __metadata("design:returntype", android.view.View)
    ], FragmentCallbacksImplementation.prototype, "onCreateView", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, android.os.Bundle, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onSaveInstanceState", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [org.nativescript.widgets.FragmentBase, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onDestroyView", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onDestroy", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [org.nativescript.widgets.FragmentBase, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onPause", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [org.nativescript.widgets.FragmentBase, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onResume", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, Function]),
        __metadata("design:returntype", void 0)
    ], FragmentCallbacksImplementation.prototype, "onStop", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [androidx.fragment.app.Fragment, Function]),
        __metadata("design:returntype", String)
    ], FragmentCallbacksImplementation.prototype, "toStringOverride", null);
    return FragmentCallbacksImplementation;
}());
export { FragmentCallbacksImplementation };
function findPageForFragment(fragment, frame) {
    var fragmentTag = fragment.getTag();
    if (Trace.isEnabled()) {
        Trace.write("Finding page for ".concat(fragmentTag, "."), Trace.categories.NativeLifecycle);
    }
    var entry;
    var current = frame._currentEntry;
    var executingContext = frame._executingContext;
    if (current && current.fragmentTag === fragmentTag) {
        entry = current;
    }
    else if (executingContext && executingContext.entry && executingContext.entry.fragmentTag === fragmentTag) {
        entry = executingContext.entry;
    }
    var page;
    if (entry) {
        entry.recreated = true;
        page = entry.resolvedPage;
    }
    if (page) {
        var callbacks = fragment[CALLBACKS];
        callbacks.frame = frame;
        callbacks.entry = entry;
        entry.fragment = fragment;
        _updateTransitions(entry);
    }
    else {
        throw new Error("Could not find a page for ".concat(fragmentTag, "."));
    }
}
//# sourceMappingURL=fragment-callbacks.js.map