import { CoreTypes } from '../../../core-types';
import { Trace } from '../../../trace';
import { CSSType, View } from '../../core/view';
import { GridLayout } from '../grid-layout';
import { Animation } from '../../animation';
import { isNumber } from '../../../utils/types';
var RootLayoutBase = /** @class */ (function (_super) {
    __extends(RootLayoutBase, _super);
    function RootLayoutBase() {
        var _this = _super.call(this) || this;
        _this.popupViews = [];
        global.rootLayout = _this;
        return _this;
    }
    RootLayoutBase.prototype.onLoaded = function () {
        // get actual content count of rootLayout (elements between the <RootLayout> tags in the template).
        // All popups will be inserted dynamically at a higher index
        this.staticChildCount = this.getChildrenCount();
        _super.prototype.onLoaded.call(this);
    };
    RootLayoutBase.prototype._onLivesync = function (context) {
        var handled = false;
        if (this.popupViews.length > 0) {
            this.closeAll();
            handled = true;
        }
        if (_super.prototype._onLivesync.call(this, context)) {
            handled = true;
        }
        return handled;
    };
    /**
     * Ability to add any view instance to composite views like layers.
     *
     * @param view
     * @param options
     * @returns
     */
    RootLayoutBase.prototype.open = function (view, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        return new Promise(function (resolve, reject) {
            if (!(view instanceof View)) {
                return reject(new Error("Invalid open view: ".concat(view)));
            }
            if (_this.hasChild(view)) {
                return reject(new Error("".concat(view, " has already been added")));
            }
            var toOpen = [];
            var enterAnimationDefinition = options.animation ? options.animation.enterFrom : null;
            // keep track of the views locally to be able to use their options later
            _this.popupViews.push({ view: view, options: options });
            if (options.shadeCover) {
                // perf optimization note: we only need 1 layer of shade cover
                // we just update properties if needed by additional overlaid views
                if (_this.shadeCover) {
                    // overwrite current shadeCover options if topmost popupview has additional shadeCover configurations
                    toOpen.push(_this.updateShadeCover(_this.shadeCover, options.shadeCover));
                }
                else {
                    toOpen.push(_this.openShadeCover(options.shadeCover));
                }
            }
            view.opacity = 0; // always begin with view invisible when adding dynamically
            _this.insertChild(view, _this.getChildrenCount() + 1);
            toOpen.push(new Promise(function (res, rej) {
                setTimeout(function () {
                    // only apply initial state and animate after the first tick - ensures safe areas and other measurements apply correctly
                    _this.applyInitialState(view, enterAnimationDefinition);
                    _this.getEnterAnimation(view, enterAnimationDefinition)
                        .play()
                        .then(function () {
                        _this.applyDefaultState(view);
                        view.notify({ eventName: 'opened', object: view });
                        res();
                    }, function (err) {
                        rej(new Error("Error playing enter animation: ".concat(err)));
                    });
                });
            }));
            Promise.all(toOpen).then(function () {
                resolve();
            }, function (err) {
                reject(err);
            });
        });
    };
    /**
     * Ability to remove any view instance from composite views.
     * Optional animation parameter to overwrite close animation declared when opening popup.
     *
     * @param view
     * @param exitTo
     * @returns
     */
    RootLayoutBase.prototype.close = function (view, exitTo) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var _a, _b, _c, _d, _e, _f;
            if (!(view instanceof View)) {
                return reject(new Error("Invalid close view: ".concat(view)));
            }
            if (!_this.hasChild(view)) {
                return reject(new Error("Unable to close popup. ".concat(view, " not found")));
            }
            var toClose = [];
            var popupIndex = _this.getPopupIndex(view);
            var poppedView = _this.popupViews[popupIndex];
            var cleanupAndFinish = function () {
                view.notify({ eventName: 'closed', object: view });
                _this.removeChild(view);
                resolve();
            };
            // use exitAnimation that is passed in and fallback to the exitAnimation passed in when opening
            var exitAnimationDefinition = exitTo || ((_b = (_a = poppedView === null || poppedView === void 0 ? void 0 : poppedView.options) === null || _a === void 0 ? void 0 : _a.animation) === null || _b === void 0 ? void 0 : _b.exitTo);
            // Remove view from tracked popupviews
            _this.popupViews.splice(popupIndex, 1);
            toClose.push(new Promise(function (res, rej) {
                if (exitAnimationDefinition) {
                    _this.getExitAnimation(view, exitAnimationDefinition)
                        .play()
                        .then(res, function (err) {
                        rej(new Error("Error playing exit animation: ".concat(err)));
                    });
                }
                else {
                    res();
                }
            }));
            if (_this.shadeCover) {
                // Update shade cover with the topmost popupView options (if not specifically told to ignore)
                if (_this.popupViews.length) {
                    if (!((_d = (_c = poppedView === null || poppedView === void 0 ? void 0 : poppedView.options) === null || _c === void 0 ? void 0 : _c.shadeCover) === null || _d === void 0 ? void 0 : _d.ignoreShadeRestore)) {
                        var shadeCoverOptions = (_e = _this.popupViews[_this.popupViews.length - 1].options) === null || _e === void 0 ? void 0 : _e.shadeCover;
                        if (shadeCoverOptions) {
                            toClose.push(_this.updateShadeCover(_this.shadeCover, shadeCoverOptions));
                        }
                    }
                }
                else {
                    // Remove shade cover animation if this is the last opened popup view
                    toClose.push(_this.closeShadeCover((_f = poppedView === null || poppedView === void 0 ? void 0 : poppedView.options) === null || _f === void 0 ? void 0 : _f.shadeCover));
                }
            }
            Promise.all(toClose).then(function () {
                cleanupAndFinish();
            }, function (err) {
                reject(err);
            });
        });
    };
    RootLayoutBase.prototype.closeAll = function () {
        var toClose = [];
        var views = this.popupViews.map(function (popupView) { return popupView.view; });
        // Close all views at the same time and wait for all of them
        for (var _i = 0, views_1 = views; _i < views_1.length; _i++) {
            var view = views_1[_i];
            toClose.push(this.close(view));
        }
        return Promise.all(toClose);
    };
    RootLayoutBase.prototype.getShadeCover = function () {
        return this.shadeCover;
    };
    RootLayoutBase.prototype.openShadeCover = function (options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        return new Promise(function (resolve) {
            if (_this.shadeCover) {
                if (Trace.isEnabled()) {
                    Trace.write("RootLayout shadeCover already open.", Trace.categories.Layout, Trace.messageType.warn);
                }
                resolve();
            }
            else {
                // Create the one and only shade cover
                var shadeCover_1 = _this.createShadeCover();
                shadeCover_1.on('loaded', function () {
                    _this._initShadeCover(shadeCover_1, options);
                    _this.updateShadeCover(shadeCover_1, options).then(function () {
                        resolve();
                    });
                });
                _this.shadeCover = shadeCover_1;
                // Insert shade cover at index right above the first layout
                _this.insertChild(_this.shadeCover, _this.staticChildCount + 1);
            }
        });
    };
    RootLayoutBase.prototype.closeShadeCover = function (shadeCoverOptions) {
        var _this = this;
        if (shadeCoverOptions === void 0) { shadeCoverOptions = {}; }
        return new Promise(function (resolve) {
            // if shade cover is displayed and the last popup is closed, also close the shade cover
            if (_this.shadeCover) {
                return _this._closeShadeCover(_this.shadeCover, shadeCoverOptions).then(function () {
                    if (_this.shadeCover) {
                        _this.shadeCover.off('loaded');
                        if (_this.shadeCover.parent) {
                            _this.removeChild(_this.shadeCover);
                        }
                    }
                    _this.shadeCover = null;
                    // cleanup any platform specific details related to shade cover
                    _this._cleanupPlatformShadeCover();
                    resolve();
                });
            }
            resolve();
        });
    };
    RootLayoutBase.prototype.topmost = function () {
        return this.popupViews.length ? this.popupViews[this.popupViews.length - 1].view : null;
    };
    // bring any view instance open on the rootlayout to front of all the children visually
    RootLayoutBase.prototype.bringToFront = function (view, animated) {
        var _this = this;
        if (animated === void 0) { animated = false; }
        return new Promise(function (resolve, reject) {
            var _a;
            if (!(view instanceof View)) {
                return reject(new Error("Invalid bringToFront view: ".concat(view)));
            }
            if (!_this.hasChild(view)) {
                return reject(new Error("".concat(view, " not found or already at topmost")));
            }
            var popupIndex = _this.getPopupIndex(view);
            // popupview should be present and not already the topmost view
            if (popupIndex < 0 || popupIndex == _this.popupViews.length - 1) {
                return reject(new Error("".concat(view, " not found or already at topmost")));
            }
            // keep the popupViews array in sync with the stacking of the views
            var currentView = _this.popupViews[_this.getPopupIndex(view)];
            _this.popupViews.splice(_this.getPopupIndex(view), 1);
            _this.popupViews.push(currentView);
            var exitAnimation = _this.getViewExitState(view);
            if (animated && exitAnimation) {
                _this.getExitAnimation(view, exitAnimation)
                    .play()
                    .then(function () {
                    _this._bringToFront(view);
                    var initialState = _this.getViewInitialState(currentView.view);
                    if (initialState) {
                        _this.applyInitialState(view, initialState);
                        _this.getEnterAnimation(view, initialState)
                            .play()
                            .then(function () {
                            _this.applyDefaultState(view);
                        })
                            .catch(function (ex) {
                            reject(new Error("Error playing enter animation: ".concat(ex)));
                        });
                    }
                    else {
                        _this.applyDefaultState(view);
                    }
                })
                    .catch(function (ex) {
                    _this._bringToFront(view);
                    reject(new Error("Error playing exit animation: ".concat(ex)));
                });
            }
            else {
                _this._bringToFront(view);
            }
            // update shadeCover to reflect topmost's shadeCover options
            var shadeCoverOptions = (_a = currentView === null || currentView === void 0 ? void 0 : currentView.options) === null || _a === void 0 ? void 0 : _a.shadeCover;
            if (shadeCoverOptions) {
                _this.updateShadeCover(_this.shadeCover, shadeCoverOptions);
            }
            resolve();
        });
    };
    RootLayoutBase.prototype.getPopupIndex = function (view) {
        return this.popupViews.findIndex(function (popupView) { return popupView.view === view; });
    };
    RootLayoutBase.prototype.getViewInitialState = function (view) {
        var _a, _b, _c;
        var popupIndex = this.getPopupIndex(view);
        if (popupIndex === -1) {
            return;
        }
        var initialState = (_c = (_b = (_a = this.popupViews[popupIndex]) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.animation) === null || _c === void 0 ? void 0 : _c.enterFrom;
        if (!initialState) {
            return;
        }
        return initialState;
    };
    RootLayoutBase.prototype.getViewExitState = function (view) {
        var _a, _b, _c;
        var popupIndex = this.getPopupIndex(view);
        if (popupIndex === -1) {
            return;
        }
        var exitAnimation = (_c = (_b = (_a = this.popupViews[popupIndex]) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.animation) === null || _c === void 0 ? void 0 : _c.exitTo;
        if (!exitAnimation) {
            return;
        }
        return exitAnimation;
    };
    RootLayoutBase.prototype.applyInitialState = function (targetView, enterFrom) {
        var animationOptions = __assign(__assign({}, defaultTransitionAnimation), (enterFrom || {}));
        targetView.translateX = animationOptions.translateX;
        targetView.translateY = animationOptions.translateY;
        targetView.scaleX = animationOptions.scaleX;
        targetView.scaleY = animationOptions.scaleY;
        targetView.rotate = animationOptions.rotate;
        targetView.opacity = animationOptions.opacity;
    };
    RootLayoutBase.prototype.applyDefaultState = function (targetView) {
        targetView.translateX = 0;
        targetView.translateY = 0;
        targetView.scaleX = 1;
        targetView.scaleY = 1;
        targetView.rotate = 0;
        targetView.opacity = 1;
    };
    RootLayoutBase.prototype.getEnterAnimation = function (targetView, enterFrom) {
        var animationOptions = __assign(__assign({}, defaultTransitionAnimation), (enterFrom || {}));
        return new Animation([
            {
                target: targetView,
                translate: { x: 0, y: 0 },
                scale: { x: 1, y: 1 },
                rotate: 0,
                opacity: 1,
                duration: animationOptions.duration,
                curve: animationOptions.curve,
            },
        ]);
    };
    RootLayoutBase.prototype.getExitAnimation = function (targetView, exitTo) {
        return new Animation([this.getExitAnimationDefinition(targetView, exitTo)]);
    };
    RootLayoutBase.prototype.getExitAnimationDefinition = function (targetView, exitTo) {
        return __assign(__assign(__assign({ target: targetView }, defaultTransitionAnimation), (exitTo || {})), { translate: { x: isNumber(exitTo.translateX) ? exitTo.translateX : defaultTransitionAnimation.translateX, y: isNumber(exitTo.translateY) ? exitTo.translateY : defaultTransitionAnimation.translateY }, scale: { x: isNumber(exitTo.scaleX) ? exitTo.scaleX : defaultTransitionAnimation.scaleX, y: isNumber(exitTo.scaleY) ? exitTo.scaleY : defaultTransitionAnimation.scaleY } });
    };
    RootLayoutBase.prototype.createShadeCover = function () {
        var shadeCover = new GridLayout();
        shadeCover.verticalAlignment = 'bottom';
        return shadeCover;
    };
    RootLayoutBase.prototype.updateShadeCover = function (shade, shadeOptions) {
        var _this = this;
        if (shadeOptions === void 0) { shadeOptions = {}; }
        if (shadeOptions.tapToClose !== undefined && shadeOptions.tapToClose !== null) {
            shade.off('tap');
            if (shadeOptions.tapToClose) {
                shade.on('tap', function () {
                    _this.closeAll();
                });
            }
        }
        return this._updateShadeCover(shade, shadeOptions);
    };
    RootLayoutBase.prototype.hasChild = function (view) {
        return this.getChildIndex(view) >= 0;
    };
    RootLayoutBase.prototype._bringToFront = function (view) { };
    RootLayoutBase.prototype._initShadeCover = function (view, shadeOption) { };
    RootLayoutBase.prototype._updateShadeCover = function (view, shadeOption) {
        return new Promise(function () { });
    };
    RootLayoutBase.prototype._closeShadeCover = function (view, shadeOptions) {
        return new Promise(function () { });
    };
    RootLayoutBase.prototype._cleanupPlatformShadeCover = function () { };
    RootLayoutBase = __decorate([
        CSSType('RootLayout'),
        __metadata("design:paramtypes", [])
    ], RootLayoutBase);
    return RootLayoutBase;
}(GridLayout));
export { RootLayoutBase };
export function getRootLayout() {
    return global.rootLayout;
}
export var defaultTransitionAnimation = {
    translateX: 0,
    translateY: 0,
    scaleX: 1,
    scaleY: 1,
    rotate: 0,
    opacity: 1,
    duration: 300,
    curve: CoreTypes.AnimationCurve.easeIn,
};
export var defaultShadeCoverTransitionAnimation = __assign(__assign({}, defaultTransitionAnimation), { opacity: 0 });
export var defaultShadeCoverOptions = {
    opacity: 0.5,
    color: '#000000',
    tapToClose: true,
    animation: {
        enterFrom: defaultShadeCoverTransitionAnimation,
        exitTo: defaultShadeCoverTransitionAnimation,
    },
    ignoreShadeRestore: false,
};
//# sourceMappingURL=root-layout-common.js.map