import { View, CSSType } from '../core/view';
import { LayoutBase } from '../layouts/layout-base';
import { Property } from '../core/properties';
import { Trace } from '../../trace';
/**
 * Proxy view container that adds all its native children directly to the parent.
 * To be used as a logical grouping container of views.
 *
 * @nsView ProxyViewContainer
 */
// Cases to cover:
// * Child is added to the attached proxy. Handled in _addViewToNativeVisualTree.
// * Proxy (with children) is added to the DOM. In _addViewToNativeVisualTree _addViewToNativeVisualTree recursively when the proxy is added to the parent.
// * Child is removed from attached proxy. Handled in _removeViewFromNativeVisualTree.
// * Proxy (with children) is removed form the DOM. In _removeViewFromNativeVisualTree recursively when the proxy is removed from its parent.
var ProxyViewContainer = /** @class */ (function (_super) {
    __extends(ProxyViewContainer, _super);
    function ProxyViewContainer() {
        var _this = _super.call(this) || this;
        _this.proxiedLayoutProperties = new Set();
        _this.nativeViewProtected = undefined;
        return _this;
    }
    Object.defineProperty(ProxyViewContainer.prototype, "ios", {
        // No native view for proxy container.
        // @ts-ignore
        get: function () {
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ProxyViewContainer.prototype, "android", {
        // @ts-ignore
        get: function () {
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ProxyViewContainer.prototype, "isLayoutRequested", {
        // get nativeView(): any {
        //     return null;
        // }
        get: function () {
            // Always return false so all layout requests from children bubble up.
            return false;
        },
        enumerable: false,
        configurable: true
    });
    ProxyViewContainer.prototype.createNativeView = function () {
        return undefined;
    };
    ProxyViewContainer.prototype._getNativeViewsCount = function () {
        var result = 0;
        this.eachChildView(function (cv) {
            result += cv._getNativeViewsCount();
            return true;
        });
        return result;
    };
    ProxyViewContainer.prototype._eachLayoutView = function (callback) {
        this.eachChildView(function (cv) {
            if (!cv.isCollapsed) {
                cv._eachLayoutView(callback);
            }
            return true;
        });
    };
    ProxyViewContainer.prototype._setupUI = function (context, atIndex, parentIsLoaded) {
        var processChildren = false;
        if (this.reusable && this._context === context) {
            processChildren = true;
        }
        _super.prototype._setupUI.call(this, context, atIndex, parentIsLoaded);
        if (this.reusable && processChildren) {
            this.eachChild(function (child) {
                var oldReusable = child.reusable;
                child.reusable = true;
                child._setupUI(context);
                child.reusable = oldReusable;
                return true;
            });
        }
    };
    ProxyViewContainer.prototype._tearDownUI = function (force) {
        _super.prototype._tearDownUI.call(this, force);
        if (this.reusable && !force) {
            this.eachChild(function (child) {
                var oldReusable = child.reusable;
                child.reusable = true;
                child._tearDownUI();
                child.reusable = oldReusable;
                return true;
            });
        }
    };
    ProxyViewContainer.prototype._addViewToNativeVisualTree = function (child, atIndex) {
        var _this = this;
        if (Trace.isEnabled()) {
            Trace.write('ProxyViewContainer._addViewToNativeVisualTree for a child ' + child + ' ViewContainer.parent: ' + this.parent, Trace.categories.ViewHierarchy);
        }
        _super.prototype._addViewToNativeVisualTree.call(this, child);
        layoutProperties.forEach(function (propName) {
            var proxyPropName = makeProxyPropName(propName);
            child[proxyPropName] = child[propName];
            if (_this.proxiedLayoutProperties.has(propName)) {
                _this._applyLayoutPropertyToChild(child, propName, _this[propName]);
            }
        });
        var parent = this.parent;
        if (parent instanceof View) {
            var baseIndex = 0;
            var insideIndex = 0;
            if (parent instanceof LayoutBase) {
                // Get my index in parent and convert it to native index.
                baseIndex = parent._childIndexToNativeChildIndex(parent.getChildIndex(this));
            }
            if (atIndex !== undefined) {
                insideIndex = this._childIndexToNativeChildIndex(atIndex);
            }
            else {
                // Add last;
                insideIndex = this._getNativeViewsCount();
            }
            if (Trace.isEnabled()) {
                Trace.write('ProxyViewContainer._addViewToNativeVisualTree at: ' + atIndex + ' base: ' + baseIndex + ' additional: ' + insideIndex, Trace.categories.ViewHierarchy);
            }
            return parent._addViewToNativeVisualTree(child, baseIndex + insideIndex);
        }
        return false;
    };
    ProxyViewContainer.prototype._removeViewFromNativeVisualTree = function (child) {
        if (Trace.isEnabled()) {
            Trace.write('ProxyViewContainer._removeViewFromNativeVisualTree for a child ' + child + ' ViewContainer.parent: ' + this.parent, Trace.categories.ViewHierarchy);
        }
        _super.prototype._removeViewFromNativeVisualTree.call(this, child);
        var parent = this.parent;
        if (parent instanceof View) {
            return parent._removeViewFromNativeVisualTree(child);
        }
    };
    /*
     * Some layouts (e.g. GridLayout) need to get notified when adding and
     * removing children, so that they can update private measure data.
     *
     * We register our children with the parent to avoid breakage.
     */
    ProxyViewContainer.prototype._registerLayoutChild = function (child) {
        var parent = this.parent;
        if (parent instanceof LayoutBase) {
            parent._registerLayoutChild(child);
        }
    };
    ProxyViewContainer.prototype._unregisterLayoutChild = function (child) {
        var parent = this.parent;
        if (parent instanceof LayoutBase) {
            parent._unregisterLayoutChild(child);
        }
    };
    /*
     * Register/unregister existing children with the parent layout.
     */
    ProxyViewContainer.prototype._parentChanged = function (oldParent) {
        // call super in order to execute base logic like clear inherited properties, etc.
        _super.prototype._parentChanged.call(this, oldParent);
        var addingToParent = this.parent && !oldParent;
        var newLayout = this.parent;
        var oldLayout = oldParent;
        if (addingToParent && newLayout instanceof LayoutBase) {
            this.eachLayoutChild(function (child) {
                newLayout._registerLayoutChild(child);
                return true;
            });
        }
        else if (oldLayout instanceof LayoutBase) {
            this.eachLayoutChild(function (child) {
                oldLayout._unregisterLayoutChild(child);
                return true;
            });
        }
    };
    /**
     * Layout property changed, proxy the new value to the child view(s)
     */
    ProxyViewContainer.prototype._changedLayoutProperty = function (propName, value) {
        var _this = this;
        var numChildren = this._getNativeViewsCount();
        if (numChildren > 1) {
            Trace.write("ProxyViewContainer._changeLayoutProperty - you're setting '" + propName + "' for " + this + ' with more than one child. Probably this is not what you want, consider wrapping it in a StackLayout ', Trace.categories.ViewHierarchy, Trace.messageType.error);
        }
        this.eachLayoutChild(function (child) {
            _this._applyLayoutPropertyToChild(child, propName, value);
            return true;
        });
        this.proxiedLayoutProperties.add(propName);
    };
    /**
     * Apply the layout property to the child view.
     */
    ProxyViewContainer.prototype._applyLayoutPropertyToChild = function (child, propName, value) {
        var proxyPropName = makeProxyPropName(propName);
        if (proxyPropName in child) {
            if (child[propName] !== child[proxyPropName]) {
                // Value was set directly on the child view, don't override.
                if (Trace.isEnabled()) {
                    Trace.write('ProxyViewContainer._applyLayoutPropertyToChild child ' + child + ' has its own value [' + child[propName] + '] for [' + propName + ']', Trace.categories.ViewHierarchy);
                }
                return;
            }
        }
        child[propName] = value;
        child[proxyPropName] = value;
    };
    ProxyViewContainer = __decorate([
        CSSType('ProxyViewContainer'),
        __metadata("design:paramtypes", [])
    ], ProxyViewContainer);
    return ProxyViewContainer;
}(LayoutBase));
export { ProxyViewContainer };
// Layout propeties to be proxyed to the child views
var layoutProperties = [
    // AbsoluteLayout
    'left',
    'top',
    // DockLayout
    'dock',
    // FlexLayout
    'flexDirection',
    'flexWrap',
    'justifyContent',
    'alignItems',
    'alignContent',
    'order',
    'flexGrow',
    'flexShrink',
    'flexWrapBefore',
    'alignSelf',
    'flexFlow',
    'flex',
    // GridLayout
    'column',
    'columnSpan',
    'col',
    'colSpan',
    'row',
    'rowSpan',
];
var _loop_1 = function (name_1) {
    var proxyProperty = new Property({
        name: name_1,
        valueChanged: function (target, oldValue, value) {
            target._changedLayoutProperty(name_1, value);
        },
    });
    proxyProperty.register(ProxyViewContainer);
};
// Override the inherited layout properties
for (var _i = 0, layoutProperties_1 = layoutProperties; _i < layoutProperties_1.length; _i++) {
    var name_1 = layoutProperties_1[_i];
    _loop_1(name_1);
}
function makeProxyPropName(propName) {
    return "_proxy:".concat(propName);
}
//# sourceMappingURL=index.js.map