import { CssAnimationProperty } from '../core/properties';
import { KeyframeAnimationInfo } from '../animation/keyframe-animation';
import { timeConverter, animationTimingFunctionConverter } from '../styling/converters';
import { transformConverter } from '../styling/style-properties';
import { cleanupImportantFlags } from './css-utils';
var ANIMATION_PROPERTY_HANDLERS = Object.freeze({
    'animation-name': function (info, value) { return (info.name = value.replace(/['"]/g, '')); },
    'animation-duration': function (info, value) { return (info.duration = timeConverter(value)); },
    'animation-delay': function (info, value) { return (info.delay = timeConverter(value)); },
    'animation-timing-function': function (info, value) { return (info.curve = animationTimingFunctionConverter(value)); },
    'animation-iteration-count': function (info, value) { return (info.iterations = value === 'infinite' ? Number.POSITIVE_INFINITY : parseFloat(value)); },
    'animation-direction': function (info, value) { return (info.isReverse = value === 'reverse'); },
    'animation-fill-mode': function (info, value) { return (info.isForwards = value === 'forwards' || value === 'both'); },
});
var CssAnimationParser = /** @class */ (function () {
    function CssAnimationParser() {
    }
    CssAnimationParser.keyframeAnimationsFromCSSDeclarations = function (declarations) {
        if (declarations === null || declarations === undefined) {
            return undefined;
        }
        var animations = new Array();
        var animationInfo = undefined;
        declarations.forEach(function (_a) {
            var property = _a.property, value = _a.value;
            if (property === 'animation') {
                keyframeAnimationsFromCSSProperty(value, animations);
            }
            else {
                var propertyHandler = ANIMATION_PROPERTY_HANDLERS[property];
                if (propertyHandler) {
                    if (animationInfo === undefined) {
                        animationInfo = new KeyframeAnimationInfo();
                        animations.push(animationInfo);
                    }
                    propertyHandler(animationInfo, value);
                }
            }
        });
        return animations.length === 0 ? undefined : animations;
    };
    CssAnimationParser.keyframesArrayFromCSS = function (keyframes) {
        var parsedKeyframes = new Array();
        for (var _i = 0, keyframes_1 = keyframes; _i < keyframes_1.length; _i++) {
            var keyframe = keyframes_1[_i];
            var declarations = parseKeyframeDeclarations(keyframe.declarations);
            for (var _a = 0, _b = keyframe.values; _a < _b.length; _a++) {
                var time = _b[_a];
                if (time === 'from') {
                    time = 0;
                }
                else if (time === 'to') {
                    time = 1;
                }
                else {
                    time = parseFloat(time) / 100;
                    if (time < 0) {
                        time = 0;
                    }
                    if (time > 100) {
                        time = 100;
                    }
                }
                var current = parsedKeyframes[time];
                if (current === undefined) {
                    current = {};
                    current.duration = time;
                    current.declarations = [];
                    parsedKeyframes[time] = current;
                }
                for (var _c = 0, _d = keyframe.declarations; _c < _d.length; _c++) {
                    var declaration = _d[_c];
                    if (declaration.property === 'animation-timing-function') {
                        current.curve = animationTimingFunctionConverter(declaration.value);
                    }
                }
                current.declarations = current.declarations.concat(declarations);
            }
        }
        var array = [];
        for (var parsedKeyframe in parsedKeyframes) {
            array.push(parsedKeyframes[parsedKeyframe]);
        }
        array.sort(function (a, b) {
            return a.duration - b.duration;
        });
        return array;
    };
    return CssAnimationParser;
}());
export { CssAnimationParser };
/**
 * @see https://w3c.github.io/csswg-drafts/css-animations/#propdef-animation
 * @see https://developer.mozilla.org/en-US/docs/Web/CSS/animation
 * @internal - exported for testing
 * @param value
 * @param animations
 */
export function keyframeAnimationsFromCSSProperty(value, animations) {
    if (typeof value !== 'string') {
        return;
    }
    if (value.trim().length === 0) {
        return;
    }
    /**
     * Matches whitespace except if the whitespace is contained in parenthesis - ex. cubic-bezier(1, 1, 1, 1).
     */
    var VALUE_SPLIT_RE = /\s(?![^(]*\))/;
    /**
     * Matches commas except if the comma is contained in parenthesis - ex. cubic-bezier(1, 1, 1, 1).
     */
    var MULTIPLE_SPLIT_RE = /,(?![^(]*\))/;
    var isTime = function (v) { return !!v.match(/\dm?s$/g); };
    var isTimingFunction = function (v) { return !!v.match(/ease|linear|ease-in|ease-out|ease-in-out|spring|cubic-bezier/g); };
    var isIterationCount = function (v) { return !!v.match(/infinite|[\d.]+$/g); };
    var isDirection = function (v) { return !!v.match(/normal|reverse|alternate|alternate-reverse/g); };
    var isFillMode = function (v) { return !!v.match(/none|forwards|backwards|both/g); };
    var isPlayState = function (v) { return !!v.match(/running|paused/g); };
    var values = value.split(MULTIPLE_SPLIT_RE);
    var _loop_1 = function (parsedValue) {
        var animationInfo = new KeyframeAnimationInfo();
        var parts = parsedValue.trim().split(VALUE_SPLIT_RE);
        var _a = parts.filter(isTime), duration = _a[0], delay = _a[1];
        var timing = parts.filter(isTimingFunction)[0];
        var iterationCount = parts.filter(isIterationCount)[0];
        var direction = parts.filter(isDirection)[0];
        var fillMode = parts.filter(isFillMode)[0];
        var playState = parts.filter(isPlayState)[0];
        var name_1 = parts.filter(function (v) {
            // filter out "consumed" values
            return ![duration, delay, timing, iterationCount, direction, fillMode, playState].filter(Boolean).includes(v);
        })[0];
        // console.log({
        // 	duration,
        // 	delay,
        // 	timing,
        // 	iterationCount,
        // 	direction,
        // 	fillMode,
        // 	playState,
        // 	name,
        // });
        if (duration) {
            ANIMATION_PROPERTY_HANDLERS['animation-duration'](animationInfo, duration);
        }
        if (delay) {
            ANIMATION_PROPERTY_HANDLERS['animation-delay'](animationInfo, delay);
        }
        if (timing) {
            ANIMATION_PROPERTY_HANDLERS['animation-timing-function'](animationInfo, timing);
        }
        if (iterationCount) {
            ANIMATION_PROPERTY_HANDLERS['animation-iteration-count'](animationInfo, iterationCount);
        }
        if (direction) {
            ANIMATION_PROPERTY_HANDLERS['animation-direction'](animationInfo, direction);
        }
        if (fillMode) {
            ANIMATION_PROPERTY_HANDLERS['animation-fill-mode'](animationInfo, fillMode);
        }
        if (playState) {
            // TODO: implement play state? Currently not supported...
        }
        if (name_1) {
            ANIMATION_PROPERTY_HANDLERS['animation-name'](animationInfo, name_1);
        }
        else {
            // based on the SPEC we should set the name to 'none' if no name is provided
            // however we just don't set the name at all.
            // perhaps we should set it to 'none' and handle it accordingly.
            // animationInfo.name = 'none'
        }
        animations.push(animationInfo);
    };
    for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
        var parsedValue = values_1[_i];
        _loop_1(parsedValue);
    }
}
export function parseKeyframeDeclarations(unparsedKeyframeDeclarations) {
    var declarations = unparsedKeyframeDeclarations.reduce(function (declarations, _a) {
        var unparsedProperty = _a.property, unparsedValue = _a.value;
        var property = CssAnimationProperty._getByCssName(unparsedProperty);
        unparsedValue = cleanupImportantFlags(unparsedValue, property === null || property === void 0 ? void 0 : property.cssLocalName);
        if (typeof unparsedProperty === 'string' && (property === null || property === void 0 ? void 0 : property._valueConverter)) {
            declarations[property.name] = property._valueConverter(unparsedValue);
        }
        else if (unparsedProperty === 'transform') {
            var transformations = transformConverter(unparsedValue);
            Object.assign(declarations, transformations);
        }
        return declarations;
    }, {});
    return Object.keys(declarations).map(function (property) { return ({
        property: property,
        value: declarations[property],
    }); });
}
//# sourceMappingURL=css-animation-parser.js.map