import { makeValidator, makeParser } from '../core/properties';
import { Trace } from '../../trace';
var Font = /** @class */ (function () {
    function Font(fontFamily, fontSize, fontStyle, fontWeight, fontScale, fontVariationSettings) {
        this.fontFamily = fontFamily;
        this.fontSize = fontSize;
        this.fontVariationSettings = fontVariationSettings;
        this.fontStyle = fontStyle !== null && fontStyle !== void 0 ? fontStyle : FontStyle.NORMAL;
        this.fontWeight = fontWeight !== null && fontWeight !== void 0 ? fontWeight : FontWeight.NORMAL;
        this.fontScale = fontScale !== null && fontScale !== void 0 ? fontScale : 1;
    }
    Object.defineProperty(Font.prototype, "isItalic", {
        get: function () {
            return this.fontStyle === FontStyle.ITALIC;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Font.prototype, "isBold", {
        get: function () {
            return this.fontWeight === FontWeight.SEMI_BOLD || this.fontWeight === FontWeight.BOLD || this.fontWeight === '700' || this.fontWeight === FontWeight.EXTRA_BOLD || this.fontWeight === FontWeight.BLACK;
        },
        enumerable: false,
        configurable: true
    });
    Font.equals = function (value1, value2) {
        // both values are falsy
        if (!value1 && !value2) {
            return true;
        }
        // only one is falsy
        if (!value1 || !value2) {
            return false;
        }
        return value1.fontFamily === value2.fontFamily && value1.fontSize === value2.fontSize && value1.fontStyle === value2.fontStyle && value1.fontWeight === value2.fontWeight && value1.fontScale === value2.fontScale && FontVariationSettings.toString(value1.fontVariationSettings) === FontVariationSettings.toString(value2.fontVariationSettings);
    };
    Font.default = undefined;
    return Font;
}());
export { Font };
export var FontStyle;
(function (FontStyle) {
    FontStyle.NORMAL = 'normal';
    FontStyle.ITALIC = 'italic';
    FontStyle.isValid = makeValidator(FontStyle.NORMAL, FontStyle.ITALIC);
    FontStyle.parse = makeParser(FontStyle.isValid);
})(FontStyle || (FontStyle = {}));
export var FontWeight;
(function (FontWeight) {
    FontWeight.THIN = '100';
    FontWeight.EXTRA_LIGHT = '200';
    FontWeight.LIGHT = '300';
    FontWeight.NORMAL = 'normal';
    FontWeight.MEDIUM = '500';
    FontWeight.SEMI_BOLD = '600';
    FontWeight.BOLD = 'bold';
    FontWeight.EXTRA_BOLD = '800';
    FontWeight.BLACK = '900';
    FontWeight.isValid = makeValidator(FontWeight.THIN, FontWeight.EXTRA_LIGHT, FontWeight.LIGHT, FontWeight.NORMAL, '400', FontWeight.MEDIUM, FontWeight.SEMI_BOLD, FontWeight.BOLD, '700', FontWeight.EXTRA_BOLD, FontWeight.BLACK);
    FontWeight.parse = makeParser(FontWeight.isValid);
})(FontWeight || (FontWeight = {}));
export var FontVariationSettings;
(function (FontVariationSettings) {
    function parse(fontVariationSettings) {
        if (!fontVariationSettings) {
            return null;
        }
        var allowedValues = ['normal', 'inherit', 'initial', 'revert', 'revert-layer', 'unset'];
        var variationSettingsValue = fontVariationSettings.trim();
        if (allowedValues.indexOf(variationSettingsValue.toLowerCase()) !== -1) {
            return null;
        }
        var chunks = variationSettingsValue.split(',');
        if (chunks.length) {
            var parsed = [];
            for (var _i = 0, chunks_1 = chunks; _i < chunks_1.length; _i++) {
                var chunk = chunks_1[_i];
                var trimmedChunk = chunk.trim();
                var axisChunks = trimmedChunk.split(' ');
                if (axisChunks.length === 2) {
                    var axisName = axisChunks[0].trim();
                    var axisValue = parseFloat(axisChunks[1]);
                    // See https://drafts.csswg.org/css-fonts/#font-variation-settings-def.
                    // Axis name strings longer or shorter than four characters are invalid.
                    if (!isNaN(axisValue) && axisName.length === 6 && ((axisName.startsWith("'") && axisName.endsWith("'")) || (axisName.startsWith('"') && axisName.endsWith('"')))) {
                        // Remove quotes as they might cause problems when using name as an object key
                        var unquotedAxisName = axisName.substring(1, axisName.length - 1);
                        parsed.push({ axis: unquotedAxisName, value: axisValue });
                    }
                    else {
                        Trace.write('Invalid value (font-variation-settings): ' + variationSettingsValue, Trace.categories.Error, Trace.messageType.error);
                    }
                }
                else {
                    Trace.write('Invalid value (font-variation-settings): ' + variationSettingsValue, Trace.categories.Error, Trace.messageType.error);
                }
            }
            return parsed;
        }
        Trace.write('Invalid value (font-variation-settings): ' + variationSettingsValue, Trace.categories.Error, Trace.messageType.error);
    }
    FontVariationSettings.parse = parse;
    function toString(fontVariationSettings) {
        if (fontVariationSettings === null || fontVariationSettings === void 0 ? void 0 : fontVariationSettings.length) {
            return fontVariationSettings.map(function (_a) {
                var axis = _a.axis, value = _a.value;
                return "'".concat(axis, "' ").concat(value);
            }).join(', ');
        }
        return null;
    }
    FontVariationSettings.toString = toString;
})(FontVariationSettings || (FontVariationSettings = {}));
export function parseFontFamily(value) {
    if (!value) {
        return [];
    }
    return value
        .split(',')
        .map(function (v) { return (v || '').trim().replace(/['"]+/g, ''); })
        .filter(function (v) { return !!v; });
}
export var genericFontFamilies;
(function (genericFontFamilies) {
    genericFontFamilies.serif = 'serif';
    genericFontFamilies.sansSerif = 'sans-serif';
    genericFontFamilies.monospace = 'monospace';
    genericFontFamilies.system = 'system';
})(genericFontFamilies || (genericFontFamilies = {}));
var styles = new Set();
[FontStyle.NORMAL, FontStyle.ITALIC].forEach(function (val, i, a) { return styles.add(val); });
// http://www.w3schools.com/cssref/pr_font_weight.asp
//- normal(same as 400)
//- bold(same as 700)
//- 100(Thin) (API16 -thin)
//- 200(Extra Light / Ultra Light) (API16 -light)
//- 300(Light) (API16 -light)
//- 400(Normal)
//- 500(Medium) (API21 -medium)
//- 600(Semi Bold / Demi Bold) (API21 -medium)
//- 700(Bold) (API16 -bold)
//- 800(Extra Bold / Ultra Bold) (API16 -bold)
//- 900(Black / Heavy) (API21 -black)
var weights = new Set();
[FontWeight.THIN, FontWeight.EXTRA_LIGHT, FontWeight.LIGHT, FontWeight.NORMAL, '400', FontWeight.MEDIUM, FontWeight.SEMI_BOLD, FontWeight.BOLD, '700', FontWeight.EXTRA_BOLD, FontWeight.BLACK].forEach(function (val, i, a) { return weights.add(val); });
export function parseFont(fontValue) {
    var result = {
        fontStyle: 'normal',
        fontVariant: 'normal',
        fontWeight: 'normal',
    };
    var parts = fontValue.split(/\s+/);
    var part;
    while ((part = parts.shift())) {
        if (part === 'normal') {
            // nothing to do here
        }
        else if (part === 'small-caps') {
            // The only supported font variant in shorthand font
            result.fontVariant = part;
        }
        else if (styles.has(part)) {
            result.fontStyle = part;
        }
        else if (weights.has(part)) {
            result.fontWeight = part;
        }
        else if (!result.fontSize) {
            var sizes = part.split('/');
            result.fontSize = sizes[0];
            result.lineHeight = sizes.length > 1 ? sizes[1] : undefined;
        }
        else {
            result.fontFamily = part;
            if (parts.length) {
                result.fontFamily += ' ' + parts.join(' ');
            }
            break;
        }
    }
    return result;
}
/**
 * Kind of hack.
 * Used to search font variation axis names, since iOS for some reason requires names
 * but tags are the standards.
 */
export function fuzzySearch(query, dataset) {
    var q = query ? query.trim().toLowerCase() : '';
    var result = [];
    if (!q.length) {
        return null;
    }
    dataset.forEach(function (item) {
        var s = item.trim().toLowerCase();
        var n = -1;
        for (var _i = 0, q_1 = q; _i < q_1.length; _i++) {
            var char_1 = q_1[_i];
            n = s.indexOf(char_1, n + 1);
            if (!~n) {
                return;
            }
        }
        result.push(item);
    });
    return result.length ? result : null;
}
//# sourceMappingURL=font-common.js.map