import { Font as FontBase, parseFontFamily, genericFontFamilies, FontWeight, FontVariationSettings } from './font-common';
import { Trace } from '../../trace';
import { SDK_VERSION } from '../../utils/constants';
import * as fs from '../../file-system';
import { ad } from '../../utils';
export * from './font-common';
var FONTS_BASE_PATH = '/fonts/';
var typefaceCache = new Map();
var appAssets;
var Font = /** @class */ (function (_super) {
    __extends(Font, _super);
    function Font() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Font.prototype.withFontFamily = function (family) {
        return new Font(family, this.fontSize, this.fontStyle, this.fontWeight, 1, this.fontVariationSettings);
    };
    Font.prototype.withFontStyle = function (style) {
        return new Font(this.fontFamily, this.fontSize, style, this.fontWeight, 1, this.fontVariationSettings);
    };
    Font.prototype.withFontWeight = function (weight) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, weight, 1, this.fontVariationSettings);
    };
    Font.prototype.withFontSize = function (size) {
        return new Font(this.fontFamily, size, this.fontStyle, this.fontWeight, 1, this.fontVariationSettings);
    };
    Font.prototype.withFontScale = function (scale) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, this.fontWeight, 1, this.fontVariationSettings);
    };
    Font.prototype.withFontVariationSettings = function (variationSettings) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, this.fontWeight, 1, variationSettings);
    };
    Font.prototype.getAndroidTypeface = function () {
        if (!this._typeface) {
            this._typeface = createTypeface(this);
        }
        return this._typeface;
    };
    Font.prototype.getUIFont = function (defaultFont) {
        return undefined;
    };
    Font.default = new Font(undefined, undefined);
    return Font;
}(FontBase));
export { Font };
function computeFontCacheKey(fontFamily, font) {
    var sep = ':';
    return [fontFamily, String(FontVariationSettings.toString(font.fontVariationSettings)).replace(/'/g, '').replace(/[\s,]/g, '_')].join(sep);
}
function loadFontFromFile(fontFamily, font) {
    var _a;
    var cacheKey = SDK_VERSION >= 26 ? computeFontCacheKey(fontFamily, font) : fontFamily;
    appAssets = appAssets || ad.getApplicationContext().getAssets();
    if (!appAssets) {
        return null;
    }
    var result = typefaceCache.get(cacheKey);
    // Check for undefined explicitly as null mean we tried to load the font, but failed.
    if (result === undefined) {
        result = null;
        var fontAssetPath = void 0;
        var basePath = fs.path.join(fs.knownFolders.currentApp().path, 'fonts', fontFamily);
        if (fs.File.exists(basePath + '.ttf')) {
            fontAssetPath = FONTS_BASE_PATH + fontFamily + '.ttf';
        }
        else if (fs.File.exists(basePath + '.otf')) {
            fontAssetPath = FONTS_BASE_PATH + fontFamily + '.otf';
        }
        else if (Trace.isEnabled()) {
            Trace.write('Could not find font file for ' + fontFamily, Trace.categories.Error, Trace.messageType.error);
        }
        if (fontAssetPath) {
            try {
                fontAssetPath = fs.path.join(fs.knownFolders.currentApp().path, fontAssetPath);
                if (SDK_VERSION >= 26) {
                    var builder = new android.graphics.Typeface.Builder(fontAssetPath);
                    if (builder) {
                        builder.setFontVariationSettings(((_a = font.fontVariationSettings) === null || _a === void 0 ? void 0 : _a.length) ? FontVariationSettings.toString(font.fontVariationSettings) : '');
                        result = builder.build();
                    }
                    else {
                        result = android.graphics.Typeface.createFromFile(fontAssetPath);
                        if (Trace.isEnabled()) {
                            Trace.write('Could not create builder for ' + fontFamily, Trace.categories.Error, Trace.messageType.error);
                        }
                    }
                }
                else {
                    result = android.graphics.Typeface.createFromFile(fontAssetPath);
                }
            }
            catch (e) {
                if (Trace.isEnabled()) {
                    Trace.write('Error loading font asset: ' + fontAssetPath, Trace.categories.Error, Trace.messageType.error);
                }
            }
        }
        typefaceCache.set(cacheKey, result);
    }
    return result;
}
function createTypeface(font) {
    var fontStyle = 0;
    if (font.isBold) {
        fontStyle |= android.graphics.Typeface.BOLD;
    }
    if (font.isItalic) {
        fontStyle |= android.graphics.Typeface.ITALIC;
    }
    //http://stackoverflow.com/questions/19691530/valid-values-for-androidfontfamily-and-what-they-map-to
    var fontFamilies = parseFontFamily(font.fontFamily);
    var result = null;
    for (var _i = 0, fontFamilies_1 = fontFamilies; _i < fontFamilies_1.length; _i++) {
        var fontFamily = fontFamilies_1[_i];
        switch (fontFamily.toLowerCase()) {
            case genericFontFamilies.serif:
                result = android.graphics.Typeface.create('serif' + getFontWeightSuffix(font.fontWeight), fontStyle);
                break;
            case genericFontFamilies.sansSerif:
            case genericFontFamilies.system:
                result = android.graphics.Typeface.create('sans-serif' + getFontWeightSuffix(font.fontWeight), fontStyle);
                break;
            case genericFontFamilies.monospace:
                result = android.graphics.Typeface.create('monospace' + getFontWeightSuffix(font.fontWeight), fontStyle);
                break;
            default: {
                result = loadFontFromFile(fontFamily, font);
                if (result && fontStyle) {
                    result = android.graphics.Typeface.create(result, fontStyle);
                }
                break;
            }
        }
        if (result) {
            // Found the font!
            break;
        }
    }
    if (!result) {
        result = android.graphics.Typeface.create('sans-serif' + getFontWeightSuffix(font.fontWeight), fontStyle);
    }
    return result;
}
function getFontWeightSuffix(fontWeight) {
    if (typeof fontWeight === 'number') {
        fontWeight = (fontWeight + '');
    }
    switch (fontWeight) {
        case FontWeight.THIN:
            return SDK_VERSION >= 16 ? '-thin' : '';
        case FontWeight.EXTRA_LIGHT:
        case FontWeight.LIGHT:
            return SDK_VERSION >= 16 ? '-light' : '';
        case FontWeight.NORMAL:
        case '400':
        case undefined:
        case null:
            return '';
        case FontWeight.MEDIUM:
        case FontWeight.SEMI_BOLD:
            return SDK_VERSION >= 21 ? '-medium' : '';
        case FontWeight.BOLD:
        case '700':
        case FontWeight.EXTRA_BOLD:
            return '';
        case FontWeight.BLACK:
            return SDK_VERSION >= 21 ? '-black' : '';
        default:
            throw new Error("Invalid font weight: \"".concat(fontWeight, "\""));
    }
}
//# sourceMappingURL=font.android.js.map