import { Font as FontBase, parseFontFamily, FontWeight, FontVariationSettings, fuzzySearch } from './font-common';
import { Trace } from '../../trace';
import * as fs from '../../file-system';
export { FontStyle, FontWeight, FontVariationSettings, parseFont } from './font-common';
var uiFontCache = new Map();
function computeFontCacheKey(fontDescriptor) {
    var fontFamily = fontDescriptor.fontFamily, fontSize = fontDescriptor.fontSize, fontWeight = fontDescriptor.fontWeight, fontVariationSettings = fontDescriptor.fontVariationSettings, isBold = fontDescriptor.isBold, isItalic = fontDescriptor.isItalic;
    var sep = ':';
    return [fontFamily.join(sep), fontSize, fontWeight, String(FontVariationSettings.toString(fontVariationSettings)).replace(/'/g, '').replace(/[\s,]/g, '_'), isBold, isItalic].join(sep);
}
function getUIFontCached(fontDescriptor) {
    var _a;
    var cacheKey = computeFontCacheKey(fontDescriptor);
    if (uiFontCache.has(cacheKey)) {
        if (Trace.isEnabled()) {
            Trace.write("UIFont reuse from cache: ".concat(JSON.stringify(fontDescriptor), ", cache size: ").concat(uiFontCache.size), Trace.categories.Style, Trace.messageType.info);
        }
        return uiFontCache.get(cacheKey);
    }
    var uiFont = NativeScriptUtils.createUIFont(fontDescriptor);
    if ((_a = fontDescriptor.fontVariationSettings) === null || _a === void 0 ? void 0 : _a.length) {
        var font = CGFontCreateWithFontName(uiFont.fontName);
        var variationAxes = CGFontCopyVariationAxes(font);
        // This can be null if font doesn't support axes
        if (variationAxes === null || variationAxes === void 0 ? void 0 : variationAxes.count) {
            var variationSettings = NSMutableDictionary.new();
            var variationAxesCount = variationAxes.count;
            var variationAxesNames = [];
            for (var i = 0, length_1 = variationAxes.count; i < length_1; i++) {
                variationAxesNames.push(String(variationAxes[i].objectForKey('kCGFontVariationAxisName')));
            }
            for (var _i = 0, _b = fontDescriptor.fontVariationSettings; _i < _b.length; _i++) {
                var variationSetting = _b[_i];
                var axisName = fuzzySearch(variationSetting.axis, variationAxesNames);
                if (axisName === null || axisName === void 0 ? void 0 : axisName.length) {
                    variationSettings.setValueForKey(variationSetting.value, axisName[0]);
                }
            }
            font = CGFontCreateCopyWithVariations(font, variationSettings);
            uiFont = CTFontCreateWithGraphicsFont(font, fontDescriptor.fontSize, null, null);
        }
    }
    uiFontCache.set(cacheKey, uiFont);
    if (Trace.isEnabled()) {
        Trace.write("UIFont creation: ".concat(JSON.stringify(fontDescriptor), ", cache size: ").concat(uiFontCache.size), Trace.categories.Style, Trace.messageType.info);
    }
    return uiFont;
}
var Font = /** @class */ (function (_super) {
    __extends(Font, _super);
    function Font(family, size, style, weight, scale, variationSettings) {
        return _super.call(this, family, size, style, weight, scale, variationSettings) || this;
    }
    Font.prototype.withFontFamily = function (family) {
        return new Font(family, this.fontSize, this.fontStyle, this.fontWeight, this.fontScale, this.fontVariationSettings);
    };
    Font.prototype.withFontStyle = function (style) {
        return new Font(this.fontFamily, this.fontSize, style, this.fontWeight, this.fontScale, this.fontVariationSettings);
    };
    Font.prototype.withFontWeight = function (weight) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, weight, this.fontScale, this.fontVariationSettings);
    };
    Font.prototype.withFontSize = function (size) {
        return new Font(this.fontFamily, size, this.fontStyle, this.fontWeight, this.fontScale, this.fontVariationSettings);
    };
    Font.prototype.withFontScale = function (scale) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, this.fontWeight, scale, this.fontVariationSettings);
    };
    Font.prototype.withFontVariationSettings = function (variationSettings) {
        return new Font(this.fontFamily, this.fontSize, this.fontStyle, this.fontWeight, this.fontScale, variationSettings);
    };
    Font.prototype.getUIFont = function (defaultFont) {
        return getUIFontCached({
            fontFamily: parseFontFamily(this.fontFamily),
            // Apply a11y scale and calculate proper font size (avoid applying multiplier to native point size as it's messing calculations)
            fontSize: this.fontSize ? this.fontSize * this.fontScale : defaultFont.pointSize,
            fontWeight: getNativeFontWeight(this.fontWeight),
            fontVariationSettings: this.fontVariationSettings,
            isBold: this.isBold,
            isItalic: this.isItalic,
        });
    };
    Font.prototype.getAndroidTypeface = function () {
        return undefined;
    };
    Font.default = new Font(undefined, undefined);
    return Font;
}(FontBase));
export { Font };
function getNativeFontWeight(fontWeight) {
    if (typeof fontWeight === 'number') {
        fontWeight = (fontWeight + '');
    }
    switch (fontWeight) {
        case FontWeight.THIN:
            return UIFontWeightUltraLight;
        case FontWeight.EXTRA_LIGHT:
            return UIFontWeightThin;
        case FontWeight.LIGHT:
            return UIFontWeightLight;
        case FontWeight.NORMAL:
        case '400':
        case undefined:
        case null:
            return UIFontWeightRegular;
        case FontWeight.MEDIUM:
            return UIFontWeightMedium;
        case FontWeight.SEMI_BOLD:
            return UIFontWeightSemibold;
        case FontWeight.BOLD:
        case '700':
            return UIFontWeightBold;
        case FontWeight.EXTRA_BOLD:
            return UIFontWeightHeavy;
        case FontWeight.BLACK:
            return UIFontWeightBlack;
        default:
            console.log("Invalid font weight: \"".concat(fontWeight, "\""));
    }
}
export var ios;
(function (ios) {
    function registerFont(fontFile) {
        var filePath = fs.path.join(fs.knownFolders.currentApp().path, 'fonts', fontFile);
        if (!fs.File.exists(filePath)) {
            filePath = fs.path.join(fs.knownFolders.currentApp().path, fontFile);
        }
        var fontData = NSFileManager.defaultManager.contentsAtPath(filePath);
        if (!fontData) {
            throw new Error('Could not load font from: ' + fontFile);
        }
        var provider = CGDataProviderCreateWithCFData(fontData);
        var font = CGFontCreateWithDataProvider(provider);
        if (!font) {
            throw new Error('Could not load font from: ' + fontFile);
        }
        var error = new interop.Reference();
        if (!CTFontManagerRegisterGraphicsFont(font, error)) {
            if (Trace.isEnabled()) {
                Trace.write('Error occur while registering font: ' + CFErrorCopyDescription(error.value), Trace.categories.Error, Trace.messageType.error);
            }
        }
    }
    ios.registerFont = registerFont;
})(ios || (ios = {}));
function registerFontsInFolder(fontsFolderPath) {
    var fontsFolder = fs.Folder.fromPath(fontsFolderPath);
    fontsFolder.eachEntity(function (fileEntity) {
        if (fs.Folder.exists(fs.path.join(fontsFolderPath, fileEntity.name))) {
            return true;
        }
        if (fileEntity instanceof fs.File && (fileEntity.extension === '.ttf' || fileEntity.extension === '.otf')) {
            ios.registerFont(fileEntity.name);
        }
        return true;
    });
}
function registerCustomFonts() {
    var appDir = fs.knownFolders.currentApp().path;
    var fontsDir = fs.path.join(appDir, 'fonts');
    if (fs.Folder.exists(fontsDir)) {
        registerFontsInFolder(fontsDir);
    }
}
registerCustomFonts();
//# sourceMappingURL=font.ios.js.map