// Types
import { unsetValue, CssProperty, CssAnimationProperty, ShorthandProperty, InheritedCssProperty } from '../core/properties';
import { Style } from './style';
import { Color } from '../../color';
import { Font, parseFont, FontStyle, FontWeight, FontVariationSettings } from './font';
import { Background } from './background';
import { layout, hasDuplicates } from '../../utils';
import { radiansToDegrees } from '../../utils/number-utils';
import { decompose2DTransformMatrix, getTransformMatrix, matrixArrayToCssMatrix, multiplyAffine2d } from '../../matrix';
import { Trace } from '../../trace';
import { CoreTypes } from '../../core-types';
import { parseBackground } from '../../css/parser';
import { LinearGradient } from './linear-gradient';
import { parseCSSShadow } from './css-shadow';
function equalsCommon(a, b) {
    if (a == 'auto') {
        // tslint:disable-line
        return b == 'auto'; // tslint:disable-line
    }
    if (typeof a === 'number') {
        if (b == 'auto') {
            // tslint:disable-line
            return false;
        }
        if (typeof b === 'number') {
            return a == b; // tslint:disable-line
        }
        if (!b) {
            return false;
        }
        return b.unit == 'dip' && a == b.value; // tslint:disable-line
    }
    if (b == 'auto') {
        // tslint:disable-line
        return false;
    }
    if (typeof b === 'number') {
        return a ? a.unit == 'dip' && a.value == b : false; // tslint:disable-line
    }
    if (!a || !b) {
        return false;
    }
    return a.value == b.value && a.unit == b.unit; // tslint:disable-line
}
function convertToStringCommon(length) {
    if (length == 'auto') {
        // tslint:disable-line
        return 'auto';
    }
    if (typeof length === 'number') {
        return length.toString();
    }
    var val = length.value;
    if (length.unit === '%') {
        val *= 100;
    }
    return val + length.unit;
}
function toDevicePixelsCommon(length, auto, parentAvailableWidth) {
    if (auto === void 0) { auto = Number.NaN; }
    if (parentAvailableWidth === void 0) { parentAvailableWidth = Number.NaN; }
    if (length == 'auto') {
        // tslint:disable-line
        return auto;
    }
    if (typeof length === 'number') {
        return layout.round(layout.toDevicePixels(length));
    }
    if (!length) {
        return auto;
    }
    switch (length.unit) {
        case 'px':
            return layout.round(length.value);
        case '%':
            return layout.round(parentAvailableWidth * length.value);
        case 'dip':
        default:
            return layout.round(layout.toDevicePixels(length.value));
    }
}
export var PercentLength;
(function (PercentLength) {
    function parse(fromValue) {
        if (fromValue == 'auto') {
            // tslint:disable-line
            return 'auto';
        }
        if (typeof fromValue === 'string') {
            var stringValue = fromValue.trim();
            var percentIndex = stringValue.indexOf('%');
            if (percentIndex !== -1) {
                var value = void 0;
                // if only % or % is not last we treat it as invalid value.
                if (percentIndex !== stringValue.length - 1 || percentIndex === 0) {
                    value = Number.NaN;
                }
                else {
                    // Normalize result to values between -1 and 1
                    value = parseFloat(stringValue.substring(0, stringValue.length - 1).trim()) / 100;
                }
                if (isNaN(value) || !isFinite(value)) {
                    throw new Error("Invalid value: ".concat(fromValue));
                }
                return { unit: '%', value: value };
            }
            else if (stringValue.indexOf('px') !== -1) {
                stringValue = stringValue.replace('px', '').trim();
                var value = parseFloat(stringValue);
                if (isNaN(value) || !isFinite(value)) {
                    throw new Error("Invalid value: ".concat(fromValue));
                }
                return { unit: 'px', value: value };
            }
            else {
                var value = parseFloat(stringValue);
                if (isNaN(value) || !isFinite(value)) {
                    throw new Error("Invalid value: ".concat(fromValue));
                }
                return value;
            }
        }
        else {
            return fromValue;
        }
    }
    PercentLength.parse = parse;
    PercentLength.equals = equalsCommon;
    PercentLength.toDevicePixels = toDevicePixelsCommon;
    PercentLength.convertToString = convertToStringCommon;
})(PercentLength || (PercentLength = {}));
export var Length;
(function (Length) {
    function parse(fromValue) {
        if (fromValue == 'auto') {
            // tslint:disable-line
            return 'auto';
        }
        if (typeof fromValue === 'string') {
            var stringValue = fromValue.trim();
            if (stringValue.indexOf('px') !== -1) {
                stringValue = stringValue.replace('px', '').trim();
                var value = parseFloat(stringValue);
                if (isNaN(value) || !isFinite(value)) {
                    throw new Error("Invalid value: ".concat(stringValue));
                }
                return { unit: 'px', value: value };
            }
            else {
                var value = parseFloat(stringValue);
                if (isNaN(value) || !isFinite(value)) {
                    throw new Error("Invalid value: ".concat(stringValue));
                }
                return value;
            }
        }
        else {
            return fromValue;
        }
    }
    Length.parse = parse;
    Length.equals = equalsCommon;
    Length.toDevicePixels = toDevicePixelsCommon;
    Length.convertToString = convertToStringCommon;
})(Length || (Length = {}));
export var minWidthProperty = new CssProperty({
    name: 'minWidth',
    cssName: 'min-width',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectiveMinWidth = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
minWidthProperty.register(Style);
export var minHeightProperty = new CssProperty({
    name: 'minHeight',
    cssName: 'min-height',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectiveMinHeight = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
minHeightProperty.register(Style);
export var widthProperty = new CssAnimationProperty({
    name: 'width',
    cssName: 'width',
    defaultValue: 'auto',
    equalityComparer: Length.equals,
    // TODO: CSSAnimationProperty was needed for keyframe (copying other impls), but `affectsLayout` does not exist
    //       on the animation property, so fake it here. x_x
    valueChanged: function (target, oldValue, newValue) {
        if (global.isIOS) {
            var view = target.viewRef.get();
            if (view) {
                view.requestLayout();
            }
        }
    },
    valueConverter: PercentLength.parse,
});
widthProperty.register(Style);
export var heightProperty = new CssAnimationProperty({
    name: 'height',
    cssName: 'height',
    defaultValue: 'auto',
    equalityComparer: Length.equals,
    // TODO: CSSAnimationProperty was needed for keyframe (copying other impls), but `affectsLayout` does not exist
    //       on the animation property, so fake it here. -_-
    valueChanged: function (target, oldValue, newValue) {
        if (global.isIOS) {
            var view = target.viewRef.get();
            if (view) {
                view.requestLayout();
            }
        }
    },
    valueConverter: PercentLength.parse,
});
heightProperty.register(Style);
var marginProperty = new ShorthandProperty({
    name: 'margin',
    cssName: 'margin',
    getter: function () {
        if (PercentLength.equals(this.marginTop, this.marginRight) && PercentLength.equals(this.marginTop, this.marginBottom) && PercentLength.equals(this.marginTop, this.marginLeft)) {
            return this.marginTop;
        }
        return "".concat(PercentLength.convertToString(this.marginTop), " ").concat(PercentLength.convertToString(this.marginRight), " ").concat(PercentLength.convertToString(this.marginBottom), " ").concat(PercentLength.convertToString(this.marginLeft));
    },
    converter: convertToMargins,
});
marginProperty.register(Style);
export var marginLeftProperty = new CssProperty({
    name: 'marginLeft',
    cssName: 'margin-left',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueConverter: PercentLength.parse,
});
marginLeftProperty.register(Style);
export var marginRightProperty = new CssProperty({
    name: 'marginRight',
    cssName: 'margin-right',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueConverter: PercentLength.parse,
});
marginRightProperty.register(Style);
export var marginTopProperty = new CssProperty({
    name: 'marginTop',
    cssName: 'margin-top',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueConverter: PercentLength.parse,
});
marginTopProperty.register(Style);
export var marginBottomProperty = new CssProperty({
    name: 'marginBottom',
    cssName: 'margin-bottom',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueConverter: PercentLength.parse,
});
marginBottomProperty.register(Style);
var paddingProperty = new ShorthandProperty({
    name: 'padding',
    cssName: 'padding',
    getter: function () {
        if (Length.equals(this.paddingTop, this.paddingRight) && Length.equals(this.paddingTop, this.paddingBottom) && Length.equals(this.paddingTop, this.paddingLeft)) {
            return this.paddingTop;
        }
        return "".concat(Length.convertToString(this.paddingTop), " ").concat(Length.convertToString(this.paddingRight), " ").concat(Length.convertToString(this.paddingBottom), " ").concat(Length.convertToString(this.paddingLeft));
    },
    converter: convertToPaddings,
});
paddingProperty.register(Style);
export var paddingLeftProperty = new CssProperty({
    name: 'paddingLeft',
    cssName: 'padding-left',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectivePaddingLeft = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
paddingLeftProperty.register(Style);
export var paddingRightProperty = new CssProperty({
    name: 'paddingRight',
    cssName: 'padding-right',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectivePaddingRight = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
paddingRightProperty.register(Style);
export var paddingTopProperty = new CssProperty({
    name: 'paddingTop',
    cssName: 'padding-top',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectivePaddingTop = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
paddingTopProperty.register(Style);
export var paddingBottomProperty = new CssProperty({
    name: 'paddingBottom',
    cssName: 'padding-bottom',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.effectivePaddingBottom = Length.toDevicePixels(newValue, 0);
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
    valueConverter: Length.parse,
});
paddingBottomProperty.register(Style);
export var horizontalAlignmentProperty = new CssProperty({
    name: 'horizontalAlignment',
    cssName: 'horizontal-align',
    defaultValue: CoreTypes.HorizontalAlignment.stretch,
    affectsLayout: global.isIOS,
    valueConverter: CoreTypes.HorizontalAlignment.parse,
});
horizontalAlignmentProperty.register(Style);
export var verticalAlignmentProperty = new CssProperty({
    name: 'verticalAlignment',
    cssName: 'vertical-align',
    defaultValue: CoreTypes.VerticalAlignmentText.stretch,
    affectsLayout: global.isIOS,
    valueConverter: CoreTypes.VerticalAlignmentText.parse,
});
verticalAlignmentProperty.register(Style);
function parseThickness(value) {
    if (typeof value === 'string') {
        var arr = value.split(/[ ,]+/);
        var top_1;
        var right = void 0;
        var bottom = void 0;
        var left = void 0;
        if (arr.length === 1) {
            top_1 = arr[0];
            right = arr[0];
            bottom = arr[0];
            left = arr[0];
        }
        else if (arr.length === 2) {
            top_1 = arr[0];
            bottom = arr[0];
            right = arr[1];
            left = arr[1];
        }
        else if (arr.length === 3) {
            top_1 = arr[0];
            right = arr[1];
            left = arr[1];
            bottom = arr[2];
        }
        else if (arr.length === 4) {
            top_1 = arr[0];
            right = arr[1];
            bottom = arr[2];
            left = arr[3];
        }
        else {
            throw new Error('Expected 1, 2, 3 or 4 parameters. Actual: ' + value);
        }
        return {
            top: top_1,
            right: right,
            bottom: bottom,
            left: left,
        };
    }
    else {
        return value;
    }
}
function convertToMargins(value) {
    if (typeof value === 'string' && value !== 'auto') {
        var thickness = parseThickness(value);
        return [
            [marginTopProperty, PercentLength.parse(thickness.top)],
            [marginRightProperty, PercentLength.parse(thickness.right)],
            [marginBottomProperty, PercentLength.parse(thickness.bottom)],
            [marginLeftProperty, PercentLength.parse(thickness.left)],
        ];
    }
    else {
        return [
            [marginTopProperty, value],
            [marginRightProperty, value],
            [marginBottomProperty, value],
            [marginLeftProperty, value],
        ];
    }
}
function convertToPaddings(value) {
    if (typeof value === 'string' && value !== 'auto') {
        var thickness = parseThickness(value);
        return [
            [paddingTopProperty, Length.parse(thickness.top)],
            [paddingRightProperty, Length.parse(thickness.right)],
            [paddingBottomProperty, Length.parse(thickness.bottom)],
            [paddingLeftProperty, Length.parse(thickness.left)],
        ];
    }
    else {
        return [
            [paddingTopProperty, value],
            [paddingRightProperty, value],
            [paddingBottomProperty, value],
            [paddingLeftProperty, value],
        ];
    }
}
export var rotateProperty = new CssAnimationProperty({
    name: 'rotate',
    cssName: 'rotate',
    defaultValue: 0,
    valueConverter: parseFloat,
});
rotateProperty.register(Style);
export var rotateXProperty = new CssAnimationProperty({
    name: 'rotateX',
    cssName: 'rotatex',
    defaultValue: 0,
    valueConverter: parseFloat,
});
rotateXProperty.register(Style);
export var rotateYProperty = new CssAnimationProperty({
    name: 'rotateY',
    cssName: 'rotatey',
    defaultValue: 0,
    valueConverter: parseFloat,
});
rotateYProperty.register(Style);
export var perspectiveProperty = new CssAnimationProperty({
    name: 'perspective',
    cssName: 'perspective',
    defaultValue: 1000,
    valueConverter: parseFloat,
});
perspectiveProperty.register(Style);
export var scaleXProperty = new CssAnimationProperty({
    name: 'scaleX',
    cssName: 'scaleX',
    defaultValue: 1,
    valueConverter: parseFloat,
});
scaleXProperty.register(Style);
export var scaleYProperty = new CssAnimationProperty({
    name: 'scaleY',
    cssName: 'scaleY',
    defaultValue: 1,
    valueConverter: parseFloat,
});
scaleYProperty.register(Style);
function parseDIPs(value) {
    if (value.indexOf('px') !== -1) {
        return layout.toDeviceIndependentPixels(parseFloat(value.replace('px', '').trim()));
    }
    else {
        return parseFloat(value.replace('dip', '').trim());
    }
}
export var translateXProperty = new CssAnimationProperty({
    name: 'translateX',
    cssName: 'translateX',
    defaultValue: 0,
    valueConverter: parseDIPs,
});
translateXProperty.register(Style);
export var translateYProperty = new CssAnimationProperty({
    name: 'translateY',
    cssName: 'translateY',
    defaultValue: 0,
    valueConverter: parseDIPs,
});
translateYProperty.register(Style);
var transformProperty = new ShorthandProperty({
    name: 'transform',
    cssName: 'transform',
    getter: function () {
        var scaleX = this.scaleX;
        var scaleY = this.scaleY;
        var translateX = this.translateX;
        var translateY = this.translateY;
        var rotate = this.rotate;
        var rotateX = this.rotateX;
        var rotateY = this.rotateY;
        var result = '';
        if (translateX !== 0 || translateY !== 0) {
            result += "translate(".concat(translateX, ", ").concat(translateY, ") ");
        }
        if (scaleX !== 1 || scaleY !== 1) {
            result += "scale(".concat(scaleX, ", ").concat(scaleY, ") ");
        }
        if (rotateX !== 0 || rotateY !== 0 || rotate !== 0) {
            result += "rotate(".concat(rotateX, ", ").concat(rotateY, ", ").concat(rotate, ") ");
            result += "rotate (".concat(rotate, ")");
        }
        return result.trim();
    },
    converter: convertToTransform,
});
transformProperty.register(Style);
var IDENTITY_TRANSFORMATION = {
    translate: { x: 0, y: 0 },
    rotate: { x: 0, y: 0, z: 0 },
    scale: { x: 1, y: 1 },
};
var TRANSFORM_SPLITTER = new RegExp(/\s*(.+?)\((.*?)\)/g);
var TRANSFORMATIONS = Object.freeze(['rotate', 'rotateX', 'rotateY', 'rotate3d', 'translate', 'translate3d', 'translateX', 'translateY', 'scale', 'scale3d', 'scaleX', 'scaleY']);
var STYLE_TRANSFORMATION_MAP = Object.freeze({
    scale: function (value) { return ({ property: 'scale', value: value }); },
    scale3d: function (value) { return ({ property: 'scale', value: value }); },
    scaleX: function (_a) {
        var x = _a.x;
        return ({
            property: 'scale',
            value: { x: x, y: IDENTITY_TRANSFORMATION.scale.y },
        });
    },
    scaleY: function (_a) {
        var y = _a.y;
        return ({
            property: 'scale',
            value: { y: y, x: IDENTITY_TRANSFORMATION.scale.x },
        });
    },
    translate: function (value) { return ({ property: 'translate', value: value }); },
    translate3d: function (value) { return ({ property: 'translate', value: value }); },
    translateX: function (_a) {
        var x = _a.x;
        return ({
            property: 'translate',
            value: { x: x, y: IDENTITY_TRANSFORMATION.translate.y },
        });
    },
    translateY: function (_a) {
        var y = _a.y;
        return ({
            property: 'translate',
            value: { y: y, x: IDENTITY_TRANSFORMATION.translate.x },
        });
    },
    rotate3d: function (value) { return ({ property: 'rotate', value: value }); },
    rotateX: function (x) { return ({
        property: 'rotate',
        value: {
            x: x,
            y: IDENTITY_TRANSFORMATION.rotate.y,
            z: IDENTITY_TRANSFORMATION.rotate.z,
        },
    }); },
    rotateY: function (y) { return ({
        property: 'rotate',
        value: {
            x: IDENTITY_TRANSFORMATION.rotate.x,
            y: y,
            z: IDENTITY_TRANSFORMATION.rotate.z,
        },
    }); },
    rotate: function (z) { return ({
        property: 'rotate',
        value: {
            x: IDENTITY_TRANSFORMATION.rotate.x,
            y: IDENTITY_TRANSFORMATION.rotate.y,
            z: z,
        },
    }); },
});
function convertToTransform(value) {
    if (value === unsetValue) {
        value = 'none';
    }
    var _a = transformConverter(value), translate = _a.translate, rotate = _a.rotate, scale = _a.scale;
    return [
        [translateXProperty, translate.x],
        [translateYProperty, translate.y],
        [scaleXProperty, scale.x],
        [scaleYProperty, scale.y],
        [rotateProperty, rotate.z],
        [rotateXProperty, rotate.x],
        [rotateYProperty, rotate.y],
    ];
}
export function transformConverter(text) {
    var transformations = parseTransformString(text);
    if (text === 'none' || text === '' || !transformations.length) {
        return IDENTITY_TRANSFORMATION;
    }
    var usedTransforms = transformations.map(function (t) { return t.property; });
    if (!hasDuplicates(usedTransforms)) {
        var fullTransformations_1 = __assign({}, IDENTITY_TRANSFORMATION);
        transformations.forEach(function (transform) {
            fullTransformations_1[transform.property] = transform.value;
        });
        return fullTransformations_1;
    }
    var affineMatrix = transformations.map(getTransformMatrix).reduce(multiplyAffine2d);
    var cssMatrix = matrixArrayToCssMatrix(affineMatrix);
    return decompose2DTransformMatrix(cssMatrix);
}
// using general regex and manually checking the matched
// properties is faster than using more specific regex
// https://jsperf.com/cssparse
function parseTransformString(text) {
    var matches = [];
    var match;
    while ((match = TRANSFORM_SPLITTER.exec(text)) !== null) {
        var property = match[1];
        var value = convertTransformValue(property, match[2]);
        if (TRANSFORMATIONS.indexOf(property) !== -1) {
            matches.push(normalizeTransformation({ property: property, value: value }));
        }
    }
    return matches;
}
function normalizeTransformation(_a) {
    var property = _a.property, value = _a.value;
    return STYLE_TRANSFORMATION_MAP[property](value);
}
function convertTransformValue(property, stringValue) {
    // eslint-disable-next-line prefer-const
    var _a = stringValue.split(',').map(parseFloat), x = _a[0], y = _a[1], z = _a[2];
    if (property === 'translate') {
        y !== null && y !== void 0 ? y : (y = IDENTITY_TRANSFORMATION.translate.y);
    }
    else {
        y !== null && y !== void 0 ? y : (y = x);
        z !== null && z !== void 0 ? z : (z = y);
    }
    if (property === 'rotate' || property === 'rotateX' || property === 'rotateY') {
        return stringValue.slice(-3) === 'rad' ? radiansToDegrees(x) : x;
    }
    return { x: x, y: y, z: z };
}
// Background properties.
var backgroundProperty = new ShorthandProperty({
    name: 'background',
    cssName: 'background',
    getter: function () {
        return "".concat(this.backgroundColor, " ").concat(this.backgroundImage, " ").concat(this.backgroundRepeat, " ").concat(this.backgroundPosition);
    },
    converter: convertToBackgrounds,
});
backgroundProperty.register(Style);
export var backgroundInternalProperty = new CssProperty({
    name: 'backgroundInternal',
    cssName: '_backgroundInternal',
    defaultValue: Background.default,
});
backgroundInternalProperty.register(Style);
// const pattern: RegExp = /url\(('|")(.*?)\1\)/;
export var backgroundImageProperty = new CssProperty({
    name: 'backgroundImage',
    cssName: 'background-image',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withImage(newValue);
    },
    equalityComparer: function (value1, value2) {
        if (value1 instanceof LinearGradient && value2 instanceof LinearGradient) {
            return LinearGradient.equals(value1, value2);
        }
        else {
            return value1 === value2;
        }
    },
    valueConverter: function (value) {
        if (typeof value === 'string') {
            var parsed = parseBackground(value);
            if (parsed) {
                value = typeof parsed.value.image === 'object' ? LinearGradient.parse(parsed.value.image) : value;
            }
        }
        return value;
    },
});
backgroundImageProperty.register(Style);
export var backgroundColorProperty = new CssAnimationProperty({
    name: 'backgroundColor',
    cssName: 'background-color',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withColor(newValue);
    },
    equalityComparer: Color.equals,
    valueConverter: function (value) { return new Color(value); },
});
backgroundColorProperty.register(Style);
export var backgroundRepeatProperty = new CssProperty({
    name: 'backgroundRepeat',
    cssName: 'background-repeat',
    valueConverter: CoreTypes.BackgroundRepeat.parse,
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withRepeat(newValue);
    },
});
backgroundRepeatProperty.register(Style);
export var backgroundSizeProperty = new CssProperty({
    name: 'backgroundSize',
    cssName: 'background-size',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withSize(newValue);
    },
});
backgroundSizeProperty.register(Style);
export var backgroundPositionProperty = new CssProperty({
    name: 'backgroundPosition',
    cssName: 'background-position',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withPosition(newValue);
    },
});
backgroundPositionProperty.register(Style);
function convertToBackgrounds(value) {
    if (typeof value === 'string') {
        var backgrounds = parseBackground(value).value;
        var backgroundColor = unsetValue;
        if (backgrounds.color) {
            backgroundColor = backgrounds.color instanceof Color ? backgrounds.color : new Color(backgrounds.color);
        }
        var backgroundImage = void 0;
        if (typeof backgrounds.image === 'object' && backgrounds.image) {
            backgroundImage = LinearGradient.parse(backgrounds.image);
        }
        else {
            backgroundImage = backgrounds.image || unsetValue;
        }
        var backgroundRepeat = backgrounds.repeat || unsetValue;
        var backgroundPosition = backgrounds.position ? backgrounds.position.text : unsetValue;
        return [
            [backgroundColorProperty, backgroundColor],
            [backgroundImageProperty, backgroundImage],
            [backgroundRepeatProperty, backgroundRepeat],
            [backgroundPositionProperty, backgroundPosition],
        ];
    }
    else {
        return [
            [backgroundColorProperty, unsetValue],
            [backgroundImageProperty, unsetValue],
            [backgroundRepeatProperty, unsetValue],
            [backgroundPositionProperty, unsetValue],
        ];
    }
}
function parseBorderColor(value) {
    var result = {
        top: undefined,
        right: undefined,
        bottom: undefined,
        left: undefined,
    };
    if (value.indexOf('rgb') === 0 || value.indexOf('hsl') === 0) {
        result.top = result.right = result.bottom = result.left = new Color(value);
        return result;
    }
    var arr = value.split(/[ ,]+/);
    if (arr.length === 1) {
        var arr0 = new Color(arr[0]);
        result.top = arr0;
        result.right = arr0;
        result.bottom = arr0;
        result.left = arr0;
    }
    else if (arr.length === 2) {
        var arr0 = new Color(arr[0]);
        var arr1 = new Color(arr[1]);
        result.top = arr0;
        result.right = arr1;
        result.bottom = arr0;
        result.left = arr1;
    }
    else if (arr.length === 3) {
        var arr0 = new Color(arr[0]);
        var arr1 = new Color(arr[1]);
        var arr2 = new Color(arr[2]);
        result.top = arr0;
        result.right = arr1;
        result.bottom = arr2;
        result.left = arr1;
    }
    else if (arr.length === 4) {
        var arr0 = new Color(arr[0]);
        var arr1 = new Color(arr[1]);
        var arr2 = new Color(arr[2]);
        var arr3 = new Color(arr[3]);
        result.top = arr0;
        result.right = arr1;
        result.bottom = arr2;
        result.left = arr3;
    }
    else {
        throw new Error("Expected 1, 2, 3 or 4 parameters. Actual: ".concat(value));
    }
    return result;
}
// Border Color properties.
var borderColorProperty = new ShorthandProperty({
    name: 'borderColor',
    cssName: 'border-color',
    getter: function () {
        if (Color.equals(this.borderTopColor, this.borderRightColor) && Color.equals(this.borderTopColor, this.borderBottomColor) && Color.equals(this.borderTopColor, this.borderLeftColor)) {
            return this.borderTopColor;
        }
        else {
            return "".concat(this.borderTopColor, " ").concat(this.borderRightColor, " ").concat(this.borderBottomColor, " ").concat(this.borderLeftColor);
        }
    },
    converter: function (value) {
        if (typeof value === 'string') {
            var fourColors = parseBorderColor(value);
            return [
                [borderTopColorProperty, fourColors.top],
                [borderRightColorProperty, fourColors.right],
                [borderBottomColorProperty, fourColors.bottom],
                [borderLeftColorProperty, fourColors.left],
            ];
        }
        else {
            return [
                [borderTopColorProperty, value],
                [borderRightColorProperty, value],
                [borderBottomColorProperty, value],
                [borderLeftColorProperty, value],
            ];
        }
    },
});
borderColorProperty.register(Style);
export var borderTopColorProperty = new CssProperty({
    name: 'borderTopColor',
    cssName: 'border-top-color',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withBorderTopColor(newValue);
    },
    equalityComparer: Color.equals,
    valueConverter: function (value) { return new Color(value); },
});
borderTopColorProperty.register(Style);
export var borderRightColorProperty = new CssProperty({
    name: 'borderRightColor',
    cssName: 'border-right-color',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withBorderRightColor(newValue);
    },
    equalityComparer: Color.equals,
    valueConverter: function (value) { return new Color(value); },
});
borderRightColorProperty.register(Style);
export var borderBottomColorProperty = new CssProperty({
    name: 'borderBottomColor',
    cssName: 'border-bottom-color',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withBorderBottomColor(newValue);
    },
    equalityComparer: Color.equals,
    valueConverter: function (value) { return new Color(value); },
});
borderBottomColorProperty.register(Style);
export var borderLeftColorProperty = new CssProperty({
    name: 'borderLeftColor',
    cssName: 'border-left-color',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withBorderLeftColor(newValue);
    },
    equalityComparer: Color.equals,
    valueConverter: function (value) { return new Color(value); },
});
borderLeftColorProperty.register(Style);
// Border Width properties.
var borderWidthProperty = new ShorthandProperty({
    name: 'borderWidth',
    cssName: 'border-width',
    getter: function () {
        if (Length.equals(this.borderTopWidth, this.borderRightWidth) && Length.equals(this.borderTopWidth, this.borderBottomWidth) && Length.equals(this.borderTopWidth, this.borderLeftWidth)) {
            return this.borderTopWidth;
        }
        else {
            return "".concat(Length.convertToString(this.borderTopWidth), " ").concat(Length.convertToString(this.borderRightWidth), " ").concat(Length.convertToString(this.borderBottomWidth), " ").concat(Length.convertToString(this.borderLeftWidth));
        }
    },
    converter: function (value) {
        if (typeof value === 'string' && value !== 'auto') {
            var borderWidths = parseThickness(value);
            return [
                [borderTopWidthProperty, borderWidths.top],
                [borderRightWidthProperty, borderWidths.right],
                [borderBottomWidthProperty, borderWidths.bottom],
                [borderLeftWidthProperty, borderWidths.left],
            ];
        }
        else {
            return [
                [borderTopWidthProperty, value],
                [borderRightWidthProperty, value],
                [borderBottomWidthProperty, value],
                [borderLeftWidthProperty, value],
            ];
        }
    },
});
borderWidthProperty.register(Style);
export var borderTopWidthProperty = new CssProperty({
    name: 'borderTopWidth',
    cssName: 'border-top-width',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-top-width should be Non-Negative Finite number. Value: ".concat(value));
        }
        var view = target.viewRef.get();
        if (view) {
            view.effectiveBorderTopWidth = value;
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
        target.backgroundInternal = target.backgroundInternal.withBorderTopWidth(value);
    },
    valueConverter: Length.parse,
});
borderTopWidthProperty.register(Style);
export var borderRightWidthProperty = new CssProperty({
    name: 'borderRightWidth',
    cssName: 'border-right-width',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-right-width should be Non-Negative Finite number. Value: ".concat(value));
        }
        var view = target.viewRef.get();
        if (view) {
            view.effectiveBorderRightWidth = value;
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
        target.backgroundInternal = target.backgroundInternal.withBorderRightWidth(value);
    },
    valueConverter: Length.parse,
});
borderRightWidthProperty.register(Style);
export var borderBottomWidthProperty = new CssProperty({
    name: 'borderBottomWidth',
    cssName: 'border-bottom-width',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-bottom-width should be Non-Negative Finite number. Value: ".concat(value));
        }
        var view = target.viewRef.get();
        if (view) {
            view.effectiveBorderBottomWidth = value;
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
        target.backgroundInternal = target.backgroundInternal.withBorderBottomWidth(value);
    },
    valueConverter: Length.parse,
});
borderBottomWidthProperty.register(Style);
export var borderLeftWidthProperty = new CssProperty({
    name: 'borderLeftWidth',
    cssName: 'border-left-width',
    defaultValue: CoreTypes.zeroLength,
    affectsLayout: global.isIOS,
    equalityComparer: Length.equals,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-left-width should be Non-Negative Finite number. Value: ".concat(value));
        }
        var view = target.viewRef.get();
        if (view) {
            view.effectiveBorderLeftWidth = value;
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
        target.backgroundInternal = target.backgroundInternal.withBorderLeftWidth(value);
    },
    valueConverter: Length.parse,
});
borderLeftWidthProperty.register(Style);
// Border Radius properties.
var borderRadiusProperty = new ShorthandProperty({
    name: 'borderRadius',
    cssName: 'border-radius',
    getter: function () {
        if (Length.equals(this.borderTopLeftRadius, this.borderTopRightRadius) && Length.equals(this.borderTopLeftRadius, this.borderBottomRightRadius) && Length.equals(this.borderTopLeftRadius, this.borderBottomLeftRadius)) {
            return this.borderTopLeftRadius;
        }
        return "".concat(Length.convertToString(this.borderTopLeftRadius), " ").concat(Length.convertToString(this.borderTopRightRadius), " ").concat(Length.convertToString(this.borderBottomRightRadius), " ").concat(Length.convertToString(this.borderBottomLeftRadius));
    },
    converter: function (value) {
        if (typeof value === 'string') {
            var borderRadius = parseThickness(value);
            return [
                [borderTopLeftRadiusProperty, borderRadius.top],
                [borderTopRightRadiusProperty, borderRadius.right],
                [borderBottomRightRadiusProperty, borderRadius.bottom],
                [borderBottomLeftRadiusProperty, borderRadius.left],
            ];
        }
        else {
            return [
                [borderTopLeftRadiusProperty, value],
                [borderTopRightRadiusProperty, value],
                [borderBottomRightRadiusProperty, value],
                [borderBottomLeftRadiusProperty, value],
            ];
        }
    },
});
borderRadiusProperty.register(Style);
export var borderTopLeftRadiusProperty = new CssProperty({
    name: 'borderTopLeftRadius',
    cssName: 'border-top-left-radius',
    defaultValue: 0,
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-top-left-radius should be Non-Negative Finite number. Value: ".concat(value));
        }
        target.backgroundInternal = target.backgroundInternal.withBorderTopLeftRadius(value);
    },
    valueConverter: Length.parse,
    equalityComparer: Length.equals,
});
borderTopLeftRadiusProperty.register(Style);
export var borderTopRightRadiusProperty = new CssProperty({
    name: 'borderTopRightRadius',
    cssName: 'border-top-right-radius',
    defaultValue: 0,
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-top-right-radius should be Non-Negative Finite number. Value: ".concat(value));
        }
        target.backgroundInternal = target.backgroundInternal.withBorderTopRightRadius(value);
    },
    valueConverter: Length.parse,
    equalityComparer: Length.equals,
});
borderTopRightRadiusProperty.register(Style);
export var borderBottomRightRadiusProperty = new CssProperty({
    name: 'borderBottomRightRadius',
    cssName: 'border-bottom-right-radius',
    defaultValue: 0,
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-bottom-right-radius should be Non-Negative Finite number. Value: ".concat(value));
        }
        target.backgroundInternal = target.backgroundInternal.withBorderBottomRightRadius(value);
    },
    valueConverter: Length.parse,
    equalityComparer: Length.equals,
});
borderBottomRightRadiusProperty.register(Style);
export var borderBottomLeftRadiusProperty = new CssProperty({
    name: 'borderBottomLeftRadius',
    cssName: 'border-bottom-left-radius',
    defaultValue: 0,
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var value = Length.toDevicePixels(newValue, 0);
        if (!isNonNegativeFiniteNumber(value)) {
            throw new Error("border-bottom-left-radius should be Non-Negative Finite number. Value: ".concat(value));
        }
        target.backgroundInternal = target.backgroundInternal.withBorderBottomLeftRadius(value);
    },
    valueConverter: Length.parse,
    equalityComparer: Length.equals,
});
borderBottomLeftRadiusProperty.register(Style);
var boxShadowProperty = new CssProperty({
    name: 'boxShadow',
    cssName: 'box-shadow',
    valueChanged: function (target, oldValue, newValue) {
        target.backgroundInternal = target.backgroundInternal.withBoxShadow(newValue
            ? {
                inset: newValue.inset,
                offsetX: Length.toDevicePixels(newValue.offsetX, 0),
                offsetY: Length.toDevicePixels(newValue.offsetY, 0),
                blurRadius: Length.toDevicePixels(newValue.blurRadius, 0),
                spreadRadius: Length.toDevicePixels(newValue.spreadRadius, 0),
                color: newValue.color,
            }
            : null);
    },
    valueConverter: function (value) {
        return parseCSSShadow(value);
    },
});
boxShadowProperty.register(Style);
function isNonNegativeFiniteNumber(value) {
    return isFinite(value) && !isNaN(value) && value >= 0;
}
var supportedPaths = ['rect', 'circle', 'ellipse', 'polygon', 'inset'];
function isClipPathValid(value) {
    if (!value) {
        return true;
    }
    var functionName = value.substring(0, value.indexOf('(')).trim();
    return supportedPaths.indexOf(functionName) !== -1;
}
export var clipPathProperty = new CssProperty({
    name: 'clipPath',
    cssName: 'clip-path',
    valueChanged: function (target, oldValue, newValue) {
        if (!isClipPathValid(newValue)) {
            throw new Error('clip-path is not valid.');
        }
        target.backgroundInternal = target.backgroundInternal.withClipPath(newValue);
    },
});
clipPathProperty.register(Style);
function isFloatValueConverter(value) {
    var newValue = parseFloat(value);
    if (isNaN(newValue)) {
        throw new Error("Invalid value: ".concat(newValue));
    }
    return newValue;
}
export var zIndexProperty = new CssProperty({
    name: 'zIndex',
    cssName: 'z-index',
    valueConverter: isFloatValueConverter,
});
zIndexProperty.register(Style);
function opacityConverter(value) {
    var newValue = parseFloat(value);
    if (!isNaN(newValue) && 0 <= newValue && newValue <= 1) {
        return newValue;
    }
    throw new Error("Opacity should be between [0, 1]. Value: ".concat(newValue));
}
export var opacityProperty = new CssAnimationProperty({
    name: 'opacity',
    cssName: 'opacity',
    defaultValue: 1,
    valueConverter: opacityConverter,
});
opacityProperty.register(Style);
export var colorProperty = new InheritedCssProperty({
    name: 'color',
    cssName: 'color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
colorProperty.register(Style);
export var fontInternalProperty = new CssProperty({
    name: 'fontInternal',
    cssName: '_fontInternal',
});
fontInternalProperty.register(Style);
export var fontFamilyProperty = new InheritedCssProperty({
    name: 'fontFamily',
    cssName: 'font-family',
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var currentFont = target.fontInternal || Font.default;
        if (currentFont.fontFamily !== newValue) {
            var newFont = currentFont.withFontFamily(newValue);
            target.fontInternal = Font.equals(Font.default, newFont) ? unsetValue : newFont;
        }
    },
});
fontFamilyProperty.register(Style);
export var fontScaleInternalProperty = new InheritedCssProperty({
    name: 'fontScaleInternal',
    cssName: '_fontScaleInternal',
    defaultValue: 1.0,
    valueConverter: function (v) { return parseFloat(v); },
});
fontScaleInternalProperty.register(Style);
export var fontSizeProperty = new InheritedCssProperty({
    name: 'fontSize',
    cssName: 'font-size',
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        if (target.viewRef['handleFontSize'] === true) {
            return;
        }
        var currentFont = target.fontInternal || Font.default;
        if (currentFont.fontSize !== newValue) {
            var newFont = currentFont.withFontSize(newValue);
            target.fontInternal = Font.equals(Font.default, newFont) ? unsetValue : newFont;
        }
    },
    valueConverter: function (v) { return parseFloat(v); },
});
fontSizeProperty.register(Style);
export var fontStyleProperty = new InheritedCssProperty({
    name: 'fontStyle',
    cssName: 'font-style',
    affectsLayout: global.isIOS,
    defaultValue: FontStyle.NORMAL,
    valueConverter: FontStyle.parse,
    valueChanged: function (target, oldValue, newValue) {
        var currentFont = target.fontInternal || Font.default;
        if (currentFont.fontStyle !== newValue) {
            var newFont = currentFont.withFontStyle(newValue);
            target.fontInternal = Font.equals(Font.default, newFont) ? unsetValue : newFont;
        }
    },
});
fontStyleProperty.register(Style);
export var fontWeightProperty = new InheritedCssProperty({
    name: 'fontWeight',
    cssName: 'font-weight',
    affectsLayout: global.isIOS,
    defaultValue: FontWeight.NORMAL,
    valueConverter: FontWeight.parse,
    valueChanged: function (target, oldValue, newValue) {
        var currentFont = target.fontInternal || Font.default;
        if (currentFont.fontWeight !== newValue) {
            var newFont = currentFont.withFontWeight(newValue);
            target.fontInternal = Font.equals(Font.default, newFont) ? unsetValue : newFont;
        }
    },
});
fontWeightProperty.register(Style);
var fontProperty = new ShorthandProperty({
    name: 'font',
    cssName: 'font',
    getter: function () {
        return "".concat(this.fontStyle, " ").concat(this.fontWeight, " ").concat(this.fontSize, " ").concat(this.fontFamily);
    },
    converter: function (value) {
        if (value === unsetValue) {
            return [
                [fontStyleProperty, unsetValue],
                [fontWeightProperty, unsetValue],
                [fontSizeProperty, unsetValue],
                [fontFamilyProperty, unsetValue],
            ];
        }
        else {
            var font = parseFont(value);
            var fontSize = parseFloat(font.fontSize);
            return [
                [fontStyleProperty, font.fontStyle],
                [fontWeightProperty, font.fontWeight],
                [fontSizeProperty, fontSize],
                [fontFamilyProperty, font.fontFamily],
            ];
        }
    },
});
fontProperty.register(Style);
export var fontVariationSettingsProperty = new InheritedCssProperty({
    name: 'fontVariationSettings',
    cssName: 'font-variation-settings',
    affectsLayout: global.isIOS,
    valueChanged: function (target, oldValue, newValue) {
        var currentFont = target.fontInternal || Font.default;
        if (currentFont.fontVariationSettings !== newValue) {
            var newFont = currentFont.withFontVariationSettings(newValue);
            target.fontInternal = Font.equals(Font.default, newFont) ? unsetValue : newFont;
        }
    },
    valueConverter: function (value) {
        return FontVariationSettings.parse(value);
    },
});
fontVariationSettingsProperty.register(Style);
export var visibilityProperty = new CssProperty({
    name: 'visibility',
    cssName: 'visibility',
    defaultValue: CoreTypes.Visibility.visible,
    affectsLayout: global.isIOS,
    valueConverter: CoreTypes.Visibility.parse,
    valueChanged: function (target, oldValue, newValue) {
        var view = target.viewRef.get();
        if (view) {
            view.isCollapsed = newValue === CoreTypes.Visibility.collapse;
        }
        else {
            Trace.write("".concat(newValue, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
        }
    },
});
visibilityProperty.register(Style);
export var androidElevationProperty = new CssProperty({
    name: 'androidElevation',
    cssName: 'android-elevation',
    valueConverter: parseFloat,
});
androidElevationProperty.register(Style);
export var androidDynamicElevationOffsetProperty = new CssProperty({
    name: 'androidDynamicElevationOffset',
    cssName: 'android-dynamic-elevation-offset',
    valueConverter: parseFloat,
});
androidDynamicElevationOffsetProperty.register(Style);
//# sourceMappingURL=style-properties.js.map