import { unsetValue, _evaluateCssVariableExpression, _evaluateCssCalcExpression, isCssVariable, isCssVariableExpression, isCssCalcExpression } from '../core/properties';
import { StyleSheetSelectorScope, SelectorsMatch, fromAstNode, MEDIA_QUERY_SEPARATOR, matchMediaQueryString } from './css-selector';
import { Trace } from '../../trace';
import { File, knownFolders, path } from '../../file-system';
import { Application } from '../../application';
import { profile } from '../../profiling';
var keyframeAnimationModule;
function ensureKeyframeAnimationModule() {
    if (!keyframeAnimationModule) {
        keyframeAnimationModule = require('../animation/keyframe-animation');
    }
}
import { sanitizeModuleName } from '../../utils/common';
import { resolveModuleName } from '../../module-name-resolver';
import { cleanupImportantFlags } from './css-utils';
var cssAnimationParserModule;
function ensureCssAnimationParserModule() {
    if (!cssAnimationParserModule) {
        cssAnimationParserModule = require('./css-animation-parser');
    }
}
var parser = 'css-tree';
try {
    var appConfig = require('~/package.json');
    if (appConfig) {
        if (appConfig.cssParser === 'rework') {
            parser = 'rework';
        }
        else if (appConfig.cssParser === 'nativescript') {
            parser = 'nativescript';
        }
    }
}
catch (e) {
    //
}
var mergedApplicationCssSelectors = [];
var applicationCssSelectors = [];
var applicationAdditionalSelectors = [];
var mergedApplicationCssKeyframes = [];
var applicationCssKeyframes = [];
var applicationAdditionalKeyframes = [];
var applicationCssSelectorVersion = 0;
var tagToScopeTag = new Map();
var currentScopeTag = null;
var animationsSymbol = Symbol('animations');
var kebabCasePattern = /-([a-z])/g;
var pattern = /('|")(.*?)\1/;
/**
 * Evaluate css-variable and css-calc expressions
 */
function evaluateCssExpressions(view, property, value) {
    var newValue = _evaluateCssVariableExpression(view, property, value);
    if (newValue === 'unset') {
        return unsetValue;
    }
    value = newValue;
    try {
        value = _evaluateCssCalcExpression(value);
    }
    catch (e) {
        Trace.write("Failed to evaluate css-calc for property [".concat(property, "] for expression [").concat(value, "] to ").concat(view, ". ").concat(e.stack), Trace.categories.Error, Trace.messageType.error);
        return unsetValue;
    }
    return value;
}
export function mergeCssSelectors() {
    mergedApplicationCssSelectors = applicationCssSelectors.slice();
    mergedApplicationCssSelectors.push.apply(mergedApplicationCssSelectors, applicationAdditionalSelectors);
}
export function mergeCssKeyframes() {
    mergedApplicationCssKeyframes = applicationCssKeyframes.slice();
    mergedApplicationCssKeyframes.push.apply(mergedApplicationCssKeyframes, applicationAdditionalKeyframes);
}
var CSSSource = /** @class */ (function () {
    function CSSSource(_ast, _url, _file, _source) {
        this._ast = _ast;
        this._url = _url;
        this._file = _file;
        this._source = _source;
        this._selectors = [];
        this._keyframes = [];
        this.parse();
    }
    CSSSource.fromDetect = function (cssOrAst, fileName) {
        if (typeof cssOrAst === 'string') {
            // raw-loader
            return CSSSource.fromSource(cssOrAst, fileName);
        }
        else if (typeof cssOrAst === 'object') {
            if (cssOrAst.default) {
                cssOrAst = cssOrAst.default;
            }
            if (cssOrAst.type === 'stylesheet' && cssOrAst.stylesheet && cssOrAst.stylesheet.rules) {
                // css-loader
                return CSSSource.fromAST(cssOrAst, fileName);
            }
        }
        // css2json-loader
        return CSSSource.fromSource(cssOrAst.toString(), fileName);
    };
    CSSSource.fromURI = function (uri) {
        // webpack modules require all file paths to be relative to /app folder
        var appRelativeUri = CSSSource.pathRelativeToApp(uri);
        var sanitizedModuleName = sanitizeModuleName(appRelativeUri);
        var resolvedModuleName = resolveModuleName(sanitizedModuleName, 'css');
        try {
            var cssOrAst = global.loadModule(resolvedModuleName, true);
            if (cssOrAst) {
                return CSSSource.fromDetect(cssOrAst, resolvedModuleName);
            }
        }
        catch (e) {
            if (Trace.isEnabled()) {
                Trace.write("Could not load CSS from ".concat(uri, ": ").concat(e), Trace.categories.Error, Trace.messageType.warn);
            }
        }
        return CSSSource.fromFile(appRelativeUri);
    };
    CSSSource.pathRelativeToApp = function (uri) {
        if (!uri.startsWith('/')) {
            return uri;
        }
        var appPath = knownFolders.currentApp().path;
        if (!uri.startsWith(appPath)) {
            Trace.write("".concat(uri, " does not start with ").concat(appPath), Trace.categories.Error, Trace.messageType.error);
            return uri;
        }
        var relativeUri = ".".concat(uri.substring(appPath.length));
        return relativeUri;
    };
    CSSSource.fromFile = function (url) {
        // .scss, .sass, etc. css files in vanilla app are usually compiled to .css so we will try to load a compiled file first.
        var cssFileUrl = url.replace(/\..\w+$/, '.css');
        if (cssFileUrl !== url) {
            var cssFile = CSSSource.resolveCSSPathFromURL(cssFileUrl);
            if (cssFile) {
                return new CSSSource(undefined, url, cssFile, undefined);
            }
        }
        var file = CSSSource.resolveCSSPathFromURL(url);
        return new CSSSource(undefined, url, file, undefined);
    };
    CSSSource.fromFileImport = function (url, importSource) {
        var file = CSSSource.resolveCSSPathFromURL(url, importSource);
        return new CSSSource(undefined, url, file, undefined);
    };
    CSSSource.resolveCSSPathFromURL = function (url, importSource) {
        var app = knownFolders.currentApp().path;
        var file = resolveFileNameFromUrl(url, app, File.exists, importSource);
        return file;
    };
    CSSSource.fromSource = function (source, url) {
        return new CSSSource(undefined, url, undefined, source);
    };
    CSSSource.fromAST = function (ast, url) {
        return new CSSSource(ast, url, undefined, undefined);
    };
    Object.defineProperty(CSSSource.prototype, "selectors", {
        get: function () {
            return this._selectors;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CSSSource.prototype, "keyframes", {
        get: function () {
            return this._keyframes;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CSSSource.prototype, "source", {
        get: function () {
            return this._source;
        },
        enumerable: false,
        configurable: true
    });
    CSSSource.prototype.load = function () {
        var file = File.fromPath(this._file);
        this._source = file.readTextSync();
    };
    CSSSource.prototype.parse = function () {
        try {
            if (!this._ast) {
                if (!this._source && this._file) {
                    this.load();
                }
                // [object Object] check guards against empty app.css file
                if (this._source && this.source !== '[object Object]') {
                    this.parseCSSAst();
                }
            }
            if (this._ast) {
                this.createSelectorsAndKeyframes();
            }
            else {
                this._selectors = [];
            }
        }
        catch (e) {
            if (Trace.isEnabled()) {
                Trace.write('Css styling failed: ' + e, Trace.categories.Style, Trace.messageType.error);
            }
            this._selectors = [];
        }
    };
    CSSSource.prototype.parseCSSAst = function () {
        if (this._source) {
            if (__CSS_PARSER__ === 'css-tree') {
                var cssTreeParse = require('../../css/css-tree-parser').cssTreeParse;
                this._ast = cssTreeParse(this._source, this._file);
            }
            else if (__CSS_PARSER__ === 'nativescript') {
                var CSS3Parser = require('../../css/CSS3Parser').CSS3Parser;
                var CSSNativeScript = require('../../css/CSSNativeScript').CSSNativeScript;
                var cssparser = new CSS3Parser(this._source);
                var stylesheet = cssparser.parseAStylesheet();
                var cssNS = new CSSNativeScript();
                this._ast = cssNS.parseStylesheet(stylesheet);
            }
            else if (__CSS_PARSER__ === 'rework') {
                var parseCss = require('../../css').parse;
                this._ast = parseCss(this._source, { source: this._file });
            }
        }
    };
    CSSSource.prototype.createSelectorsAndKeyframes = function () {
        if (this._ast) {
            var nodes = this._ast.stylesheet.rules;
            var rulesets = [];
            var keyframes = [];
            // When css2json-loader is enabled, imports are handled there and removed from AST rules
            populateRulesFromImports(nodes, rulesets, keyframes);
            _populateRules(nodes, rulesets, keyframes);
            if (rulesets && rulesets.length) {
                ensureCssAnimationParserModule();
                rulesets.forEach(function (rule) {
                    rule[animationsSymbol] = cssAnimationParserModule.CssAnimationParser.keyframeAnimationsFromCSSDeclarations(rule.declarations);
                });
            }
            this._selectors = rulesets;
            this._keyframes = keyframes;
        }
    };
    CSSSource.prototype.toString = function () {
        return this._file || this._url || '(in-memory)';
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CSSSource.prototype, "load", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CSSSource.prototype, "parse", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CSSSource.prototype, "parseCSSAst", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CSSSource.prototype, "createSelectorsAndKeyframes", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String, String]),
        __metadata("design:returntype", String)
    ], CSSSource, "resolveCSSPathFromURL", null);
    return CSSSource;
}());
function populateRulesFromImports(nodes, rulesets, keyframes) {
    var imports = nodes.filter(function (r) { return r.type === 'import'; });
    if (!imports.length) {
        return;
    }
    var urlFromImportObject = function (importObject) {
        var importItem = importObject['import'];
        var urlMatch = importItem && importItem.match(pattern);
        return urlMatch && urlMatch[2];
    };
    var sourceFromImportObject = function (importObject) { return importObject['position'] && importObject['position']['source']; };
    var toUrlSourcePair = function (importObject) { return ({
        url: urlFromImportObject(importObject),
        source: sourceFromImportObject(importObject),
    }); };
    var getCssFile = function (_a) {
        var url = _a.url, source = _a.source;
        return (source ? CSSSource.fromFileImport(url, source) : CSSSource.fromURI(url));
    };
    var cssFiles = imports
        .map(toUrlSourcePair)
        .filter(function (_a) {
        var url = _a.url;
        return !!url;
    })
        .map(getCssFile);
    for (var _i = 0, cssFiles_1 = cssFiles; _i < cssFiles_1.length; _i++) {
        var cssFile = cssFiles_1[_i];
        if (cssFile) {
            rulesets.push.apply(rulesets, cssFile.selectors);
            keyframes.push.apply(keyframes, cssFile.keyframes);
        }
    }
}
export function _populateRules(nodes, rulesets, keyframes, mediaQueryString) {
    for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {
        var node = nodes_1[_i];
        if (isKeyframe(node)) {
            var keyframeRule = {
                name: node.name,
                keyframes: node.keyframes,
                mediaQueryString: mediaQueryString,
            };
            keyframes.push(keyframeRule);
        }
        else if (isMedia(node)) {
            // Media query is composite in the case of nested media queries
            var compositeMediaQuery = mediaQueryString ? mediaQueryString + MEDIA_QUERY_SEPARATOR + node.media : node.media;
            _populateRules(node.rules, rulesets, keyframes, compositeMediaQuery);
        }
        else if (isRule(node)) {
            var ruleset = fromAstNode(node);
            ruleset.mediaQueryString = mediaQueryString;
            rulesets.push(ruleset);
        }
    }
}
export function removeTaggedAdditionalCSS(tag) {
    var selectorsChanged = false;
    var keyframesChanged = false;
    var updated = false;
    for (var i = 0; i < applicationAdditionalSelectors.length; i++) {
        if (applicationAdditionalSelectors[i].tag === tag) {
            applicationAdditionalSelectors.splice(i, 1);
            i--;
            selectorsChanged = true;
        }
    }
    for (var i = 0; i < applicationAdditionalKeyframes.length; i++) {
        if (applicationAdditionalKeyframes[i].tag === tag) {
            applicationAdditionalKeyframes.splice(i, 1);
            i--;
            keyframesChanged = true;
        }
    }
    if (selectorsChanged) {
        mergeCssSelectors();
        updated = true;
    }
    if (keyframesChanged) {
        mergeCssKeyframes();
        updated = true;
    }
    if (updated) {
        applicationCssSelectorVersion++;
    }
    return updated;
}
export function addTaggedAdditionalCSS(cssText, tag) {
    var _a = CSSSource.fromDetect(cssText, undefined), selectors = _a.selectors, keyframes = _a.keyframes;
    var tagScope = currentScopeTag || (tag && tagToScopeTag.has(tag) && tagToScopeTag.get(tag)) || null;
    if (tagScope && tag) {
        tagToScopeTag.set(tag, tagScope);
    }
    var selectorsChanged = false;
    var keyframesChanged = false;
    var updated = false;
    if (selectors && selectors.length) {
        selectorsChanged = true;
        if (tag != null || tagScope != null) {
            for (var i = 0, length_1 = selectors.length; i < length_1; i++) {
                selectors[i].tag = tag;
                selectors[i].scopedTag = tagScope;
            }
        }
        applicationAdditionalSelectors.push.apply(applicationAdditionalSelectors, selectors);
        mergeCssSelectors();
        updated = true;
    }
    if (keyframes && keyframes.length) {
        keyframesChanged = true;
        if (tag != null || tagScope != null) {
            for (var i = 0, length_2 = keyframes.length; i < length_2; i++) {
                keyframes[i].tag = tag;
                keyframes[i].scopedTag = tagScope;
            }
        }
        applicationAdditionalKeyframes.push.apply(applicationAdditionalKeyframes, keyframes);
        mergeCssKeyframes();
        updated = true;
    }
    if (updated) {
        applicationCssSelectorVersion++;
    }
    return updated;
}
var onCssChanged = profile('"style-scope".onCssChanged', function (args) {
    if (args.cssText) {
        var _a = CSSSource.fromSource(args.cssText, args.cssFile), selectors = _a.selectors, keyframes = _a.keyframes;
        var updated = false;
        if (selectors) {
            applicationAdditionalSelectors.push.apply(applicationAdditionalSelectors, selectors);
            mergeCssSelectors();
            updated = true;
        }
        if (keyframes) {
            applicationAdditionalKeyframes.push.apply(applicationAdditionalKeyframes, keyframes);
            mergeCssKeyframes();
            updated = true;
        }
        if (updated) {
            applicationCssSelectorVersion++;
        }
    }
    else if (args.cssFile) {
        loadCss(args.cssFile, null, null);
    }
});
function onLiveSync(args) {
    loadCss(Application.getCssFileName(), null, null);
}
var loadCss = profile("\"style-scope\".loadCss", function (cssModule) {
    if (!cssModule) {
        return;
    }
    // safely remove "./" as global CSS should be resolved relative to app folder
    if (cssModule.startsWith('./')) {
        cssModule = cssModule.substring(2);
    }
    var _a = CSSSource.fromURI(cssModule), selectors = _a.selectors, keyframes = _a.keyframes;
    var updated = false;
    // Check for existing application css selectors too in case the app is undergoing a live-sync
    if (selectors.length > 0 || applicationCssSelectors.length > 0) {
        applicationCssSelectors = selectors;
        mergeCssSelectors();
        updated = true;
    }
    // Check for existing application css keyframes too in case the app is undergoing a live-sync
    if (keyframes.length > 0 || applicationCssKeyframes.length > 0) {
        applicationCssKeyframes = keyframes;
        mergeCssKeyframes();
        updated = true;
    }
    if (updated) {
        applicationCssSelectorVersion++;
    }
});
global.NativeScriptGlobals.events.on('cssChanged', onCssChanged);
global.NativeScriptGlobals.events.on('livesync', onLiveSync);
// Call to this method is injected in the application in:
//  - no-snapshot - code injected in app.ts by [bundle-config-loader](https://github.com/NativeScript/nativescript-dev-webpack/blob/9b1e34d8ef838006c9b575285c42d2304f5f02b5/bundle-config-loader.ts#L85-L92)
//  - with-snapshot - code injected in snapshot bundle by [NativeScriptSnapshotPlugin](https://github.com/NativeScript/nativescript-dev-webpack/blob/48b26f412fd70c19dc0b9c7763e08e9505a0ae11/plugins/NativeScriptSnapshotPlugin/index.js#L48-L56)
// Having the app.css loaded in snapshot provides significant boost in startup (when using the ns-theme ~150 ms). However, because app.css is resolved at build-time,
// when the snapshot is created - there is no way to use file qualifiers or change the name of on app.css
export var loadAppCSS = profile('"style-scope".loadAppCSS', function (args) {
    loadCss(args.cssFile, null, null);
    global.NativeScriptGlobals.events.off('loadAppCss', loadAppCSS);
});
if (Application.hasLaunched()) {
    loadAppCSS({
        eventName: 'loadAppCss',
        object: Application,
        cssFile: Application.getCssFileName(),
    }, null, null);
}
else {
    global.NativeScriptGlobals.events.on('loadAppCss', loadAppCSS);
}
var CssState = /** @class */ (function () {
    function CssState(viewRef) {
        var _this = this;
        this.viewRef = viewRef;
        this._appliedPropertyValues = CssState.emptyPropertyBag;
        this._onDynamicStateChangeHandler = function () { return _this.updateDynamicState(); };
    }
    /**
     * Called when a change had occurred that may invalidate the statically matching selectors (class, id, ancestor selectors).
     * As a result, at some point in time, the selectors matched have to be requerried from the style scope and applied to the view.
     */
    CssState.prototype.onChange = function () {
        var view = this.viewRef.get();
        if (view && view.isLoaded) {
            this.unsubscribeFromDynamicUpdates();
            this.updateMatch();
            this.subscribeForDynamicUpdates();
            this.updateDynamicState();
        }
        else {
            this._matchInvalid = true;
        }
    };
    CssState.prototype.isSelectorsLatestVersionApplied = function () {
        var view = this.viewRef.get();
        if (!view) {
            Trace.write("isSelectorsLatestVersionApplied returns default value \"false\" because \"this.viewRef\" cleared.", Trace.categories.Style, Trace.messageType.warn);
            return false;
        }
        return this.viewRef.get()._styleScope.getSelectorsVersion() === this._appliedSelectorsVersion;
    };
    CssState.prototype.onLoaded = function () {
        if (this._matchInvalid) {
            this.updateMatch();
        }
        this.subscribeForDynamicUpdates();
        this.updateDynamicState();
    };
    CssState.prototype.onUnloaded = function () {
        this.unsubscribeFromDynamicUpdates();
        this.stopKeyframeAnimations();
    };
    CssState.prototype.updateMatch = function () {
        var _a;
        var view = this.viewRef.get();
        if (view && view._styleScope) {
            this._match = (_a = view._styleScope.matchSelectors(view)) !== null && _a !== void 0 ? _a : CssState.emptyMatch;
            this._appliedSelectorsVersion = view._styleScope.getSelectorsVersion();
        }
        else {
            this._match = CssState.emptyMatch;
        }
        this._matchInvalid = false;
    };
    CssState.prototype.updateDynamicState = function () {
        var _this = this;
        var view = this.viewRef.get();
        if (!view) {
            Trace.write("updateDynamicState not executed to view because \".viewRef\" is cleared", Trace.categories.Style, Trace.messageType.warn);
            return;
        }
        var matchingSelectors = this._match.selectors.filter(function (sel) { return (sel.dynamic ? sel.match(view) : true); });
        // Ideally we should return here if there are no matching selectors, however
        // if there are property removals, returning here would not remove them
        // this is seen in STYLE test in automated.
        // if (!matchingSelectors || matchingSelectors.length === 0) {
        // 		return;
        // }
        view._batchUpdate(function () {
            _this.stopKeyframeAnimations();
            _this.setPropertyValues(matchingSelectors);
            _this.playKeyframeAnimations(matchingSelectors);
        });
    };
    CssState.prototype.playKeyframeAnimations = function (matchingSelectors) {
        var animations = [];
        matchingSelectors.forEach(function (selector) {
            var _a;
            var ruleAnimations = (_a = selector.ruleset) === null || _a === void 0 ? void 0 : _a[animationsSymbol];
            if (ruleAnimations) {
                ensureKeyframeAnimationModule();
                for (var _i = 0, ruleAnimations_1 = ruleAnimations; _i < ruleAnimations_1.length; _i++) {
                    var animationInfo = ruleAnimations_1[_i];
                    var animation = keyframeAnimationModule.KeyframeAnimation.keyframeAnimationFromInfo(animationInfo);
                    if (animation) {
                        animations.push(animation);
                    }
                }
            }
        });
        if ((this._playsKeyframeAnimations = animations.length > 0)) {
            var view_1 = this.viewRef.get();
            if (!view_1) {
                Trace.write("KeyframeAnimations cannot play because \".viewRef\" is cleared", Trace.categories.Animation, Trace.messageType.warn);
                return;
            }
            animations.map(function (animation) { return animation.play(view_1); });
            Object.freeze(animations);
            this._appliedAnimations = animations;
        }
    };
    CssState.prototype.stopKeyframeAnimations = function () {
        if (!this._playsKeyframeAnimations) {
            return;
        }
        this._appliedAnimations.filter(function (animation) { return animation.isPlaying; }).forEach(function (animation) { return animation.cancel(); });
        this._appliedAnimations = CssState.emptyAnimationArray;
        var view = this.viewRef.get();
        if (view) {
            view.style['keyframe:rotate'] = unsetValue;
            view.style['keyframe:rotateX'] = unsetValue;
            view.style['keyframe:rotateY'] = unsetValue;
            view.style['keyframe:scaleX'] = unsetValue;
            view.style['keyframe:scaleY'] = unsetValue;
            view.style['keyframe:translateX'] = unsetValue;
            view.style['keyframe:translateY'] = unsetValue;
            view.style['keyframe:backgroundColor'] = unsetValue;
            view.style['keyframe:opacity'] = unsetValue;
        }
        else {
            Trace.write("KeyframeAnimations cannot be stopped because \".viewRef\" is cleared", Trace.categories.Animation, Trace.messageType.warn);
        }
        this._playsKeyframeAnimations = false;
    };
    /**
     * Calculate the difference between the previously applied property values,
     * and the new set of property values that have to be applied for the provided selectors.
     * Apply the values and ensure each property setter is called at most once to avoid excessive change notifications.
     * @param matchingSelectors
     */
    CssState.prototype.setPropertyValues = function (matchingSelectors) {
        var view = this.viewRef.get();
        if (!view) {
            Trace.write("".concat(matchingSelectors, " not set to view's property because \".viewRef\" is cleared"), Trace.categories.Style, Trace.messageType.warn);
            return;
        }
        var newPropertyValues = new view.style.PropertyBag();
        matchingSelectors.forEach(function (selector) { return selector.ruleset.declarations.forEach(function (declaration) { return (newPropertyValues[declaration.property] = declaration.value); }); });
        var oldProperties = this._appliedPropertyValues;
        // Update values for the scope's css-variables
        view.style.resetScopedCssVariables();
        var valuesToApply = {};
        var cssExpsProperties = {};
        var replacementFunc = function (g) { return g[1].toUpperCase(); };
        for (var property in newPropertyValues) {
            var value = cleanupImportantFlags(newPropertyValues[property], property);
            var isCssExp = isCssVariableExpression(value) || isCssCalcExpression(value);
            if (isCssExp) {
                // we handle css exp separately because css vars must be evaluated first
                cssExpsProperties[property] = value;
                continue;
            }
            delete oldProperties[property];
            if (property in oldProperties && oldProperties[property] === value) {
                // Skip unchanged values
                continue;
            }
            if (isCssVariable(property)) {
                view.style.setScopedCssVariable(property, value);
                delete newPropertyValues[property];
                continue;
            }
            valuesToApply[property] = value;
        }
        //we need to parse CSS vars first before evaluating css expressions
        for (var property in cssExpsProperties) {
            delete oldProperties[property];
            var value = evaluateCssExpressions(view, property, cssExpsProperties[property]);
            if (property in oldProperties && oldProperties[property] === value) {
                // Skip unchanged values
                continue;
            }
            if (value === unsetValue) {
                delete newPropertyValues[property];
            }
            if (isCssVariable(property)) {
                view.style.setScopedCssVariable(property, value);
                delete newPropertyValues[property];
            }
            valuesToApply[property] = value;
        }
        // Unset removed values
        for (var property in oldProperties) {
            if (property in view.style) {
                view.style["css:".concat(property)] = unsetValue;
            }
            else {
                var camelCasedProperty = property.replace(kebabCasePattern, replacementFunc);
                view[camelCasedProperty] = unsetValue;
            }
        }
        // Set new values to the style
        for (var property in valuesToApply) {
            var value = valuesToApply[property];
            try {
                if (property in view.style) {
                    view.style["css:".concat(property)] = value;
                }
                else {
                    var camelCasedProperty = property.replace(kebabCasePattern, replacementFunc);
                    view[camelCasedProperty] = value;
                }
            }
            catch (e) {
                Trace.write("Failed to apply property [".concat(property, "] with value [").concat(value, "] to ").concat(view, ". ").concat(e.stack), Trace.categories.Error, Trace.messageType.error);
            }
        }
        this._appliedPropertyValues = newPropertyValues;
    };
    CssState.prototype.subscribeForDynamicUpdates = function () {
        var _this = this;
        var changeMap = this._match.changeMap;
        changeMap.forEach(function (changes, view) {
            if (changes.attributes) {
                changes.attributes.forEach(function (attribute) {
                    view.addEventListener(attribute + 'Change', _this._onDynamicStateChangeHandler);
                });
            }
            if (changes.pseudoClasses) {
                changes.pseudoClasses.forEach(function (pseudoClass) {
                    var eventName = ':' + pseudoClass;
                    view.addEventListener(':' + pseudoClass, _this._onDynamicStateChangeHandler);
                    if (view[eventName]) {
                        view[eventName](+1);
                    }
                });
            }
        });
        this._appliedChangeMap = changeMap;
    };
    CssState.prototype.unsubscribeFromDynamicUpdates = function () {
        var _this = this;
        this._appliedChangeMap.forEach(function (changes, view) {
            if (changes.attributes) {
                changes.attributes.forEach(function (attribute) {
                    view.removeEventListener(attribute + 'Change', _this._onDynamicStateChangeHandler);
                });
            }
            if (changes.pseudoClasses) {
                changes.pseudoClasses.forEach(function (pseudoClass) {
                    var eventName = ':' + pseudoClass;
                    view.removeEventListener(eventName, _this._onDynamicStateChangeHandler);
                    if (view[eventName]) {
                        view[eventName](-1);
                    }
                });
            }
        });
        this._appliedChangeMap = CssState.emptyChangeMap;
    };
    CssState.prototype.toString = function () {
        var view = this.viewRef.get();
        if (!view) {
            Trace.write("toString() of CssState cannot execute correctly because \".viewRef\" is cleared", Trace.categories.Animation, Trace.messageType.warn);
            return '';
        }
        return "".concat(view, "._cssState");
    };
    CssState.emptyChangeMap = Object.freeze(new Map());
    CssState.emptyPropertyBag = {};
    CssState.emptyAnimationArray = Object.freeze([]);
    CssState.emptyMatch = {
        selectors: [],
        changeMap: new Map(),
        addAttribute: function () { },
        addPseudoClass: function () { },
        properties: null,
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CssState.prototype, "updateMatch", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], CssState.prototype, "updateDynamicState", null);
    return CssState;
}());
export { CssState };
CssState.prototype._appliedChangeMap = CssState.emptyChangeMap;
CssState.prototype._appliedAnimations = CssState.emptyAnimationArray;
CssState.prototype._matchInvalid = true;
var StyleScope = /** @class */ (function () {
    function StyleScope() {
        this._css = '';
        this._localCssSelectors = [];
        this._localCssKeyframes = [];
        this._localCssSelectorVersion = 0;
        this._localCssSelectorsAppliedVersion = 0;
        this._applicationCssSelectorsAppliedVersion = 0;
        this._cssFiles = [];
    }
    Object.defineProperty(StyleScope.prototype, "css", {
        get: function () {
            return this._css;
        },
        set: function (value) {
            this.setCss(value);
        },
        enumerable: false,
        configurable: true
    });
    StyleScope.prototype.addCss = function (cssString, cssFileName) {
        this.appendCss(cssString, cssFileName);
    };
    StyleScope.prototype.addCssFile = function (cssFileName) {
        this.appendCss(null, cssFileName);
    };
    StyleScope.prototype.changeCssFile = function (cssFileName) {
        if (!cssFileName) {
            return;
        }
        this._cssFiles.push(cssFileName);
        currentScopeTag = cssFileName;
        var cssFile = CSSSource.fromURI(cssFileName);
        currentScopeTag = null;
        this._css = cssFile.source;
        this._localCssSelectors = cssFile.selectors;
        this._localCssKeyframes = cssFile.keyframes;
        this._localCssSelectorVersion++;
        this.ensureSelectors();
    };
    StyleScope.prototype.setCss = function (cssString, cssFileName) {
        this._css = cssString;
        var cssFile = CSSSource.fromSource(cssString, cssFileName);
        this._localCssSelectors = cssFile.selectors;
        this._localCssKeyframes = cssFile.keyframes;
        this._localCssSelectorVersion++;
        this.ensureSelectors();
    };
    StyleScope.prototype.appendCss = function (cssString, cssFileName) {
        var _a, _b;
        if (!cssString && !cssFileName) {
            return;
        }
        if (cssFileName) {
            this._cssFiles.push(cssFileName);
            currentScopeTag = cssFileName;
        }
        var cssFile = cssString ? CSSSource.fromSource(cssString, cssFileName) : CSSSource.fromURI(cssFileName);
        currentScopeTag = null;
        this._css = this._css + cssFile.source;
        (_a = this._localCssSelectors).push.apply(_a, cssFile.selectors);
        (_b = this._localCssKeyframes).push.apply(_b, cssFile.keyframes);
        this._localCssSelectorVersion++;
        this.ensureSelectors();
    };
    StyleScope.prototype.getKeyframeAnimationWithName = function (animationName) {
        if (!this._mergedCssKeyframes) {
            return null;
        }
        var keyframeRule = this.findKeyframeRule(animationName);
        ensureKeyframeAnimationModule();
        var animation = new keyframeAnimationModule.KeyframeAnimationInfo();
        ensureCssAnimationParserModule();
        animation.keyframes = keyframeRule ? cssAnimationParserModule.CssAnimationParser.keyframesArrayFromCSS(keyframeRule.keyframes) : null;
        return animation;
    };
    StyleScope.prototype.ensureSelectors = function () {
        if (!this.isApplicationCssSelectorsLatestVersionApplied() || !this.isLocalCssSelectorsLatestVersionApplied() || !this._mergedCssSelectors) {
            this._createSelectors();
        }
        return this.getSelectorsVersion();
    };
    StyleScope.prototype._increaseApplicationCssSelectorVersion = function () {
        applicationCssSelectorVersion++;
    };
    StyleScope.prototype.isApplicationCssSelectorsLatestVersionApplied = function () {
        return this._applicationCssSelectorsAppliedVersion === applicationCssSelectorVersion;
    };
    StyleScope.prototype.isLocalCssSelectorsLatestVersionApplied = function () {
        return this._localCssSelectorsAppliedVersion === this._localCssSelectorVersion;
    };
    StyleScope.prototype._createSelectors = function () {
        var _this = this;
        var toMerge = [];
        var toMergeKeyframes = [];
        toMerge.push.apply(toMerge, mergedApplicationCssSelectors.filter(function (v) { return !v.scopedTag || _this._cssFiles.indexOf(v.scopedTag) >= 0; }));
        toMergeKeyframes.push.apply(toMergeKeyframes, mergedApplicationCssKeyframes.filter(function (v) { return !v.scopedTag || _this._cssFiles.indexOf(v.scopedTag) >= 0; }));
        this._applicationCssSelectorsAppliedVersion = applicationCssSelectorVersion;
        toMerge.push.apply(toMerge, this._localCssSelectors);
        toMergeKeyframes.push.apply(toMergeKeyframes, this._localCssKeyframes);
        this._localCssSelectorsAppliedVersion = this._localCssSelectorVersion;
        if (toMerge.length > 0) {
            this._mergedCssSelectors = toMerge;
            this._selectorScope = new StyleSheetSelectorScope(this._mergedCssSelectors);
        }
        else {
            this._mergedCssSelectors = null;
            this._selectorScope = null;
        }
        this._mergedCssKeyframes = toMergeKeyframes.length > 0 ? toMergeKeyframes : null;
    };
    // HACK: This @profile decorator creates a circular dependency
    // HACK: because the function parameter type is evaluated with 'typeof'
    StyleScope.prototype.matchSelectors = function (view) {
        var match;
        // should be (view: ViewBase): SelectorsMatch<ViewBase>
        this.ensureSelectors();
        if (this._selectorScope) {
            match = this._selectorScope.query(view);
            // Make sure to re-apply keyframes to matching selectors as a media query keyframe might be applicable at this point
            this._applyKeyframesToSelectors(match.selectors);
        }
        else {
            match = null;
        }
        return match;
    };
    StyleScope.prototype.query = function (node) {
        this.ensureSelectors();
        var match = this.matchSelectors(node);
        return match ? match.selectors : [];
    };
    StyleScope.prototype.getSelectorsVersion = function () {
        // The counters can only go up. So we can return just appVersion + localVersion
        // The 100000 * appVersion is just for easier debugging
        return 100000 * this._applicationCssSelectorsAppliedVersion + this._localCssSelectorsAppliedVersion;
    };
    StyleScope.prototype._applyKeyframesToSelectors = function (selectors) {
        if (!(selectors === null || selectors === void 0 ? void 0 : selectors.length)) {
            return;
        }
        for (var i = selectors.length - 1; i >= 0; i--) {
            var ruleset = selectors[i].ruleset;
            var animations = ruleset[animationsSymbol];
            if (animations != null && animations.length) {
                ensureCssAnimationParserModule();
                for (var _i = 0, animations_1 = animations; _i < animations_1.length; _i++) {
                    var animation = animations_1[_i];
                    var keyframeRule = this.findKeyframeRule(animation.name);
                    animation.keyframes = keyframeRule ? cssAnimationParserModule.CssAnimationParser.keyframesArrayFromCSS(keyframeRule.keyframes) : null;
                }
            }
        }
    };
    StyleScope.prototype.getAnimations = function (ruleset) {
        return ruleset[animationsSymbol];
    };
    StyleScope.prototype.findKeyframeRule = function (animationName) {
        if (!this._mergedCssKeyframes) {
            return null;
        }
        // Cache media query results to avoid validations of other identical queries
        var validatedMediaQueries;
        // Iterate in reverse order as the last usable keyframe rule matters the most
        for (var i = this._mergedCssKeyframes.length - 1; i >= 0; i--) {
            var rule = this._mergedCssKeyframes[i];
            if (rule.name !== animationName) {
                continue;
            }
            if (!rule.mediaQueryString) {
                return rule;
            }
            if (!validatedMediaQueries) {
                validatedMediaQueries = [];
            }
            var isMatchingAllQueries = matchMediaQueryString(rule.mediaQueryString, validatedMediaQueries);
            if (isMatchingAllQueries) {
                return rule;
            }
        }
        return null;
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String, Object]),
        __metadata("design:returntype", void 0)
    ], StyleScope.prototype, "setCss", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [String, Object]),
        __metadata("design:returntype", void 0)
    ], StyleScope.prototype, "appendCss", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], StyleScope.prototype, "_createSelectors", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", SelectorsMatch)
    ], StyleScope.prototype, "matchSelectors", null);
    return StyleScope;
}());
export { StyleScope };
export function resolveFileNameFromUrl(url, appDirectory, fileExists, importSource) {
    var fileName = typeof url === 'string' ? url.trim() : '';
    if (fileName.indexOf('~/') === 0) {
        fileName = fileName.replace('~/', '');
    }
    var isAbsolutePath = fileName.indexOf('/') === 0;
    var absolutePath = isAbsolutePath ? fileName : path.join(appDirectory, fileName);
    if (fileExists(absolutePath)) {
        return absolutePath;
    }
    if (!isAbsolutePath) {
        if (fileName[0] === '~' && fileName[1] !== '/' && fileName[1] !== '"') {
            fileName = fileName.substring(1);
        }
        if (importSource) {
            var importFile = resolveFilePathFromImport(importSource, fileName);
            if (fileExists(importFile)) {
                return importFile;
            }
        }
        var external_1 = path.join(appDirectory, 'tns_modules', fileName);
        if (fileExists(external_1)) {
            return external_1;
        }
    }
    return null;
}
function resolveFilePathFromImport(importSource, fileName) {
    var importSourceParts = importSource.split(path.separator);
    var fileNameParts = fileName
        .split(path.separator)
        // exclude the dot-segment for current directory
        .filter(function (p) { return !isCurrentDirectory(p); });
    // remove current file name
    importSourceParts.pop();
    // remove element in case of dot-segment for parent directory or add file name
    fileNameParts.forEach(function (p) { return (isParentDirectory(p) ? importSourceParts.pop() : importSourceParts.push(p)); });
    return importSourceParts.join(path.separator);
}
export var applyInlineStyle = profile(function applyInlineStyle(view, styleStr) {
    var localStyle = "local { ".concat(styleStr, " }");
    var inlineRuleSet = CSSSource.fromSource(localStyle).selectors;
    // Reset unscoped css-variables
    view.style.resetUnscopedCssVariables();
    // Set all the css-variables first, so we can be sure they are up-to-date
    inlineRuleSet[0].declarations.forEach(function (d) {
        // Use the actual property name so that a local value is set.
        var property = d.property;
        if (isCssVariable(property)) {
            view.style.setUnscopedCssVariable(property, d.value);
        }
    });
    inlineRuleSet[0].declarations.forEach(function (d) {
        // Use the actual property name so that a local value is set.
        var property = d.property;
        try {
            if (isCssVariable(property)) {
                // Skip css-variables, they have been handled
                return;
            }
            var value = evaluateCssExpressions(view, property, d.value);
            if (property in view.style) {
                view.style[property] = value;
            }
            else {
                view[property] = value;
            }
        }
        catch (e) {
            Trace.write("Failed to apply property [".concat(d.property, "] with value [").concat(d.value, "] to ").concat(view, ". ").concat(e), Trace.categories.Error, Trace.messageType.error);
        }
    });
    // This is needed in case of changes to css-variable or css-calc expressions.
    view._onCssStateChange();
});
function isCurrentDirectory(uriPart) {
    return uriPart === '.';
}
function isParentDirectory(uriPart) {
    return uriPart === '..';
}
function isMedia(node) {
    return node.type === 'media';
}
function isKeyframe(node) {
    return node.type === 'keyframes';
}
function isRule(node) {
    return node.type === 'rule';
}
//# sourceMappingURL=style-scope.js.map