import { Observable } from '../../data/observable';
import { Screen } from '../../platform';
import { isNumber, CORE_ANIMATION_DEFAULTS } from '../../utils';
import { querySelectorAll } from '../core/view-base';
// always increment when adding new transitions to be able to track their state
export var SharedTransitionAnimationType;
(function (SharedTransitionAnimationType) {
    SharedTransitionAnimationType[SharedTransitionAnimationType["present"] = 0] = "present";
    SharedTransitionAnimationType[SharedTransitionAnimationType["dismiss"] = 1] = "dismiss";
})(SharedTransitionAnimationType || (SharedTransitionAnimationType = {}));
var SharedTransitionObservable = /** @class */ (function (_super) {
    __extends(SharedTransitionObservable, _super);
    function SharedTransitionObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    // @ts-ignore
    SharedTransitionObservable.prototype.on = function (eventNames, callback, thisArg) {
        _super.prototype.on.call(this, eventNames, callback, thisArg);
    };
    return SharedTransitionObservable;
}(Observable));
var sharedTransitionEvents;
var currentStack;
/**
 * Shared Element Transitions (preview)
 * Allows you to auto animate between shared elements on two different screesn to create smooth navigational experiences.
 * View components can define sharedTransitionTag="name" alone with a transition through this API.
 */
var SharedTransition = /** @class */ (function () {
    function SharedTransition() {
    }
    /**
     * Configure a custom transition with presentation/dismissal options.
     * @param transition The custom Transition instance.
     * @param options
     * @returns a configured SharedTransition instance for use with navigational APIs.
     */
    SharedTransition.custom = function (transition, options) {
        SharedTransition.updateState(transition.id, __assign(__assign({}, (options || {})), { instance: transition, activeType: SharedTransitionAnimationType.present }));
        var pageEnd = options === null || options === void 0 ? void 0 : options.pageEnd;
        if (isNumber(pageEnd === null || pageEnd === void 0 ? void 0 : pageEnd.duration)) {
            // Android uses milliseconds/iOS uses seconds
            // users pass in milliseconds
            transition.setDuration(__APPLE__ ? (pageEnd === null || pageEnd === void 0 ? void 0 : pageEnd.duration) / 1000 : pageEnd === null || pageEnd === void 0 ? void 0 : pageEnd.duration);
        }
        return { instance: transition };
    };
    /**
     * Listen to various shared element transition events.
     * @returns Observable
     */
    SharedTransition.events = function () {
        if (!sharedTransitionEvents) {
            sharedTransitionEvents = new SharedTransitionObservable();
        }
        return sharedTransitionEvents;
    };
    /**
     * Notify a Shared Transition event.
     * @param id transition instance id
     * @param eventName Shared Transition event name
     * @param type TransitionNavigationType
     * @param action SharedTransitionEventAction
     */
    SharedTransition.notifyEvent = function (eventName, data) {
        switch (eventName) {
            case SharedTransition.startedEvent:
            case SharedTransition.interactiveUpdateEvent:
                SharedTransition.inProgress = true;
                break;
            default:
                SharedTransition.inProgress = false;
                break;
        }
        SharedTransition.events().notify({
            eventName: eventName,
            data: data,
        });
    };
    /**
     * Update transition state.
     * @param id Transition instance id
     * @param state SharedTransitionState
     */
    SharedTransition.updateState = function (id, state) {
        if (!currentStack) {
            currentStack = [];
        }
        var existingTransition = SharedTransition.getState(id);
        if (existingTransition) {
            // updating existing
            for (var key in state) {
                existingTransition[key] = state[key];
                // console.log(' ... updating state: ', key, state[key])
            }
        }
        else {
            currentStack.push(state);
        }
    };
    /**
     * Get current state for any transition.
     * @param id Transition instance id
     */
    SharedTransition.getState = function (id) {
        return currentStack === null || currentStack === void 0 ? void 0 : currentStack.find(function (t) { var _a; return ((_a = t.instance) === null || _a === void 0 ? void 0 : _a.id) === id; });
    };
    /**
     * Finish transition state.
     * @param id Transition instance id
     */
    SharedTransition.finishState = function (id) {
        var index = currentStack === null || currentStack === void 0 ? void 0 : currentStack.findIndex(function (t) { var _a; return ((_a = t.instance) === null || _a === void 0 ? void 0 : _a.id) === id; });
        if (index > -1) {
            currentStack.splice(index, 1);
        }
    };
    /**
     * Gather view collections based on sharedTransitionTag details.
     * @param fromPage Page moving away from
     * @param toPage Page moving to
     * @returns Collections of views pertaining to shared elements or particular pages
     */
    SharedTransition.getSharedElements = function (fromPage, toPage) {
        // 1. Presented view: gather all sharedTransitionTag views
        var presentedSharedElements = querySelectorAll(toPage, 'sharedTransitionTag').filter(function (v) { return !v.sharedTransitionIgnore && typeof v.sharedTransitionTag === 'string'; });
        // console.log('presented sharedTransitionTag total:', presentedSharedElements.length);
        // 2. Presenting view: gather all sharedTransitionTag views
        var presentingSharedElements = querySelectorAll(fromPage, 'sharedTransitionTag').filter(function (v) { return !v.sharedTransitionIgnore && typeof v.sharedTransitionTag === 'string'; });
        // console.log(
        // 	'presenting sharedTransitionTags:',
        // 	presentingSharedElements.map((v) => v.sharedTransitionTag)
        // );
        // 3. only handle sharedTransitionTag on presenting which match presented
        var presentedTags = presentedSharedElements.map(function (v) { return v.sharedTransitionTag; });
        return {
            sharedElements: presentingSharedElements.filter(function (v) { return presentedTags.includes(v.sharedTransitionTag); }),
            presented: presentedSharedElements,
            presenting: presentingSharedElements,
        };
    };
    /**
     * When the transition starts.
     */
    SharedTransition.startedEvent = 'SharedTransitionStartedEvent';
    /**
     * When the transition finishes.
     */
    SharedTransition.finishedEvent = 'SharedTransitionFinishedEvent';
    /**
     * When the interactive transition cancels.
     */
    SharedTransition.interactiveCancelledEvent = 'SharedTransitionInteractiveCancelledEvent';
    /**
     * When the interactive transition updates with the percent value.
     */
    SharedTransition.interactiveUpdateEvent = 'SharedTransitionInteractiveUpdateEvent';
    /**
     * Enable to see various console logging output of Shared Element Transition behavior.
     */
    SharedTransition.DEBUG = false;
    return SharedTransition;
}());
export { SharedTransition };
/**
 * Get dimensional rectangle (x,y,width,height) from properties with fallbacks for any undefined values.
 * @param props combination of properties conformed to SharedTransitionPageProperties
 * @param defaults fallback properties when props doesn't contain a value for it
 * @returns { x,y,width,height }
 */
export function getRectFromProps(props, defaults) {
    defaults = __assign({ x: 0, y: 0, width: getPlatformWidth(), height: getPlatformHeight() }, (defaults || {}));
    return {
        x: isNumber(props === null || props === void 0 ? void 0 : props.x) ? props === null || props === void 0 ? void 0 : props.x : defaults.x,
        y: isNumber(props === null || props === void 0 ? void 0 : props.y) ? props === null || props === void 0 ? void 0 : props.y : defaults.y,
        width: isNumber(props === null || props === void 0 ? void 0 : props.width) ? props === null || props === void 0 ? void 0 : props.width : defaults.width,
        height: isNumber(props === null || props === void 0 ? void 0 : props.height) ? props === null || props === void 0 ? void 0 : props.height : defaults.height,
    };
}
/**
 * Get spring properties with default fallbacks for any undefined values.
 * @param props various spring related properties conforming to SharedSpringProperties
 * @returns
 */
export function getSpringFromProps(props) {
    return {
        tension: isNumber(props === null || props === void 0 ? void 0 : props.tension) ? props === null || props === void 0 ? void 0 : props.tension : CORE_ANIMATION_DEFAULTS.spring.tension,
        friction: isNumber(props === null || props === void 0 ? void 0 : props.friction) ? props === null || props === void 0 ? void 0 : props.friction : CORE_ANIMATION_DEFAULTS.spring.friction,
        mass: isNumber(props === null || props === void 0 ? void 0 : props.mass) ? props === null || props === void 0 ? void 0 : props.mass : CORE_ANIMATION_DEFAULTS.spring.mass,
        velocity: isNumber(props === null || props === void 0 ? void 0 : props.velocity) ? props === null || props === void 0 ? void 0 : props.velocity : CORE_ANIMATION_DEFAULTS.spring.velocity,
        delay: isNumber(props === null || props === void 0 ? void 0 : props.delay) ? props === null || props === void 0 ? void 0 : props.delay : 0,
    };
}
/**
 * Page starting defaults for provided type.
 * @param type TransitionNavigationType
 * @returns { x,y,width,height }
 */
export function getPageStartDefaultsForType(type) {
    return {
        x: type === 'page' ? getPlatformWidth() : 0,
        y: type === 'page' ? 0 : getPlatformHeight(),
        width: getPlatformWidth(),
        height: getPlatformHeight(),
    };
}
function getPlatformWidth() {
    return __ANDROID__ ? Screen.mainScreen.widthPixels : Screen.mainScreen.widthDIPs;
}
function getPlatformHeight() {
    return __ANDROID__ ? Screen.mainScreen.heightPixels : Screen.mainScreen.heightDIPs;
}
//# sourceMappingURL=shared-transition.js.map