import { Trace } from '../trace';
import { getFileExtension } from './common';
import { SDK_VERSION } from './constants';
import { android as AndroidUtils } from './native-helper';
import { topmost } from '../ui/frame/frame-stack';
export { clearInterval, clearTimeout, setInterval, setTimeout } from '../timer';
export * from './common';
export * from './constants';
export * from './debug';
export * from './layout-helper';
export * from './macrotask-scheduler';
export * from './mainthread-helper';
export * from './native-helper';
export * from './types';
var MIN_URI_SHARE_RESTRICTED_APK_VERSION = 24;
export function GC() {
    gc();
}
export function releaseNativeObject(object) {
    __releaseNativeCounterpart(object);
}
export function openUrl(location) {
    var context = AndroidUtils.getApplicationContext();
    try {
        var intent = new android.content.Intent(android.content.Intent.ACTION_VIEW, android.net.Uri.parse(location.trim()));
        intent.addFlags(android.content.Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }
    catch (e) {
        // We don't do anything with an error. We just output it
        Trace.write("Failed to start activity for handling URL: ".concat(location), Trace.categories.Error, Trace.messageType.error);
        return false;
    }
    return true;
}
export function openUrlAsync(location) {
    return new Promise(function (resolve, reject) {
        try {
            var context = AndroidUtils.getApplicationContext();
            var intent = new android.content.Intent(android.content.Intent.ACTION_VIEW, android.net.Uri.parse(location.trim()));
            intent.addFlags(android.content.Intent.FLAG_ACTIVITY_NEW_TASK);
            context.startActivity(intent);
            resolve(true);
        }
        catch (e) {
            // We don't do anything with an error. We just output it
            Trace.write("Failed to start activity for handling URL: ".concat(location), Trace.categories.Error, Trace.messageType.error);
            resolve(false);
        }
    });
}
/**
 * Check whether external storage is read only
 *
 * @returns {boolean} whether the external storage is read only
 */
function isExternalStorageReadOnly() {
    var extStorageState = android.os.Environment.getExternalStorageState();
    if (android.os.Environment.MEDIA_MOUNTED_READ_ONLY === extStorageState) {
        return true;
    }
    return false;
}
/**
 * Checks whether external storage is available
 *
 * @returns {boolean} whether external storage is available
 */
function isExternalStorageAvailable() {
    var extStorageState = android.os.Environment.getExternalStorageState();
    if (android.os.Environment.MEDIA_MOUNTED === extStorageState) {
        return true;
    }
    return false;
}
/**
 * Detect the mimetype of a file at a given path
 *
 * @param {string} filePath
 * @returns {string} mimetype
 */
function getMimeTypeNameFromExtension(filePath) {
    var mimeTypeMap = android.webkit.MimeTypeMap.getSingleton();
    var extension = getFileExtension(filePath).replace('.', '').toLowerCase();
    return mimeTypeMap.getMimeTypeFromExtension(extension);
}
/**
 * Open a file
 *
 * @param {string} filePath
 * @returns {boolean} whether opening the file succeeded or not
 */
export function openFile(filePath, title) {
    if (title === void 0) { title = 'Open File...'; }
    var context = AndroidUtils.getApplicationContext();
    try {
        // Ensure external storage is available
        if (!isExternalStorageAvailable()) {
            Trace.write("\nExternal storage is unavailable (please check app permissions).\nApplications cannot access internal storage of other application on Android (see: https://developer.android.com/guide/topics/data/data-storage).\n", Trace.categories.Error, Trace.messageType.error);
            return false;
        }
        // Ensure external storage is available
        if (isExternalStorageReadOnly()) {
            Trace.write('External storage is read only', Trace.categories.Error, Trace.messageType.error);
            return false;
        }
        // Determine file mimetype & start creating intent
        var mimeType = getMimeTypeNameFromExtension(filePath);
        var intent = new android.content.Intent(android.content.Intent.ACTION_VIEW);
        var chooserIntent = android.content.Intent.createChooser(intent, title);
        intent.addFlags(android.content.Intent.FLAG_ACTIVITY_NEW_TASK);
        chooserIntent.addFlags(android.content.Intent.FLAG_ACTIVITY_NEW_TASK);
        // Android SDK <28 only requires starting the chooser Intent straight forwardly
        if (SDK_VERSION < MIN_URI_SHARE_RESTRICTED_APK_VERSION) {
            Trace.write("detected sdk version ".concat(SDK_VERSION, " (< ").concat(MIN_URI_SHARE_RESTRICTED_APK_VERSION, "), using simple openFile"), Trace.categories.Debug);
            intent.setDataAndType(android.net.Uri.fromFile(new java.io.File(filePath)), mimeType);
            context.startActivity(chooserIntent);
            return true;
        }
        Trace.write("detected sdk version ".concat(SDK_VERSION, " (>= ").concat(MIN_URI_SHARE_RESTRICTED_APK_VERSION, "), using URI openFile"), Trace.categories.Debug);
        // Android SDK 24+ introduced file system permissions changes that disallow
        // exposing URIs between applications
        //
        // see: https://developer.android.com/reference/android/os/FileUriExposedException
        // see: https://github.com/NativeScript/NativeScript/issues/5661#issuecomment-456405380
        var providerName = "".concat(context.getPackageName(), ".provider");
        Trace.write("fully-qualified provider name [".concat(providerName, "]"), Trace.categories.Debug);
        var apkURI = androidx.core.content.FileProvider.getUriForFile(context, providerName, new java.io.File(filePath));
        // Set flags & URI as data type on the view action
        intent.addFlags(android.content.Intent.FLAG_GRANT_READ_URI_PERMISSION);
        chooserIntent.addFlags(android.content.Intent.FLAG_GRANT_READ_URI_PERMISSION);
        // Finish intent setup
        intent.setDataAndType(apkURI, mimeType);
        context.startActivity(chooserIntent);
        return true;
    }
    catch (err) {
        var msg_1 = err.message ? ": ".concat(err.message) : '';
        Trace.write("Error in openFile".concat(msg_1), Trace.categories.Error, Trace.messageType.error);
        if (msg_1 && msg_1.includes('Attempt to invoke virtual method') && msg_1.includes('android.content.pm.ProviderInfo.loadXmlMetaData') && msg_1.includes('on a null object reference')) {
            // Alert user to possible fix
            Trace.write("\nPlease ensure you have your manifest correctly configured with the FileProvider.\n(see: https://developer.android.com/reference/android/support/v4/content/FileProvider#ProviderDefinition)\n", Trace.categories.Error);
        }
        return false;
    }
}
export function isRealDevice() {
    return AndroidUtils.isRealDevice();
}
export function dismissSoftInput(nativeView) {
    AndroidUtils.dismissSoftInput(nativeView);
}
export function dismissKeyboard() {
    var _a, _b, _c, _d, _e;
    dismissSoftInput();
    var modalDialog = (_e = (_d = ((_b = (_a = topmost()) === null || _a === void 0 ? void 0 : _a._modalParent) !== null && _b !== void 0 ? _b : (_c = topmost()) === null || _c === void 0 ? void 0 : _c.modal)) === null || _d === void 0 ? void 0 : _d._dialogFragment) === null || _e === void 0 ? void 0 : _e.getDialog();
    var view = modalDialog !== null && modalDialog !== void 0 ? modalDialog : AndroidUtils.getCurrentActivity();
    if (view) {
        var focus_1 = view.getCurrentFocus();
        if (focus_1) {
            focus_1.clearFocus();
        }
    }
}
export function copyToClipboard(value) {
    try {
        var clipboard = AndroidUtils.getApplicationContext().getSystemService(android.content.Context.CLIPBOARD_SERVICE);
        var clip = android.content.ClipData.newPlainText('Clipboard value', value);
        clipboard.setPrimaryClip(clip);
    }
    catch (err) {
        console.log(err);
    }
}
//# sourceMappingURL=index.android.js.map